<?php
require_once("lib-api-action/apiaction.php");

use Billing\Exception\ConstraintViolationException;
use Billing\Manager\CryptManager;
use Billing\Object\Contact;

class UpdateClientContact extends MB_AdminAPIAction
{
	public function execute()
	{
		parent::execute();

		if ( !is_null($id = bgpc("clientContactID")) ) {
			$contact = Factory()->Contact($id);
		} else if ( !is_null($guid = bgpc("clientContactGUID")) ) {
			$contact = Factory()->Contact($guid, "guid");
		} else if ( !is_null($externalId = bgpc("contactExternalID")) ) {
			$contact = Factory()->Contact($externalId, "external_id");
		} else if ( !is_null($id = bgpc("clientID")) ) {
			$contact = Factory()->Customer($id)->PrimaryContact;
		} else if ( !is_null($guid = bgpc("clientGUID")) ) {
			$contact = Factory()->Customer($guid, "guid")->PrimaryContact;
		} else if ( !is_null($externalId = bgpc("customerExternalID")) ) {
			$contact = Factory()->Customer($externalId, "external_id")->PrimaryContact;
		} else {
			throw new ProductException(ProductException::E_OBJECT_NOT_FOUND, "Missing contact identifier");
		}
                
		$this->collectContact($contact);
        $this->validateContact($contact);
        Service()->ContactManager->update($contact);

        return $this->output();
	}

	protected function collectContact(Contact $contact)
	{
		foreach (Contact::$attrsDescriptor as $attr_name => $attr_desc) {
			switch ($attr_name) {
                case "client_contact_id":
                case "client_id":
                    continue;
                case "language_id":
                    if ("" != ($language = bgpc("language"))) {
                        $contact->language_id = Service()->LanguageManager->getID($language);
                    } else if (is_null($contact->id)) {
                        $contact->language_id = $contact->Customer->PrimaryContact->language_id;
                    }
                    continue;
                case "countries_id":
                    if ( !is_null($country = bgpc("country")) ) {
                        $contact->countries_id = Service()->CountryManager->getID($country);
                    }
                    continue;
                case "contact_password":
                    if ( !is_null($password = bgpc("contactPassword")) ) {
                        $contact->plainPassword = $password;
                    }
                    continue;  
                default:
                    if ( false === ($param_name = getDBXMLCol($attr_name)) )
                        continue;
                    if ( is_null($param_value = bgpc($param_name)) )
                        continue;
                    $contact->$attr_name = $param_value;
            }
        }
        if ( !is_null($emailActions = bgpc("emailActions")) ) {
            $email_action_ids = array();
            if ( is_array($emailActions) ) {
                foreach ( $emailActions as $email_action_name ) {
                    $email_action_ids[] = getAPITypeID("emailAction", $email_action_name);
                }
            }
            $contact->emailActionIDs = $email_action_ids;
        }
	}

    protected function validateContact(Contact $contact)
    {
        if (count($violations = Service()->ContactManager->validate($contact))) {
            throw new ConstraintViolationException($violations);
        }
    }
}