<?php
require_once ("lib-action/action.php");
require_once ("lib-billing/include/accounting/Accounting.php");
require_once ("lib-billing/include/accounting/General.php");

class AddGeneralJournalEntryResponse extends MB_Action
{
	public function execute()
    {
		parent::execute();

        $valid = true;
        $entries = array();
        // get ledger entries from request
        try {
            $entries = $this->getRequestLedgerEntryData("addGeneral");
        } catch(ProductException $e) {
            $this->smartyVars["failreason"][] = $e->getMessage();
            $valid = false;
        }
		// validate ledger entries, make sure they add up
		if ($valid && (0 == count($entries) || 0 == $this->validateLedgerEntryData($entries))) {
            $this->smartyVars["failreason"][] = blmsg('TRANS_INVALID_LEDGER_ENTRIES');
            $valid = false;
        }

		// check the currency id for sanity
		if (strlen($_REQUEST["currencyID"])!=3){
			$this->smartyVars["failreason"][] = blmsg('TRANS_NEED_CURRENCY_ID');
			$valid = false;
		}
        
        $this->smartyVars["cango"] = ($valid) ? 1 : 0;

		// if we cant go, lets redisplay the form with
		if (!(int)$this->smartyVars["cango"]){
			$this->smartyVars["currencyID"] = $_REQUEST["currencyID"];
			$this->smartyVars["generalDescription"] = $_REQUEST["generalDescription"];
			$this->smartyVars["generalDateCreatedYear"] = $_REQUEST["generalYear"];
			$this->smartyVars["generalDateCreatedMonth"] = $_REQUEST["generalMonth"];
			$this->smartyVars["generalDateCreatedDay"] = $_REQUEST["generalDay"];
			$this->addLedgerAccounts();
			$this->smartyVars["existingEntries"] = $entries;
            $this->smartyVars["buttons"] = array(
                array('id' => 'billing-id-save', 'class' => 'action', 'name' => 'save', 'value' => "TRANS_CREATE", 'type' => 'submit', 'label' => 'TRANS_CREATE'),
            );
            $this->addTemplate("accounting/AddGeneralJournalEntry", $this->smartyVars);
		} else {

			// ok, going to build inserts because everything is ok.
			$amount = 0;
			foreach ($entries as $row){
				$amount += $row["generalLedgerDebitAmount"];
			}
			if (checkdate($_REQUEST["generalMonth"],$_REQUEST["generalDay"],$_REQUEST["generalYear"])) {
				if (intval($_REQUEST["generalHour"]) <= 23) {
					$generalHour = (int) $_REQUEST["generalHour"];
				} else {
					$generalHour = date("h");
				}
				if (intval($_REQUEST["generalMinute"]) <= 59) {
					$generalMinute = (int) $_REQUEST["generalMinute"];
				} else {
					$generalMinute = date("i");
				}
				$generalDateCreated = mktime($generalHour,$generalMinute,0,$_REQUEST["generalMonth"],$_REQUEST["generalDay"],$_REQUEST["generalYear"]);
			} else {
				$generalDateCreated = time();
			}

            $generalDescription = bgpc("generalDescription");
            $currencyID = bgpc("currencyID");

			// run the general journal entry object with its options
			$gen = new General();
            $gen->setVendorID(getCurrentSession()->getVendorId());
            $gen->setClientID(0);
			$gen->setInvoiceID(0);
			$gen->setDescription($generalDescription);
			$gen->setTotalDebits($amount);
			$gen->setTotalCredits($amount);
			$gen->setCurrencyID($currencyID);
			$gen->setLedgerEntries($entries);
			$gen->updateTimestamp = $generalDateCreated;
			$gen->execute();

			if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
				redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
			} else {
				redirect("accounting.php?action=".getActionID("ShowGeneralJournal")."&sid=".secureInput($_REQUEST["sid"]));
			}

		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
