<?php
set_time_limit(0);
require_once("lib-action/action.php");
require_once ("lib-billing/include/accounting/Accounting.php");


class ShowBalanceSheet extends MB_Action {

	public function __construct()
    {
		parent::MB_Action();
		list($m, $d, $y) = explode("/", $_REQUEST["dateRange"]);
		$this->endTime = @mktime(0,0,0,$m,$d+1,$y)-1;
		unset($_REQUEST["dateRange"]);

	}

	public function execute() {
		global $CONF;
		parent::execute();
	    $this->smartyVars["nowDate"] = date("m/d/Y");
	    $this->smartyVars["nowDateParts"] = explode("/", date("m/d/Y"));
	    $this->smartyVars["accountTypes"] = $CONF["ledgerCategories"];
	    $this->smartyVars["endDate"] = $this->endTime;

		// --------------------------------------------------------------
		// Will return all ledger accounts defined...
		// Example Record:
	    // 		[ledgerAccountID] => 1
	    // 		[parentLedgerAccountID] => 0
	    // 		[ledgerAccountNumber] => 10000
	    // 		[ledgerAccountType] => 1
	    // 		[ledgerAccountName] => cash
	    // 		[ledgerAccountTransKey] => TRANS_CASH
		// --------------------------------------------------------------
	    $accountsXML = "
	                <mbapi>
	                   <command>GetAccountingLedgerAccounts</command>
	                   <showXMLHeader>0</showXMLHeader>
	                   <params>
	                     <sortDir>ASC</sortDir>
	                     <sortColumn>ledgerAccountNumber</sortColumn>
	                     <recordOffset>0</recordOffset>
	                     <numRecords>-1</numRecords>
	                   </params>
	                </mbapi>
	                ";
	    $accounts = dispatchMBAPI($accountsXML, 1);
	    $this->smartyVars["ledgerAccounts"] = array();
	    if (isset($accounts["mbapi"][0]["results"][0]["ledger_accounts"][0]["account"])){
	        foreach ($accounts["mbapi"][0]["results"][0]["ledger_accounts"][0]["account"] as $ledgerAccount){
	            $this->smartyVars["ledgerAccounts"][$ledgerAccount["ledgerAccountID"][0]] = collapseArrayElements($ledgerAccount);
	        }
	    }

		// --------------------------------------------------------------
		// Will return all ledger entries grouped by type and summed...
		// Example Record:
		// 		[ledgerAccountID] => 4
		// 		[currencyID] => USD
		// 		[ledgerAccountType] => 1
		// 		[ledgerAccountName] => accounts_receivable
		// 		[ledgerAccountTransKey] => TRANS_ACCOUNTS_RECEIVABLE
		// 		[generalLedgerDebitAmount] => 15881990.70
		// 		[generalLedgerCreditAmount] => 15769338.54
		// --------------------------------------------------------------
	    $ledgerEntriesXML = "
	                <mbapi>
	                  <command>GetAccountingGeneralLedger</command>
	                  <showXMLHeader>0</showXMLHeader>
	                  <params>
	                    <generalLedgerDateCreatedGreatest>".$this->endTime."</generalLedgerDateCreatedGreatest>
	                    <groupByAndSum>1</groupByAndSum>
	                    <sortDir>ASC</sortDir>
	                    <sortColumn>generalLedgerID</sortColumn>
	                    <recordOffset>0</recordOffset>
	                    <numRecords>-1</numRecords>
	                    <vendorID>" . getCurrentSession()->getVendorId() . "</vendorID>
	                  </params>
	                </mbapi>
	                ";
	    $ledgerEntries = dispatchMBAPI($ledgerEntriesXML, 1);
	    $this->smartyVars["accountTotals"] = array();
	    if (isset($ledgerEntries["mbapi"][0]["results"][0]["accounting_general_ledger"][0]["ledger"])){
	        foreach ($ledgerEntries["mbapi"][0]["results"][0]["accounting_general_ledger"][0]["ledger"] as $entry){
	            $this->smartyVars["accountTotals"][$entry["currencyID"][0]][$entry["ledgerAccountID"][0]]["accountInfo"] = collapseArrayElements($entry);
	            $this->smartyVars["accountTotals"][$entry["currencyID"][0]][$entry["ledgerAccountID"][0]]["credits"] += $entry["generalLedgerCreditAmount"][0];
	            $this->smartyVars["accountTotals"][$entry["currencyID"][0]][$entry["ledgerAccountID"][0]]["debits"] += $entry["generalLedgerDebitAmount"][0];
	        }
	    }

	    $this->smartyVars["ledgerTypeTotals"] = array();
	    $this->smartyVars["ledgerAccountsTotals"] = array();
	    foreach ($this->smartyVars["accountTotals"] as $currencyID => $accountTotals){
	        $this->smartyVars["ledgerTypeTotals"][$currencyID] = array();
	        foreach ($accountTotals as $accountID => $totalInfo){
	            $accountType = $CONF["ledgerCategories"][$totalInfo["accountInfo"]["ledgerAccountType"]][0];

	            switch ($accountType){

	                case "asset":
	                    $totalInfo["total"] = $totalInfo["debits"]-$totalInfo["credits"];
	                    $this->smartyVars["ledgerAccountsTotals"][$currencyID]["asset"][] = $totalInfo;
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["asset"] -= $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["asset"] += $totalInfo["debits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] -= $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] += $totalInfo["debits"];
	                break;

	                case "liability":
	                    $totalInfo["total"] = $totalInfo["credits"]-$totalInfo["debits"];
	                    $this->smartyVars["ledgerAccountsTotals"][$currencyID]["liability"][] = $totalInfo;
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["liability"] += $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["liability"] -= $totalInfo["debits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] -= $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] += $totalInfo["debits"];
	                break;

	                case "equity":
	                    $totalInfo["total"] = $totalInfo["credits"]-$totalInfo["debits"];
	                    $this->smartyVars["ledgerAccountsTotals"][$currencyID]["equity"][] = $totalInfo;
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] += $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] -= $totalInfo["debits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] += $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] -= $totalInfo["debits"];
	                break;

	                case "revenue":
	                case "otherRev":
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equityretained"] += $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equityretained"] -= $totalInfo["debits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] += $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] -= $totalInfo["debits"];
	                break;

	                case "cogs":
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equityretained"] += $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equityretained"] -= $totalInfo["debits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] += $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] -= $totalInfo["debits"];
	                break;

	                case "expense":
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equityretained"] += $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equityretained"] -= $totalInfo["debits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] += $totalInfo["credits"];
	                    $this->smartyVars["ledgerTypeTotals"][$currencyID]["equity"] -= $totalInfo["debits"];
	                break;

	            }

	        }
	    }

	    ksort($this->smartyVars["ledgerTypeTotals"]);

		$this->addTemplate("accounting/ShowBalanceSheet");

	} // execute
}
