<?php
set_time_limit(0);
require_once("lib-action/action.php");
require_once ("lib-billing/include/accounting/Accounting.php");

class ShowProfitAndLoss extends MB_Action
{

	public function __construct()
    {
		parent::MB_Action();

        $this->beginMonth = ((int)$_REQUEST["profitAndLossBeginMonth"])?(int)$_REQUEST["profitAndLossBeginMonth"]:1;
        $this->beginDay = ((int)$_REQUEST["profitAndLossBeginDay"])?(int)$_REQUEST["profitAndLossBeginDay"]:1;
        $this->beginYear = ((int)$_REQUEST["profitAndLossBeginYear"])?(int)$_REQUEST["profitAndLossBeginYear"]:date("Y");

        $this->endMonth = ((int)$_REQUEST["profitAndLossEndMonth"])?(int)$_REQUEST["profitAndLossEndMonth"]:1;
        $this->endDay = ((int)$_REQUEST["profitAndLossEndDay"])?(int)$_REQUEST["profitAndLossEndDay"]:1;
        $this->endYear = ((int)$_REQUEST["profitAndLossEndYear"])?(int)$_REQUEST["profitAndLossEndYear"]:date("Y");

        $this->startTime = mktime(0,0,0,$this->beginMonth,$this->beginDay,$this->beginYear);
        $this->endTime = mktime(0,0,0,$this->endMonth,$this->endDay+1,$this->endYear)-1;

        $this->startYearTime = mktime(0,0,0,1,1,$this->beginYear);

    }

	public function execute()
    {
		global $CONF;
		parent::execute();

        $this->smartyVars["accountTypes"] = $CONF["ledgerCategories"];

        $this->smartyVars["startDate"] = $this->startTime;
        $this->smartyVars["endDate"] = $this->endTime;

        $accountsXML = "
                    <mbapi>
                       <command>GetAccountingLedgerAccounts</command>
                       <showXMLHeader>0</showXMLHeader>
                       <params>
                         <sortDir>ASC</sortDir>
                         <sortColumn>ledgerAccountNumber</sortColumn>
                         <recordOffset>0</recordOffset>
                         <numRecords>-1</numRecords>
                       </params>
                    </mbapi>
                    ";
        $accounts = dispatchMBAPI($accountsXML, 1);

        $this->smartyVars["ledgerAccounts"] = array();
        if (isset($accounts["mbapi"][0]["results"][0]["ledger_accounts"][0]["account"])){
            foreach ($accounts["mbapi"][0]["results"][0]["ledger_accounts"][0]["account"] as $ledgerAccount){
                $this->smartyVars["ledgerAccounts"][$ledgerAccount["ledgerAccountID"][0]] = collapseArrayElements($ledgerAccount);
            }
        }

        $ledgerEntriesXML = "
                    <mbapi>
                      <command>GetAccountingGeneralLedger</command>
                      <showXMLHeader>0</showXMLHeader>
                      <params>
                        <generalLedgerDateCreatedLeast>".$this->startTime."</generalLedgerDateCreatedLeast>
                        <generalLedgerDateCreatedGreatest>".$this->endTime."</generalLedgerDateCreatedGreatest>
                        <groupByAndSum>1</groupByAndSum>
                        <sortDir>ASC</sortDir>
                        <sortColumn>generalLedgerID</sortColumn>
                        <recordOffset>0</recordOffset>
                        <numRecords>-1</numRecords>
                        <vendorID>" . getCurrentSession()->getVendorId() . "</vendorID>
                      </params>
                    </mbapi>
                    ";
        $ledgerEntries = dispatchMBAPI($ledgerEntriesXML, 1);

        $ytdLedgerEntriesXML = "
                    <mbapi>
                      <command>GetAccountingGeneralLedger</command>
                      <showXMLHeader>0</showXMLHeader>
                      <params>
                        <generalLedgerDateCreatedLeast>".$this->startYearTime."</generalLedgerDateCreatedLeast>
                        <generalLedgerDateCreatedGreatest>".$this->endTime."</generalLedgerDateCreatedGreatest>
                        <groupByAndSum>1</groupByAndSum>
                        <sortDir>ASC</sortDir>
                        <sortColumn>generalLedgerID</sortColumn>
                        <recordOffset>0</recordOffset>
                        <numRecords>-1</numRecords>
                        <vendorID>" . getCurrentSession()->getVendorId() . "</vendorID>
                      </params>
                    </mbapi>
                    ";
        $ytdLedgerEntries = dispatchMBAPI($ytdLedgerEntriesXML, 1);

        $this->smartyVars["accountTotals"] = array();
        $this->smartyVars["ytdAccountTotals"] = array();
        $CONF['debug']->debugMessage("Entering ledger loops", 1);

        if (isset($ledgerEntries["mbapi"][0]["results"][0]["accounting_general_ledger"][0]["ledger"])){
             foreach ($ledgerEntries["mbapi"][0]["results"][0]["accounting_general_ledger"][0]["ledger"] as $entry){
                 $this->smartyVars["accountTotals"][$entry["currencyID"][0]][$entry["ledgerAccountID"][0]]["accountInfo"] = collapseArrayElements($entry);
                 $this->smartyVars["accountTotals"][$entry["currencyID"][0]][$entry["ledgerAccountID"][0]]["credits"] += $entry["generalLedgerCreditAmount"][0];
                 $this->smartyVars["accountTotals"][$entry["currencyID"][0]][$entry["ledgerAccountID"][0]]["debits"] += $entry["generalLedgerDebitAmount"][0];
             }
        }

        if (isset($ytdLedgerEntries["mbapi"][0]["results"][0]["accounting_general_ledger"][0]["ledger"])){
             foreach ($ytdLedgerEntries["mbapi"][0]["results"][0]["accounting_general_ledger"][0]["ledger"] as $entry){
                 $this->smartyVars["ytdAccountTotals"][$entry["currencyID"][0]][$entry["ledgerAccountID"][0]]["accountInfo"] = collapseArrayElements($entry);
                 $this->smartyVars["ytdAccountTotals"][$entry["currencyID"][0]][$entry["ledgerAccountID"][0]]["credits"] += $entry["generalLedgerCreditAmount"][0];
                 $this->smartyVars["ytdAccountTotals"][$entry["currencyID"][0]][$entry["ledgerAccountID"][0]]["debits"] += $entry["generalLedgerDebitAmount"][0];
             }
        }

        $this->smartyVars["ledgerTypeTotals"] = array();
        $this->smartyVars["ledgerAccountsTotals"] = array();
        $CONF['debug']->debugMessage("Setting type totals and account totals for ytd", 1);

//////////////////////////////////////////////////////////////////////////////////////////////
        foreach ($this->smartyVars["ytdAccountTotals"] as $currencyID => $accountTotals){
            if (!$this->smartyVars["ledgerTypeTotals"][$currencyID])$this->smartyVars["ledgerTypeTotals"][$currencyID] = array();
            foreach ($accountTotals as $accountID => $totalInfo){
                $accountType = $CONF["ledgerCategories"][$totalInfo["accountInfo"]["ledgerAccountType"]][0];

                switch ($accountType){
                    case "revenue":
                        $totalInfo["ytdtotal"] = $totalInfo["credits"]-$totalInfo["debits"];
                        $this->smartyVars["ledgerAccountsTotals"][$currencyID]["revenue"][$totalInfo["accountInfo"]["ledgerAccountID"]] = $totalInfo;
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdrevenue"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdrevenue"] -= $totalInfo["debits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdtotal"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdtotal"] -= $totalInfo["debits"];
                    break;
                    case "otherRev":
                        $totalInfo["ytdtotal"] = $totalInfo["credits"]-$totalInfo["debits"];
                        $this->smartyVars["ledgerAccountsTotals"][$currencyID]["otherRev"][$totalInfo["accountInfo"]["ledgerAccountID"]] = $totalInfo;
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdotherRev"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdotherRev"] -= $totalInfo["debits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdtotal"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdtotal"] -= $totalInfo["debits"];
                    break;
                    case "cogs":
                        $totalInfo["ytdtotal"] = $totalInfo["debits"]-$totalInfo["credits"];
                        $this->smartyVars["ledgerAccountsTotals"][$currencyID]["cogs"][$totalInfo["accountInfo"]["ledgerAccountID"]] = $totalInfo;
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdcogs"] -= $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdcogs"] += $totalInfo["debits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdtotal"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdtotal"] -= $totalInfo["debits"];
                    break;
                    case "expense":
                        $totalInfo["ytdtotal"] = $totalInfo["debits"]-$totalInfo["credits"];
                        $this->smartyVars["ledgerAccountsTotals"][$currencyID]["expense"][$totalInfo["accountInfo"]["ledgerAccountID"]] = $totalInfo;
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdexpense"] -= $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdexpense"] += $totalInfo["debits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdtotal"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdtotal"] -= $totalInfo["debits"];
                    break;
                    case "otherExp":
                        $totalInfo["ytdtotal"] = $totalInfo["debits"]-$totalInfo["credits"];
                        $this->smartyVars["ledgerAccountsTotals"][$currencyID]["otherExp"][$totalInfo["accountInfo"]["ledgerAccountID"]] = $totalInfo;
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdotherExp"] -= $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdotherExp"] += $totalInfo["debits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdtotal"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["ytdtotal"] -= $totalInfo["debits"];
                    break;
                }

            }
        }

        $CONF['debug']->debugMessage("Setting Range type totals and account totals", 1);

//////////////////////////////////////////////////////////////////////////////////////////////
        foreach ($this->smartyVars["accountTotals"] as $currencyID => $accountTotals){
            if (!$this->smartyVars["ledgerTypeTotals"])$this->smartyVars["ledgerTypeTotals"][$currencyID] = array();
            foreach ($accountTotals as $accountID => $totalInfo){
                $accountType = $CONF["ledgerCategories"][$totalInfo["accountInfo"]["ledgerAccountType"]][0];

                switch ($accountType){
                    case "revenue":
                        $totalInfo["total"] = $totalInfo["credits"]-$totalInfo["debits"];
                        $this->smartyVars["ledgerAccountsTotals"][$currencyID]["revenue"][$totalInfo["accountInfo"]["ledgerAccountID"]]["total"] = $totalInfo["total"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["revenue"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["revenue"] -= $totalInfo["debits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] -= $totalInfo["debits"];
                    break;
                    case "otherRev":
                        $totalInfo["total"] = $totalInfo["credits"]-$totalInfo["debits"];
                        $this->smartyVars["ledgerAccountsTotals"][$currencyID]["otherRev"][$totalInfo["accountInfo"]["ledgerAccountID"]]["total"] = $totalInfo["total"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["otherRev"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["otherRev"] -= $totalInfo["debits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] -= $totalInfo["debits"];
                    break;
                    case "cogs":
                        $totalInfo["total"] = $totalInfo["debits"]-$totalInfo["credits"];
                        $this->smartyVars["ledgerAccountsTotals"][$currencyID]["cogs"][$totalInfo["accountInfo"]["ledgerAccountID"]]["total"] = $totalInfo["total"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["cogs"] -= $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["cogs"] += $totalInfo["debits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] -= $totalInfo["debits"];
                    break;
                    case "expense":
                        $totalInfo["total"] = $totalInfo["debits"]-$totalInfo["credits"];
                        $this->smartyVars["ledgerAccountsTotals"][$currencyID]["expense"][$totalInfo["accountInfo"]["ledgerAccountID"]]["total"] = $totalInfo["total"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["expense"] -= $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["expense"] += $totalInfo["debits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] -= $totalInfo["debits"];
                    break;
                    case "otherExp":
                        $totalInfo["total"] = $totalInfo["debits"]-$totalInfo["credits"];
                        $this->smartyVars["ledgerAccountsTotals"][$currencyID]["otherExp"][$totalInfo["accountInfo"]["ledgerAccountID"]]["total"] = $totalInfo["total"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["otherExp"] -= $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["otherExp"] += $totalInfo["debits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] += $totalInfo["credits"];
                        $this->smartyVars["ledgerTypeTotals"][$currencyID]["total"] -= $totalInfo["debits"];
                    break;
                }

            }
        }

        $CONF['debug']->debugMessage("Getting ready to display template", 1);
        ksort($this->smartyVars["ledgerTypeTotals"]);
		$this->addTemplate("accounting/ShowProfitAndLoss");

	} // execute
}
