<?php

require_once("lib-action/action.php");
require_once("lib-mbapi/mbapi.php");

class ShowAdminGroupDetails extends MB_Action
{
	public $params = array("adminGroupID");

	public function execute()
	{
		parent::execute();
		if(!$adminGroupID = bgpc("adminGroupID")) {
            billing_put_error(blmsg("TRANS_NO_ADMIN_GROUPS_FOUND"));
            redirect('admins.php', getActionID("ShowAdminGroups"));
		}

        // --------------------------------------
        // Fetch config_groups for the nav menu
        // --------------------------------------
        $xmlParams = array (
            "parentConfigGroupID"=>0,
            "getNumChildren"=>1,
            "getLevelOneChildren"=>1
        );
        $tmp = $this->params;
        $this->params = array();
        $query = $this->getMBAPIQuery("GetConfigGroups","",$xmlParams);
        $this->params = $tmp;
        $navConfigGroupData = dispatchMBAPI($query, 1);
        $navConfigGroupsResults =& $navConfigGroupData["mbapi"][0]["results"][0];
        // fix child config groups structure when only 1 child group exists
        $numNavGroups = $navConfigGroupData["mbapi"][0]["header"][0]["numResults"][0];
        for($i=0;$i<$numNavGroups;$i++) {
            $c =& $navConfigGroupsResults["configGroups"][0]["configGroup"][$i];
            if((int) $c["numChildren"][0]) {
                $c["levelOneConfigGroups"][0]["levelOneConfigGroup"][0] =
                    array(0 => @$c["levelOneConfigGroups"][0]["levelOneConfigGroup"][0]);
            }
        }

        // fetch all admins
        $query = $this->getMBAPIQuery("GetAdmins","",array("sortDir"=>"asc","sortColumn"=>"adminLastName"),0,0,0,0,0,$this->params);
        $data = dispatchMBAPI($query, 1);

        // assign all admins
        $allAdmins = $data["mbapi"][0]["results"][0]["admins"][0]["admin"];
        if ($data["mbapi"][0]["header"][0]["numResults"][0]==1) {
            $allAdmins = array(0=>$data["mbapi"][0]["results"][0]["admins"][0]["admin"][0]);
        }

        if(!is_array($allAdmins)) {
            billing_put_error(blmsg("TRANS_UNABLE_TO_FETCH_ALL_ADMINS"));
            redirect('admins.php', getActionID("ShowAdminGroups"));
        }

        // collect ALL adminIDs in an array to use later
        $allAdminIDs = array();
        foreach($allAdmins as $admin) {
            $allAdminIDs[] = $admin["adminID"][0];
        }

        // fetch this admin group
        $query = $this->getMBAPIQuery("GetAdminGroups","",array  (
            "getAdminData" => 1,
            "getActionData" => 1,
        ));
        $data = dispatchMBAPI($query, 1);

        if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {
            if(((int) $data["mbapi"][0]["header"][0]["numResults"][0]) > 0) {

                if ($data["mbapi"][0]["results"][0]["adminGroups"][0]["adminGroup"][0]['isRootGroup'][0]) {
                    $this->removeMasterAdminFromMbapiResult($data["mbapi"][0]["results"][0]["adminGroups"][0]["adminGroup"][0]['admins'][0]['admin']);
                    $this->removeMasterAdminIdFromCollectionAdminIds($allAdminIDs);
                }

                $smartyVars = array();

                // --------------------------------------
                // Data from: admin_groups
                // --------------------------------------

                // set adminGroup smarty var (only one group returned)
                $smartyVars["adminGroup"] =& $data["mbapi"][0]["results"][0]["adminGroups"][0]["adminGroup"][0];

                // --------------------------------------
                // Relations from: admin_group_rel
                // --------------------------------------

                // set adminsInGroup smarty var & correct offset if only 1 admin is returned
                // this will be an array of all admin data where the admin's are assigned to this group
                $smartyVars["adminsInGroup"] = array();
                if(is_array($smartyVars["adminGroup"]["admins"][0]["admin"])) {
                    $smartyVars["adminsInGroup"] = $smartyVars["adminGroup"]["admins"][0]["admin"];
                    if(count($smartyVars["adminGroup"]["admins"][0]["admin"])==1) {
                        $smartyVars["adminsInGroup"] = array($smartyVars["adminGroup"]["admins"][0]["admin"][0]);
                    }
                    foreach($smartyVars["adminsInGroup"] as $admin) {
                        if(is_array($admin["adminID"]) && count($admin["adminID"]) && ((int) $admin["adminID"][0])) {
                            $smartyVars["adminInOptions"][$admin["adminID"][0]] = $admin["adminLastName"][0].", "
                                .$admin["adminFirstName"][0];
                        }
                    }
                }

                // nuke nested admins data from adminGroup param, this makes the smarty data easier to deal with
                unset($smartyVars["adminGroup"]["admins"]);

                // assign all adminIDs in the group to $adminIDsInGroup for later use
                $adminIDsInGroup = array();
                foreach($smartyVars["adminsInGroup"] as $admin) {
                    if(is_array($admin["adminID"]) && count($admin["adminID"])) {
                        $adminIDsInGroup[] = $admin["adminID"][0];
                    }
                }

                // assign all adminIDs NOT in the group to $adminIDsNotInGroup for later use
                $adminIDsNotInGroup = array();
                foreach($allAdminIDs as $adminID) {
                    if (!in_array($adminID,$adminIDsInGroup)) {
                        $adminIDsNotInGroup[] = $adminID;
                    }
                }

                // set adminsNotInGroup smarty var
                // this will be an array of all admin data where the admin's are NOT assigned to this group
                $lookupAdminID = array_flip($allAdminIDs);
                $smartyVars["adminsNotInGroup"] = array();
                foreach($adminIDsNotInGroup as $adminID) {
                    $smartyVars["adminsNotInGroup"][] = $allAdmins[$lookupAdminID[$adminID]];
                }

                // set adminInOptions for the select list of admins NOT in this group
                foreach($smartyVars["adminsNotInGroup"] as $admin) {
                    if (((int) $admin["adminID"][0])) {
                        $smartyVars["adminNotInOptions"][$admin["adminID"][0]] = $admin["adminLastName"][0].", ".$admin["adminFirstName"][0];
                    }
                }

                // --------------------------------------
                // Relations from: admin_group_action_rel
                // --------------------------------------

                $formElements = $actionCategoriesEven = $actionCategoriesOdd = $adminGroupActions = array();
                $rawCategories = getActionCategories();
                if (is_array($smartyVars["adminGroup"]["actions"][0]["actionName"])) {
                    $adminGroupActions = $smartyVars["adminGroup"]["actions"][0]["actionName"];
                }

                $i = 1;

                foreach($rawCategories as $catName => $actions) {
                    if ($i %2 === 0) {
                        foreach($actions as $actionName) {
                            if (in_array($actionName,$adminGroupActions)) {
                                $actionCategoriesEven[$catName]["comboAdd"][] = $actionName;
                            } else {
                                $actionCategoriesEven[$catName]["comboRemove"][] = $actionName;
                            }
                        }
                        $formElements[] = "comboAdd_$catName";
                    } else {
                        foreach($actions as $actionName) {
                            if (in_array($actionName,$adminGroupActions)) {
                                $actionCategoriesOdd[$catName]["comboAdd"][] = $actionName;
                            } else {
                                $actionCategoriesOdd[$catName]["comboRemove"][] = $actionName;
                            }
                        }
                        $formElements[] = "comboAdd_$catName";
                    }
                    $i++;
                }

                // --------------------------------------
                // Show template
                // --------------------------------------

                $smartyVars["navConfigGroups"] =& $navConfigGroupsResults["configGroups"][0]["configGroup"];
                $smartyVars = collapseArrayElements($smartyVars);
                $smartyVars["adminGroupActions"] =& $adminGroupActions;
                $smartyVars["actionCategoriesEven"] =& $actionCategoriesEven;
                $smartyVars["actionCategoriesOdd"] =& $actionCategoriesOdd;
                $smartyVars["formElements"] =& $formElements;
                $smartyVars["numActionCategories"] = $i-1;
                $this->addTemplate("admins/ShowAdminGroupDetails",$smartyVars, 1);

            } else {
                $this->addError('TRANS_ACTIONERROR', "AdminGroup does not exist.");
            }
        }

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

	public function removeMasterAdminFromMbapiResult(&$admins)
	{
		if (Service()->AdminManager->getMasterAdmin()->admin_username == $admins[0]['adminUsername'][0]) {
			unset($admins[0]['adminID'][0]);
		}

		return $admins;
	}

	public function removeMasterAdminIdFromCollectionAdminIds(&$adminIds)
	{
		$keyForAdminId = array_search(Service()->AdminManager->getMasterAdmin()->id, $adminIds);
		unset($adminIds[$keyForAdminId]);

		return $adminIds;
	}
}