<?php

require_once("lib-action/action.php");

use Billing\Object\Admin;

class ShowAdmins extends MB_Action
{
	public function execute()
	{
		parent::execute();

		if (1 == bgpc("sync")) {
			$this->processSync();
		}

		$page = UIFactory()->Page();
		$page->title = blmsg("TRANS_ADMINS");
		$page->parent_title = blmsg("TRANS_MANAGE_SYSTEM");
		$page->parent_url = admin_action_url("configs", "system", "show");

		require_once("UI/List/UIAdminList.php");
		$page->addListPanel(new UIAdminList());

		$this->smartyVars["page"] = $page;
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}

	private function processSync()
	{
		$adminIDs = bgpc("id", array());
		if (empty($adminIDs)) {
			return;
		}
		$errors = array();
		$leftSyncingAdmins = false;
		foreach ($adminIDs as $adminID) {
			try {
				$admin = Factory()->Admin($adminID);
				if ($admin->sync_tasks & Admin::SYNC_CHECK_UPDATE_STATUS) {
					// check previous update operation status only
				} else {
					$admin->sync_tasks |= Admin::SYNC_NEED_UPDATE | Admin::SYNC_NEED_REPAIR_SSO;
				}
				Service()->AdminManager->sync($admin);
				if (!is_null($admin->sync_error)) {
					$errors[] = blmsg("TRANS_SYNC_ADMIN_ERROR_MSG", array("ADMIN_ID" => $admin->id, "ERROR" => $admin->sync_error));
				} else if (!$admin->isSynced() && $admin->isSyncable()) {
					$leftSyncingAdmins = true;
				}
			} catch (ProductException $e) {
				$errors[] = blmsg("TRANS_SYNC_ADMIN_ERROR_MSG", array("ADMIN_ID" => $adminID, "ERROR" => $e->getMessage()));
			}
		}
		if (empty($errors)) {
			getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_SYNC_ADMINS_SUCCESS_MSG");
			if ($leftSyncingAdmins) {
				billing_put_warning(blmsg("TRANS_SYNC_ADMINS_PENDING_MSG"));
			}
		} else {
			getCurrentSession()->Storage->errorMessages[] = blmsg(
				"TRANS_SYNC_ADMINS_FAILURE_MSG",
				array("ERROR" => billing_safetyhtml(implode("\n", $errors)))
			);
		}
	}
}
