<?php

use Billing\Manager\AdminManager;
use Billing\Manager\SecurityManager;

class Admins_MyProfileController extends Billing_Zend_Controller_Action_AdminController
{
    public function init()
    {
        parent::init();
        $this->_backUrl = $this->view->adminActionUrl('dashboard', 'home', 'index');
    }

    public function editAction()
    {
        $admin = Service()->AdminManager->getAdmin();

        $this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_ADMIN_EDIT_MY_PROFILE'));
        $editForm = new Billing_Zend_Form_Final_AdminProfileForm(array('context' => array(
            'cancelLink' =>  $this->_backUrl,
            'admin' => $admin,
            'showAdminGroups' => false
        )));
        
        if (Service()->SecurityManager->isPasswordExpired()) {
            $this->_status->addWarning($this->_translator->translate("TRANS_ADMIN_PASSWORD_WAS_EXPIRED_NEED_TO_CHANGE", array("DAYS"  => Settings()->security_PasswordExpirationInterval)));
        }

        if ($admin->isSyncable() && !is_null($admin->sync_error)) {
            $this->_status->addError($this->_translator->translate("TRANS_SYNC_THIS_ADMIN_ERROR_MSG", array("ERROR" => billing_safetyhtml($admin->sync_error))));
        }
                
        if ($this->_request->isPost() && $editForm->isValid($this->_request->getPost()))
        {
            try {
                $editForm->processForm();
                $this->_redirectWithMessage($this->_backUrl, $this->_translator->translate("TRANS_ADMIN_EDIT_MY_PROFILE_SUCCESS_MSG"), "info");
            } catch (ProductException $e) {
                $this->_status->addError($e->getMessage());
            }
        }
        $this->view->form = $editForm;
    }
}