<?php

require_once("lib-action/action.php");
require_once("lib-mbapi/mbapi.php");
require_once("lib-billing/include/config/internal/certs.php");
require_once("lib-billing/include/config/internal/datasources.php");
require_once("lib-tk/toolkit.php");


class OrderCertCSRRequestResponse extends MB_Action
{

	function OrderCertCSRRequestResponse() {
		parent::MB_Action();
		$this->stringRep = "OrderCertCSRRequestResponse";

		$this->params = array(
			"CSRCSR",
			"RIRenewalIndicator",
			"UPGCUIndicator",
			"certID",
			"clientID");
	}

	function execute() {
		global $CONF;
		parent::execute();

		$this->setReferralActionID( getActionID("OrderCertCSRRequest") );
		$this->setReferralActionFile( "certs.php" );

		$success = $this->validateMultiInputs(
				array(
					"CSRCSR" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
					), array(), array(), 0, array("emptyCSR" => 1, "certID" => (int)$_REQUEST["certID"]));

		if ($success) {
			$query = $this->getMBAPIQuery("GetCerts", "", array("certID" => (int)$_REQUEST["certID"]));
			$data = grabSimpleMBAPIResults( dispatchMBAPI( $query, 1) );
			$data = $data[0];

			$orderParams = array(
					"subCommand" => "ValidateOrderParameters",
					"ProductCode" => $data["ODProductCode"],
					"Method" => "RESELLER",
					"ValidityPeriod" => $data["ODValidityPeriod"],
					"ServerCount" => (notZero($data["ODServerCount"]) ? $data["ODServerCount"] : 1),
					"CSR" => $_REQUEST["CSRCSR"],
					"WebServerType" => "other",
					"WildCard" => 'false',
					"CUIndicator" => 'false',
					"CUCertificate" => null,
					"RenewalIndicator" => 'false',
					"RenewalBehavior" => "GEOTRUST",
					"SpecialInstructions" => null
					);
			if (notNull( $data["orID"] )) {
				$orderParams["orID"] = $data["orID"];
			}

			dbg( "########## Package sent to SSLF ############" );
			dbg( $orderParams );

			$query = $this->getMBAPIQuery( "ProcessSSLfactory", "", $orderParams, 0, 0, 0, 0, 1, array_keys( $_REQUEST ) );
			$data = dispatchMBAPI( $query, 1 );

			dbg( "########## Returned Package from SSLF #############" );
			dbg($data);

			if ($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {
					$results = correctArray($data["mbapi"][0]["results"][0]);

					$params = array(
							"CSRDomainName" => $results["PARSEDCSR"][0]["DOMAINNAME"],
							"CSRCountry" => $results["PARSEDCSR"][0]["COUNTRY"],
							"CSREmail" => $results["PARSEDCSR"][0]["EMAIL"],
							"CSRLocality" => $results["PARSEDCSR"][0]["LOCALITY"],
							"CSROrganization" => $results["PARSEDCSR"][0]["ORGANIZATION"],
							"CSROrganizationUnit" => $results["PARSEDCSR"][0]["ORGANIZATIONUNIT"],
							"CSRState" => $results["PARSEDCSR"][0]["STATE"],
							"CSRIsValidDomainName" => $results["PARSEDCSR"][0]["ISVALIDTRUEDOMAINNAME"],
							"CSRHasBadExtensions" => $results["PARSEDCSR"][0]["HASBADEXTENSIONS"],
							"CSRStartDate" => time(),

							"ODPrice" => $results["PRICE"],
							"orSuccesscode" => $results["ORDERRESPONSEHEADER"][0]["SUCESSCODE"],
							"orID" => $results["or_id"]);

					$this->applyRequestTo( $params );


					if (notZero($data["ODServerCount"]) && notZero( $params["ODServerCount"]))
						// no server count made
						$params["ODServerCount"] = 1;


					$setQuery = new MBQuery("SetCerts", "update", $params);

					if ($this->addErrorsFromResult( $r =& $setQuery->getResult() )) {
						redirect( "certs.php", "OrderCertCSRVerify", array("certID" => (int)$_REQUEST["certID"]) );
					} else {
						$this->addError('TRANS_ACTIONERROR', "Cert failed to record properly");
					}
			}
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}