<?php

require_once("lib-action/action.php");
require_once("UI/Page.php");

use Billing\Object\BillingAccount as BillingAccount;

class SelectClientContactPage extends UIPage
{
	public function createControls()
	{
		require_once("UI/PageController.php");

		$input_type = UIFactory()->InputDropDown("clientContactID",	blmsg("TRANS_CLIENT_CONTACT_NAME"));
		$client_contacts = Factory()->ContactList(array('client_id' =>  $this->getValue("clientID")));;
		foreach ( $client_contacts as $client_contact )
			$input_type->addOption(	$client_contact->id, $client_contact->contact_fullname );

		$input_panel = UIFactory()->InputPanel(blmsg("TRANS_CLIENT_CONTACT_CHOOSING"));
		$input_panel->addInputControl($input_type);
		$this->addInputPanel($input_panel);
	}

	public function processInput()
	{
		dispatchAction("clients", getActionID("AddBillingAccount"));
	}
}

class AddBillingAccount extends MB_Action
{
	public $params = array("clientID", "clientContactID");

	public function execute()
	{
		parent::execute();
		$clientID = getParam('clientID');
		$clientContactID = getParam('clientContactID');
		$command = bgpc("command");
		if(!$clientID) {
			$this->addError('TRANS_ACTIONERROR', "No client ID given.");
		} else if(!$clientContactID || "cancel" == $command) {
			$this->createSelectClientContactPage($clientID);
			return $this->hasExecutedSuccessfully();
		} else {
			global $CONF;
			$query = $this->getMBAPIQuery("GetClients", "", array("getContactData"=> "4"));
			$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
			$data = dispatchMBAPI($query, 1);
			$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);

			$results =& $data["mbapi"][0]["results"][0];
			if ($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors())
			{
				$client = Factory()->Customer($clientID);

				$this->smartyVars["client"] = collapseArrayElements($results["clients"][0]["client"][0]);
				$this->smartyVars["contact"] = collapseArrayElements($results["clients"][0]["client"][0]["contacts"][0]["contact"][0]);
				$this->smartyVars["billingAccountTypes"] = Service()->BillingAccountManager->getAllowedBillingAccountTypes($client);
				$this->smartyVars["bankAccountTypes"] = $CONF["types"]["bankAccount"];
				$this->smartyVars["cardTypes"] = $CONF["types"]["card"];
				$this->smartyVars["cardExtraTypes"] = $CONF["types"]["cardExtraIDs"];

                require_once("objects/PaymentGateway.php");
                require_once("object_managers/PaymentGatewayManager.php");
				$bt_gateways = Factory()->PaymentGatewayList(array("payment_gateway_type" => PaymentGateway::TYPE_HSPC_BANK_TRANSFER, 'vendor_id' => $client->vendor_id));
                if (!Service()->PaymentGatewayManager->isHSPCAvailable()) {
                    billing_put_warning(blmsg('TRANS_PBA_PLUGINS_SERVICE_NOT_AVAILABLE'));
                }
				if (0 < count($bt_gateways) && Service()->PaymentGatewayManager->isHSPCAvailable()) {
					$fake_billing_account = Factory()->BillingAccount();
					foreach ($bt_gateways as $bt_gateway) {
						require_once('ModuleTool.php');
						$plugin = ModuleTool::getModuleInstance("gateway", "HSPCBankTransfer", array("gateway_id" => $bt_gateway->id));
						$fake_billing_account->payment_gateway_id = $bt_gateway->id;
						$adapter_input_fields = $plugin->getPaymentMethodAdapter($fake_billing_account)->getInputFields(true);
						$input_fields = array();
						foreach ($adapter_input_fields as $field_name => $field_opts) {
							$input_fields[] = array(
								"field_name" => "{$field_name}_{$bt_gateway->id}",
								"field_title" => $field_opts[0],
								"field_desc" => $field_opts[2],
							);
						}
						$this->smartyVars["ddbp_accounts"][] = array(
							"gateway_id" => $bt_gateway->id,
							"gateway_name" => $bt_gateway->payment_gateway_name,
							"gateway_desc" => $bt_gateway->payment_gateway_description,
							"fields" => $input_fields,
						);
					}
					$this->smartyVars["selected_ddbp_gateway_id"] = bgpc("selected_ddbp_gateway_id");
				}

				$formDefaults = array(
					"billingAccountOrder" => "0",
					"billingAccountPercent" => "100.0",
				);
				if ($_REQUEST["comboAction"] == "AddNewClient") {
					$formDefaults["comboAction"] = "AddNewClient";
				}

				$this->mergeFormState($formDefaults);

				// do we have an lek stored?
				$this->smartyVars["needKey"] = Service()->CryptManager->isEncryptionEnabled() && !Service()->CryptManager->isLekEnabled() ? true : false;

                $this->smartyVars["ccAccount"] = BillingAccount::TYPE_CREDIT_CARD;
                $this->smartyVars["manualAccount"] = BillingAccount::TYPE_MANUAL_PAYMENT;
                $this->smartyVars["thirdPartyAccount"] = BillingAccount::TYPE_THIRD_PARTY_GATEWAY;
                $this->smartyVars["bankTransferAccount"] = BillingAccount::TYPE_HSPC_BANK_TRANSFER;
                $this->smartyVars["buttons"] = array(
                    array('id' => 'billing-id-save', 'class' => 'action', 'name' => 'save', 'value' => "TRANS_SAVE_BILLING_ACCOUNT", 'type' => 'submit', 'label' => 'TRANS_SAVE_BILLING_ACCOUNT'),
                );

                $this->addTemplate("clients/AddBillingAccount", $this->smartyVars);
			}
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();

	}

	private function createSelectClientContactPage ( $clientID )
    {
    	$accounts_list_url = action_url("clients", "ShowBillingAccounts", array("clientID" => $clientID));
		$controller  = UIFactory()->PageController($accounts_list_url);
		$page1 = new SelectClientContactPage($controller);

		$page1->title = blmsg("TRANS_ADD_BILLING_ACCOUNT");
		$page1->parent_title = blmsg("TRANS_MANAGE_BILLING_ACCOUNTS");
		$page1->parent_url = $accounts_list_url;
		$page1->setValue( "clientID", $clientID );

		$controller->addPage($page1);
		$controller->processRequest();
		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return;
    }
}
