<?php
require_once("lib-action/action.php");

use Billing\Skin;
use Billing\Object\Customer;
use Billing\Object\EmailTemplateGroup;
use Billing\Object\PaymentTransaction;
use Billing\Object\Reseller;
use Billing\Object\BillingAccount as BillingAccount;

class AddClient extends MB_Action
{
	public function execute()
	{
		global $CONF;
		parent::execute();

		$this->smartyVars["paymentList"] = PaymentTransaction::payment_types();
		$this->smartyVars["statusList"] = blmsg_array(Customer::statuses());

		$this->mergeLanguageData($CONF["user"]["locale"]->language);

        $company = Service()->CompanyManager->getCompanyByVendorId(getCurrentSession()->getVendorId());
		$this->mergeCountryData($company->Country->countries_iso_2);
		$this->smartyVars["emailActions"] = blmsg_array(EmailTemplateGroup::types());
		foreach (Customer::types() as $key => $type) {
			$this->smartyVars["clientTypes"][$key] = blmsg($type);
		}

		$this->mergeThemeData();

		$formDefaults = array(
			"clientActive" => 1,
			"countriesID" => $this->smartyVars["selectedCountry"],
			"languageID"  => $this->smartyVars["selectedLanguage"],
			"contactEmailFormat" => 1,
			"applyTax" => 1,
		);
		$this->smartyVars['attributeGroups'] =array();
        foreach(Factory()->AttributeGroupList(array("vendor_id" => getCurrentSession()->getVendorId(), "group_active" => 1)) as $group) {
            $this->smartyVars['attributeGroups'][$group->group_id] = $group->group_name;
        }

		$this->smartyVars["billingAccountTypes"] = $CONF["types"]["billingAccount"];
		unset($this->smartyVars["billingAccountTypes"][getAPITypeID("billingAccount", "hspcbanktransfer")]);
		$this->smartyVars["bankAccountTypes"] = $CONF["types"]["bankAccount"];
		$this->smartyVars["cardTypes"] = $CONF["types"]["card"];
		$this->smartyVars["cardExtraTypes"] = $CONF["types"]["cardExtraIDs"];
		$this->smartyVars["needKey"] = Service()->CryptManager->isEncryptionEnabled() && !Service()->CryptManager->isLekEnabled() ? true : false;
		$this->mergeFormState($formDefaults);

		$this->smartyVars["skinList"] = Skin::getSkins();
		if (! isset($this->smartyVars["formValues"]["skinID"]))
			$this->smartyVars["formValues"]["skinID"] = Skin::DEFAULT_SKIN_ID;

        $this->smartyVars["billingSecureUrl"] = billing_base_url();
        $this->smartyVars["permissionCustomerGroups"] = true;
        if ($this->_isReseller()) {
            $this->smartyVars["permissionCustomerGroups"] = $this->_getReseller()->hasPermission(\Billing\Object\Reseller::PERMISSION_CUSTOMER_GROUPS);
        }

        $this->smartyVars["ccAccount"] = BillingAccount::TYPE_CREDIT_CARD;
        $this->smartyVars["manualAccount"] = BillingAccount::TYPE_MANUAL_PAYMENT;
        $this->smartyVars["thirdPartyAccount"] = BillingAccount::TYPE_THIRD_PARTY_GATEWAY;
        $this->smartyVars["bankTransferAccount"] = BillingAccount::TYPE_HSPC_BANK_TRANSFER;

        $this->smartyVars["formValues"]["contactPrimary"] = 1;
        $this->addTemplate("clients/AddClient", $this->smartyVars);

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}