<?php

require_once("lib-action/action.php");

use Billing\Manager\CryptManager;
use Billing\Object\Contact;

class AddClientContactResponse extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$this->setReferralActionID(getActionID("AddClientContact"));
		$this->setReferralActionFile("clients.php");

        $validate = array(
            "clientID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
            "contactFirstName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
            "contactLastName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
            "contactEmail" => array(INPUT_RESTRICTION_EMAIL|INPUT_RESTRICTION_REQUIRED|INPUT_RESTRICTION_NOTINCOL, array("source" => "contacts", "col" => "contactEmail")),
            "contactEmailFormat" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
            "contactCompany" => INPUT_RESTRICTION_STRING,
            "contactAddress1" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
            "contactAddress2" => INPUT_RESTRICTION_STRING,
            "contactCity" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
            "contactState" => INPUT_RESTRICTION_STRING,
            "contactZip" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
            "countriesID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
            "languageID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
            "contactPhone1" => INPUT_RESTRICTION_PHONE|INPUT_RESTRICTION_REQUIRED,
            "contactPhone2" => INPUT_RESTRICTION_PHONE,
            "contactFax" => INPUT_RESTRICTION_PHONE,
            "contactMobilePhone" => INPUT_RESTRICTION_PHONE,
            "contactCounty" => INPUT_RESTRICTION_STRING,
            "emailActionIDs" => INPUT_RESTRICTION_ARRAY,
            "contactUseSpecifiedEmailRulesOnly" => INPUT_RESTRICTION_BOOL,
            "contactUsername" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
            "contactPassword" => array(INPUT_RESTRICTION_PASSWORDCONFIRM|INPUT_RESTRICTION_REQUIRED, array("confirm" => "contactPasswordConfirm")),
        );

        $this->_validateZip();
        $this->_validateAndTransformCodeState();

		$contact = Factory()->Contact();
        $this->_collectContact($contact);
        if (count($violations = Service()->ContactManager->validate($contact))) {
            $this->_addInvalidParams($violations, $validate);
        }

		$this->validateMultiInputs($validate);
        if (count($violations)) {
            $this->validationRedirect(array(), array(), true);
        }

		try {
			Service()->ContactManager->update($contact);

			if ( "" != ($redirectURL = bgpc("redirectURL")) && "" != ($redirectAction = bgpc("redirectAction")) ) {
				redirect($redirectURL."?action=".getActionID($redirectAction)."&".bgpc("redirectArgs"));
			}
			redirect("clients.php", getActionID("ShowClientContacts"), array("clientID" => $contact->client_id));
		} catch (ProductException $e) {
			billing_put_error($e->getMessage());
			$this->validationRedirect(array(), array(), true);
        }
		return $this->hasExecutedSuccessfully();
	}

    private function _collectContact(Contact $contact)
    {
		foreach (Contact::$attrsDescriptor as $attr_name => $attr_desc) {
			switch ($attr_name) {
                case 'contact_primary':
                    break;
                case 'contact_password':
                    $contact->plainPassword = bgpc('contactPassword');
                    break;
                default:
                    if (false === ($param_name = getDBXMLCol($attr_name)) || is_null($param_value = bgpc($param_name))) {
                        break;
                    }
                    $contact->$attr_name = trim($param_value);
			}
		}
		$contact->contact_primary = 0;
		if (!is_null($emailActionIDs = bgpc('emailActionIDs'))) {
			$contact->emailActionIDs = $emailActionIDs;
		}
    }
}
