<?php
require_once("lib-action/action.php");

use Billing\Manager\CryptManager;
use Billing\Object\BillingAccount;
use Billing\Object\Customer;

class AddClientResponse extends MB_Action
{
	public function execute()
	{
		global $CONF;
		parent::execute();
        if (is_null($reseller = getCurrentSession()->getReseller())) {
		    $_REQUEST['companyID'] = Service()->CompanyManager->getProviderCompany()->id;
        } else {
            $_REQUEST['companyID'] = $reseller->company_id;
        }

		$validate = array(
			"clientType" => INPUT_RESTRICTION_UNSIGNED_INT,
			"clientMailInvoices" => INPUT_RESTRICTION_BOOL,
			"clientBillingDay" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
			"applyTax" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
			"taxIDNumber" => INPUT_RESTRICTION_STRING,
			"skinID" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"groupID" => INPUT_RESTRICTION_UNSIGNED_INT,
			"companyID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
			"clientStatus" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
			"clientActive" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
			"contactFirstName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactLastName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactUsername" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactEmail" => array(INPUT_RESTRICTION_EMAIL|INPUT_RESTRICTION_REQUIRED|INPUT_RESTRICTION_NOTINCOL, array("source" => "contacts", "col" => "contactEmail")),
			"contactEmailFormat" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
			"contactCompany" => INPUT_RESTRICTION_STRING,
			"contactAddress1" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactAddress2" => INPUT_RESTRICTION_STRING,
			"contactCity" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactState" => INPUT_RESTRICTION_STRING,
			"contactZip" => array(INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED|INPUT_RESTRICTION_MAXLEN,array('maxLength' => 10)),
			"countriesID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
			"languageID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
			"contactPhone1" => INPUT_RESTRICTION_PHONE|INPUT_RESTRICTION_REQUIRED,
			"contactPhone2" => INPUT_RESTRICTION_PHONE,
			"contactFax" => INPUT_RESTRICTION_PHONE,
			"contactPrimary" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
			"contactMobilePhone" => INPUT_RESTRICTION_PHONE,
			"contactCounty" => INPUT_RESTRICTION_STRING,
			"contactPassword" => array(INPUT_RESTRICTION_PASSWORDCONFIRM|INPUT_RESTRICTION_REQUIRED, array("confirm" => "contactPasswordConfirm")),
			"comboAction" => INPUT_RESTRICTION_STRING,
			"contactUseSpecifiedEmailRulesOnly" => INPUT_RESTRICTION_BOOL,
			"emailActionIDs" => INPUT_RESTRICTION_ARRAY,
			// account
			"billingAccountType" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
			// card account
		);

		$billingAccountType = bgpc("billingAccountType");
		switch ($billingAccountType)
		{
			case getAPITypeID("billingAccount", "card"):
				$validate["billingCardNameOnCard"] = INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED;
				$validate["billingCardType"] = array(
					INPUT_RESTRICTION_INSET|INPUT_RESTRICTION_REQUIRED,
					array('set' => $CONF['types']['card'])
				);
				$validate["billingCardExpMonth"] = INPUT_RESTRICTION_MONTH|INPUT_RESTRICTION_REQUIRED;
				$validate["billingCardExpYear"] = INPUT_RESTRICTION_YEAR|INPUT_RESTRICTION_REQUIRED;
				$validate["billingCardNum"] = array(
					INPUT_RESTRICTION_INT|INPUT_RESTRICTION_MAXLEN|INPUT_RESTRICTION_MINLEN|INPUT_RESTRICTION_CARD|INPUT_RESTRICTION_REQUIRED,
					array(
						'maxLength' => 19,
						'minLength' => 13,
						'allowedCardTypes' => array(getAPITypeName("card", bgpc('billingCardType'))),
					),
				);
				break;
			case getAPITypeID("billingAccount", "echeck"):
			case getAPITypeID("billingAccount", "bank"):
			case getAPITypeID("billingAccount", "wire"):
				$validate["billingBankType"] = INPUT_RESTRICTION_UNSIGNED_INT;
				$validate["billingBankName"] = INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED;
				$validate["billingBankState"] = INPUT_RESTRICTION_STRING;
				$validate["billingBankAbaCode"] = INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED;
				$validate["billingBankAccountNum"] = INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED;
				$validate["billingBankLicenseNum"] = INPUT_RESTRICTION_STRING;
				$validate["billingBankLicenseState"] = INPUT_RESTRICTION_STRING;
				$validate["billingBankDob"] = INPUT_RESTRICTION_STRING;
				break;
		}

		if (!is_null(bgpc("contactUsername"))) {
			if (false !== $this->validateParam("contactUsername", array(INPUT_RESTRICTION_REQUIRED|INPUT_RESTRICTION_NOTINCOL, array("source" => "contacts", "col" => "contactUsername"))))
			$this->validateParam("contactUsername", array(INPUT_RESTRICTION_REQUIRED|INPUT_RESTRICTION_NOTINCOL, array("source" => "admins", "col" => "adminUsername")));
			$validate["contactUsername"] = INPUT_RESTRICTION_REQUIRED;
			if (!Service()->ContactManager->isUsernameUnique(bgpc("contactUsername"))) {
				$this->invalidParams[] = "contactUsername-" . base64_encode(INPUT_RESTRICTION_NOTINCOL);
			}
		}

		$this->setReferralActionID(getActionID("AddClient"));
		$this->setReferralActionFile("clients.php");

        $this->_validateZip();
        $this->_validateAndTransformCodeState();

        $customer = Factory()->Customer();
        $this->_updateCustomer($customer, $validate);
        if (count($violations = Service()->CustomerManager->validate($customer))) {
            $this->_addInvalidParams($violations, $validate);
        }

        $billingAccount = Factory()->BillingAccount();
        $this->_updateBillingAccount($billingAccount, $validate);

		$this->validateMultiInputs($validate);
        if (count($violations)) {
            $this->validationRedirect(array(), array(), true);
        }

		try {
			Service()->CustomerManager->update($customer);
		} catch (ProductException $e) {
			billing_put_error($e->getMessage());
			$this->validationRedirect(array(), array(), true);
		}

		try {
            $billingAccount->client_contact_id = $customer->PrimaryContact->id;
			Service()->BillingAccountManager->update($billingAccount);
            $customer->default_billing_account_id = $billingAccount->id;
            $customer->update();
		} catch (ProductException $e) {
			billing_put_error($e->getMessage());
		}

		billing_put_info(blmsg('TRANS_ADD_NEW_CLIENT_SUCCESS_MSG', array('NAME' => billing_safetyhtml($customer->PrimaryContact->contact_fullname))));
		redirect('clients.php', getActionID('ShowClientDetails'), array('clientID' => $customer->id));
	}

    private function _updateCustomer(Customer $customer, array $validate)
    {
        $primaryContact = $customer->PrimaryContact;
		$customerFields = array('apply_tax', 'tax_id_number', 'countries_id', 'client_status', 'client_billing_day', 'client_type', 'group_id');
        foreach ($customerFields as $fieldName) {
            if (false === ($paramName = getDBXMLCol($fieldName)) || null === ($paramValue = bgpc($paramName)) || !isset($validate[$paramName]))  {
                continue;
            }
            $customer->$fieldName = $paramValue;
        }
        $customer->client_status_reason = blmsg('TRANS_MANUALLY_CREATED_BY_ADMIN');
        if ($reseller = $this->_getReseller()) {
            $customer->vendor_id = $reseller->id;
            $customer->company_id = $reseller->company_id;
        } else {
            $customer->company_id = Service()->CompanyManager->getProviderCompany()->id;
        }
		$contactFields = array('contact_email', 'contact_username', 'contact_company', 'contact_first_name', 'contact_last_name', 'contact_address_1',
			'contact_address_2', 'contact_city', 'contact_state', 'contact_county', 'contact_zip', 'countries_id', 'language_id', 'contact_phone1',
			'contact_phone2', 'contact_fax', 'contact_mobile_phone', 'contact_email_format', 'contact_use_specified_email_rules_only',
		);
        foreach ($contactFields as $fieldName) {
            if (false === ($paramName = getDBXMLCol($fieldName)) || null === ($paramValue = bgpc($paramName)) || !isset($validate[$paramName]))  {
                continue;
            }
            $primaryContact->$fieldName = trim($paramValue);
        }
        $primaryContact->plainPassword = bgpc('contactPassword');
        $primaryContact->emailActionIDs = bgpc("emailActionIDs", array());
        $customer->client_company = $primaryContact->contact_company;
    }

    private function _updateBillingAccount(BillingAccount $billingAccount, array $validate)
    {
		$billingAccountFields = array('billing_account_type',
			'billing_card_name_on_card', 'billing_card_type', 'billing_card_num', 'billing_card_exp_month', 'billing_card_exp_year', 'billing_card_issue_num', 'billing_card_start_date',
			'billing_bank_type', 'billing_bank_name', 'billing_bank_state', 'billing_bank_aba_code', 'billing_bank_account_num', 'billing_bank_license_num', 'billing_bank_license_state', 'billing_bank_dob',
		);
        foreach ($billingAccountFields as $fieldName) {
            if (false === ($paramName = getDBXMLCol($fieldName)) || null === ($paramValue = bgpc($paramName)) || !isset($validate[$paramName]))  {
                continue;
            }
            $billingAccount->$fieldName = $paramValue;
        }
    }
}
