<?php

require_once("lib-action/action.php");

use Billing\Object\GroupAttributes;

class EditBasicClientAttributes extends MB_Action
{
    public $params = array('clientID');

	public function execute()
    {
		parent::execute();

		$clientID = getParam('clientID');
		if(!$clientID) {
			$this->addError('TRANS_ACTIONERROR', "No client ID given.");
		} else {
            $customer = Factory()->Customer($clientID);
            $this->_setResellerTemplateVariables($customer);
			$client = oneById("clients", $clientID, array("mergePrimaryContact" => 1));

			// retrive customer attributes, for admin own customers only
			$keyedAttributes = array();
			if (is_null($customer->vendor_id)) {
				foreach (Service()->AttributeManager->getCustomerAttributes() as $customerAttribute) {
					$keyedAttributes[$customerAttribute->id] = $customerAttribute->attribute_name;
				}
				foreach (Service()->AttributeManager->getCustomerGroupAttributes() as $customerGroupAttribute) {
					$keyedAttributes[$customerGroupAttribute->id] = $customerGroupAttribute->attribute_name;
				}
			}

			$namedAttributes = array_flip($keyedAttributes);
            $attributeGroups = $attributeGroupValues = array();
            foreach(Factory()->AttributeGroupList(array("vendor_id" => getCurrentSession()->getVendorId(), "group_active" => 1)) as $group) {
                $attributeGroups[$group->group_id] = $group->group_name;
                $attributeGroupValues[$group->group_id] = array(
                    GroupAttributes::MAXIMUM_DISCOUNT => !is_null($group->GroupAttributes[GroupAttributes::MAXIMUM_DISCOUNT]->value) ? $group->GroupAttributes[GroupAttributes::MAXIMUM_DISCOUNT]->value : "" ,
                    GroupAttributes::MINIMUM_DISCOUNT => !is_null($group->GroupAttributes[GroupAttributes::MINIMUM_DISCOUNT]->value) ? $group->GroupAttributes[GroupAttributes::MINIMUM_DISCOUNT]->value : "" ,
                );
                if (!is_null($value = $group->GroupAttributes[GroupAttributes::SUSPENTION_EXEMPT]->value)) {
                    $attributeGroupValues[$group->group_id][GroupAttributes::SUSPENTION_EXEMPT] = (1 == $value)
                        ? blmsg("TRANS_YES")
                        : blmsg("TRANS_NO");
                } else {
                    $attributeGroupValues[$group->group_id][GroupAttributes::SUSPENTION_EXEMPT] ="";
                }
            }
            $this->smartyVars["attributeGroups"] = $attributeGroups;
            $this->smartyVars["attributeGroupValues"] = $attributeGroupValues;
            $this->smartyVars["suspentionExemptAttribute"] = GroupAttributes::SUSPENTION_EXEMPT;

            $programaticAttributes = GroupAttributes::types();
			$defaultAttrs = array_keys($programaticAttributes);
			foreach ($defaultAttrs as $attrName){
				if (!$namedAttributes[$attrName]){
					$namedAttributes[$attrName] = "x";
				}
			}
            ksort($namedAttributes);
			$this->smartyVars["customerGroupAttributes"] = array_intersect_key($namedAttributes, $programaticAttributes);
			$this->smartyVars["namedAttributes"] = array_diff_key($namedAttributes, $programaticAttributes);
			$this->smartyVars["programaticAttributes"] = $programaticAttributes;
			$clientAttributes = arrayFor("clientAttributes", array("clientID" => $clientID));

			$this->smartyVars["attributeValues"] = array();
			foreach ($clientAttributes as $attr){
				$this->smartyVars["attributeValues"][$attr["attributeID"]] = $attr;
			}

			$this->smartyVars["client"] = $client;
			$this->mergeFormState($client);
            $this->smartyVars["buttons"] = array(
                array('id' => 'billing-id-save', 'class' => 'action', 'name' => 'save', 'value' => "TRANS_SAVE", 'type' => 'submit', 'label' => 'TRANS_SAVE'),
            );
            $this->addTemplate("clients/EditBasicClientAttributes", $this->smartyVars);
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
