<?php

require_once("lib-action/action.php");

use Billing\Object\GroupAttributes;

class EditBasicClientAttributesResponse extends MB_Action
{
	public $params = array("clientID");

	public function execute()
	{
		parent::execute();

		$customer = Factory()->Customer(getParam('clientID'));

		if (!getCurrentSession()->isValidToken()) {
			billing_put_error(blmsg("TRANS_ERROR_TOKEN"));
			redirect("clients.php", "EditBasicClientAttributes", array("clientID" => $customer->id));
		}

		// update customer group
		$customer->group_id = bgpc("groupID");
		$customer->update();

		$keyedAttributes = optionsFor("attributes", array());
		$namedAttributes = array_flip($keyedAttributes);
		dbg("attributes:");
		dbg($keyedAttributes);
		dbg($namedAttributes);

		$programaticAttributes = GroupAttributes::types();
		$defaultAttrs = array_keys($programaticAttributes);
		foreach ($defaultAttrs as $attrName){
			if (!$namedAttributes[$attrName]){
				$namedAttributes[$attrName] = "x";
			}
		}

		$clientAttributes = arrayFor("clientAttributes", array("clientID" => $customer->id));
		$keyedClientAttributes = array();
		foreach ($clientAttributes as $attr){
			if (!isset($_REQUEST["activeAttributes"][$keyedAttributes[$attr["attributeID"]]])){
				deleteForKey("clientAttributes", $attr["clientAttributeID"]);
			} else {
				$keyedClientAttributes[$keyedAttributes[$attr["attributeID"]]] = $attr["clientAttributeID"];
			}
		}

		foreach ((array)$_REQUEST["activeAttributes"] as $aName => $active){
			$attributeID = $namedAttributes[$aName];
			if ($attributeID=="x"){
				$namedAttributes[$aName] = $attributeID = insertFor("attributes", array(
					"attributeName" => $aName,
					"attributeType" => 0,
					"attributeSubtype" => 0,
				));
			}

			$clientAttributeID = $keyedClientAttributes[$aName];
			if (!(int)$clientAttributeID){
				$keyedClientAttributes[$aName] = $clientAttributeID = insertFor("clientAttributes", array(
					"attributeID" => $attributeID,
					"clientID" => $customer->id,
					"clientAttributeValue" => $_REQUEST["attributeValues"][$aName],
				));
			} else {
				updateFor("clientAttributes", array(
					"clientAttributeID" => $clientAttributeID,
					"clientAttributeValue" => $_REQUEST["attributeValues"][$aName],
				));
			}

		}

		$successMessage = ($customer->isReseller()) ? blmsg("TRANS_EDIT_CLIENT_ATTRIBUTES_FOR_RESELLER_SUCCESS_MSG")
			: blmsg("TRANS_EDIT_CLIENT_ATTRIBUTES_FOR_CUSTOMER_SUCCESS_MSG");
		billing_put_info($successMessage);
		redirect("clients.php", "ShowClientDetails", array("clientID" => $customer->id));

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
