<?php
require_once("lib-action/action.php");

use Billing\Manager\CryptManager;
use Billing\Manager\SecurityManager;
use Billing\Object\BillingAccount;
use Billing\Object\Plan;

class EditBillingAccount extends MB_Action
{
	public $params = array("clientID", "clientContactID", "billingAccountID");

    protected $_templateName = 'EditBillingAccount';
    protected $_customerAccountLocaleKey = 'TRANS_ACTION_LOG_COMMENT_CLIENT_BILLING_ACCOUNT';
    protected $_resellerAccountLocaleKey = 'TRANS_ACTION_LOG_COMMENT_RESELLER_BILLING_ACCOUNT';
    protected $_getDecryptedData = false;

    public function execute()
    {
        parent::execute();

        $clientID = bgpc('clientID');
        $clientContactID = bgpc('clientContactID');
        $billingAccountID = bgpc('billingAccountID');
        try {
            /** @var $billingAccount BillingAccount */
            $billingAccount = Factory()->BillingAccount($billingAccountID);
            if (!$clientContactID && $billingAccountID){
                $clientContactID = $billingAccount->client_contact_id;
            }
            if (!$clientID && $clientContactID){
                $clientID = Factory()->Contact($clientContactID)->client_id;
            }
        } catch (ProductException $e) {
            if (ProductException::E_OBJECT_NOT_FOUND == $e->getCode()) {
                billing_put_error(blmsg($e->getMessage()));
                redirect('clients.php', getActionID("ShowBillingAccounts"));
            }
            throw $e;
        }
        $this->smartyVars["canDecryptData"] = Service()->CryptManager->canDecryptData();
        $query = $this->getMBAPIQuery("GetClients", "",  array("getContactData"=>"4", "getBillingData"=>"4", "clientContactID" =>$clientContactID));
        $data = dispatchMBAPI($query, 1);

        $results =& $data["mbapi"][0]["results"][0];

        if ($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {
            $customer = Factory()->Customer($clientID);

            if ($billingAccount->isBlockedByQuarantine()) {
                $this->smartyVars["isBlockedByQuarantine"] = $billingAccount->isBlockedByQuarantine();
                $quarantineDate = $billingAccount->billing_account_date_failed + Settings($customer->vendor_id)->paymentQueue_quarantinePeriodDuration * BillingAccount::SEC_IN_HOUR;
                $this->smartyVars["billingAccountQuarantineDate"] = BillingLocale::getDateTimeMedium($quarantineDate);
            }
            if ($billingAccount->isBlockedByLimitOfFailures()) {
                $this->smartyVars["isBlockedByLimitOfFailures"] = $billingAccount->isBlockedByLimitOfFailures();
                $this->smartyVars["billingAccountQuantityFailures"] = $billingAccount->billing_account_failures_num;
            }

            $comment = $customer->isReseller()
                    ?  blmsg($this->_resellerAccountLocaleKey, array("ACCOUNT_NAME" => $billingAccount->getAccountNameForList(), "ACCOUNT_ID" => $billingAccount->id, "RESELLER_ID" => $customer->getReseller()->id), Service()->SecurityManager->getActionLogLocale())
                    :  blmsg($this->_customerAccountLocaleKey, array("ACCOUNT_NAME" => $billingAccount->getAccountNameForList(), "ACCOUNT_ID" => $billingAccount->id, "CLIENT_ID" => $customer->id), Service()->SecurityManager->getActionLogLocale());

            Service()->SecurityManager->pushCurrentActionIntoLog(
                SecurityManager::USER_TYPE_CLIENT,
                $clientID,
                $customer->PrimaryContact->contact_username,
                "TRANS_ACTION_LOG_REASON_SHOW_BILLING_ACCOUNT_DETAILS",
                $comment
            );

            $this->smartyVars["client"] = correctArray($results["clients"][0]["client"][0]);
            $this->smartyVars["contact"] = correctArray($results["clients"][0]["client"][0]["contacts"][0]["contact"][0]);
            $this->smartyVars["billingAccount"] = collapseArrayElements($results["clients"][0]["client"][0]["billingAccounts"][0]["billingAccount"][0]);
            if ($this->_getDecryptedData) {
                if (!is_null($billingAccount->billing_card_num)) {
                    $this->smartyVars["billingAccount"]["decryptedCard"] = $billingAccount->billing_card_num;
                } elseif (!is_null($billingAccount->billing_bank_account_num)) {
                    $this->smartyVars["billingAccount"]["decryptedAccountNum"] = $billingAccount->billing_bank_account_num;
                }
            }
            $this->smartyVars["billingAccountTypes"] = Service()->BillingAccountManager->getAllowedBillingAccountTypes($customer);
            $this->smartyVars["billingAccountTypeName"] = BillingAccount::typeName($this->smartyVars["billingAccount"]["billingAccountType"]);
            $this->smartyVars["ccAccount"] = BillingAccount::TYPE_CREDIT_CARD;
            $this->smartyVars["manualAccount"] = BillingAccount::TYPE_MANUAL_PAYMENT;
            $this->smartyVars["thirdPartyAccount"] = BillingAccount::TYPE_THIRD_PARTY_GATEWAY;
            $this->smartyVars["bankTransferAccount"] = BillingAccount::TYPE_HSPC_BANK_TRANSFER;
            $this->smartyVars["bankAccountTypes"] = blmsg_array(PaymentGateway::bank_types());
            $this->smartyVars["cardTypes"] = blmsg_array(PaymentGateway::card_types());
            $this->smartyVars["cardExtraTypes"] = PaymentGateway::getCardTypeIdsWithExtraFields();

            if (BillingAccount::TYPE_HSPC_BANK_TRANSFER == $billingAccount->billing_account_type) {
                $bt_gateways = Factory()->PaymentGatewayList(array("payment_gateway_type" => PaymentGateway::TYPE_HSPC_BANK_TRANSFER, 'vendor_id' => $customer->vendor_id));
                if (0 < count($bt_gateways)) {
                    $billing_account = Factory()->BillingAccount($billingAccountID);
                    $billing_account_gateway_id = $billing_account->payment_gateway_id;
                    $additional_form_values = array();
                    foreach ($bt_gateways as $bt_gateway) {
                        $plugin = ModuleTool::getModuleInstance("gateway", "HSPCBankTransfer", array("gateway_id" => $bt_gateway->id));
                        $billing_account->payment_gateway_id = $bt_gateway->id;
                        $adapter_input_fields = $plugin->getPaymentMethodAdapter($billing_account)->getInputFields(false, !$this->_getDecryptedData);
                        $input_fields = array();
                        foreach ($adapter_input_fields as $field_name => $field_opts) {
                            $input_fields[] = array(
                                "field_name" => $field_name = "{$field_name}_{$bt_gateway->id}",
                                "field_title" => $field_opts[0],
                                "field_desc" => $field_opts[2]
                            );
                            $additional_form_values[$field_name] = $field_opts[1];
                        }
                        $this->smartyVars["ddbp_accounts"][] = array(
                            "gateway_id" => $bt_gateway->id,
                            "gateway_name" => $bt_gateway->payment_gateway_name,
                            "gateway_desc" => $bt_gateway->payment_gateway_description,
                            "fields" => $input_fields,
                        );
                    }
                    $billing_account->payment_gateway_id = $billing_account_gateway_id;
                    $this->smartyVars["selected_ddbp_gateway_id"] = bgpc("selected_ddbp_gateway_id", $billing_account_gateway_id);
                }
            }

            $defaults = $this->smartyVars["billingAccount"];
            if (!is_null($billingAccountType = bgpc('billingAccountType')))	{
                $defaults = array('billingAccountType' => $billingAccountType);
            }

            $this->mergeFormState($defaults);
            $this->smartyVars["paymentQueue_enableAdvancedShareEditing"] = Settings($customer->vendor_id)->paymentQueue_enableAdvancedShareEditing;
            if(is_array($additional_form_values)) {
                $this->smartyVars["formValues"] = array_merge($this->smartyVars["formValues"], $additional_form_values);
            }

            $this->_addDoubleListSelectorSubscriptions($clientID, $billingAccountID);
            $this->smartyVars["buttons"] = array(
                array('id' => 'billing-id-update', 'class' => 'action', 'name' => 'update', 'value' => "TRANS_UPDATE_BILLING_ACCOUNT", 'type' => 'submit', 'label' => 'TRANS_UPDATE_BILLING_ACCOUNT'),
            );

            $this->addTemplate("clients/{$this->_templateName}", $this->smartyVars);
        }


        $this->addErrorTemplates();
        return $this->hasExecutedSuccessfully();
    }

    private function _addDoubleListSelectorSubscriptions($clientId, $billingAccountId)
    {
        $this->smartyVars["doubleListSelectorOptions"] = array();
        foreach (Factory()->ClientBillingAccountShareList(array('billing_account_id' => $billingAccountId)) as $share) {
			if ($share->billing_account_share_value == 0) {
				continue;
			}
            $this->smartyVars["doubleListSelectorOptions"][$share->package_id] = array(
                'id' => $share->package_id,
                'title' => $share->Subscription->displayName,
                'selected' => true
            );
		}

        foreach(Factory()->SubscriptionList(array('client_id' => $clientId, 'package_active' => 1)) as $subscription) {
            if (!isset($this->smartyVars["doubleListSelectorOptions"][$subscription->id])
                    && Plan::TYPE_FEE != $subscription->package_type) {
                $this->smartyVars["doubleListSelectorOptions"][$subscription->id] = array(
                    'id' => $subscription->id,
                    'title' => $subscription->displayName
                );
            }
        }

        asort($this->smartyVars["doubleListSelectorOptions"]);
        $this->smartyVars["doubleListSelectorAttribs"] = array('id' => 'subscriptions');

        $GLOBALS["tpl"]->assign_by_ref("doubleListSelector", new Billing_Zend_View_Helper_DoubleListSelector());
    }
}
