<?php
require_once("lib-action/genericactions.php");

use Billing\Manager\CryptManager;
use Billing\ModuleTool;
use Billing\Object\BillingAccount;

class EditBillingAccountResponse extends EditResponseAction
{
	public function __construct()
	{
		if(!$billingAccount = oneOf("billingAccounts", array("billingAccountID" => (int) $_REQUEST["billingAccountID"]))) {
			$this->addError(MBAPI_ACTIONERROR, "No valid match.");
			return false;
		}
		$this->d = new DataSource("billingAccounts");
		$description = $this->d->getDescription();
		parent::__construct();
		unset($description['columns']['clientContactID']);

		$success_validation = true;

		$billingAccountType = bgpc("billingAccountType");
		if($billingAccountType == getAPITypeID("billingAccount", "manual") || $billingAccountType == getAPITypeID("billingAccount", "secondary")) {
			unset($description["columns"]['billingBankType']);
			unset($description["columns"]['billingBankName']);
			unset($description["columns"]['billingBankState']);
			unset($description["columns"]['billingBankABACode']);
			unset($description["columns"]['billingBankAccountNum']);
			unset($description["columns"]['billingBankLicenseNum']);
			unset($description["columns"]['billingBankLicenseState']);
			unset($description["columns"]['billingBankDob']);

			unset($description["columns"]['billingCardNameOnCard']);
			unset($description["columns"]['billingCardType']);
			unset($description["columns"]['billingCardNum']);
			unset($description["columns"]['billingCardExpMonth']);
			unset($description["columns"]['billingCardExpYear']);
			unset($description["columns"]['billingCardIssueNum']);
			unset($description["columns"]['billingCardStartDate']);
		} else if($billingAccountType == getAPITypeID("billingAccount", "card")) {
			unset($description["columns"]['billingBankType']);
			unset($description["columns"]['billingBankName']);
			unset($description["columns"]['billingBankState']);
			unset($description["columns"]['billingBankABACode']);
			unset($description["columns"]['billingBankAccountNum']);
			unset($description["columns"]['billingBankLicenseNum']);
			unset($description["columns"]['billingBankLicenseState']);
			unset($description["columns"]['billingBankDob']);

			$billingCardType = (int) $_REQUEST["billingCardType"];
			$cardExtraTypes = getAPITypes("cardExtraIDs");
			if(!in_array($billingCardType, $cardExtraTypes)) {
				unset($description["columns"]['billingCardIssueNum']);
				unset($description["columns"]['billingCardStartDate']);
			}
			if (!$_REQUEST["billingCardNum"] || $_REQUEST["billingCardNum"]==""){
				unset($description["columns"]['billingCardNum']);
			}
		} else {
			unset($description["columns"]['billingCardNameOnCard']);
			unset($description["columns"]['billingCardType']);
			unset($description["columns"]['billingCardNum']);
			unset($description["columns"]['billingCardExpMonth']);
			unset($description["columns"]['billingCardExpYear']);
			unset($description["columns"]['billingCardIssueNum']);
			unset($description["columns"]['billingCardStartDate']);
		}
		$this->d->setDescription($description);
		switch ($billingAccountType)
		{
			case getAPITypeID("billingAccount", "card"):
				$validation_fields = array(
					"billingCardType" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
					"billingCardNameOnCard" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
					"billingCardExpMonth" => INPUT_RESTRICTION_MONTH|INPUT_RESTRICTION_REQUIRED,
					"billingCardExpYear" => INPUT_RESTRICTION_YEAR|INPUT_RESTRICTION_REQUIRED,
					"billingCardNum" => array(
						INPUT_RESTRICTION_INT|INPUT_RESTRICTION_MAXLEN|INPUT_RESTRICTION_MINLEN|INPUT_RESTRICTION_CARD,
						array(
							'maxLength' => 19,
							'minLength' => 13,
							'allowedCardTypes' => array(getAPITypeName("card", bgpc('billingCardType'))),
						),
					),
                );
				break;
			case getAPITypeID("billingAccount", "echeck"):
			case getAPITypeID("billingAccount", "bank"):
			case getAPITypeID("billingAccount", "wire"):
				$validation_fields = array(
					"billingBankType" => INPUT_RESTRICTION_UNSIGNED_INT,
					"billingBankName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
					"billingBankState" => INPUT_RESTRICTION_STRING,
					"billingBankAbaCode" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
					"billingBankAccountNum" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
					"billingBankLicenseNum" => INPUT_RESTRICTION_STRING,
					"billingBankLicenseState" => INPUT_RESTRICTION_STRING,
                );
				break;
			case getAPITypeID("billingAccount", "hspcbanktransfer"):
				$gateway_id = bgpc("selected_ddbp_gateway_id");

				$plugin = ModuleTool::getModuleInstance("gateway", "HSPCBankTransfer", array("gateway_id" => $gateway_id));
                $billing_account = new BillingAccount(bgpc('billingAccountID'));
				// check vendors of client and payment gateway
				if (!$billing_account->Contact->Customer->hasSameVendor(Factory()->PaymentGateway($gateway_id))) {
					throw new ForbiddenOperationException();
				}
				$billing_account->payment_gateway_id = $gateway_id;
				$adapter = $plugin->getPaymentMethodAdapter($billing_account);
				$adapter->collectInputFields("_{$gateway_id}");
				$adapter_input_fields = $adapter->getInputFields();
				$validation_fields = array();
				foreach (array_keys($adapter_input_fields) as $field_name) {
					$validation_fields["{$field_name}_{$gateway_id}"] = INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED;
				}
				$adapter_validation = $adapter->validate();
				if (HSPC_PM_Adapter::VALIDATION_OK != $adapter_validation) {
					$success_validation = false;
					billing_put_error($adapter_validation);
				}
				break;
		}
		$this->setReferralActionID(getActionID("EditBillingAccount"));
		$this->setReferralActionFile("clients.php");
		$args = array(
			"billingAccountID" => bgpc("billingAccountID"),
			"clientID" => bgpc("clientID"),
			"clientContactID" => bgpc("clientContactID"),
			"billingAccountType" => $billingAccountType,
			"selected_ddbp_gateway_id" => $gateway_id,
		);
		$success_validation = $this->validateMultiInputs( $validation_fields, array(), array(), 1, $args ) && $success_validation;
		if (! $success_validation) {
			$this->validationRedirect($args, array(), true); //Force redirect
			exit;
		}
	}

	public function editResponseParamHandler()
	{
		$billingAccount = new BillingAccount(bgpc('billingAccountID'));
		$this->d->mergeParams(array('dataEncryptionKey' => $billingAccount->data_encryption_key));
		return true;
	}

	public function editResponseHandler()
	{
		// valid match assumed due to earlier check
		$billingAccount = Factory()->BillingAccount(bgpc("billingAccountID"));
		$clientID = $billingAccount->Contact->Customer->id;

		if ((int)$_REQUEST['defaultBillingAccount']){
			updateForKey(":clientsGeneric", $clientID, array(
				"defaultBillingAccountID" => $billingAccount->id,
			));
		}

		loadConfigs("payment", "paymentQueue");

        $subscriptions = array();
        foreach ((array)$_REQUEST["subscriptions"] as $subscriptionId => $subscriptionIsEnabled){
            if ($subscriptionIsEnabled) {
                $subscriptions[] = Factory()->Subscription($subscriptionId);
            }
        }
        Service()->BillingAccountManager->updateBillingAccountShares($billingAccount, $subscriptions);

		if (BillingAccount::TYPE_HSPC_BANK_TRANSFER == ($billingAccountType = bgpc("billingAccountType"))) {
			$gateway_id = bgpc("selected_ddbp_gateway_id");
			$plugin = ModuleTool::getModuleInstance("gateway", "HSPCBankTransfer", array("gateway_id" => $gateway_id));
			$billingAccount->payment_gateway_id = $gateway_id;
            $billingAccount->billing_account_type = $billingAccountType;
			$adapter = $plugin->getPaymentMethodAdapter($billingAccount);
			$adapter->collectInputFields("_{$gateway_id}");
			$adapter->save();
		}

		//////////////////////////
		return true;
	}

	public function overrideValidateColumnParamType($column, $columnType, $mode, $paramType, $isRequired)
	{
		switch($column) {
			case "billingCardNum":
				if(empty($_REQUEST["billingCardNum"])) {
					break;
				}
				$paramType |= INPUT_RESTRICTION_MAXLEN|INPUT_RESTRICTION_MINLEN|INPUT_RESTRICTION_CARD;
				$args = array(
					'maxLength' => 19,
					'minLength' => 13,
					'allowedCardTypes' => array(getAPITypeName("card", $_REQUEST['billingCardType'])),
				);
				break;
			default:
				$args = array();
				break;
		}

		return array($paramType, $isRequired, $args);
	}

	public function redirect()
	{
		if(!$this->goBack()) {
			$billingAccounts = new DataSource("billingAccounts", array("billingAccountID" => (int) $_REQUEST["billingAccountID"]));
			$contacts = new DataSource("contacts");
			$clients = new DataSource("clients");

			$clientID = $contacts->getIDByID($clients, $contactID = $billingAccounts->getIDFor(new DataSource("contacts")));

			billing_put_info(blmsg("TRANS_UPDATE_BILLING_ACCOUNT_SUCCESS_MSG"));
			redirect("clients.php", getActionID("ShowBillingAccounts"), array("clientID" => $clientID, "contact" => $contactID));
		}
	}
}
