<?php

require_once ("lib-action/action.php");
require_once("lib-billing/include/config/internal/mbapimap/clientBillingAccounts.php");

use Billing\Manager\SecurityManager;
use Billing\Object\Customer;
use Billing\Skin;

class EditClient extends MB_Action
{
	public $params = array("clientID");

	public function execute()
	{
		global $CONF;
		parent::execute();

		$clientID = getParam('clientID');
		if ( !$clientID ) {
			$this->addError(blmsg("TRANS_ACTIONERROR"), "No client ID given.");
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

		Service()->SecurityManager->pushCurrentActionIntoLog(
			SecurityManager::USER_TYPE_CLIENT,
			$clientID,
			Factory()->Customer($clientID)->PrimaryContact->contact_username,
			"TRANS_ACTION_LOG_REASON_SHOW_USER_DETAILS",
			blmsg("TRANS_ACTION_LOG_COMMENT_CLIENT", array("CLIENT_ID" => $clientID), Service()->SecurityManager->getActionLogLocale())
		);

		$query = $this->getMBAPIQuery("GetClients", "", array("mergePrimaryContact" => 1));
		$data = dispatchMBAPI($query, 1);

		$results =& $data["mbapi"][0]["results"][0];
		if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {

			$this->smartyVars["clientTypes"] = blmsg_array(Customer::types());
			$this->smartyVars["statusList"] = blmsg_array(Customer::statuses());
			$this->smartyVars["paymentList"] = $CONF["types"]["payments"];
			$this->mergeThemeData();

			$client = collapseArrayElements($results["clients"][0]["client"]);
			$this->smartyVars["client"] = $client;

			$formDefaults = $client;
			$this->mergeFormState($formDefaults);

			$this->smartyVars["selectableBillingAccounts"] = array('') + optionsFor("clientBillingAccounts", array("clientID"=>$clientID));

			$this->smartyVars["skinList"] = Skin::getSkins();

			$this->smartyVars["billingDays"][0] = blmsg("TRANS_OBEY_SYSTEMWIDE_SETTINGS");
			for ($day = 1; $day <=28; $day++) {
				$this->smartyVars["billingDays"][$day] = $day;
			}

            $this->smartyVars["buttons"] = array(
                array('id' => 'billing-id-save', 'class' => 'action', 'name' => 'save', 'value' => "TRANS_SAVE", 'type' => 'submit', 'label' => 'TRANS_SAVE'),
                array('id' => 'billing-id-reset', 'name' => 'reset', 'value' => "TRANS_RESET", 'type' => 'reset', 'label' => 'TRANS_RESET')
            );
			$this->addTemplate("clients/EditClient", $this->smartyVars);
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
