<?php
require_once("lib-action/action.php");

use Billing\Manager\SecurityManager;

class EditClientContact extends MB_Action
{
	public $params = array("clientContactID");

	public function __construct()
	{
		$_REQUEST["clientContactID"] = $this->_getClientContact()->id;
		parent::__construct();
	}

	public function execute()
	{
		global $CONF;
		parent::execute();

		$clientContactID = getParam("clientContactID");

		$clientContact = Factory()->Contact($clientContactID);

        if ($clientContact->Customer->isReseller()) {
            redirect(admin_action_url("resellers", "reseller-profile", "index", array("id" => $clientContact->Customer->getReseller()->id)));
        }

        Service()->SecurityManager->pushCurrentActionIntoLog(
			SecurityManager::USER_TYPE_CLIENT,
			$clientContact->client_id,
			$clientContact->Customer->PrimaryContact->contact_username,
			"TRANS_ACTION_LOG_REASON_SHOW_USER_DETAILS",
			blmsg(
				"TRANS_ACTION_LOG_COMMENT_CLIENT_CONTACT",
				array("CLIENT_ID" => $clientContact->client_id, "CONTACT_ID" => $clientContact->id),
				Service()->SecurityManager->getActionLogLocale()
			)
		);

		if(getParam("clientContactID")) {
			$clientContactID = (int) getParam("clientContactID");
			$query = $this->getMBAPIQuery("GetClients", "", array("clientContactID" => $clientContactID, "getContactData" => "4", "mergePrimaryContact" => 0));
			$data = dispatchMBAPI($query, 1);
		} else {
			$query = $this->getMBAPIQuery("GetClients", "", array("getContactData" => "4", "mergePrimaryContact" => 1));
			$data = dispatchMBAPI($query, 1);
		}

		$results =& $data["mbapi"][0]["results"][0];
		if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {

			$this->mergeCountryData();
			$this->mergeLanguageData();
			$client = correctArray($results["clients"][0]["client"][0]);
			$contact = correctArray($results["clients"][0]["client"][0]["contacts"][0]["contact"][0]);
			$this->smartyVars["client"] = $client;
			$this->smartyVars["contact"] = $contact;

			$selectedEmailActionIDs = array();
			if ($contact["emailActionIDs"]) {
				if ($contact["emailActionIDs"][0]
					&& is_array($contact["emailActionIDs"][0])
					&& isset($contact["emailActionIDs"][0]["emailActionID"]))
				{
					$selectedEmailActionIDs = array();
					foreach ($contact["emailActionIDs"] as $emailAction) {
						$selectedEmailActionIDs[] = $emailAction["emailActionID"];
					}
				} else {
					$selectedEmailActionIDs = $contact["emailActionIDs"];
				}
			}

			$this->smartyVars["selectedEmailActionIDs"] = $selectedEmailActionIDs;

			require_once("lib-billing/include/config/internal/emailactions.php");
			$this->smartyVars["emailActions"] = blmsg_array($CONF["types"]["emailAction"]);

			$formDefaults = $contact;
			$this->mergeFormState($formDefaults);

			// grab secret questions
			$this->smartyVars["secretQuestions"] = getTypes("clientSecretQuestions");
			$this->smartyVars["contactAnswers"] = arrayFor("contactSecretAnswers", array(
				"clientContactID" => (int)$_REQUEST["clientContactID"]));

			$this->smartyVars["isUsernameChangeable"] = Service()->ContactManager->isUsernameChangeable($clientContact);
			if ($clientContact->isSyncable() && !is_null($clientContact->sync_error)) {
				$this->smartyVars["errorMessages"][] = blmsg("TRANS_SYNC_THIS_CONTACT_ERROR_MSG", array("ERROR" => nl2br(billing_safetyhtml($clientContact->sync_error))));
			}

            $this->smartyVars["billingSecureUrl"] = billing_base_url();
            $this->addTemplate("clients/EditClientContact", $this->smartyVars);
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
