<?php

require_once("lib-action/action.php");

use Billing\Object\Customer;
use Billing\Object\Plan;
use Billing\Object\Subscription;
use Billing\TaskManager\Struct\Task;
use Billing\TaskManager\Client as TaskManagerClient;

class EditNewOrdersResponse extends MB_Action
{
	public $params = array("clientID");

	public function execute()
	{
		parent::execute();

		if($_REQUEST["referralActionID"] == getActionID("ShowDashboardNewOrders")) {
			$this->setReferralActionID(getActionID("ShowDashboard"));
			$this->setReferralActionFile("dashboard.php");
		} else {
			$this->setReferralActionID(getActionID("ShowNewOrders"));
			$this->setReferralActionFile("clients.php");
		}

		// These are accepted clients, update their status to active
		// AND -- If their packages are pending, update their status to active
		// EXCEPT -- If their packages are in the queue, leave them alone for the queue to run
		$clientIDs = $this->_filterCustomersByVendor((array)$_REQUEST["activate"]);
		foreach($clientIDs as $clientID) {
			$packages = correctArray(grabResultsFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetPackages",
					"params" => array(
						"clientID" => (int) $clientID,
						"getPackageAttributeData" => 1,
					),
				),
			)));
			foreach ($packages as $package) {
				if (1 == $package["packageActive"] && Subscription::STATUS_PENDING == $package["packageStatus"] &&
					!in_array($package["packageType"], array(Plan::TYPE_HOSTING, Plan::TYPE_SSL_CERT, Plan::TYPE_RESELLER))
				) {
                    $subscription = Factory()->Subscription($package["packageID"]);
                    $subscription->setStatus(\Billing\Object\Subscription::STATUS_ACTIVE, blmsg('TRANS_SUBSCRIPTION_STATUS_REASON_SET_ACTIVE_IF_NOT_PROVISIONING'));
                    $subscription->update();
				}
			}

			try {
				$customer = Factory()->Customer($clientID);
				$customer->client_status = Customer::STATUS_ACTIVE;
				$customer->client_status_reason = $customer->isReseller() ?
					blmsg("TRANS_RESELLER_NEW_ACTIVE_STATUS_REASON") : blmsg("TRANS_CUSTOMER_NEW_ACTIVE_STATUS_REASON");
				Service()->CustomerManager->update($customer);
			} catch (ProductException $e) {
				billing_put_error($e->getMessage());
				redirect("dashboard.php", getActionID("ShowDashboard"));
			}
			if ($customer->isReseller()) {
				Service()->ResellerManager->repairSSO($customer->getReseller());
			}
		}

		// These are not accepted clients, update their status to canceled
		// AND -- If their packages are pending, update their status to canceled
		// EXCEPT -- If their packages are in the queue, update the queue item to canceled
		// ELSE -- If their packages are already active, send them to the queue to be deleted
		$clientIDs = $this->_filterCustomersByVendor((array)$_REQUEST["cancel"]);
		$pendingPackageIDs = NULL;

		foreach($clientIDs as $clientID) {
			$packages = correctArray(grabResultsFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetPackages",
					"params" => array(
						"clientID" => (int) $clientID,
						"getPackageAttributeData" => 1,
					),
				),
			)));

			$customer = Factory()->Customer($clientID);
			$statusReason = $customer->isReseller() ? blmsg("TRANS_RESELLER_NEW_CANCEL_STATUS_REASON") : blmsg("TRANS_CUSTOMER_NEW_CANCEL_STATUS_REASON");
			foreach ($packages as $package){
				$panelTied = FALSE;
				//Pull the server-group tied packages out
				foreach ((array) $package["packageAttributes"] as $attribute){
					if((int)$attribute["serverGroupID"]) {
						$panelTied = TRUE;
						break;
					}
				}

				$subscription = Factory()->Subscription($package["packageID"]);

				dbg("Flip the package status to canceled");
                $subscription->setStatus(Subscription::STATUS_CANCELED, $statusReason);
				$subscription->package_active = 0;
				Service()->SubscriptionManager->update($subscription);

				// IF Package record is ACTIVE
				// IF Package __IS__ tied to a panel
				if($package["packageActive"] && $panelTied) {
					if($package["packageStatus"]  == Subscription::STATUS_PENDING) {
						dbg("Flip any tasks to canceled if they exist for this package");
						TaskManagerClient::cancelTasks(
							array("attributes" => array("subscriptionId" => $package["packageID"])),
							Task::STATUS_CANCELLED,
							"TRANS_TASK_CANCELLED_REASON_SUBSCRIPTION_CANCELLED"
						);
					} else if ($package["packageStatus"]  == Subscription::STATUS_ACTIVE) {
						dbg("Create task for cancellation of subscription");
						$this->createNewTask($package, $_REQUEST["cancelAction"]);
					}
				}
			}

			//Flip client status to canceled
			dbg("Flip client status to active");
			$this->cancelCustomer($customer, Customer::STATUS_CANCELED, $statusReason);
		}

		// These are not accepted clients, update their status to fraud
		// AND -- If their packages are pending, update their status to fraud
		// EXCEPT -- If their packages are in the queue, update the queue item to canceled
		// ELSE -- If their packages are already active, send them to the queue to be deleted
		$clientIDs = $this->_filterCustomersByVendor((array)$_REQUEST["fraud"]);
		$pendingPackageIDs = NULL;

		foreach($clientIDs as $clientID) {

			$invoiceReversal = dispatchMBAPI(toXML(array(
				"mbapi" => array(
					"command" => "ProcessCreditNote",
					"params" => array(
						"clientID" => (int) $clientID,
					),
				),
			)));

			$packages = correctArray(grabResultsFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetPackages",
					"params" => array(
						"clientID" => (int) $clientID,
						"getPackageAttributeData" => 1,
					),
				),
			)));

			$customer = Factory()->Customer($clientID);
			$statusReason = $customer->isReseller() ? blmsg("TRANS_RESELLER_NEW_FRAUDULENT_STATUS_REASON") : blmsg("TRANS_CUSTOMER_NEW_FRAUDULENT_STATUS_REASON");
			foreach ($packages as $package) {
				$panelTied = FALSE;
				//Pull the server-group tied packages out
				foreach ((array) $package["packageAttributes"] as $attribute){
					if((int)$attribute["serverGroupID"]) {
						$panelTied = TRUE;
						break;
					}
				}

				$subscription = Factory()->Subscription($package["packageID"]);

				dbg("Flip the package status to canceled");
                $subscription->setStatus(Subscription::STATUS_CANCELED, $statusReason);
				$subscription->package_active = 0;
				Service()->SubscriptionManager->update($subscription);

				// IF Package record is ACTIVE
				// IF Package __IS__ tied to a panel
				if($package["packageActive"] && $panelTied) {
					if($package["packageStatus"] == Subscription::STATUS_PENDING) {
						dbg("Flip any tasks to canceled if they exist for this package");
						TaskManagerClient::cancelTasks(
							array("attributes" => array("subscriptionId" => $package["packageID"])),
							Task::STATUS_CANCELLED,
							"TRANS_TASK_CANCELLED_REASON_SUBSCRIPTION_FRAUDULENT"
						);
					} else if($package["packageStatus"] == Subscription::STATUS_ACTIVE) {
						dbg("Create task for cancellation of subscription");
						$this->createNewTask($package, $_REQUEST["cancelAction"]);
					}
				}
			}

			//Flip client status to canceled
			dbg("Flip client status to fraudulent");
			$this->cancelCustomer($customer, Customer::STATUS_FRAUDULENT, $statusReason);
		}

		redirect($this->getReferralActionFile(), $this->getReferralActionID(), array("n"=>uniqid(0) ));

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

	private function _filterCustomersByVendor($clientIds)
	{
		$result = array();
		$session = getCurrentSession();
		foreach ($clientIds as $id) {
			if ($session->checkVendor(Factory()->Customer($id))) {
				$result[] = $id;
			}
		}
		return $result;
	}

	private function createNewTask($subscriptionData, $action="suspend")
	{
		//NOTE: if the subscription status is changed from active, the task will not run.
		//The task will flip the status after running the suspend or delete calls

		$cancelActions = array("suspend","delete");
		$action = in_array($action,$cancelActions) ? $action : "suspend" ;

		$subscriptionID = (int) $subscriptionData["packageID"];
        $subscription = Factory()->Subscription($subscriptionID);

		$pushQueueItem = $serverGroupID = $configGroupID = FALSE;
		if(is_array($subscriptionData["packageAttributes"])) {
			foreach($subscriptionData["packageAttributes"] as $key => $value) {
				if($value["packageAttributeName"] == "panel_status") {
					$pushQueueItem = ($value["packageAttributeValue"] == "none")? TRUE : $pushQueueItem;
					$serverGroupID = $value["serverGroupID"];
					$configGroupID = $value["configGroupID"];
				}
			}
		}

		if(!$pushQueueItem) {
			return;
		}

		switch($action) {
			case "suspend":
				$subCommand = "panelSuspend";
				break;
			case "delete":
				$subCommand = "panelDelete";
				break;
		}

		$panelQuery = new MBQuery(
			"ProcessPanel", $subCommand,
			array(
				'subCommand' => $subCommand,
				'configGroupID' => $configGroupID,
				'serverGroupID' => $serverGroupID,
				'packageID' => $subscriptionID,
                'packageStatus' => $subscription->package_status,
                'packageStatusReason' => $subscription->package_status_reason
			)
		);
		$task = new Task(
			"ProcessPanel ({$subCommand})",
			$panelQuery->getXMLQuery(),
			array(
				"panelId" => $configGroupID,
				"panelGroupId" => $serverGroupID,
				"subscriptionId" => $subscriptionID,
			)
		);
		TaskManagerClient::scheduleMbapi($task, $subscription->vendor_id);
	}

	private function cancelCustomer(Customer $customer, $cancelStatus, $cancelStatusReason)
	{
		try {
			if ($customer->isReseller()) {
				$reseller = $customer->getReseller();
				$reseller->active = 0;
				$reseller->status = $cancelStatus;
				$reseller->status_reason = $cancelStatusReason;
				$reseller->date_cancelled = time();
				Service()->ResellerManager->update($reseller);
			} else {
				$customer->client_active = 0;
				$customer->client_status = $cancelStatus;
				$customer->client_status_reason = $cancelStatusReason;
				$customer->client_date_cancelled = time();
				Service()->CustomerManager->update($customer);
			}
		} catch (ProductException $e) {
			billing_put_error($e->getMessage());
			redirect("dashboard.php", getActionID("ShowDashboard"));
		}
	}
}
