<?php

require_once("lib-action/action.php");

use Billing\Manager\SecurityManager;
use Billing\Object\Plan;
use Billing\Object\Subscription;

class ForwardToClientInterface extends MB_Action
{
	public $params = array("clientID", "clientContactID");

	public function execute()
	{
		global $CONF;
		parent::execute();

		$clientID = (int)$_REQUEST["clientID"];
		$clientContactID = (int)$_REQUEST["clientContactID"];

		if(!notZero($clientID) || !notZero( $clientContactID)) {
			$this->addError('TRANS_ACTIONERROR', "Customer ID and Contact ID are required.");
		} else {
			$getUrl = new MBQuery("GetClients", "", array(
							"clientID" => $clientID,
							"getCompanyData" => 1));
			$urlResult = $getUrl->getResult();

			$clientInterfaceUrl = "";
			if ($urlResult->getErrorCount()) {
				$this->addError('TRANS_ACTIONERROR', "Failed to fetch Customer URL information.");
			} else {
				$urlResult = grabSimpleMBAPIResults( $urlResult->getStructure() );
				$clientInterfaceUrl = $urlResult[0]["companyClientURL"];
			}
			unset( $urlResult, $getUrl );

			if ("" == $clientInterfaceUrl && "" == ($clientInterfaceUrl = Service()->CompanyManager->getProviderCompany()->company_client_url)) {
				$this->addError('TRANS_ACTIONERROR', "Client interface location is not set in the system configuration.");
			} elseif ($this->getErrors()) {
				// do nothing
			} else {
				if ((int)$CONF["user"]["type"] == 2) { // The current user IS an admin
					if(isset($_REQUEST["subscriptionID"])) {
						$subscription = Factory()->Subscription($_REQUEST["subscriptionID"]);
						if(!is_null($subscription->getPanelId()) && $subscription->ControlPanel->isPlesk()) {
							$clientCpUrl = rtrim($subscription->ControlPanel->url, "/") . "/admin/subscription/login/externalId/{$subscription->external_id}";
							redirect_url($clientCpUrl);
						}
					}

					if('msp' == PACKAGE_TYPE) {
						$forwardToSubscription = NULL;
						if(!isset($_REQUEST["subscriptionID"])) {
							$subscriptions = Factory()->SubscriptionList(array(
								'client_id' => $clientID,
								'package_type' => Plan::TYPE_HOSTING,
								'package_status' => Subscription::STATUS_ACTIVE
							));
							foreach($subscriptions as $subscription) {
								if(is_null($subscription->getPanelId())) {
									continue;
								}
								$forwardToSubscription = $subscription;
								break;
							}
						} else {
							$forwardToSubscription = Factory()->Subscription($_REQUEST["subscriptionID"]);
						}

						if(!is_null($forwardToSubscription)) {
							$uiUrl = parse_url($forwardToSubscription->ControlPanel->ConfigParams["uiUrl"]->value);
							$clientCpUrl = sprintf($forwardToSubscription->ControlPanel->ConfigParams["clientCpUrl"]->value, $forwardToSubscription->external_id);
							$url = $uiUrl['scheme'] . '://' . $uiUrl['host'] . ':' . $uiUrl['port'] . $clientCpUrl;
							redirect_url(billing_referer_url($url));
						}
					}

					if(isset($_REQUEST["subscriptionID"])) {
						$subscription = Factory()->Subscription($_REQUEST["subscriptionID"]);
						if(!is_null($subscription->getPanelId()) && $subscription->ControlPanel->isPlesk()) {
							$clientCpUrl = $subscription->ControlPanel->ConfigParams["api"]->value . "/admin/subscription/login/externalId/{$subscription->external_id}";
							redirect_url($clientCpUrl);
						}
					}

					$clientQuery = new MBQuery("GetClients", "", array(
										"clientID" => $clientID,
										"clientContactID" => $clientContactID,
										"getContactData" => 4));
					$clientResult = $clientQuery->getResult();
					unset($clientQuery);
					if (!$clientResult->getErrorCount()) {
						// golden
						$clientResults = grabSimpleMBAPIResults($clientResult->getStructure());
						$clientResults = $clientResults[0];
						unset($clientResult);

						if (notZero( $clientResults["clientID"] ) && notZero( $clientResults["contacts"][0]["contact"][0]["clientContactID"] )) {
							$realUserID = getCurrentSession()->real_user_id;
							$realUserType = getCurrentSession()->real_user_type;

							Service()->SecurityManager->pushCurrentActionIntoLog(
								SecurityManager::USER_TYPE_CLIENT,
								$clientID,
								Factory()->Customer($clientID)->PrimaryContact->contact_username,
								"TRANS_ACTION_LOG_REASON_LOGIN",
								blmsg(
									"TRANS_ACTION_LOG_COMMENT_ADMIN_IMPERSONATED",
									array(
										"ADMIN_ID" => getCurrentSession()->user_id,
										"CLIENT_ID" => $clientID,
										"CONTACT_ID" => $clientContactID,
										"USERNAME" => $clientResults["contacts"][0]["contact"][0]["contactUsername"],
									),
									Service()->SecurityManager->getActionLogLocale()
								)
							);

							$newSid = sha1(strval(mt_rand(0, 200000000)).strval(microtime()).strval(mt_rand(0, 200000000)));

							$newSessionQuery = new MBQuery("SetSession", "insert", array(
													"userID" => $clientContactID,
													"userType" => 1, // regular user type
													"realUserID" => $realUserID,
													"realUserType" => $realUserType,
													"sessionIP" => $CONF["user"]["ip"],
													"sessionActive" => 1,
													"sessionID" => $newSid,
													"sessionDateActive" => time(),
													"sessionDateCreated" => time(),
													"sessionDateExpired" => $sessionExpired = (int)(time() + Settings()->session_AdminSessionDuration * 60)));
							$newSessionQuery->getResult();	// create the new session
							unset($newSessionQuery);

                            getCurrentSession()->Storage->impContactSessions[$clientContactID] = $newSid;

							$clientInterfaceUrl = rtrim(billing_referer_url($clientInterfaceUrl), "/");
							$clientInterfaceUrl = "{$clientInterfaceUrl}/index.php?ctr=dashboard&act=".getActionID("ShowDashboard")."&impContactId=".$clientContactID;
                            if (bgpc("impResellerId")) {
                                $clientInterfaceUrl .= "&impResellerId=".bgpc("impResellerId");
                            }
							if (headers_sent() && PHP_SAPI!="cli") {
								dbg("Redirecting VIA Javascript");
								echo "<script language=\"javascript\" type=\"text/javascript\">"
									."//<![CDATA[\n"
									."\tdocument.location.href = '{$clientInterfaceUrl}';\n"
									."//]]>\n"
									."</script>";
							} else {
								dbg("Redirecting VIA Header Redirect to {$clientInterfaceUrl}");
								mbHeader("Location: {$clientInterfaceUrl}");
							}
						}
					} else {
						$this->addError(blmsg("TRANS_ACTIONERROR"), "Unable to fetch Customer Information.");
					}
				} else {
					$this->addError(blmsg("TRANS_ACTIONERROR"), "Must be an admin to process.");
				}
			}
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}