<?php

require_once("lib-action/action.php");
require_once("lib-billing/include/accounting/Accounting.php");

use Billing\Object\LedgerAccount;

/**
 * This class is a child class of MB_Action. It takes $_REQUEST["clientID"] and displays all
 * the client accounting detail information.
 *
 * @author  Kris Bailey
 */
class ShowClientARHistory extends MB_Action
{
    public function __construct()
    {
        $_REQUEST["clientID"] = $this->_getClient()->id;

        parent::MB_Action();
        $this->stringRep = "ShowClientARHistory";
        $this->params = array("clientID");
    }

	public function execute()
    {
		global $CONF;
		parent::execute();
		dbg("at top of action");

		$clientID = ((int) $_REQUEST["clientID"]);
		if(!$clientID) {
			$this->addError('TRANS_ACTIONERROR', "No client ID given.");
		} else {

			// lets get the accounting data here:

			$vendorId = Factory()->Customer($clientID)->vendor_id;

			$params = array(
				"ledgerAccountID" => getGeneralLedgerAccountID(LedgerAccount::NAME_ACCOUNTS_RECEIVABLE, $vendorId),
			);

			$query = $this->getMBAPIQuery("GetAccountingGeneralLedger", "", $params, 1, 1, 1, 1);
			$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
			$data = dispatchMBAPI($query, 1);
			$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);
			dbg("got the data back from the command");
			$results =& $data["mbapi"][0]["results"][0];
			if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {

				if ($results["accounting_general_ledger"] && $results["accounting_general_ledger"][0]["ledger"]){
					$this->smartyVars["ledgerData"] = correctArray($results["accounting_general_ledger"][0]["ledger"]);
				} else {
					$this->smartyVars["ledgerData"] = array();
				}
				$invoiceIDs = array();
				$ledgerEntryIDs = array();
				foreach ($this->smartyVars["ledgerData"] as $row){
					$invoiceIDs[] = $row["invoiceID"];
					$ledgerEntryIDs[] = $row["generalLedgerID"];
				}

				$invoiceData = correctArray(grabResultsFromMBAPIArray2(array(
					"mbapi" => array(
						"command" => "GetInvoices",
						"params" => array(
							"invoiceID" => $invoiceIDs,
							"getInvoiceLineItemsData" => 1,
						),
					),
				)));
				$this->smartyVars["keyedInvoices"] = array();
				$this->smartyVars["lineItemsToPackages"] = array();
				if (is_array($invoiceData)){
					foreach ($invoiceData as $invoice){
						foreach ((array)$invoice["lineItems"] as $li){
							$this->smartyVars["lineItemNames"][$li["lineItemID"]] = array(
								"name" => "(".$li["packageID"].") ".billing_safetyhtml($li["lineItemDescription"]),
								"packageID" => $li["packageID"],
								"lineItemID" => $li["lineItemID"],
							);
						}
						$this->smartyVars["keyedInvoices"][$invoice["invoiceID"]] = $invoice;
					}
				}
				// ok, we have successfully gotten the accounting data

				// lets get the packages for those ledger entry ids
				$controlData = correctArray(grabResultsFromMBAPIArray2(array(
					"mbapi" => array(
						"command" => "GetAccountingControlRel",
						"params" => array(
							"groupingType" => 1,
							"controlType" => getAPITypeID("controls", "lineitems"),
							"generalLedgerID" => $ledgerEntryIDs,
						),
					),
				)));
				dbg($controlData);
				$this->smartyVars["controlData"] = $controlData;
				$this->smartyVars["ledgerPackageNames"] = array();
				foreach ((array)$controlData as $data){
					$thisLineItem = $data["controlID"];
					foreach ((array)$data["generalLedgerIDs"] as $glid){
						$this->smartyVars["ledgerPackageNames"][$glid] = $this->smartyVars["lineItemNames"][$thisLineItem];
					}
				}

				// lets get this clients data
				$this->smartyVars["client"] = correctArray(grabFirstResultFromMBAPIArray2(array(
					"mbapi" => array(
						"command" => "GetClients",
						"params" => array(
							"clientID" => (int)$_REQUEST["clientID"],
							"mergePrimaryContact" => 1,
						),
					),
				)));

				$this->smartyVars["paginationLinks"] = collapseArrayElements($results["paginationLinks"][0]);

				$this->addTemplate("clients/ShowClientARHistory", $this->smartyVars);

			}

		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
