<?php
require_once("lib-action/action.php");

use Billing\Object\Contact;

class ShowClientContacts extends MB_Action
{
	public function execute()
	{
		parent::execute();

		if (1 == bgpc("sync")) {
			$this->processSync();
		}

		$clientID = bgpc("clientID");

		$page  = UIFactory()->Page();
		$page->title = blmsg("TRANS_CONTACTS");

		if (!is_null($clientID)) {
			$customer = Factory()->Customer($clientID);
			$page->parent_title = billing_safetyhtml($customer->PrimaryContact->contact_fullname);
			$page->parent_url = action_url("clients", "ShowClientDetails", array("clientID"	=> $clientID));
		} else {
			$page->parent_title = blmsg("TRANS_CLIENTS");
			$page->parent_url = action_url("clients", "ShowClients");
		}

		require_once("UI/List/UIClientContactList.php");
		$page->addListPanel(new UIClientContactList($clientID));
		$this->smartyVars["page"] = $page;
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}

	private function processSync()
	{
		$contactIDs = bgpc("id", array());
		if (empty($contactIDs)) {
			return;
		}
		$errors = array();
		$leftSyncingContacts = false;
		foreach ($contactIDs as $contactID) {
			try {
				$contact = Factory()->Contact($contactID);
				if ($contact->sync_tasks & Contact::SYNC_CHECK_UPDATE_STATUS) {
					// check previous update operation status only
				} else {
					$contact->sync_tasks |= Contact::SYNC_NEED_UPDATE | Contact::SYNC_NEED_REPAIR_SSO;
				}
				Service()->ContactManager->sync($contact);
				if (!is_null($contact->sync_error)) {
					$errors[] = blmsg("TRANS_SYNC_CONTACT_ERROR_MSG", array("CONTACT_ID" => $contact->id, "ERROR" => $contact->sync_error));
				} else if (!$contact->isSynced() && $contact->isSyncable()) {
					$leftSyncingContacts = true;
				}
			} catch (ProductException $e) {
				$errors[] = blmsg("TRANS_SYNC_CONTACT_ERROR_MSG", array("CONTACT_ID" => $contactID, "ERROR" => $e->getMessage()));
			}
		}
		if (empty($errors)) {
			billing_put_info(blmsg("TRANS_SYNC_CONTACTS_SUCCESS_MSG"));
			if ($leftSyncingContacts) {
				billing_put_warning(blmsg("TRANS_SYNC_CONTACTS_PENDING_MSG"));
			}
		} else {
			billing_put_error(blmsg("TRANS_SYNC_CONTACTS_FAILURE_MSG", array("ERROR" => billing_safetyhtml(implode("\n", $errors)))));
		}
	}
}
