<?php

require_once("lib-action/action.php");
require_once("lib-billing/include/accounting/Accounting.php");

use Billing\Object\LedgerAccount;

/**
 * This class is a child class of MB_Action. It takes $_REQUEST["clientID"] and displays all
 * the client accounting detail information.
 *
 * @author  Michael Fountain
 */
class ShowClientCreditHistory extends MB_Action
{
    public function __construct()
    {
        $_REQUEST["clientID"] = $this->_getClient()->id;

        parent::MB_Action();
        $this->stringRep = "ShowClientCreditHistory";
        $this->params = array("clientID");
    }

	public function execute()
    {
		parent::execute();
		dbg("at top of action");

		$clientID = ((int) $_REQUEST["clientID"]);
		if(!$clientID) {
			$this->addError('TRANS_ACTIONERROR', "No client ID given.");
		} else {

			// lets get the accounting data here:

			$vendorId = Factory()->Customer($clientID)->vendor_id;

			$params = array(
				"ledgerAccountID" => getGeneralLedgerAccountID(LedgerAccount::NAME_CUSTOMER_CREDITS, $vendorId)
			);

			$query = $this->getMBAPIQuery("GetAccountingGeneralLedger", "", $params, 1, 1, 1, 1);
			$data = dispatchMBAPI($query, 1);
			dbg("got the data back from the command");
			$results =& $data["mbapi"][0]["results"][0];
			if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {

				if ($results["accounting_general_ledger"] && $results["accounting_general_ledger"][0]["ledger"]){
					$this->smartyVars["ledgerData"] = correctArray($results["accounting_general_ledger"][0]["ledger"]);
				} else {
					$this->smartyVars["ledgerData"] = array();
				}
				$invoiceIDs = array();
				foreach ($this->smartyVars["ledgerData"] as $row){
					$invoiceIDs[] = $row["invoiceID"];
				}

				$invoiceData = correctArray(grabResultsFromMBAPIArray(array(
					"mbapi" => array(
						"command" => "GetInvoices",
						"params" => array(
							"invoiceID" => $invoiceIDs,
						),
					),
				)));
				$this->smartyVars["keyedInvoices"] = array();
				if (is_array($invoiceData)){
					foreach ($invoiceData as $invoice){
						$this->smartyVars["keyedInvoices"][$invoice["invoiceID"]] = $invoice;
					}
				}
				// ok, we have successfully gotten the accounting data

				// lets get this clients data
				$this->smartyVars["client"] = correctArray(grabFirstResultFromMBAPIArray(array(
					"mbapi" => array(
						"command" => "GetClients",
						"params" => array(
							"clientID" => (int)$_REQUEST["clientID"],
							"mergePrimaryContact" => 1,
						),
					),
				)));

				$this->smartyVars["paginationLinks"] = collapseArrayElements($results["paginationLinks"][0]);

				$this->addTemplate("clients/ShowClientCreditHistory", $this->smartyVars);

			}

		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
