<?php

require_once("lib-action/action.php");
use Billing\Skin;
use Billing\Object\Contact;
use Billing\Object\Plan;
use Billing\Object\Store;
use Billing\Object\BillingAccount;
use Billing\Manager\SecurityManager;
use Billing\Object\LedgerAccount;

class ShowClientDetails extends MB_Action
{
	const PACKAGE_INFO_EMAIL_RESENT_SUCCESSFULLY = 1;
	const PACKAGE_INFO_EMAIL_RESENT_UNSUCCESSFULLY = 0;

    public function __construct()
    {
        $_REQUEST["clientID"] = $this->_getClient()->id;
        parent::__construct();
        $this->params = array("clientID");
    }

    public function execute()
    {
        global $CONF;
        parent::execute();
        $clientID = bgpc("clientID");
        if (bgpc('resendPackageInfoEmail') && ($configGroupId = bgpc('configGroupID')) && ($subscriptionId = bgpc('packageID'))) {
            require_once ('lib-mbapi/include/mail/emailTemplates/packageInfoEmail.php');
            $email = new PackageInfoEmail($subscriptionId, $configGroupId); // need package ID and configGroupID
            redirect("clients.php", "ShowClientDetails", array(
                "clientID" => $clientID,
                "resendPackageInfoEmailStatus" => $email->execute() ? self::PACKAGE_INFO_EMAIL_RESENT_SUCCESSFULLY : self::PACKAGE_INFO_EMAIL_RESENT_UNSUCCESSFULLY
            ));
            exit;
        }

        $customer = Factory()->Customer($clientID);
        $GLOBALS['__pagePath'] = array(($customer->isReseller() ? 'TRANS_MANAGE_RESELLERS' : 'TRANS_MANAGE_CLIENTS'), billing_safetyhtml($customer->PrimaryContact->contact_fullname), '');

        $this->_setResellerTemplateVariables($customer);

        Service()->SecurityManager->pushCurrentActionIntoLog(
            SecurityManager::USER_TYPE_CLIENT,
            $clientID,
            $customer->PrimaryContact->contact_username,
            "TRANS_ACTION_LOG_REASON_SHOW_USER_DETAILS",
            $customer->isReseller()
                ? blmsg("TRANS_ACTION_LOG_COMMENT_RESELLER", array("RESELLER_ID" => $customer->getReseller()->id), Service()->SecurityManager->getActionLogLocale())
                : blmsg("TRANS_ACTION_LOG_COMMENT_CLIENT", array("CLIENT_ID" => $clientID), Service()->SecurityManager->getActionLogLocale())
        );

        if (!is_null($resendPackageInfoEmailStatus = bgpc('resendPackageInfoEmailStatus'))) {
            $this->smartyVars["resendPackageInfoEmailMessage"] = blmsg(self::PACKAGE_INFO_EMAIL_RESENT_SUCCESSFULLY == $resendPackageInfoEmailStatus ? 'TRANS_PACKAGE_INFO_EMAIL_RESENT_SUCCESSFULLY' : 'TRANS_PACKAGE_INFO_EMAIL_RESENT_UNSUCCESSFULLY');
        }
        if ($planId = bgpc('productID')) {
            $this->smartyVars["productID"] = $planId;
            $this->smartyVars["mode"] = (int)$_REQUEST["mode"];
        }
        $this->smartyVars["createInvoice"] = (1 == bgpc("createInvoice", 0));

        $suppressFeePackages = bgpc('showFees') ? 0 : 1;
        $query = array(
            "mbapi" => array(
                "command" => "GetClients",
                "params" => array(
                    "clientID" => (int)$_REQUEST["clientID"],
                    "mergePrimaryContact" => 1,
                    "getPackageData" => 2, // This fetches stats from all packages even though we do not need them - to do - MF
                    "packageActive" => 1,
                    "getDomainData" => 2,
                    "getAccountingData" => 1,
                    "getCertData" => 2,
                    "getCreditData" => 1,
                    "getContactData" => 3,
                    "getBillingData" => 2,
                    "getBookmarkData" => 1,
                    "getClientGroupData" => 1,
                    "getSessionData" => 1,
                    "getTicketData" => 2, // (0=None, 1=All Data, 2=Last 5, 3=Last)
                    "getNoteData" => 2, // (0=None, 1=All Data, 2=Last 5, 3=Last)
                    "suppressFeePackages" => $suppressFeePackages,
                    "hooks" => array(
                        "hook" => array(
                            "hookFunctionName" => "calculateResellerVolumnDiscount",
                        ),
                    ),
                ),
            ),
        );
        $data = dispatchMBAPI($query, 1);
        $results =& $data["mbapi"][0]["results"][0];
        if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {
            $this->smartyVars["clientID"] = $clientID;
            $this->smartyVars["client"] = collapseArrayElements($results["clients"][0]["client"]);
            $client =& $this->smartyVars["client"];

            $fraudResultsLog = grabResultsFromMBAPIArray(array(
                "mbapi" => array(
                    "command" => "GetFraudResultsLog",
                    "params" => array(
                        "clientID" => $clientID,
                    ),
                ),
            ));
            foreach($fraudResultsLog as $key => $item) {
                $fraudResultsLog[$key]['fraudResultFinalActionUsedDesc'] = \Billing\Manager\FraudGuardianManager::getStatusById($item['fraudResultFinalActionUsed']);
            }
            $this->smartyVars["fraudResultsLog"] = $fraudResultsLog;
            // --------------------------------------------------------------
            // MBAPI call to get the default billing account information
            // --------------------------------------------------------------
            $this->smartyVars["defaultBillingAccount"] = oneByID("billingAccounts", $client["defaultBillingAccountID"]);

            // --------------------------------------------------------------
            // Set the primary attribute group for this client
            // --------------------------------------------------------------
            if ($this->smartyVars["client"]["groupID"]) {
                $this->smartyVars["clientAttributeGroup"] = nameByID("attributeGroups", $this->smartyVars["client"]["groupID"]);
            } else {
                $this->smartyVars["clientAttributeGroup"] = "";
            }


            $this->smartyVars["primaryContact"] = collapseArrayElements($results["clients"][0]["client"][0]["contacts"][0]["contact"][0]);
            $this->smartyVars["primaryContact"]["languageName"] = $this->smartyVars["client"]["languageName"];
            $this->smartyVars["clientGroups"] = isset($results["clients"][0]["client"][0]["clientGroups"]) ?
                collapseArrayElements(protectArray($results["clients"][0]["client"][0]["clientGroups"][0]["clientGroup"])) : array()
            ;
            $this->smartyVars["noteList"] = isset($results["clients"][0]["client"][0]["notes"]) ?
                collapseArrayElements(protectArray($results["clients"][0]["client"][0]["notes"][0]["note"])) : array()
            ;
            $username = $customer->PrimaryContact->contact_username;
            $this->smartyVars["isContactBlocked"] = $customer->isReseller() ? Service()->SecurityManager->isAdminBlocked($username)
                    : Service()->SecurityManager->isContactBlocked($username);

            // --------------------------------------------------------------
            // Format the ticket data
            // --------------------------------------------------------------
            $this->smartyVars["lastTickets"] =array();
            if (isset( $results["clients"][0]["client"][0]["tickets"] ) && is_array( $results["clients"][0]["client"][0]["tickets"] )) {
                $this->smartyVars["lastTickets"] = correctArray($results["clients"][0]["client"][0]["tickets"][0]["ticket"]);
            }

            $this->smartyVars["paymentList"] = getTypes("payments");
            $billingAccountTypes = getTypes("billingAccount");
            $this->smartyVars["statusList"] = getStatuses("client");
            $this->smartyVars["packageList"] = $packageStatuses = getStatuses("package");
            $this->smartyVars["domainList"] = getStatuses("package");
            $this->smartyVars["certList"] = $certStatuses = getStatuses("package");

            $this->smartyVars["skinList"] = Skin::getSkins();
            $this->smartyVars["skinID"] = "" == $client["skinID"] ? Skin::DEFAULT_SKIN_ID : $client["skinID"];

            if ($results["clients"][0]["client"][0]["countriesISO2"][0]) {
                $this->smartyVars["clientCountry"] = blmsg("TRANS_ISO_3166_".$results["clients"][0]["client"][0]["countriesISO2"][0]);
                $this->smartyVars["primaryContactCountry"] = blmsg("TRANS_ISO_3166_".$results["clients"][0]["client"][0]["contacts"][0]["contact"][0]["countriesISO2"][0]);
            }

            // accounting data we got
            $this->smartyVars["accountingData"] = collapseArrayElements($results["clients"][0]["client"][0]["accountingGeneralLedgers"][0]["accountingGeneralLedger"]);
            if (!empty($this->smartyVars["accountingData"]["currencyID"])) { // clean it up
                $this->smartyVars["accountingData"] = array($this->smartyVars["accountingData"]);
            }

            // now we can get the totals for each currency of a specific ledger account.
            // ledger accounts you may want totals for are:
            //      "cash", "cash_1", "cash_2" -> the bank accounts, basically, how much money has this client paid us
            //      "accounts_receivable" -> how much this client owes us
            //      "accounts_payable" -> how much misc things (remaining) we have to pay someone else for this client
            //      "domains_payable" -> how much money you have (remaining) to pay registrars for this client
            //      "certs_payable" -> how much money you have (remaining) to pay geotrust for this client
            //      "taxes_payable" -> how much taxes (remaining) you charged this customer to pay to x tax zone
            //      "customer_credits" -> how much money this customer has on credit with you
            //      "revenue" -> how much money you have made off this customer (counts money they paid you and money they still owe you)
            //      "cogs" -> cost of goods, how much money the things you sold this customer have cost you total
            //      "domain_cogs" -> how much money total you have paid for domains you have sold to this customer
            //      "certs_cogs" -> how much money total you have paid for certs you have sold to this customer
            //      "setup_cogs" -> how much money total you have paid in setup fees for things you have sold to this customer
            //      "usage_cogs" -> not implemented yet
            //      "expense" -> misc expenses you have incurred due to this client
            //      "refund_expense" -> how much money you have refunded to this customer
            //
            //      basically, you do:
            //       $var = $this->getAccountingCurrencyTotals($this->smartyVars["accountingData"], "ACCOUNT_NAME_HERE");
            //
            //      and $var will be an array of arrays of form: currencyID=>total, currencyID=>total
            //      example:
            //      array('USD'=>123.45,
            //            'AUD'=>234.56,
            //            'GBP'=>345.67
            //      )
            //
            // below I will do a few accounts we will want by default

            // accounts receivable, what they owe us
            $this->smartyVars["accountingAccountsReceivable"] = $this->getAccountingCurrencyTotals($this->smartyVars["accountingData"], "accounts_receivable");
            // auth only stuff, what we can get by a prior auth capture
            $this->smartyVars["accountingAuthorized"] = $this->getAccountingCurrencyTotals($this->smartyVars["accountingData"], "authorized_payments");
            // customers credits, what we owe them
            $this->smartyVars["accountingCustomerCredits"] = $this->getAccountingCurrencyTotals($this->smartyVars["accountingData"], "customer_credits");
            // revenue, what we have made off them
            $this->smartyVars["accountingRevenue"] = $this->getAccountingCurrencyTotals($this->smartyVars["accountingData"], "revenue");
            // payables, what we owe someone currently for this customer
            $this->smartyVars["accountingPayables"] = $this->getAccountingCurrencyTotals($this->smartyVars["accountingData"], array("accounts_payable", "domains_payable", "certs_payable", "taxes_payable"));
            // cogs, what we have paid other people for this customer
            $this->smartyVars["accountingCogs"] = $this->getAccountingCurrencyTotals($this->smartyVars["accountingData"], array("cogs", "domains_cogs", "certs_cogs", "setup_cogs"));
            // refund, what we have refunded this customer
            $this->smartyVars["accountingRefunds"] = $this->getAccountingCurrencyTotals($this->smartyVars["accountingData"], "refund_expense");
            // expense, misc expenses we have incurred for this customer
            $this->smartyVars["accountingExpenses"] = $this->getAccountingCurrencyTotals($this->smartyVars["accountingData"], "expense");

            ksort($this->smartyVars["accountingAccountsReceivable"]);
            ksort($this->smartyVars["accountingCustomerCredits"]);
            ksort($this->smartyVars["accountingRevenue"]);
            ksort($this->smartyVars["accountingPayables"]);
            ksort($this->smartyVars["accountingCogs"]);
            ksort($this->smartyVars["accountingRefunds"]);
            ksort($this->smartyVars["accountingExpenses"]);


            // count package status
            if (isset($results["clients"][0]["client"][0]["packages"][0]["package"]) && is_array($results["clients"][0]["client"][0]["packages"][0]["package"])) {
                foreach($results["clients"][0]["client"][0]["packages"][0]["package"] as $package) {
                    if (isset($this->smartyVars["packageStatuses"][$package["packageStatus"][0]])) {
                        $this->smartyVars["packageStatuses"][$package["packageStatus"][0]] += $package["packageCount"][0];
                    } else {
                        $this->smartyVars["packageStatuses"][$package["packageStatus"][0]] = $package["packageCount"][0];
                    }
                }
            }

            // count domain status
            if (isset($results["clients"][0]["client"][0]["domains"][0]["domain"]) && is_array($results["clients"][0]["client"][0]["domains"][0]["domain"])) {
                foreach($results["clients"][0]["client"][0]["domains"][0]["domain"] as $domain) {
                    if (isset($this->smartyVars["domainStatuses"][$domain["packageStatus"][0]])) {
                        $this->smartyVars["domainStatuses"][$domain["packageStatus"][0]] += $domain["domainPackageCount"][0];
                    } else {
                        $this->smartyVars["domainStatuses"][$domain["packageStatus"][0]] = $domain["domainPackageCount"][0];
                    }
                }
            }

            // count cert status
            if (isset($results["clients"][0]["client"][0]["certs"][0]["cert"]) && is_array($results["clients"][0]["client"][0]["certs"][0]["cert"])) {
                foreach($results["clients"][0]["client"][0]["certs"][0]["cert"] as $cert) {
                    if (isset($this->smartyVars["certStatuses"][$cert["packageStatus"][0]])) {
                        $this->smartyVars["certStatuses"][$cert["packageStatus"][0]] += $cert["certPackageCount"][0];
                    } else {
                        $this->smartyVars["certStatuses"][$cert["packageStatus"][0]] = $cert["certPackageCount"][0];
                    }
                }
            }

            // count billing account types
            if (isset($results["clients"][0]["client"][0]["billingAccounts"][0]["billingAccount"]) && is_array($results["clients"][0]["client"][0]["billingAccounts"][0]["billingAccount"])) {
                foreach($results["clients"][0]["client"][0]["billingAccounts"][0]["billingAccount"] as $billingAccount) {
                    if (isset($this->smartyVars["billingAccountTypes"][$billingAccount["billingAccountType"][0]])) {
                        $this->smartyVars["billingAccountTypes"][$billingAccount["billingAccountType"][0]] += $billingAccount["billingAccountCount"][0];
                    } else {
                        $this->smartyVars["billingAccountTypes"][$billingAccount["billingAccountType"][0]] = $billingAccount["billingAccountCount"][0];
                    }
                }
            }


            /*
            //Fix the invoice list.  It is returning each line item instead of just the invoice
            $displayInvoices = NULL;
            $invoices = array();
            $invoiceLineItems = correctArray($results["clients"][0]["client"][0]["invoices"][0]["invoice"]);
            $invoiceCount = 0;
            if(is_array($invoiceLineItems)) {

                //$invoices["totalCount"] = count($invoiceLineItems);

                foreach($invoiceLineItems as $lineItem) {
                    $invoices[$lineItem["currencyID"]]["amountDue"] += $lineItem["invoiceAmount"];
                    $invoices[$lineItem["currencyID"]]["amountPaid"] += $lineItem["invoiceAmountPaid"];
                    if($lineItem["invoiceAmount"] === $lineItem["invoiceAmountPaid"]) {
                        $invoices[$lineItem["currencyID"]]["paid"]++;
                    }
                    if($lineItem["invoiceAmount"] > $lineItem["invoiceAmountPaid"]) {
                        $invoices[$lineItem["currencyID"]]["due"]++;
                    }
                //Get 5 invoices for display
                    if (count($displayInvoices)<5){
                        $displayInvoices[$lineItem["invoiceID"]] = array(
                            "invoiceNumber" => $lineItem["invoiceNumber"],
                            "invoiceStatus" => $lineItem["invoiceStatus"],
                            "amount"  => $lineItem["invoiceAmount"],
                            "paid" => $lineItem["invoiceAmountPaid"]);
                    }
                }
                $this->smartyVars["displayInvoices"] = $displayInvoices;
                $this->smartyVars["invoiceDetails"] = $invoices;
            }
            */

            // --------------------------------------------------------------
            // MBAPI call to lookup currency IDs
            // --------------------------------------------------------------
            $currencyIDs = collapseArrayElements(dispatchMBAPI($this->getMBAPIQuery("GetCurrencyIDs", "", array()), 1));
            dbg($currencyIDs);
            $currencyIDs = $currencyIDs["mbapi"]["results"]["currencyID"];
            if (!is_array($currencyIDs)){
                $currencyIDs = array($currencyIDs);
            }
            foreach ($currencyIDs as $cid){
                $this->smartyVars["currencyIDs"][$cid] = $cid;
            }

            // --------------------------------------------------------------
            // MBAPI call to lookup the client attributes
            // --------------------------------------------------------------

            $this->smartyVars["clientAttributes"] = arrayFor(
                "clientAttributes",
                array(
                    "clientID" => $clientID,
                    "attribute" => array(),
                )
            );

            $this->smartyVars["programaticAttributes"] = getTypes("programaticAttributes");

            // --------------------------------------------------------------
            // MBAPI call to lookup the last 5 emails sent to the client
            // --------------------------------------------------------------
            $lastEmails = new MBQuery("GetEmailQueue", "", array(
                "clientContactID" => $this->smartyVars["primaryContact"]["clientContactID"],
                "numRecords" => 5,
                "suppressEmailText" => 1,
                "sortDir" => "desc",
                "sortColumn" => "emailDateSent"));
            $lastEmails = $lastEmails->getResult();
            if ($lastEmails->getSuccess()) {
                $this->smartyVars["lastEmails"] = grabSimpleMBAPIResults( $lastEmails->getStructure() );
            }
            unset($lastEmails);

            // --------------------------------------------------------------
            // MBAPI call to get the sums of all invoices by status
            // --------------------------------------------------------------
            $invoiceStats = new MBQuery("GetInvoices", "", array(
                "clientID" => $clientID,
                "getInvoiceSums" => 1));
            $results = $invoiceStats->getResult();
            $invoiceStats = grabSimpleMBAPIResults($results->getStructure());
            $n = count($invoiceStats);
            $this->smartyVars["invoiceStats"] = array();
            require_once("objects/Invoice.php");
            for ($i=0; $i<$n; $i++) {
                $invoiceStatus = $invoiceStats[$i]["invoiceStatus"];
                $currencyID = $invoiceStats[$i]["currencyID"];
                $numInvoices = $invoiceStats[$i]["numInvoices"];
                $sumInvoicesAmount = $invoiceStats[$i]["sumInvoicesAmount"];
                $sumInvoicesAmountPaid = $invoiceStats[$i]["sumInvoicesAmountPaid"];
                $this->smartyVars["invoiceStats"][$currencyID][$invoiceStatus] = array(
                    "invoiceStatus" => $invoiceStatus,
                    "invoiceStatusName" => Invoice::statusName($invoiceStatus),
                    "numInvoices" => $numInvoices,
                    "sumInvoicesAmount" => $sumInvoicesAmount,
                    "sumInvoicesAmountPaid" => $sumInvoicesAmountPaid,
                    "sumInvoicesAmountDue" => $sumInvoicesAmount - $sumInvoicesAmountPaid,
                );
                sort($this->smartyVars["invoiceStats"][$currencyID]);
            }


            // --------------------------------------------------------------
            // Set locale data
            // --------------------------------------------------------------
            $tmp = explode(";", $CONF["user"]["locale"]->localeArray["locale_pattern_currency"]);
            $tmp[0] = preg_replace("/^¤#0.00/", "¤#,##0.00", $tmp[0]);
            $tmp[1] = preg_replace("/^¤#0.00/", "¤#,##0.00", $tmp[1]);
            $CONF["user"]["locale"]->localeArray["locale_pattern_currency"] = $tmp[0].";".$tmp[1];
            $this->smartyVars["localeArray"] =  $CONF["user"]["locale"]->localeArray;
            $params = array('vendor_id' => $this->_getReseller() ? $this->_getReseller()->id : null, 'product_is_addon' => 0);
            if ($customer->isReseller()) {
                $params["product_type"] = Plan::TYPE_RESELLER;
            }
            $productList = Factory()->PlanList($params);
            if (!$customer->isReseller()) {
                foreach($productList as $key => $product) {
                    if (Plan::TYPE_RESELLER == $product->product_type) {
                        unset($productList[$key]);
                    }
                }
            }
            $this->smartyVars["productCount"] = count($productList);

            // --------------------------------------------------------------
            // Set ledger data names
            // --------------------------------------------------------------

            $this->smartyVars["includeAjaxJS"] = 1;
            $this->mergeThemeData();
            $this->smartyVars["cycles"] = getCyclesArray();
            $this->smartyVars["resendEmailPackages"] = correctArray(getMBAPIArray("GetResendEmailPackages", array(
                "clientID" => $clientID,
            )));

            $this->addLedgerAccounts();
            $this->smartyVars["defaultLegerAccount"] = getGeneralLedgerAccountID(LedgerAccount::NAME_CUSTOMER_EXPENSE, getCurrentSession()->getVendorId());;

            $orderFormsList = array();
            $orderFormsCurrenciesList = array();
            $type = $customer->isReseller() ? Store::TYPE_RESELLER : Store::TYPE_CUSTOMER;
            foreach ( Factory()->StoreList(array('vendor_id' => getCurrentSession()->getVendorId(), 'order_form_type' => $type)) as $orderForm ) {
                $orderFormsList[$orderForm->id] = $orderForm->order_form_name;
                $orderFormsCurrenciesList[$orderForm->id] = $orderForm->currency_id;
            }
            if ( false === asort($orderFormsList) ) {
                global $php_errormsg;
                throw new Exception("Failed to sort order forms: {$php_errormsg}");
            }
            $this->smartyVars["orderForms"] = $orderFormsList;
            $this->smartyVars["orderFormsCurrencies"] = $orderFormsCurrenciesList;

            $contactIDs = IDsOf(":contacts", array(
                "clientID" => $clientID,
            ));
            $this->smartyVars["billingAccounts"] = array();
            foreach(Factory()->BillingAccountList(array("billing_account_active" => 1, "client_contact_id" => $contactIDs)) as $account ){
                $this->smartyVars["billingAccounts"][$account->id] = Factory()->Contact($account->client_contact_id)->contact_fullname." - ".$account->getAccountName();
            }

            if ($customer->isReseller()) {
                $reseller = $customer->getReseller();
                if ($reseller->isSyncable() && !is_null($reseller->sync_error)) {
                    $this->smartyVars["errorMessages"][] = blmsg("TRANS_SYNC_THIS_RESELLER_ERROR_MSG", array("ERROR" => nl2br(billing_safetyhtml($reseller->sync_error))));
                }
            } else {
                if ($customer->isSyncable() && !is_null($customer->sync_error)) {
                    $this->smartyVars["errorMessages"][] = blmsg("TRANS_SYNC_THIS_CUSTOMER_ERROR_MSG", array("ERROR" => nl2br(billing_safetyhtml($customer->sync_error))));
                }
            }
            $this->smartyVars["enableMergeCustomers"] = Config()->general_enableMergeCustomers;
            $this->addTemplate("clients/ShowClientDetails");
        }

        $this->addErrorTemplates();
        return $this->hasExecutedSuccessfully;
    }
}