<?php
require_once("lib-action/action.php");

use Billing\Object\Customer;

class ShowClients extends MB_Action
{
	public function execute()
	{
		parent::execute();

		if (1 == bgpc("sync")) {
			$this->processSync();
		}

		$page  = UIFactory()->Page();
		if (bgpc('search')) {
			$page->title = blmsg("TRANS_CLIENTS_SEARCH", array('SEARCH' => billing_safetyhtml(bgpc('search'))));
		} else {
			$page->title = blmsg("TRANS_CLIENTS");
            $payoutData = Service()->AffiliateProgramManager->getPayoutsData(getCurrentSession()->getVendorId());
			if (0 < Service()->AffiliateProgramManager->getAffiliateProgramsCount(getCurrentSession()->getVendorId()) || !empty($payoutData)) {
				require_once("UI/Tabs/UICustomersScreenTabs.php");
				$tabs = new UICustomersScreenTabs();
				$tabs->setCurrent("customers");
				$page->setScreenTabs($tabs);
			}
		}
		$page->parent_url = action_url("dashboard", "ShowDashboard");

		require_once("UI/List/UIClientList.php");
		$page->addListPanel(new UIClientList());

		$this->smartyVars["page"] = $page;
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}

	private function processSync()
	{
		$customerIDs = bgpc("id", array());
		if (empty($customerIDs)) {
			return;
		}
		$errors = array();
		$leftSyncingCustomers = false;
		foreach ($customerIDs as $customerID) {
			try {
				$customer = Factory()->Customer($customerID);
				if ($customer->sync_tasks & Customer::SYNC_CHECK_UPDATE_STATUS) {
					// check previous update operation status only
				} else {
					$customer->sync_tasks |= Customer::SYNC_NEED_REPAIR;
				}
				Service()->CustomerManager->sync($customer);
				if (!is_null($customer->sync_error)) {
					$errors[] = blmsg("TRANS_SYNC_CUSTOMER_ERROR_MSG", array("CUSTOMER_ID" => $customer->id, "ERROR" => $customer->sync_error));
				} else if (!$customer->isSynced() && $customer->isSyncable()) {
					$leftSyncingCustomers = true;
				}
			} catch (ProductException $e) {
				$errors[] = blmsg("TRANS_SYNC_CUSTOMER_ERROR_MSG", array("CUSTOMER_ID" => $customerID, "ERROR" => $e->getMessage()));
			}
		}
		if (empty($errors)) {
			billing_put_info(blmsg("TRANS_SYNC_CUSTOMERS_SUCCESS_MSG"));
			if ($leftSyncingCustomers) {
				billing_put_warning(blmsg("TRANS_SYNC_CONTACTS_PENDING_MSG"));
			}
		} else {
			billing_put_error(blmsg("TRANS_SYNC_CUSTOMERS_FAILURE_MSG", array("ERROR" => billing_safetyhtml(implode("\n", $errors)))));
		}
	}
}
