<?php
require_once ("lib-action/action.php");

class ShowNewOrders extends MB_Action
{
	public function __construct()
    {
		parent::__construct();

		$this->params = array(
			"clientID",
			"clientCompany",
			"contactFirstName",
			"contactLastName",
			"contactEmail",
			"contactAddress_1",
			"contactAddress_2",
			"contactCity",
			"contactZip",
			"contactUsername"
		);

		// if we are searching for any possible match, set VALID params now
		if ($_REQUEST["query"]!="") {
			// test if the searching for a specific column
			if ($_REQUEST["queryColumn"]!="") {
				$_REQUEST[$_REQUEST["queryColumn"]] = $_REQUEST["query"];
			} else {
				// test if query is an int, if yes, assume we are searching for the ID
				if (((int) $_REQUEST["query"])) {
					$_REQUEST["clientID"] = $_REQUEST["query"];
				} else {
					// if not an int, then search all other non-ID columns
					foreach($this->params as $value) {
						if ($value!="clientID") {
							$_REQUEST[$value] = $_REQUEST["query"];
						}
					}
				}
			}
		}

		$this->sortColumns = array(
			"clientID",
			"contactFirstName",
			"contactLastName",
			"clientCompany",
			"contactEmail"
		);
	}

	public function execute()
    {
		parent::execute();

 		$getMoreData = array(
 			"vendorID" => getCurrentSession()->getVendorId(),
			"clientActive" => 1,
			"clientStatus" => getAPIStatusID("client", "pending"),
			"getAccountingData" => 1,
			"getContactData" => 3, // (0=None, 1=All Data, 2=Sum by Count, 3=Single Record by ContactID)
			"getPackageData" => 1, // (0=None, 1=All Data, 2=Sum by Status, 3=Sum by Billing Cycle, 4=Sum by Product ID)
			"getDomainData" => 1, // (0=None, 1=All Data, 2=Sum by Status, 3=Sum by TLD)
			"getInvoiceData" => 1,
			"getShareData" => 1,
			"getCompanyData" => 1,
			"getClientLastAccess" => 1,
			"getTicketData" => 3, // (0=None, 1=All Data, 2=Last 5, 3=Last)
			"getNoteData" => 3, // (0=None, 1=All Data, 2=Last 5, 3=Last)
			"mergePrimaryContact" => 1
		);

		$query = $this->getMBAPIQuery("GetClients", "", $getMoreData, 1, 1, 1, 1);
		$data = dispatchMBAPI($query, 1);
		$results =& $data["mbapi"][0]["results"][0];

		if($this->addMBAPIErrors($data) === 0) {
			if(isset($results["clients"][0]["client"])) {
				$clients = correctArray($results["clients"][0]["client"]);
			} else {
				$clients = array();
			}
            foreach ($clients as &$client_opts) {
                $customer = Factory()->Customer($client_opts["clientID"]);
                $client_opts["contactFullName"] = $customer->PrimaryContact->contact_fullname;
                $client_opts["isReseller"] = $customer->isReseller();
            }
			$this->smartyVars["clients"] = $clients;

			$noteList = array();
			$c = count($clients);
			for($i = 0; $i < $c; $i++) {
				// leave this uncollapsed so that the template code can work with both singular and plural results; see the comment in the template for details
				$noteData =& $results["clients"][0]["client"][$i]["notes"][0]["note"];
				$noteList[$results["clients"][0]["client"][$i]["clientID"][0]] = collapseArrayElements(protectArray($noteData));

				$getHdTickets = new MBQuery("GetHelpdeskTickets", "", array(
										"userType" => 1,
										"userID" => $results["clients"][0]["client"][ $i ]["clientID"][0]),
											array(
										"sortColumn" => "hdTicketDateModified",
										"sortDir" => "desc",
										"numRecords" => 1));
				$hdTicket = $getHdTickets->getResult();
				$hdTicket = grabSimpleMBAPIResults( $hdTicket->getStructure() );
				$this->smartyVars["clients"][ $i ]["helpdeskTicket"] = $hdTicket[0];

				unset($hdTicket, $getHdTickets);
			}
			$this->smartyVars["noteList"] = $noteList;

			// assign filtered columns
			if (isset($_REQUEST["clientStatus"]) && is_numeric($_REQUEST["clientStatus"])) {
				$this->smartyVars["clientStatus"] = ((int) $_REQUEST["clientStatus"]);
			}

			$this->smartyVars["clientStatuses"] = getStatuses("client");
			$this->smartyVars["paginationLinks"] = collapseArrayElements($results["paginationLinks"][0]);
			$this->addTemplate("clients/ShowNewOrders", $this->smartyVars);

		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}

