<?php

require_once("lib-action/action.php");

use Billing\Object\BillingAccount;

class TakeClientPaymentResponse extends MB_Action
{
	public $params = array("clientID", "invoiceIDs");

	public function execute()
	{
		global $CONF;
		parent::execute();
		loadConfigs("payment");

		$customer = Factory()->Customer(getParam("clientID"));

		if (!getCurrentSession()->isValidToken()) {
			billing_put_error(blmsg("TRANS_ERROR_TOKEN"));
			redirect("clients.php", "TakeClientPayment", array("clientID" => $customer->id));
		}

		$invoiceIDs = bgpc("invoiceIDs", array());
		if (empty($invoiceIDs)) {
			billing_put_error(blmsg("TRANS_NO_INVOICES_TO_PAY_SELECTED_MSG"));
			redirect("clients.php", getActionID("TakeClientPayment"), array("clientID" => $customer->id));
		}
		$this->smartyVars["client"] = oneByID("clients", $customer->id, array("mergePrimaryContact" => 1));

		$this->paymentAction = Settings($customer->vendor_id)->paymentQueue_transactionAction;
		if ($this->paymentAction!="auth"){
			$this->paymentAction = "charge";
		}
		$runCmd = array(
			"mbapi" => array(
				"command" => "ProcessPaymentQueue",
				"params" => array(
					"invoiceID" =>  $invoiceIDs,
					"generateNew" => 1,
					"runPaymentQueue" => 1,
					"getTransactionData" => 1,
					"transactionAction" => $this->paymentAction,
					"ignoreQuarantinePeriod" => (int)bgpc("ignoreQuarantinePeriod"),
                    "ignoreFailuresNum" => (int)bgpc("ignoreFailuresNum"),
					"batchRun" => 1,
				),
			),
		);

        $invoiceAmount = 0.00;
        foreach($invoiceIDs as $id) {
        	$invoice = Factory()->Invoice($id);
            $invoiceAmount += $invoice->invoice_amount - $invoice->invoice_amount_paid;
        }
		$results = dispatchMBAPI(toXML($runCmd), 1);
		if ($this->addMBAPIErrors($results) === 0 && !$this->getNumErrors()) {

			$this->smartyVars["transactionInfo"] = correctArray($results["mbapi"][0]["results"][0]);

			$billingAccountIDs = array();
            $transactionAmount = 0.00;
			foreach ((array)$this->smartyVars["transactionInfo"]["transaction"] as $tr){
				$billingAccountIDs[] = $tr["billingAccountID"];
                $transactionAmount += $tr["transactionAmount"];
			}
			$billingAccounts = arrayFor("billingAccounts", array(
				"billingAccountID" => $billingAccountIDs,
			));
			$this->smartyVars["billingAccounts"] = array();
			foreach ($billingAccounts as $x=>$ba){
				$this->smartyVars["billingAccounts"][$ba["billingAccountID"]] =& $billingAccounts[$x];
			}
			$this->smartyVars["paymentStatuses"] = $CONF["statuses"]["transactions"];
			if (!isset($this->smartyVars["transactionInfo"]["transaction"])){
                $this->smartyVars["warningMessages"][] = (is_null($customer->vendor_id))
                    ? blmsg("TRANS_TAKE_CLIENT_PAYMENT_NO_TRANSACTION_CREATED")
                    : blmsg("TRANS_TAKE_RESELLER_PAYMENT_NO_TRANSACTION_CREATED");
			} elseif (abs($invoiceAmount - $transactionAmount) > 0.001) {
				$this->smartyVars["warningMessages"][] = (is_null($customer->vendor_id))
                    ? blmsg("TRANS_TAKE_CLIENT_PAYMENT_PARTLY_PAID")
                    : blmsg("TRANS_TAKE_RESELLER_PAYMENT_PARTLY_PAID");
            }

			$this->smartyVars["billingAccountTypes"] = BillingAccount::types();
		} else {
			$this->smartyVars["errors"] = correctArray($results["mbapi"][0]["header"][0]["errors"][0]["error"]);
		}

		$this->addTemplate("clients/TakeClientPaymentResponse", $this->smartyVars);
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
