<?php
use Billing\Manager\BillingAccountManager;
use Billing\Object\BillingAccount;
use Billing\Exception\ForbiddenOperationException;

class Clients_BillingAccountController extends Billing_Zend_Controller_Action_AdminController
{
    const TYPE_QUARANTINE = 1;
    const TYPE_LIMIT_FAILURES = 2;

    public function quarantineStopAction()
    {
        $this->_unlock(self::TYPE_QUARANTINE);
    }

    public function limitFailuresResetAction()
    {
        $this->_unlock(self::TYPE_LIMIT_FAILURES);
    }

    private function _unlock($type)
    {
        $billingAccount = Factory()->BillingAccount($this->getRequest()->getParam('id'));
        try{
            if (!getCurrentSession()->checkVendor($billingAccount->Contact->Customer)) {
                throw new ForbiddenOperationException();
            }

            if (self::TYPE_QUARANTINE == $type) {
                $billingAccount->billing_account_date_failed = 0;
            } else {
                $billingAccount->billing_account_failures_num = 0;
            }
            $billingAccount->update();

            $this->_redirectWithMessage($this->view->ActionUrl('clients', 'EditBillingAccount', array(
                    'clientID' => $billingAccount->Contact->Customer->id,
                    'clientContactID' => $billingAccount->Contact->id,
                    'billingAccountID' => $billingAccount->id
                )),
                $this->view->translate((self::TYPE_QUARANTINE == $type) ? 'TRANS_BILLING_ACCOUNT_QUARANTINE_STOP_SUCCESS' : 'TRANS_BILLING_ACCOUNT_LIMIT_FAILURES_RESET_SUCCESS'),
                "info"
            );
        } catch (ProductException $e) {
            $this->_redirectWithMessage($this->view->ActionUrl('clients', 'ShowClients'), $e->getMessage(), "error");
        }
    }
}