<?php

class Companies_BrandingController extends Billing_Zend_Controller_Action_AdminController
{
	public function init()
	{
		parent::init();
		if("ShowDashboard" == bgpc("redirectAction")) {
			$this->_backUrl = $this->view->ActionUrl('dashboard', 'ShowDashboard');
		} else {
			$this->_backUrl = $this->view->adminActionUrl('companies', 'branding');
		}
	}

    public function indexAction()
    {
    	if ($reseller = getCurrentSession()->getReseller()) {
			$this->view->company = Factory()->Company($reseller->company_id);
		} else {
			$this->view->company = Service()->CompanyManager->getProviderCompany();
		}

        $this->view->domain = $domain = $this->_getDomain();

    	$toolButtons = array(
			'addDomain' => array(
				'label' => 'TRANS_MANAGE_SYSTEM_BRANDING_DOMAIN_ADD',
				'class' => 'sb-plan-convert',
				'href' => $this->view->adminActionUrl('companies', 'branding', 'add-domain'),
			),
			'removeDomain' => array(
				'label' => 'TRANS_MANAGE_SYSTEM_BRANDING_DOMAIN_DELETE',
				'class' => 'sb-plan-convert',
				'href' => $this->view->adminActionUrl('companies', 'branding', 'remove-domain'),
			),
    		'urls' => array(
				'label' => 'TRANS_MANAGE_SYSTEM_BRANDING_URLS',
				'class' => 'sb-plan-settings',
				'href' => $this->view->adminActionUrl('companies', 'branding', 'urls')
			)
		);
		if (getCurrentSession()->isReseller() && !getCurrentSession()->isImpersonated()) {
			unset($toolButtons['urls']);
		}
		if ($domain) {
			unset($toolButtons['addDomain']);
		} else {
			unset($toolButtons['removeDomain']);
		}
		// in current implementation admin has no ability to add branding
		if (!getCurrentSession()->isReseller()) {
			unset($toolButtons['addDomain']);
			unset($toolButtons['removeDomain']);
		}
		if ('msp' != PACKAGE_TYPE) {
			unset($toolButtons['addDomain']);
			unset($toolButtons['removeDomain']);
		}
		$this->view->toolButtons = !empty($toolButtons) ? $toolButtons : null;
    }

    public function urlsAction()
    {
    	if ($reseller = getCurrentSession()->getReseller()) {
			$company = Factory()->Company($reseller->company_id);
		} else {
			$company = Service()->CompanyManager->getProviderCompany();
		}
        $form = new Billing_Zend_Form_Final_BrandingUrlsForm($company, $reseller, array('cancelUrl' => $this->_backUrl));
		if ($this->_request->isPost()) {
			try {
				if ($form->isValid($this->_request->getPost())) {
                    $form->process();
                    $this->_redirectWithMessage($this->_backUrl, $this->_translator->translate('TRANS_MANAGE_SYSTEM_BRANDING_URLS_EDIT_SUCCESS'));
                }
			} catch (ProductException $e) {
				$this->_status->addError($e->getMessage());
			}
		}
		$this->view->form = $form;
    }

    public function addDomainAction()
    {
    	if ('msp' != PACKAGE_TYPE) {
    		$this->_redirectWithMessage($this->view->adminActionUrl('companies', 'branding'), $this->_translator->translate('TRANS_MANAGE_SYSTEM_BRANDING_DOMAIN_RESTRICTED'));
    	}
        $reseller = getCurrentSession()->getReseller();
        $form = new Billing_Zend_Form_Final_BrandingAddDomainForm($reseller, array('cancelUrl' => $this->_backUrl));
		if ($this->_request->isPost()) {
			try {
				if ($form->isValid($this->_request->getPost())) {
                    $form->process();
                    $this->_redirectWithMessage($this->_backUrl, $this->_translator->translate('TRANS_MANAGE_SYSTEM_BRANDING_DOMAIN_ADD_SUCCESS'));
                }
			} catch (ProductException $e) {
				$this->_status->addError($e->getMessage());
			}
		}
		$this->view->form = $form;
    }

	public function removeDomainAction()
    {
    	if ('msp' != PACKAGE_TYPE) {
    		$this->_redirectWithMessage($this->view->adminActionUrl('companies', 'branding'), $this->_translator->translate('TRANS_MANAGE_SYSTEM_BRANDING_DOMAIN_RESTRICTED'));
    	}
        $reseller = getCurrentSession()->getReseller();
        $form = new Billing_Zend_Form_Final_BrandingDeleteDomainForm($reseller, array('cancelUrl' => $this->_backUrl));
		if ($this->_request->isPost()) {
			try {
				if ($form->isValid($this->_request->getPost())) {
                    $form->process();
                    $this->_redirectWithMessage($this->_backUrl, $this->_translator->translate('TRANS_MANAGE_SYSTEM_BRANDING_DOMAIN_DELETE_SUCCESS'));
                }
			} catch (ProductException $e) {
				$this->_status->addError($e->getMessage());
			}
		}
		$this->view->form = $form;
    }

    private function _getDomain()
    {
        $attributes = $this->_getAdminAttributes();
        return isset($attributes['branding_domain']) ? $attributes['branding_domain']->value : null;
    }

	private function _setDomain($domain)
    {
    	$attributes = $this->_getAdminAttributes();
    	$attributes['branding_domain']->value = $domain;
    	$attributes->update();
    }

    private function _clearDomain()
    {
    	$attributes = $this->_getAdminAttributes();
    	unset($attributes['branding_domain']);
    	$attributes->update();
    }

    private function _getAdminAttributes()
    {

    	$adminId = ($reseller = getCurrentSession()->getReseller()) ? $reseller->Admin->id : Service()->AdminManager->getMasterAdmin()->id;
        $panel = Service()->ControlPanelManager->getMasterPanel();
        return Factory()->AdminPanelParams(array($adminId, $panel->id));
    }

    private $_backUrl = NULL;
}