<?php
require_once("lib-action/action.php");
require_once("objects/ConfigGroup.php");
require_once("UI/Page.php");

use Billing\Database\Transaction;
use Billing\Checker;
use Billing\Manager\InvoiceManager;

class EditConfigParamsPage extends UIPage
{
    const PARAM_NAME_INVOICE_SEQUENCE = 'invoice_sequence';

    public function __construct(ConfigGroup $configGroup, UIPageController $controller = null)
    {
        parent::__construct($controller);
        $this->_configGroup = $configGroup;
    }

	public function validate(array &$error_messages)
	{
        $is_valid = true;
		foreach ( $this->controls as $control ) {
			if (! $control->validate($error_messages))
				$is_valid = false;
		}
        if ($this->_configGroup->config_group_id == Factory()->ConfigGroup(0)->getChild("automation")->getChild("autoSuspension")->config_group_id) {
            $params = array();
            foreach($this->inputControls as $control) {
                $params[$control->name]['value'] = $control->value;
                $params[$control->name]['title'] = $control->title;
            }
            if ($params['autoSuspendFirstNotice']['value'] > $params['autoSuspendSecondNotice']['value'] && $is_valid) {
                $error_messages[] = blmsg("TRANS_CANCELLATION_SETTINGS_NOTIFICATION_PARAM_ERROR");
                $is_valid = false;
            }
            if ($params['autoSuspendSecondNotice']['value'] > $params['autoSuspendThirdNotice']['value'] && $is_valid) {
                $error_messages[] = blmsg("TRANS_CANCELLATION_SETTINGS_NOTIFICATION_PARAM_ERROR");
                $is_valid = false;
            }
            if (($params['autoSuspendFirstNotice']['value'] > $params['autoSuspendClean']['value']
                || $params['autoSuspendSecondNotice']['value'] > $params['autoSuspendClean']['value']
                || $params['autoSuspendThirdNotice']['value'] > $params['autoSuspendClean']['value']) && $is_valid) {
                $error_messages[] = blmsg(
                    "TRANS_CANCELLATION_SETTINGS_NOTIFICATION_OF_SUSPENSION_PARAM_ERROR",
                    array(
                        "SUSPEND_PARAM" => $params['autoSuspendClean']['title'],
                    )
                );
                $is_valid = false;
            }
        }
		return $is_valid;
	}

	public function createControls()
	{
		$inputPanel = UIFactory()->InputPanel();

		$configGroupDescription = $this->_configGroup->config_group_description;
		if(!is_null($configGroupDescription)) {
			$descriptionPanel = UIFactory()->InputHTML("configGroupDescription");
			$descriptionPanel->value = blmsg($configGroupDescription);
			$inputPanel->addInputControl($descriptionPanel);
		}

		$items = array();
		foreach($this->_configGroup->ConfigParams as $configParam) {
			$items[] = array(
				'decriptor' => array(
					'id'			=> $configParam->config_param_name,
					'title'			=> $configParam->config_param_trans_key,
					'desc'			=> $configParam->config_param_description,
					'type'			=> $configParam->config_param_input_type,
					'required'		=> (1 == $configParam->config_param_required),
					'checker'		=> $this->getChecker($configParam->config_param_name),
					'extention'		=> ("f" == $configParam->config_param_input_type && "" != $configParam->config_param_value) ? blmsg("TRANS_FILE_UPLOADED_MSG") : NULL,
					'items'			=> $this->getParamValues($configParam->config_param_id)
				),
				'id' => $configParam->config_param_id,
				'value' => $configParam->config_param_value,
				'order' => $configParam->config_param_order
			);
		}
		usort($items, function($item1, $item2) {
			if ($item1['order'] == $item2['order']) {
				return 0;
			}
			return ($item1['order'] < $item2['order']) ? -1 : 1;
		});

		foreach ($items as $item) {
			$inputControl = UIFactory()->InputConfig($item['decriptor']);
			$inputControl->value = $item['value'];
			$inputPanel->addInputControl($inputControl);
			$this->inputControls[$item['id']] = $inputControl;
		}

		$this->addInputPanel($inputPanel);
		$this->getDefaultForm()->setMultipartEncryption();
		$this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;
	}

	public function processInput()
	{
		try {
			$conn = Service()->DatabaseManager->getConnection();
			$transaction = new Transaction($conn);
            foreach($this->inputControls as $id => $control) {
                $control->value = (self::PARAM_NAME_INVOICE_SEQUENCE == $control->name) ? ltrim($control->value, '0') : $control->value;
                $this->saveValue($id, $control->value, $conn);
            }
			$transaction->commit();
			getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_SETTINGS_SAVED_SUCCESSFULLY");
            redirect(admin_action_url("configs", "system", "show"));
		} catch (ProductException $e) {
			$this->addErrorMessage(UIFactory()->ErrorMessage($e->getMessage()));
		}

		return !$this->hasWarnings() && !$this->hasErrors();
	}

	protected function getSubmitButtons()
	{
		return array(UIFactory()->SubmitButton("ok", blmsg("TRANS_SAVE"), UIPageController::CMD_OK, "", "", "action"));
	}

	private function saveValue($configParamID, $configParamValue, $conn)
	{
		$invoiceLogoParam = Factory()->ConfigGroup(0)->getChild("billing")->getChild("pdfInvoice")->ConfigParams["invoice_logo"];
		if ($configParamID == $invoiceLogoParam->config_param_id) {
			$file = $_FILES[$invoiceLogoParam->config_param_name];
			if ("" == $file["name"]) {
				return;
			}

			require_once('FileDownloadClass.php');
			$ext = FileDownloadClass::getFileExt($invoiceLogoParam->config_param_name);
			$reseller = getCurrentSession()->getReseller();
			$defaultLogoName = 'logo' . ($reseller ? '-' . $reseller->id : '') . '.' . $ext;
			if (!in_array($ext, Service()->InvoiceManager->getPdfImageTypes())) {
				throw new ProductException(ProductException::E_FORBIDDEN_OPERATION, blmsg("TRANS_INCORRECT_IMAGE_FILE_TYPE", array("IMAGE_TYPE" => $ext)));
			}
			FileDownloadClass::downloadFile(Config()->invoiceLogoDirectory, $invoiceLogoParam->config_param_name, $defaultLogoName);
			$configParamValue = $defaultLogoName;
			if ($configParamValue != $invoiceLogoParam->config_param_value && InvoiceManager::DEFAULT_INVOICE_LOGO_FILE != $invoiceLogoParam->config_param_value) {
				unlink(Config()->invoiceLogoDirectory."/".$invoiceLogoParam->config_param_value);
			}
		}

		$configParamValue = (!is_array($configParamValue)) ? $configParamValue : $configParamValue[0];
		$conn->exec("UPDATE config_params SET config_param_value = ".$conn->quote($configParamValue)." WHERE config_param_id = {$configParamID}");
	}

	private function getChecker($paramName)
	{
		$checkers["sync"] = array(
			"count" => Checker::P_INT_NULL
		);

		$checkers["invoice"] = array(
			"billingDayOfMonth" => Checker::P_UINT_NOT_NULL,
			"billingProrationThreshold" => Checker::P_UINT_NOT_NULL,
			"invoice_sequence" => Checker::P_UINT_NOT_NULL,
			"invoice_generate_offset" => Checker::P_UINT_NOT_NULL,
			"invoice_domain_generate_offset" => Checker::P_UINT_NOT_NULL,
			"invoice_due_offset" => Checker::P_UINT_NOT_NULL,
		);

		$checkers["systemQueue"] = array(
			"system_queue_push_time" => Checker::P_UINT_NOT_NULL
		);

		$checkers["autoSuspension"] = array(
			"autoSuspendFirstNotice" => Checker::P_UINT_NOT_NULL,
			"autoSuspendSecondNotice" => Checker::P_UINT_NOT_NULL,
			"autoSuspendThirdNotice" => Checker::P_UINT_NOT_NULL,
			"autoSuspendClean" => Checker::P_UINT_NOT_NULL
		);

		$checkers["pdfInvoice"] = array(
			"horizontal_indent" => Checker::P_INT_NULL,
			"vertical_indent" => Checker::P_INT_NULL,
			"horizontal_size" => Checker::P_INT_NULL,
			"vertical_size" => Checker::P_INT_NULL,
			"font_size" => Checker::P_INT_NULL
		);

		$configGroupName = $this->_configGroup->config_group_name;
		return isset($checkers[$configGroupName][$paramName]) ? $checkers[$configGroupName][$paramName] : NULL;
	}

	private function getParamValues($configParamID)
	{
		$query = "SELECT config_param_value_name as value, config_param_value_trans_key as title
			FROM config_param_values
			WHERE config_param_id = {$configParamID} ORDER BY config_param_value_order";

		$values = array();
		foreach(Service()->DatabaseManager->getConnection()->getAll($query) as $value) {
			$values[] = array($value["value"], $value["title"]);
		}
		return $values;
	}

    private $_configGroup;
}

class EditConfigParams extends MB_Action
{
	public function execute()
	{
		parent::execute();

		if(!isset($_REQUEST["configGroupID"])) {
			throw new Exception("Missing parameter 'configGroupID'");
		}

		$configGroupId = bgpc("configGroupID");

		$editableConfigGroups = array("versions", "clientSidePaymentOptions", "managementrules", "sync", "invoice", "systemQueue", "autoSuspension", "display", "pdfInvoice");
		if(!in_array(Factory()->ConfigGroup($configGroupId)->config_group_name, $editableConfigGroups)) {
			throw new Exception("Config group '{$configGroupId}' is not editable");
		}

		$controller  = UIFactory()->PageController();
        $configGroup = Factory()->ConfigGroup($configGroupId);
		$controller->addPage($edit_page = new EditConfigParamsPage($configGroup, $controller));

		$edit_page->title = $configGroup->config_group_trans_key;
		$edit_page->parent_title = "TRANS_MANAGE_SYSTEM";
		$edit_page->parent_url = admin_action_url("configs", "system", "show");

		$controller->processRequest();
		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}