<?php
use Billing\Manager\CryptManager;

class Configs_EncryptionController extends Billing_Zend_Controller_Action_AdminController
{
	public function overviewAction()
	{
		$this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_GENERAL_ENCRYPTION_SETTINGS'));

		$this->view->toolButtons = array();
        $admin = Service()->AdminManager->getAdmin();
		if (Service()->CryptManager->isEncryptionEnabled()) {
            if (Service()->CryptManager->canOperateKeyEncryptionKeyPassphrase($admin)) {
    			$this->view->toolButtons['renew-key'] = array(
    				'label' => 'TRANS_ENCRYPTION_RENEW_ENCRYPTION_KEY',
    				'class' => 'sb-encryption-renew-key',
    				'href' => $this->view->adminActionUrl('configs', 'encryption', 'renew-key'),
    			);
            }
			$this->view->toolButtons['change-settings'] = array(
				'label' => 'TRANS_ENCRYPTION_CHANGE_SETTINGS',
				'class' => 'sb-encryption-change-settings',
				'href' => $this->view->adminActionUrl('configs', 'encryption', 'change-settings'),
			);
            if ( Service()->CryptManager->canOperatePrivateKeyEncryptionKey($admin) ) {
                $this->view->toolButtons['view-key'] = array(
                    'label' => 'TRANS_ENCRYPTION_DOWNLOAD_KEY',
                    'class' => 'sb-encryption-view-key',
                    'href' => $this->view->adminActionUrl('configs', 'encryption', 'view-key'),
                );
            }
            if ( Service()->CryptManager->canOperateKeyEncryptionKeyPassphrase($admin) || Service()->CryptManager->canOperatePrivateKeyEncryptionKey($admin) ) {
                $this->view->toolButtons['upload-key'] = array(
                    'label' => 'TRANS_ENCRYPTION_UPLOAD_KEY',
                    'class' => 'sb-encryption-upload-key',
                    'href' => $this->view->adminActionUrl('configs', 'encryption', 'upload-key'),
                );
            }
            $this->view->toolButtons['turn-off'] = array(
                'label' => 'TRANS_ENCRYPTION_TURN_OFF',
                'class' => 'sb-encryption-turn-off',
                'separate' => true,
                'href' => $this->view->adminActionUrl('configs', 'encryption', 'turn-off'),
            );
		} else {
            if (Service()->CryptManager->isEncryptionTurnedOn()) {
                if (Service()->CryptManager->canOperatePrivateKeyEncryptionKey(Service()->AdminManager->getAdmin())) {
                    $this->view->toolButtons['view-key'] = array(
                        'label' => 'TRANS_ENCRYPTION_DOWNLOAD_KEY',
                        'class' => 'sb-encryption-view-key',
                        'href' => $this->view->adminActionUrl('configs', 'encryption', 'view-key'),
                    );
                }
                $this->view->toolButtons['turn-off'] = array(
                    'label' => 'TRANS_ENCRYPTION_TURN_OFF',
                    'class' => 'sb-encryption-turn-off',
                    'separate' => !empty($this->view->toolButtons),
                    'href' => $this->view->adminActionUrl('configs', 'encryption', 'turn-off'),
                );
            } else {
    			$this->view->toolButtons['turn-on'] = array(
    				'label' => 'TRANS_ENABLE_ENCRYPTION',
    				'class' => 'sb-encryption-turn-on',
    				'href' => $this->view->adminActionUrl('configs', 'encryption', 'turn-on'),
    			);
            }
		}
	}

	public function turnOnAction()
	{
        $backUrl = $this->_getBackUrl();
        if (Service()->CryptManager->isEncryptionTurnedOn()) {
            $this->_redirectWithMessage($backUrl, $this->_translator->translate('TRANS_ENCRYPTION_IS_TURNED_ON'), 'error');
        }
		$this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_ENCRYPTION_TURN_ON_TITLE'));
        if (null !== Service()->CryptManager->needToReencryptFrom()) {
            $this->_status->addWarning($this->_translator->translate('TRANS_ENCRYPTION_UPDATE_WARNING'));
        }
		$turnOnForm = new Billing_Zend_Form_Final_EncryptionRenewKeyForm(array('context' => array(
			'admin' => Service()->AdminManager->getAdmin(),
			'cancelLink' => $backUrl,
            'action' => Billing_Zend_Form_Final_EncryptionRenewKeyForm::ACTION_TURN_ON,
		)));
		if ($this->_request->isPost() && $turnOnForm->isValid($this->_request->getPost())) {
			try {
				$turnOnForm->process();
                if (Service()->CryptManager->canOperatePrivateKeyEncryptionKey(Service()->AdminManager->getAdmin()))  {
                    redirect(admin_action_url('configs', 'encryption', 'view-key', array("redirectAction" => $this->getRequest()->getParam('redirectAction'))));
                }
                $this->_redirectWithMessage($backUrl, $this->_translator->translate("TRANS_ENCRYPTION_MODE_UPDATED_SUCCESSFULLY_FOR_OTHER_OWNER_MSG"), "info");

			} catch (ProductException $e) {
				$this->_status->addError($e->getMessage());
			}
		}
		$this->view->form = $turnOnForm;
	}

    public function turnOffAction()
    {
        $backUrl = $this->_getBackUrl();
        if (!Service()->CryptManager->isEncryptionTurnedOn()) {
            $this->_redirectWithMessage($backUrl, $this->_translator->translate('TRANS_ENCRYPTION_IS_TURNED_OFF'), 'error');
        }
        $this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_ENCRYPTION_TURN_OFF_TITLE'));
        $turnOffForm = new Billing_Zend_Form_Final_EncryptionTurnOffForm(array('context' => array(
            'cancelLink' => $backUrl,
        )));
        if ($this->_request->isPost() && $turnOffForm->isValid($this->_request->getPost())) {
            try {
                $turnOffForm->process();
                $this->_redirectWithMessage($backUrl, $this->_translator->translate("TRANS_ENCRYPTION_TURNED_OFF_SUCCESFULLY"));
            } catch (ProductException $e) {
                $this->_status->addError($e->getMessage());
            }
        }
        $this->view->form = $turnOffForm;
    }

    public function changeSettingsAction()
    {
        $this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_ENCRYPTION_CHANGE_SETTINGS_TITLE'));
         $backUrl = $this->_getBackUrl();
         $changeForm = new Billing_Zend_Form_Final_EncryptionChangeSettingsForm(array('context' => array(
             'cancelLink' => $backUrl,
         )));
         if ($this->_request->isPost() && $changeForm->isValid($this->_request->getPost())) {
             try {
                 $changeForm->process();
                 $this->_redirectWithMessage($backUrl, $this->_translator->translate("TRANS_ENCRYPTION_SETTINGS_SUCCESFULLY_UPDATED"));
             } catch (ProductException $e) {
                 $this->_status->addError($e->getMessage());
             }
         }
         $this->view->form = $changeForm;
    }

    public function renewKeyAction()
    {
        $backUrl = $this->_getBackUrl();
        $admin = Service()->AdminManager->getAdmin();
        if (!Service()->CryptManager->isEncryptionEnabled()) {
            $this->_redirectWithMessage($backUrl, $this->_translator->translate('TRANS_ENCRYPTION_IS_NOT_ENABLED_MSG'), 'error');
        }
        if (!Service()->CryptManager->canOperateKeyEncryptionKeyPassphrase($admin)) {
            $this->_redirectWithMessage($backUrl, $this->_translator->translate('TRANS_ENCRYPTION_CURRENT_ADMIN_DOES_NOT_HAVE_PERMISSIONS_TO_ENCRYPTION_KEY_ACTIONS'), 'error');
        }
        if (!Service()->CryptManager->canDecryptData()) {
            $this->_redirectWithMessage($backUrl, $this->_translator->translate('TRANS_ENCRYPTION_NO_PASSPHRASE_OR_PRIVATE_KEY_PRESENT_MSG'), 'error');
        }
        $this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_ENCRYPTION_RENEW_ENCRYPTION_KEY'));
        $this->_status->addWarning($this->_translator->translate('TRANS_ENCRYPTION_UPDATE_WARNING'));
		$renewKeyForm = new Billing_Zend_Form_Final_EncryptionRenewKeyForm(array('context' => array(
			'admin' => $admin,
			'cancelLink' => $backUrl,
            'action' => Billing_Zend_Form_Final_EncryptionRenewKeyForm::ACTION_RENEW,
		)));
		if ($this->_request->isPost() && $renewKeyForm->isValid($this->_request->getPost())) {
			try {
				$renewKeyForm->process();
                if (Service()->CryptManager->canOperatePrivateKeyEncryptionKey($admin, CryptManager::KEY_TYPE_NEW)) {
                    redirect(admin_action_url('configs', 'encryption', 'view-key', array(
                        'redirectAction' => $this->getRequest()->getParam('redirectAction'),
                        'type' => CryptManager::KEY_TYPE_NEW,
                    )));
                }
                $this->_redirectWithMessage($backUrl, $this->_translator->translate('TRANS_ENCRYPTION_MODE_NEW_UPDATED_SUCCESSFULLY_FOR_OTHER_OWNER_MSG'), 'info');
			} catch (ProductException $e) {
				$this->_status->addError($e->getMessage());
			}
		}
		$this->view->form = $renewKeyForm;
    }

    public function uploadKeyAction()
    {
        $this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_ENCRYPTION_UPLOAD_KEY_TITLE'));
        $backUrl = $this->_getBackUrl();
        $admin = Service()->AdminManager->getAdmin();
        if (!Service()->CryptManager->canOperateKeyEncryptionKeyPassphrase($admin) && !Service()->CryptManager->canOperatePrivateKeyEncryptionKey($admin)) {
            $this->_redirectWithMessage($backUrl, $this->_translator->translate("TRANS_ENCRYPTION_UPLOAD_KEY_ACCESS_RESTRICTED"), 'error');
        }
        $uploadKeyForm = new Billing_Zend_Form_Final_EncryptionUploadKeyForm(array('context' => array(
            'admin' => $admin,
            'cancelLink' => $backUrl,
        )));
        if ($this->_request->isPost() && $uploadKeyForm->isValid($this->_request->getPost())) {
            try {
                $uploadKeyForm->process();
                $passphraseAdmin = Service()->CryptManager->getLekBackupOperator(1);
                $privateKeyAdmin = Service()->CryptManager->getLekBackupOperator(2);
                if (Service()->CryptManager->needToUploadKeyEncryptionKeyPassphrase($passphraseAdmin)) {
                    $this->_redirectWithMessage($backUrl, $this->_translator->translate("TRANS_ENCRYPTION_KEY_PASSPHRASE_KEY_UPLOADED_SUCCESSFULLY"), "warning");
                } elseif (Service()->CryptManager->needToUploadPrivateKeyEncryptionKey($privateKeyAdmin)){
                    $this->_redirectWithMessage($backUrl, $this->_translator->translate("TRANS_ENCRYPTION_KEY_PRIVATE_KEY_UPLOADED_SUCCESSFULLY_"), "warning");
                }
                $this->_redirectWithMessage($backUrl, $this->_translator->translate("TRANS_ENCRYPTION_KEY_UPLOADED_SUCCESSFULLY"), "info");
            } catch (ProductException $e) {
                $this->_status->addError($e->getMessage());
            }
        }
        $this->view->form = $uploadKeyForm;
    }

    public function viewKeyAction()
    {
        $this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_ENCRYPTION_VIEW_KEY_TITLE'));
        $backUrl = $this->_getBackUrl();
        $admin = Service()->AdminManager->getAdmin();
        $keyType = $this->getRequest()->getParam('type', CryptManager::KEY_TYPE_CURRENT);
        if (!Service()->CryptManager->canOperatePrivateKeyEncryptionKey($admin, $keyType)) {
            $this->_redirectWithMessage($backUrl, $this->_translator->translate("TRANS_ENCRYPTION_CURRENT_ADMIN_DOES_NOT_HAVE_PERMISSIONS_TO_ENCRYPTION_KEY_ACTIONS"), "error");
        }
        try {
            $viewForm = new Billing_Zend_Form_Final_EncryptionViewKeyForm(array('context' => array(
                'keyType' => $keyType,
            )));
            if ($this->_request->isPost() && $viewForm->isValid($this->_request->getPost())) {
                if (!Service()->CryptManager->needToDownloadPrivateKeyEncryptionKey($keyType)) {
                    $this->_redirect($backUrl);
                }
                $viewForm->process();
                if (null !== Service()->CryptManager->needToReencryptFrom()) {
                    redirect(admin_action_url('configs', 'encryption', 'reencrypt', array('redirectAction' => $this->getRequest()->getParam('redirectAction'))));
                }
                $this->_redirectWithMessage($backUrl, $this->_translator->translate("TRANS_ENCRYPTION_PRIVATE_KEY_SUCCESFULLY_DOWNLOADED"));
            }
        } catch (ProductException $e) {
            $this->_status->addError($e->getMessage());
        }
        $this->view->form = $viewForm;
    }

    public function downloadKeyAction()
    {
        $keyType = $this->getRequest()->getParam('type', CryptManager::KEY_TYPE_CURRENT);
        if (!Service()->CryptManager->canOperatePrivateKeyEncryptionKey(Service()->AdminManager->getAdmin(), $keyType)) {
            $this->_redirectWithMessage(admin_action_url('dashboard', 'home', 'index'), $this->_translator->translate("TRANS_ENCRYPTION_CURRENT_ADMIN_DOES_NOT_HAVE_PERMISSIONS_TO_ENCRYPTION_KEY_ACTIONS"), "error");
        }
        $privateEncryptionKey = Service()->CryptManager->getPrivateKeyEncryptionKey($keyType);
        header("HTTP/1.1 200 OK");
        header('Content-Type:  text/plain');
        if (!isset($_SERVER['HTTP_ACCEPT_ENCODING']) || '' == $_SERVER['HTTP_ACCEPT_ENCODING']) {
            header('Content-Length: ' . mb_strlen($privateEncryptionKey, "8bit"));
        }
        header("Content-Disposition: attachment; filename=\"privateEncryptionKey.txt\"");
        if (isset($_SERVER['HTTP_USER_AGENT']) and strpos($_SERVER['HTTP_USER_AGENT'],'MSIE')) {
            header('Content-Type: application/force-download');
        } else {
            header('Content-Type: application/octet-stream');
        }
        echo $privateEncryptionKey;
        exit;
    }

	public function reencryptAction()
	{
		$this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_ENCRYPTION_REENCRYPT_BILLING_ACCOUNTS_TITLE'));
		$backUrl = $this->_getBackUrl();
        $admin = Service()->AdminManager->getAdmin();
        $keyType = in_array(Service()->CryptManager->needToReencryptFrom(), array(CryptManager::MODE_SIMPLE, CryptManager::MODE_SECURE)) ?
            CryptManager::KEY_TYPE_NEW : CryptManager::KEY_TYPE_CURRENT
        ;
        $reseller = getCurrentSession()->getReseller();
        if (
            !Service()->CryptManager->canOperatePrivateKeyEncryptionKey($admin, $keyType) &&
            !Service()->CryptManager->canOperateKeyEncryptionKeyPassphrase($admin, $keyType) &&
            ($reseller && CryptManager::MODE_PASSPHRASE != Service()->CryptManager->getEncryptionMode($reseller->id))
        ) {
            $this->_redirectWithMessage($backUrl, $this->_translator->translate('TRANS_ENCRYPTION_CURRENT_ADMIN_DOES_NOT_HAVE_PERMISSIONS_TO_ENCRYPTION_KEY_ACTIONS'), 'error');
        }
		try {
			$reencryptForm = new Billing_Zend_Form_Final_EncryptionReencryptForm(array('context' => array(
				'admin' => $admin,
			)));
			if ($this->_request->isPost() && $reencryptForm->isValid($this->_request->getPost())) {
                ignore_user_abort(true);
                set_time_limit(0);
				$reencryptForm->process();
				$this->_redirectWithMessage($backUrl, $this->_translator->translate("TRANS_ENCRYPTION_REENCRYPT_BILLING_ACCOUNTS_SUCCESS_MSG"));
			}
		} catch (ProductException $e) {
			$this->_status->addError($e->getMessage());
		}
		$this->view->form = $reencryptForm;
	}

    private function _getBackUrl()
    {
        return ('ShowDashboard' == $this->getRequest()->getParam('redirectAction')) ?
			$this->view->adminActionUrl('dashboard', 'home', 'index') :
			$this->view->adminActionUrl('configs', 'encryption', 'overview')
		;
    }
}