<?php
require_once("lib-action/action.php");

/**
 * This class is a child class of MB_Action. It lists all pending customers.
 */
class ShowDashboardNewOrders extends MB_Action
{
	public function __construct()
	{
		parent::__construct();

		// set all possible xml params here, $_GET is already cleaned
		$this->params = array(
			"clientID",
			"clientCompany",
			"contactFirstName",
			"contactLastName",
			"contactEmail",
			"contactAddress_1",
			"contactAddress_2",
			"contactCity",
			"contactZip",
			"contactUsername"
		);

		// if we are searching for any possible match, set VALID params now
		if ('' != ($query = bgpc("query"))) {
			// test if the searching for a specific column
			if ($_REQUEST["queryColumn"]!="") {
				$_REQUEST[$_REQUEST["queryColumn"]] = $query;
			} else {
				// test if query is an int, if yes, assume we are searching for the ID
				if ((int)$query) {
					$_REQUEST["clientID"] = $query;
				} else {
					// if not an int, then search all other non-ID columns
					foreach($this->params as $value) {
						if ($value!="clientID") {
							$_REQUEST[$value] = $query;
						}
					}
				}
			}
		}

		$this->sortColumns = array(
			"clientID",
			"contactFirstName",
			"contactLastName",
			"clientCompany",
			"contactEmail"
		);
	}

	public function execute()
	{
		parent::execute();

		$query = array(
			"mbapi" => array(
				"command" => "GetClients",
				"params" => array(
					"clientActive" => 1,
					"clientStatus" => getAPIStatusID("client", "pending"),
					"getInvoiceData" => 3,
					"mergePrimaryContact" => 1,
					"sortDir" => "desc",
					"sortColumn" => "clientDateCreated",
					"recordOffset" => 0,
					"numRecords" => 8,
                    "vendorID" => getCurrentSession()->getVendorId()
				),
			),
		);

		$query = toXML($query);

		require_once("lib-mbapi/mbapi.php");
		$data = dispatchMBAPI($query, 1);
		$results =& $data["mbapi"][0]["results"][0];

		if($this->addMBAPIErrors($data) === 0) {
			if(isset($results["clients"][0]["client"])) {
				$clients = correctArray($results["clients"][0]["client"]);
			} else {
				$clients = array();
			}
			foreach ($clients as &$client_opts) {
                $customer = Factory()->Customer($client_opts["clientID"]);
				$client_opts["isReseller"] = $customer->isReseller();
                $client_opts["contactFullName"] = $customer->PrimaryContact->contact_fullname;
			}
			$this->smartyVars["clients"] = $clients;
			$this->smartyVars["newOrderCount"] = $results["paginationLinks"][0]["paginationStats"][0]["totalRows"][0];
			$this->smartyVars["clientStatuses"] = getStatuses("client");
			require_once("objects/Invoice.php");
			$this->smartyVars["invoiceStatuses"] =  Invoice::statuses();
			$this->addTemplate("dashboard/ShowDashboardNewOrders", $this->smartyVars);

		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}

