<?php

require_once("lib-action/action.php");

class ShowSyncStatistics extends MB_Action
{
	public function execute()
	{
		parent::execute();
        $adminItems = array(
            "panels" => array(
				"name" => "TRANS_SYNC_STATISTICS_PANELS",
				"manager_class_name" => "ControlPanelManager",
				"get_sync_statistics_method_name" => "getSyncStatistics",
				"list_controller" => "servergroups",
				"list_action" => "ShowServerGroups",
				"order" => 1,
			),
			"administrator" => array(
				"name" => "TRANS_SYNC_STATISTICS_ADMINISTRATORS",
				"manager_class_name" => "AdminManager",
				"get_sync_statistics_method_name" => "getSyncStatistics",
				"list_controller" => "admins",
				"list_action" => "ShowAdmins",
				"order" => 2,
			),
			"reseller" => array(
				"name" => "TRANS_SYNC_STATISTICS_RESELLERS",
				"manager_class_name" => "ResellerManager",
				"get_sync_statistics_method_name" => "getSyncStatistics",
				"list_controller" => "resellers",
				"list_action" => "ShowResellers",
				"order" => 3,
			),
			"reseller_plans" => array(
				"name" => "TRANS_SYNC_STATISTICS_RESELLERS_PLANS",
				"manager_class_name" => "PlanManager",
				"get_sync_statistics_method_name" => "getResellerPlansSyncStatistics",
				"list_controller" => "products",
				"list_action" => "ShowProducts",
                "filter" => array("filter[product_is_addon]" => 0, "reseller" => 1),
				"order" => 7,
			),
        );
		$commonItems = array(
			"customer" => array(
				"name" => "TRANS_SYNC_STATISTICS_CUSTOMERS",
				"manager_class_name" => "CustomerManager",
				"get_sync_statistics_method_name" => "getSyncStatistics",
				"list_controller" => "clients",
				"list_action" => "ShowClients",
				"order" => 4,
			),
			"customer_contact" => array(
				"name" => "TRANS_SYNC_STATISTICS_CUSTOMER_CONTACTS",
				"manager_class_name" => "ContactManager",
				"get_sync_statistics_method_name" => "getSyncStatistics",
				"list_controller" => "clients",
				"list_action" => "ShowClientContacts",
				"order" => 5,
			),
			"plans" => array(
				"name" => "TRANS_SYNC_STATISTICS_PLANS",
				"manager_class_name" => "PlanManager",
				"get_sync_statistics_method_name" => "getPlansSyncStatistics",
				"list_controller" => "products",
				"list_action" => "ShowProducts",
                "filter" => array("filter[product_is_addon]" => 0),
				"order" => 6,
			),
			"addons" => array(
				"name" => "TRANS_SYNC_STATISTICS_ADDONS",
				"manager_class_name" => "PlanManager",
				"get_sync_statistics_method_name" => "getAddonsSyncStatistics",
				"list_controller" => "products",
				"list_action" => "ShowProducts",
                "filter" => array("filter[product_is_addon]" => 1),
				"order" => 8,
			),
			"subscriptions" => array(
				"name" => "TRANS_SYNC_STATISTICS_SUBSCRIPTIONS",
				"manager_class_name" => "SubscriptionManager",
				"get_sync_statistics_method_name" => "getSyncStatistics",
				"list_controller" => "packages",
				"list_action" => "ShowPackages",
				"order" => 9,
			),
		);
		$objectsDescriptor = $this->_isReseller() ? $commonItems : array_merge($adminItems, $commonItems);

		uasort($objectsDescriptor, function($left, $right) { return ($left["order"] < $right["order"]) ? -1 : 1; });

		$syncStatistics = array();
		foreach ($objectsDescriptor as $objectName => $objectDescriptor) {
			$syncStatistics[$objectName] = array();
            $managerName = $objectDescriptor['manager_class_name'];
            $methodName = $objectDescriptor['get_sync_statistics_method_name'];
			$syncStatistics[$objectName]["statistics"] = Service()->$managerName->$methodName();
            if ("panels" == $objectName) {
                $syncStatistics[$objectName]["statistics"]["synced"] = $syncStatistics[$objectName]["statistics"]["synced"] + $syncStatistics[$objectName]["statistics"]["non-syncable"];
            }
			$syncStatistics[$objectName]["label"] = blmsg($objectDescriptor["name"]);
            $filter = isset($objectDescriptor["filter"]) ? $objectDescriptor["filter"] : array();
            $syncStatistics[$objectName]["synced_url"] = action_url($objectDescriptor["list_controller"], $objectDescriptor["list_action"], array_merge($filter, array("filter[sync_status]" => "synced")));
			$syncStatistics[$objectName]["error_url"] = action_url($objectDescriptor["list_controller"], $objectDescriptor["list_action"], array_merge($filter, array("filter[sync_status]" => "error")));
			$syncStatistics[$objectName]["syncing_url"] = action_url($objectDescriptor["list_controller"], $objectDescriptor["list_action"], array_merge($filter, array("filter[sync_status]" => "syncing")));
		}
		if ("msp" == PACKAGE_TYPE) {
			$this->smartyVars["masterPanelSyncInfo"] = $syncStatistics["panels"];
			$this->smartyVars["masterPanelSyncInfo"]["label"] = blmsg("TRANS_SYSTEM_MANAGE_POA_PANEL_TITLE");
			$this->smartyVars["masterPanelSyncInfo"]["sync_url"] = action_url("modules", "EditPanelConfig", array(
				"sync" => 1,
				"configGroupID" => Service()->ControlPanelManager->getMasterPanel()->id,
				"redirectAction" => "ShowDashboard",
			));
			$this->smartyVars["masterPanelSyncInfo"]["error_url"] = action_url("modules", "EditPanelConfig", array(
				"configGroupID" => Service()->ControlPanelManager->getMasterPanel()->id,
				"redirectAction" => "ShowDashboard",
			));
			unset($syncStatistics["panels"]);
		}
		$this->smartyVars["syncStatistics"] = $syncStatistics;
		require_once("lib-tk/include/sso/SSOSettings.php");
		if (SSOSettings::spIsRegistered(NULL, SSOSettings::APP_ADMIN)) {
			$ssoSyncInfo = array(
				'status' => Service()->SsoManager->getSSOServerSyncStatus(),
				'sync_url' => action_url("sso", "SyncSSO"),
				'mode' => Settings()->sso_mode == SettingsClass::SSO_MODE_STANDARD ? "TRANS_SSO_MODE_STANDARD" : "TRANS_SSO_MODE_SIMPLIFIED",
				'mode_docs_url' => \Billing\getHelpContextUrl("sso_modes", 1158),
			);
			$this->smartyVars["ssoSyncInfo"] = $ssoSyncInfo;
		}

		$this->smartyVars["syncTaskInfo"] = Service()->SyncManager->getSyncTaskInfo();

		$this->addTemplate("dashboard/ShowSyncStatistics");
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
