<?php

use Billing\Manager\CryptManager;

class Dashboard_HomeController extends Billing_Zend_Controller_Action_AdminController
{
    public function indexAction()
    {
        $reseller = getCurrentSession()->getReseller();
        $admin = Service()->AdminManager->getAdmin();
        if (!$reseller &&
            in_array(Service()->CryptManager->needToReencryptFrom(), array(CryptManager::MODE_NONE, CryptManager::MODE_LEK, CryptManager::MODE_PASSPHRASE)) &&
            !Service()->CryptManager->isEncryptionTurnedOn()
        ) {
            $this->_status->addWarning($this->_translator->translate(
                'TRANS_ENCRYPTION_MODE_CHANGED_MSG',
                array("URL" => admin_action_url('configs', 'encryption', 'turn-on', array('redirectAction' => 'ShowDashboard')))
            ));
        }
        if (Service()->CryptManager->canOperatePrivateKeyEncryptionKey($admin)) {
            if (Service()->CryptManager->needToDownloadPrivateKeyEncryptionKey()) {
                $this->_status->addWarning($this->_translator->translate(
                    'TRANS_ENCRYPTION_PRIVATE_KEY_IS_NOT_DOWNLOADED_YET',
                    array("URL" => admin_action_url('configs', 'encryption', 'view-key', array('redirectAction' => 'ShowDashboard')))
                ));
            } else if (in_array(Service()->CryptManager->needToReencryptFrom(), array(CryptManager::MODE_NONE, CryptManager::MODE_LEK, CryptManager::MODE_PASSPHRASE))) {
                $this->_status->addWarning($this->_translator->translate(
                    'TRANS_ENCRYPTION_MODE_CHANGED_MSG',
                    array("URL" => admin_action_url('configs', 'encryption', 'reencrypt', array('redirectAction' => 'ShowDashboard')))
                ));
            }
        } else if (Service()->CryptManager->canOperateKeyEncryptionKeyPassphrase($admin) &&
            !Service()->CryptManager->needToDownloadPrivateKeyEncryptionKey() &&
            in_array(Service()->CryptManager->needToReencryptFrom(), array(CryptManager::MODE_NONE, CryptManager::MODE_LEK, CryptManager::MODE_PASSPHRASE))
        ) {
            $this->_status->addWarning($this->_translator->translate(
                'TRANS_ENCRYPTION_MODE_CHANGED_MSG',
                array("URL" => admin_action_url('configs', 'encryption', 'reencrypt', array('redirectAction' => 'ShowDashboard')))
            ));
        }
        if (Service()->CryptManager->canOperatePrivateKeyEncryptionKey($admin, CryptManager::KEY_TYPE_NEW)) {
            if (Service()->CryptManager->needToDownloadPrivateKeyEncryptionKey(CryptManager::KEY_TYPE_NEW)) {
                $this->_status->addWarning($this->_translator->translate(
                    'TRANS_ENCRYPTION_PRIVATE_KEY_NEW_IS_NOT_DOWNLOADED_YET',
                    array("URL" => admin_action_url('configs', 'encryption', 'view-key', array('redirectAction' => 'ShowDashboard', 'type' => CryptManager::KEY_TYPE_NEW)))
                ));
            } else if (in_array(Service()->CryptManager->needToReencryptFrom(), array(CryptManager::MODE_SIMPLE, CryptManager::MODE_SECURE))) {
                $this->_status->addWarning($this->_translator->translate(
                    'TRANS_ENCRYPTION_RENEW_NEED_TO_REENCRYPT_MSG',
                    array("URL" => admin_action_url('configs', 'encryption', 'reencrypt', array('redirectAction' => 'ShowDashboard')))
                ));
            }
        } else if (Service()->CryptManager->canOperateKeyEncryptionKeyPassphrase($admin, CryptManager::KEY_TYPE_NEW) &&
            !Service()->CryptManager->needToDownloadPrivateKeyEncryptionKey(CryptManager::KEY_TYPE_NEW) &&
            in_array(Service()->CryptManager->needToReencryptFrom(), array(CryptManager::MODE_SIMPLE, CryptManager::MODE_SECURE))
        ) {
            $this->_status->addWarning($this->_translator->translate(
                'TRANS_ENCRYPTION_RENEW_NEED_TO_REENCRYPT_MSG',
                array("URL" => admin_action_url('configs', 'encryption', 'reencrypt', array('redirectAction' => 'ShowDashboard')))
            ));
        }

        if ($reseller && Service()->CryptManager->isAdminEncryptionEnabled() && CryptManager::MODE_PASSPHRASE == Service()->CryptManager->getEncryptionMode($reseller->id)) {
            $this->_status->addWarning($this->_translator->translate(
                'TRANS_ENCRYPTION_MODE_CHANGED_RESELLER_MSG',
                array('URL' => admin_action_url('configs', 'encryption', 'reencrypt', array('redirectAction' => 'ShowDashboard')))
            ));
        }
        if (Service()->CryptManager->isEncryptionKeyExpired() && Service()->CryptManager->canOperateKeyEncryptionKeyPassphrase($admin)) {
            $this->_status->addWarning($this->_translator->translate(
                'TRANS_ENCRYPTION_KEY_IS_EXPIRED_MSG',
                array("URL" => admin_action_url('configs', 'encryption', 'renew-key', array('redirectAction' => 'ShowDashboard')))
            ));
        }
        if (Service()->CryptManager->needToUploadKeyEncryptionKeyPassphrase($admin)){
            $this->_status->addWarning($this->_translator->translate('TRANS_ENCRYPTION_KEY_PASSPHRASE_KEY_LOST_DESC', array("URL" => admin_action_url('configs', 'encryption', 'upload-key', array('redirectAction' => 'ShowDashboard')))));
        }
        if (Service()->CryptManager->needToUploadPrivateKeyEncryptionKey($admin)){
            $this->_status->addWarning($this->_translator->translate('TRANS_ENCRYPTION_KEY_PRIVATE_KEY_LOST_DESC', array("URL" => admin_action_url('configs', 'encryption', 'upload-key', array('redirectAction' => 'ShowDashboard')))));
        }
        if ($reseller) {
            $this->view->showQuickSetup = $reseller->hasAccessToSettings();
            if (!$this->view->showQuickSetup) {
                $this->_status->addWarning($this->_translator->translate('TRANS_RESELLER_FUNCTIONALITY_IS_LIMITED_BECAUSE_NO_ACTIVE_SUBSCRIPTION'));
            }
        } else {
            $this->view->showQuickSetup = true;
        }
        $this->view->isQuickSetupCollapsed = bgpc('dashboard-system-setup-hide', 0);
    }

    public function showBillingInfoAction()
    {
    	$this->_helper->layout->setLayout('ajax');
    	if ($reseller = getCurrentSession()->getReseller()) {
    		$billingInfo = Service()->CustomerManager->getBillingInfo($reseller->Customer);
    		if (isset($billingInfo['credit_balance']) && !empty($billingInfo['credit_balance'])) {
    			$creditData = array();
    			foreach ($billingInfo['credit_balance'] as $currency => $amount) {
    				if ($amount != 0) {
    					$creditData[$currency] = $amount;
    				}
    			}
    			if ($creditData) {
    				$this->view->creditData = $creditData;
    			}
    		}

    		$this->view->overuseData = Service()->CustomerManager->getTotalOveruse($reseller->Customer);

    		if (isset($billingInfo['invoices_due']) && !empty($billingInfo['invoices_due'])) {
    			$dueData = array();
    			foreach ($billingInfo['invoices_due'] as $currency => $amount) {
    				if ($amount != 0) {
    					$dueData[$currency] = $amount;
    				}
    			}
    			if ($dueData) {
    				$this->view->dueData = $dueData;
    			}
    		}
    	}
    }

}