<?php
use Billing\Object\AffiliateProgram;

class Discounts_AffiliateController extends Billing_Zend_Controller_Action_AdminController
{
	public function init()
	{
		parent::init();
		$this->_backUrl = $this->view->ActionUrl('discounts', 'ShowAffiliatePrograms');
	}

	public function addAction()
	{
        $this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_AFFILIATE_PROGRAM_ADD'));
        $this->_processForm( Billing_Zend_Form_Final_EditAffiliateForm::OPERATION_ADD, "TRANS_AFFILIATE_PROGRAM_ADD_SUCCESS_MSG" );
	}

	public function editAction()
	{
        $affiliateProgram = Factory()->AffiliateProgram($this->_request->getParam('id'));
        if (!$this->_checkVendor($affiliateProgram)) {
            $this->_redirectWithMessage($this->_backUrl, blmsg('TRANS_AFFILIATE_OTHERS_PROGRAMS_ERROR'), Billing_Zend_Controller_Action_Status::STATUS_ERROR);
        }
		$this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_AFFILIATE_PROGRAM_EDIT', array('NAME' => billing_safetyhtml($affiliateProgram->name))));
        $this->_processForm( Billing_Zend_Form_Final_EditAffiliateForm::OPERATION_EDIT, "TRANS_AFFILIATE_PROGRAM_EDIT_SUCCESS_MSG", $affiliateProgram );
	}

    public function removeAction()
   	{
   		$this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_AFFILIATE_PROGRAM_REMOVE'));
        $ids = $this->getRequest()->getParam('id');
        
        $idsForRemove = array();
        foreach($ids as $id) {
            $affiliateProgram = Factory()->AffiliateProgram($id);
            if ($this->_checkVendor($affiliateProgram)) {
                $idsForRemove[] = $id;
            }
        }
        
        $postData = $this->_request->getPost();
        $removeForm = new Billing_Zend_Form_Final_RemoveAffiliateForm(array('context' => array(
           'cancelLink' =>  $this->_backUrl,
           'affiliateIds' => isset($idsForRemove) ? $idsForRemove : NULL,
           'postData' => $postData,
        )));
        $this->_status->addWarning(blmsg('TRANS_AFFILIATE_PROGRAM_REMOVE_PAGE_WARNING'));
        if ($this->_request->isPost() && $removeForm->isValid($postData))
        {
           try {
               $result = $removeForm->processForm($postData);
               if ($result) {
                   $this->_redirectWithMessage($this->_backUrl, $this->_translator->translate("TRANS_AFFILIATE_PROGRAMS_REMOVING_SUCCESS_MSG"), "info");
               }
           } catch (ProductException $e) {
               $this->_status->addError($e->getMessage());
           }
        }
        $this->view->form = $removeForm;
   	}

    private function _processForm($operation, $successMessageKey, AffiliateProgram $affiliateProgram = null)
    {
        $currentForm = new Billing_Zend_Form_Final_EditAffiliateForm(array('context' => array(
            'cancelLink' =>  $this->_backUrl,
            'operation' =>  $operation,
            'affiliateProgram' => $affiliateProgram,
        )));
        if ($this->_request->isPost() && $currentForm->isValid($this->_request->getPost()))
        {
            try {
                $currentForm->processForm();
                $this->_redirectWithMessage($this->_backUrl, $this->_translator->translate($successMessageKey), "info");
            } catch (ProductException $e) {
                $this->_status->addError($e->getMessage());
            }
        }
        $this->view->form = $currentForm;
    }
    
    private function _checkVendor(AffiliateProgram $affiliateProgram) {
        return ($affiliateProgram->vendor_id == $this->_vendorId);
    }

	private $_backUrl = NULL;
}