<?php
require_once("lib-action/DomainAction.php");

class EditDomainDNSResponse extends DomainAction
{

	public $params = array("domainID","domainDNS");

	public function execute()
    {
		parent::execute();

		$domain = Factory()->Domain(bgpc("domainID"));
		$registrar = Factory()->Registrar(array($domain->registrar_id, getCurrentSession()->getVendorId()));

		if ("rucenter" == $registrar->module_name) {
			try {
				$registrar->getModule()->setDomainDNS($domain);
				$this->redirectWithInfoMessage();
			} catch (ProductException $e) {
				$this->redirectWithErrorMessage(billing_safetyhtml($e->getMessage()));
			}

		} else {

			$useDNSOld = (int)$_REQUEST["useDNSOld"];
			$useDNS = (int)$_REQUEST["useDNS"];
			$nameServers = array();
			$commandXML = "";

			$mbapiData = $this->getDomain();

			$domain = correctArray($mbapiData["mbapi"][0]["results"][0]["domains"][0]["domain"][0]);
			$domainSLD = $domain["domainSLD"];
			$domainTLD = $domain["domainTLD"];
			$domainID = $domain["domainID"];

			//If useDNS is set to No, check domain names for correct values
			if (!$useDNS) {
				$nameServers = (array)bgpc("domainDNS");
				$invalidDNS = array();
				foreach ($nameServers as $key => $value) {
					if ("" != $value && !\Billing\Checker::idnDomainName($value)) {
						$invalidDNS[$key] = blmsg("TRANS_INVALIDDOMAINNAME");
					}
				}

				if(0 != count($invalidDNS)) {
					$params = array(
						"domainID" => $this->domainID,
						"listDNS" => serialize($nameServers),
						"invalidDNS" => serialize($invalidDNS),
						"useDNS" => $useDNS
					);

					redirect("domains.php", getActionID("EditDomainDNS"), $params);
				}
			}

			//If useDNS has changed from yes to no, pass that along with the new servers
			if (!$useDNS && $useDNSOld) {
				$commandXML = "<domainDNS>0</domainDNS>";
				$nameServers = (array) $_REQUEST["domainDNS"];
				foreach ($nameServers as $value) {
					$commandXML .= "<domainNameserver>".$value."</domainNameserver>\n";
				}
			}

			// Pull the host items
			$hostDNSHostname = $hostDNSType = $hostDNSAddress = array();
			if ($useDNS) {
				$hostDNSHostname = $_REQUEST["hostDNSHostname"];
				$hostDNSType = $_REQUEST["hostDNSType"];
				$hostDNSAddress = $_REQUEST["hostDNSAddress"];

				$hostXML = "";
				foreach ($hostDNSType as $k => $value) {
					if ($value != '0') {
						$hostXML .= "<host>\n";
						$hostXML .= "  <HostName>".$hostDNSHostname[$k]."</HostName>\n";
						$hostXML .= "  <RecordType>".$value."</RecordType>\n";
						$hostXML .= "  <Address>".$hostDNSAddress[$k]."</Address>\n";
						$hostXML .= "</host>\n";
					}
				}
			}

			if ($useDNS) {
				$commandXML = "<domainDNS>1</domainDNS>";
			}
			//if no and no pass the servers
			if (!$useDNS && !$useDNSOld) {
				$nameServers = (array)$_REQUEST["domainDNS"];
				foreach ($nameServers as $value) {
					$commandXML .= "<domainNameserver>".$value."</domainNameserver>";
				}
			}


			$xml = "
			<mbapi>
				<command>ProcessRegistrar</command>
				<subCommand>setDomainDNS</subCommand>
				<params>
					<configID>0</configID>
				    <vendorID>".getCurrentSession()->getVendorId()."</vendorID>
					<domainID>$domainID</domainID>
					<domainSLD>$domainSLD</domainSLD>
					<domainTLD>$domainTLD</domainTLD>
					$commandXML
					$hostXML
				</params>
			</mbapi>";

			$data = dispatchMBAPI($xml,1);
			$this->processRegistrarResponse($data);
		}

		return $this->hasExecutedSuccessfully();
	}
}