<?php
require_once("lib-action/action.php");
require_once("lib-billing/include/config/internal/emailactions.php");

class EditEmailTemplate extends EditAction
{
	function EditEmailTemplate() {
		$this->setDataSource(new DataSource("emailTemplates"));
		EditAction::EditAction();
	}

	function editHandler() {
		$this->smartyVars["emailTemplate"]["emailTplSubject"] = utf8_html_entity_encode($this->smartyVars["emailTemplate"]["emailTplSubject"]);
		$this->smartyVars["countryTypes"] = getTypes("countryTypes");
		$this->smartyVars["regionNames"] = getRegions("names");
		$this->smartyVars["nestedRegions"] = getRegions("nested");

		$this->smartyVars["emailTemplateGroup"] = oneOf("emailTplGroups", array(
			"emailTplGroupID" => $this->smartyVars["emailTemplate"]["emailTplGroupID"]
		));
        $this->smartyVars["selectedTemplate"] = str_replace(" ", "", $this->smartyVars["emailTemplateGroup"]["emailTplGroupName"]);

		$actionName = getAPITypeName( "emailAction", $this->smartyVars["emailTemplateGroup"]["emailActionID"] );
		$fileName = mb_path("/lib-mbapi/include/mail/emailTemplates/$actionName.php");
		$this->smartyVars["canPreview"] = ($fileName) ? 1 : 0;

		$language = oneByID( 'languages', $this->smartyVars["emailTemplate"]["languageID"] );
		$country = oneByID( 'countries', $this->smartyVars["emailTemplate"]["countriesID"] );

		list($this->smartyVars['availableTemplates'] ) = $this->findAvailableEmailTemplates( $language['languageISO2'].'_'.$country['countriesISO2'] );
		if (!$this->smartyVars['availableTemplates']){
			list($this->smartyVars['availableTemplates'] ) = $this->findAvailableEmailTemplates($language['languageISO2'].'_US');
		}
		if (!$this->smartyVars['availableTemplates']){
			list($this->smartyVars['availableTemplates'] ) = $this->findAvailableEmailTemplates();
		}
		foreach ($this->smartyVars['availableTemplates'] as $k=>$v) $this->smartyVars['availableTemplates'][$k] = variableToHuman($v);

		return true;
	}

	function findAvailableEmailTemplates( $locale = "en_US" ) {
		$path = mb_path( "/lib-themes/default/lib-billing/templates/emails/" );

		if (!is_readable( $path ) || !is_dir( $path )) {
			$this->addError('TRANS_ACTIONERROR', "Email Template directory is not readable" );
			return false;
		}

		// create the directory handler
		$dh = opendir( $path );

		$availableTemplates = $htmlTemplates = $txtTemplates = array();

		// move through the files
		while( ($file = readdir( $dh )) !== false ) {
			$matches = array();

			if (preg_match( "/(^[\w,\-,_,\040]+)\.$locale\.((?:html|t[e]{0,1}xt))\.tpl$/iU", $file, $matches )) {
				$availableTemplates[ $matches[1] ] = $matches[1];

				switch( strtolower( $matches[2] ) ) {
				case 'html':
					$htmlTemplates[] = $matches[1];
					break;
				case 'txt':
				case 'text':
					$txtTemplates[] = $matches[1];
					break;
				}
			}

		}	unset( $file, $matches );

		closedir( $dh );
		unset( $dh );

		asort( $availableTemplates );
		return array( &$availableTemplates, &$htmlTemplates, &$txtTemplates );
	}
} // EditEmailTemplate
