<?php
require_once("lib-action/action.php");
require_once("lib-billing/include/config/internal/emailactions.php");

use Billing\Object\EmailTemplateGroup;

class PreviewEmailTemplate extends MB_Action
{
	public $params = array("emailTplID", "emailActionID", "preview");

	public function execute()
    {
		parent::execute();

		$emailTplID = (int)$_REQUEST["emailTplID"];
		$emailActionID = (int)$_REQUEST["emailActionID"];

		if (!notZero( $emailTplID ) || !notZero( $emailActionID )) {
			$this->addError('TRANS_ACTIONERROR', "No Email ID or No Email Action ID Given" );
		} elseif (in_array((string)$_REQUEST['preview'], array('html', 'text'))) {

			// we need the locale specific info
			$template = Factory()->EmailTemplate($emailTplID);

			// get the email action name so we can grab the file
			$actionName = getAPITypeName( "emailAction", $emailActionID );

			$fileName = mb_path("/lib-mbapi/include/mail/emailTemplates/$actionName.php");

			if (!is_readable( $fileName )) {
				$this->addError('TRANS_ACTIONERROR', "The requested email action [$actionName] does not exist." );
			} else {
				require_once($fileName);
				if (!class_exists( $actionName )) {
					$this->addError('TRANS_ACTIONERROR', "$actionName does not exists." );
				} else {
					if ( isset($_REQUEST["emailTplBody"]) ) {
						$template->email_tpl_body = utf8_html_entity_encode(bgpc("emailTplBody"));
					}
					if ( isset($_REQUEST["emailTplTextBody"]) ) {
						$template->email_tpl_text_body = utf8_html_entity_encode(bgpc("emailTplTextBody"));
					}

                    $action = $this->getEmailTemplateGroupClass($emailActionID, $actionName);
					$action->enablePreviewMode();
					$action->setTemplate($template);
					$previewData = $action->execute();

					if ( $action->getErrorCount() ) {
						show($action->getErrors());
						exit;
					}
					if ( isset($previewData[$_REQUEST['preview']]) ) {
						mbHeader('Content-Type: text/html; charset=UTF-8');
						if ( "html" == $_REQUEST['preview'] ) {
							echo $previewData[$_REQUEST['preview']];
						} else {
							echo nl2br(utf8_html_entity_encode($previewData[$_REQUEST['preview']]));
						}
						exit;
					}
					$this->addError('TRANS_ACTIONERROR', "Invalid preview type requested.");
				}
			}

		} else {
			$this->smartyVars['emailTplID'] = $_REQUEST['emailTplID'];
			$this->smartyVars['emailActionID'] = $_REQUEST['emailActionID'];
			$this->addTemplate("emails/PreviewEmailTemplate");
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

    private function getEmailTemplateGroupClass($emailActionID, $actionName)
    {
        $reseller = is_null(getCurrentSession()->getVendorId()) ? null : Factory()->Reseller(getCurrentSession()->getVendorId());
        switch($emailActionID) {
            case EmailTemplateGroup::TYPE_INVOICE_FACTURA:
                return new $actionName(null, $reseller);
            case EmailTemplateGroup::TYPE_PAYMENT_SECONDARY:
                return new $actionName(null,null,null,null,$reseller);
            case EmailTemplateGroup::TYPE_DOMAIN_RENEWAL:
            case EmailTemplateGroup::TYPE_PAYMENT_CC:
            case EmailTemplateGroup::TYPE_PAYMENT_ECHECK:
            case EmailTemplateGroup::TYPE_PAYMENT_DECLINED:
            case EmailTemplateGroup::TYPE_PAYMENT_ERROR:
                 return new $actionName(null,null,null,$reseller);
            case EmailTemplateGroup::TYPE_SUSPEND:
                return new $actionName(1,null,null,$reseller);
            default:
                return new $actionName(null,null,$reseller);
        }
    }
}
