<?php

require_once("lib-action/action.php");

use Billing\Object\BillingAccount;

class AddSingleInvoiceResponse extends MB_Action
{
	public function execute()
	{
		parent::execute();

		if(!(int)$_REQUEST["clientID"]) {
			$this->addError('TRANS_ACTIONERROR', "Client ID not found." );
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}
		$clientID = (int)$_REQUEST["clientID"];

		$client = Factory()->Customer($clientID);
		$store = Factory()->Store($_REQUEST["orderFormID"]);
		if (!$client->hasSameVendor($store)) {
			throw new ForbiddenOperationException();
		}

		$params = array(
			"clientID" => $clientID,
			"packageName" => bgpc("packageName"),
			"packageDescription" => bgpc("packageDescription"),
			"amount" => BillingLocale::getNumberFromInput(bgpc("amount")),
			"dueDate" => getMBDateFromRequest("invoiceDate"),
			"cycleID" => bgpc("cycleID"),
			"orderFormID" => bgpc("orderFormID"),
			"enableInvoiceEmail" => bgpc("enableInvoiceEmail"),
            "billingAccountID" => bgpc('billingAccountID'),
		);

		if (1 == bgpc("applyPaymentAsWell")) {
			$params["amountPaid"] = BillingLocale::getNumberFromInput(bgpc("amountPaid"));
			$params["paymentComments"] = bgpc("paymentComments");
			$params["transactionDateCreated"] = getMBDateFromRequest("transactionDateCreated");
			$params["enablePaymentEmail"] = bgpc("enablePaymentEmail");

			switch (Factory()->BillingAccount($params["billingAccountID"])->billing_account_type) {
			case BillingAccount::TYPE_CREDIT_CARD:
				$params["paymentMethod"] = "ccsingle";
				break;
			case BillingAccount::TYPE_ECHECK:
				$params["paymentMethod"] = "echeck";
				break;
			case BillingAccount::TYPE_BANK_DRAFT:
				$params["paymentMethod"] = "bank";
				break;
			case BillingAccount::TYPE_MANUAL_PAYMENT:
				$params["paymentMethod"] = "check";
				break;
			case BillingAccount::TYPE_THIRD_PARTY_GATEWAY:
				$params["paymentMethod"] = "thirdparty";
				break;
			case BillingAccount::TYPE_WIRE:
				$params["paymentMethod"] = "wire";
				break;
			case BillingAccount::TYPE_HSPC_BANK_TRANSFER:
				$params["paymentMethod"] = "hspcbanktransfer";
				break;
			default:
				$params["paymentMethod"] = "other";
			}
		}

		$results = dispatchMBAPI(array(
			"mbapi" => array(
				"command" => "ProcessSingleInvoice",
				"params" => $params,
			),
		), 1);

		$invoiceID = $results["mbapi"][0]["results"][0]["invoiceID"];
		$invoiceID = is_array($invoiceID) ? (int)$invoiceID[0] : 0;

		if ($invoiceID){
			$customer = Factory()->Customer($clientID);
			if ($customer->isReseller()) {
			    $reseller = $customer->getReseller();
			    billing_put_info(blmsg("TRANS_ADD_SINGLE_INVOICE_FOR_RESELLER_SUCCESS_MSG", array("NAME" => billing_safetyhtml($reseller->fullName))));
			} else {
			    $primaryContact = $customer->PrimaryContact;
			    billing_put_info(blmsg("TRANS_ADD_SINGLE_INVOICE_FOR_CUSTOMER_SUCCESS_MSG", 
				array("NAME" => billing_safetyhtml($primaryContact->contact_fullname))));
			}
			redirect("invoices.php", "ShowInvoiceDetails", array("invoiceID"=>$invoiceID));
		} else {
			$this->addError('TRANS_ACTIONERROR', "Error generating invoice." );
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

	}
}
