<?php

require_once("lib-action/action.php");

use Billing\Manager\PaymentGatewayManager;
use Billing\Object\BillingAccount;

class ProcessTransactionRefund extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$transactionIDs = array_unique((array)bgpc("transactionIDs"));
        $lineItemIDs = array_unique((array)bgpc("lineItemIDs"));
		$invoiceId = bgpc("invoiceID");

		// first lets test if we have a transactionID at all
		if ($transactionIDs[0]){
			// lets get all our transactions
			$this->smartyVars["transactions"] = correctArray(grabResultsFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetTransactions",
					"params" => array(
						"transactionIDs" => array(
							"transactionID" => $transactionIDs,
						),
					),
				),
			)));
			dbg($this->smartyVars["transactions"]);

			// lets get any line item info that we need
			if ($lineItemIDs[0]){
				$successRel = correctArray(grabResultsFromMBAPIArray(array(
					"mbapi" => array(
						"command" => "GetTransactionSuccessRel",
						"params" => array(
							"lineItemID" => $lineItemIDs,
							"transactionID" => $transactionIDs,
						),
					),
				)));
				$lineItemInfo = correctArray(grabResultsFromMBAPIArray(array(
					"mbapi" => array(
						"command" => "GetLineItems",
						"params" => array(
							"lineItemID" => $lineItemIDs,
						),
					),
				)));
				$this->smartyVars["lineItemNames"] = array();
				foreach ((array)$lineItemInfo as $li){
					$this->smartyVars["lineItemNames"][$li["lineItemID"]] = "(".blmsg('TRANS_ID').": ".$li["lineItemID"].") ".billing_safetyhtml($li["lineItemDescription"]);
				}
                
				$this->smartyVars["lineItemsForTransactions"] = array();
				$this->smartyVars["transactionRefundTotals"] = array();
				foreach ((array)$successRel as $sr){
					$this->smartyVars["lineItemsForTransactions"][$sr["transactionID"]][] = $sr;
					$this->smartyVars["transactionRefundTotals"][$sr["transactionID"]] += $sr["lineItemPaymentAmount"];
				}
			}
            
            $this->smartyVars["invoiceID"] = $invoiceId;
			$this->smartyVars["invoiceInfo"] = correctArray(grabFirstResultFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetInvoices",
					"params" => array(
						"invoiceID" => $invoiceId,
					),
				),
			)));

			foreach ((array)$this->smartyVars["transactions"] as $k => $v) {
                if (0 == $v["billingAccountID"]) {
                    continue;
                }
                
                $account = Factory()->BillingAccount($v["billingAccountID"]);
                if (BillingAccount::TYPE_CREDIT_CARD != $account->billing_account_type) {
                    continue;
                }
                
				$gatewayActions = array();
				if ('' != $v["paymentGatewayID"] && 1 == $account->billing_account_active && '' != $account->billing_card_num) {
                    $gateway = Factory()->PaymentGateway($v["paymentGatewayID"]);
                    if (count($this->smartyVars["lineItemsForTransactions"])) {
                        $this->smartyVars["transactions"][$k]["canPartial"] = 0; // disable partial refunds due to they are not implemented correctly
                        if (Service()->PaymentGatewayManager->isOperationSupported($gateway, PaymentGatewayManager::OPERATION_PARTIAL_REFUND)) {
                            $gatewayActions[PaymentGatewayManager::OPERATION_PARTIAL_REFUND] = blmsg("TRANS_PARTIAL_REFUND");
                            $this->smartyVars["transactions"][$k]["selectedAction"] = PaymentGatewayManager::OPERATION_PARTIAL_REFUND;
                        }
                    } else {
                        if (Service()->PaymentGatewayManager->isOperationSupported($gateway, PaymentGatewayManager::OPERATION_REFUND)) {
                            $gatewayActions[PaymentGatewayManager::OPERATION_REFUND] = blmsg("TRANS_REFUND");
                            $this->smartyVars["transactions"][$k]["selectedAction"] = PaymentGatewayManager::OPERATION_REFUND;
                        }
                        if (0.0 == Factory()->PaymentTransaction($v["transactionID"])->transaction_amount_refunded &&
                            Service()->PaymentGatewayManager->isOperationSupported($gateway, PaymentGatewayManager::OPERATION_VOID)) {
                            $gatewayActions[PaymentGatewayManager::OPERATION_VOID] = blmsg("TRANS_VOID");
                            if (!$this->smartyVars["transactions"][$k]["selectedAction"]) {
                                $this->smartyVars["transactions"][$k]["selectedAction"] = PaymentGatewayManager::OPERATION_VOID;
                            }
                        }
                    }
				} 
                $gatewayActions[PaymentGatewayManager::OPERATION_NOT_CALL_GATEWAY] = blmsg("TRANS_DO_NOT_CALL_GATEWAY");
                if (!$this->smartyVars["transactions"][$k]["selectedAction"]) {
                    $this->smartyVars["transactions"][$k]["selectedAction"] = PaymentGatewayManager::OPERATION_NOT_CALL_GATEWAY;
                }
                $this->smartyVars["transactions"][$k]["gatewayActions"] = $gatewayActions;
                $this->smartyVars["transactions"][$k]["CCtransaction"] = 1;
			}

			$this->addTemplate("invoices/ProcessTransactionRefund", $this->smartyVars);
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
