<?php

require_once("lib-action/action.php");
require_once("lib-billing/include/accounting/Refund.php");

use Billing\Object\Invoice;
use Billing\Object\LedgerAccount;
use Billing\Object\BillingAccount;
use Billing\Manager\PaymentGatewayManager;

class ProcessTransactionRefundResponse extends MB_Action
{
	public function execute()
	{
		parent::execute();

		if (is_null($transactionID = bgpc("transactionID"))) {
			throw new ProductException(ProductException::E_OBJECT_NOT_FOUND, "Transaction not found: id={$transactionID}");
		}
		if (!getCurrentSession()->isValidToken()) {
			$this->_redirectWithErrorMessage(blmsg("TRANS_ERROR_TOKEN"));
		}
		$invoiceID = bgpc("invoiceID");
		$markUnpaid = bgpc("markUnpaid");
        $gatewayAction = bgpc("gatewayAction");
        $lineItemIDs = bgpc("lineItemIDs", array());
        /* disable partial refunds due to they are not implemented correctly
        if ($_REQUEST["refundType"]=="partial"){
            $_REQUEST["refundAmount"] = BillingLocale::getNumberFromInput($ _REQUEST["lesserAmount"]);
        }
        */

		$transaction = Factory()->PaymentTransaction($transactionID);
        $successMessage = blmsg("TRANS_PROCESS_TRANSACTION_REFUND_SUCCESS_MSG");
		if (is_null($transaction->BillingAccount) || BillingAccount::TYPE_CREDIT_CARD != $transaction->BillingAccount->billing_account_type) {
			$gatewayAction = PaymentGatewayManager::OPERATION_NOT_CALL_GATEWAY;
            $successMessage = blmsg("TRANS_CANCEL_TRANSACTION_SUCCESS_MSG");
		}

		$transactionAmountRemained = round(doubleval($transaction->transaction_amount) - doubleval($transaction->transaction_amount_refunded), 2);
		$refundAmount = doubleval(bgpc("refundAmount")) ? min(doubleval(bgpc("refundAmount")), $transactionAmountRemained) : $transactionAmountRemained;
        
		try {
            switch($gatewayAction) {
                case PaymentGatewayManager::OPERATION_REFUND :
                case PaymentGatewayManager::OPERATION_PARTIAL_REFUND :
                    $success = Service()->PaymentManager->refund($transaction, $refundAmount);
                    break;
                case PaymentGatewayManager::OPERATION_NOT_CALL_GATEWAY :
                    $success = Service()->PaymentManager->refund($transaction, $refundAmount, false);
                    break;
                case PaymentGatewayManager::OPERATION_VOID :
                    $success = Service()->PaymentManager->void($transaction, $refundAmount);
                    break;
            }
			if (!$success) {
				$this->_redirectWithErrorMessage($transaction->transaction_notes);
			}
		} catch (ProductException $e) {
			$this->_redirectWithErrorMessage($e->getMessage());
		}

		$invoiceInfo = correctArray(grabFirstResultFromMBAPIArray(array(
			"mbapi" => array(
				"command" => "GetInvoices",
				"params" => array(
					"invoiceID" => $invoiceID,
					"getInvoiceLineItemsData" => 1,
				),
			),
		)));

        dispatchMBAPI(toXML(array(
            "mbapi" => array(
                "command" => "SetInvoices",
                "subCommand" => "update",
                "params" => array(
                    "invoiceID" => $invoiceID,
                    "invoiceStatus" => Invoice::STATUS_DUE,
                    "invoiceAmountPaid" => $invoiceInfo["invoiceAmountPaid"] - $refundAmount,
                ),
            ),
        )), 1);

        $setTransactionSuccessRel = array(
            "mbapi" => array(
                "command" => "SetTransactionSuccessRel",
                "subCommand" => "delete",
                "params" => array(
                    "transactionID" => $transactionID,
                )
            )
        );

        if (!empty($lineItemIDs)) {
            foreach($lineItemIDs as $lineItemId) {
                $setTransactionSuccessRel["mbapi"]["params"]["lineItemID"] = $lineItemId;
                dispatchMBAPI(toXML($setTransactionSuccessRel), 1);
            }
        } else {
            dispatchMBAPI(toXML($setTransactionSuccessRel), 1);
        }

        $processRefundAccounting = array(
            "mbapi" => array(
                "command" => "ProcessRefundAccounting",
                "params" => array(
                    "transactionID" => $transactionID,
                    "ledgerAccountName" => LedgerAccount::NAME_ACCOUNTS_RECEIVABLE,
                    "invoiceID" => $invoiceID,
                    "markUnpaid" => $markUnpaid,
                )
            )
        );

        if ($refundAmount != $transaction->transaction_amount || !empty($lineItemIDs)) {
            $processRefundAccounting["mbapi"]["params"]["refundAmount"] = $refundAmount;
        }

        $data = dispatchMBAPI(toXML($processRefundAccounting), 1);

        if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {
            billing_put_info($successMessage);
        }

		if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
			redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
		} else {
			redirect("invoices.php?action=".getActionID("ShowInvoices")."&sid=".secureInput($_REQUEST["sid"]));
		}
	}

	private function _redirectWithErrorMessage($message)
	{
		billing_put_error($message);
		redirect("invoices.php", getActionID("ProcessTransactionRefund"), array(
			"invoiceID" => bgpc("invoiceID"),
			"transactionIDs" => array(bgpc("transactionID")),
			"lineItemIDs" => bgpc("lineItemIDs", array()),
		));
	}
}