<?php
require_once ("lib-action/action.php");

use Billing\Object\PaymentTransaction;

class RemoveInvoiceTransactionResponse extends MB_Action
{
    public $params = array("transactionID");

	public function execute()
    {
		parent::execute();

		$transactionID = bgpc("transactionID");
		$invoiceID = bgpc("invoiceID");
		if(!$invoiceID || !$transactionID) {
            billing_put_error("Invalid invoice or transaction specified.");
            redirect("invoices.php", "ShowInvoices", array("invoiceID"=>$invoiceID));
		}
        // delete transaction stuff here.
        $receipts = grabResultsFromMBAPIArray(array(
            "mbapi" => array(
                "command" => "GetAccountingReceiptsJournal",
                "params" => array(
                    "transactionID" => $transactionID,
                ),
            ),
        ));

        dbg("got receipts:");
        dbg($receipts);

        foreach ($receipts as $receipt){
            dispatchMBAPI(toXML(array(
                "mbapi" => array(
                    "command" => "SetAccountingReceiptsJournal",
                    "subCommand" => "delete",
                    "params" => array(
                        "receiptJournalID" => $receipt["receiptJournalID"],
                    ),
                ),
            )), 1);
            dbg("would have deleted receipt: ".$receipt["receiptJournalID"]);
        }

        $lineItemPayments = correctArray(grabFirstResultFromMBAPIArray(array(
            "mbapi" => array(
                "command" => "GetTransactions",
                "params" => array(
                    "transactionIDs" => array("transactionID" => $transactionID),
                    "getLineItemPayments" => 1,
                ),
            ),
        )));

        dbg("got line item payments data");
        dbg($lineItemPayments);

        $invoicePayments = array();

        if (count($lineItemPayments["lineItemPayments"])){
            foreach ($lineItemPayments["lineItemPayments"] as $row){
                $invoicePayments[$row["invoiceID"]] += $row["lineItemPaymentAmount"];
            }
        }

        dbg("invoices and amounts needed to be taken off of them:");
        dbg($invoicePayments);

        if (count($invoicePayments)){
            $invoices = grabResultsFromMBAPIArray(array(
                "mbapi" => array(
                    "command" => "GetInvoices",
                    "params" => array(
                        "invoiceID" => array_keys($invoicePayments),
                    ),
                ),
            ));

            foreach ($invoices as $invoice){
                $paid = $invoice["invoiceAmountPaid"]-$invoicePayments[$invoice["invoiceID"]];
                if ($paid<0)$paid = 0;
                dispatchMBAPI(toXML(array(
                    "mbapi" => array(
                        "command" => "SetInvoices",
                        "subCommand" => "update",
                        "params" => array(
                            "invoiceID" => $invoice["invoiceID"],
                            "invoiceStatus" => getAPIStatusID("invoice", "due"),
                            "invoiceAmountPaid" => $paid,
                        ),
                    ),
                )), 1);
            }

        }

        if($lineItemPayments["transactionPaymentMethod"] == PaymentTransaction::PAYMENT_HSPCBANKTRANSFER
                && $lineItemPayments["authReturn"] != PaymentTransaction::STATUS_MANUALLY_APPLIED
        ) {
            Factory()->BatchFileTransaction($transactionID, "transaction_id")->delete();
        }

        dispatchMBAPI(toXML(array(
            "mbapi" => array(
                "command" => "SetTransaction",
                "subCommand" => "delete",
                "params" => array(
                    "transactionID" => $transactionID,
                ),
            ),
        )), 1);

        redirect("invoices.php?sid=".$_REQUEST["sid"]."&action=".getActionID("ShowInvoiceDetails")."&invoiceID=".$invoiceID);


		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}