<?php

require_once("lib-action/action.php");
require_once("UI/Page.php");

use Billing\ModuleTool;
use Billing\Object\ControlPanel;

class EditPanelConfigPage extends UIPage
{
	public function createControls()
	{
		require_once("DataObject.php");

		$this->module_input_controls = array();

		if (PACKAGE_TYPE == "msp") {
			$this->controlPanel =  Service()->ControlPanelManager->getMasterPanel();
		} elseif (!is_null($panelID = $this->getValue("configGroupID"))) {
			$this->controlPanel =  Factory()->ControlPanel($panelID);
		} else {
			$this->controlPanel =  Factory()->ControlPanel();
			$this->controlPanel->module_name = Factory()->ConfigGroup($this->getValue("parentConfigGroupID"))->config_group_name;
			$this->controlPanel->server_group_id = $this->getValue("serverGroupID");
		}
		$this->controlPanel->allowRepairCredentials = false;

		foreach (ModuleTool::getModuleControls("panel", $this->controlPanel->module_name, $this->controlPanel->id) as $inputPanel) {
			$inputPanel->initFromArray(DataObject::getMapOf($this->controlPanel->ConfigParams, "name", "value"));
			$this->module_input_controls = array_merge($this->module_input_controls, $inputPanel->getInputControls());
			if ( "" == $inputPanel->title ) {
				$inputPanel->title = blmsg("TRANS_CONFIGURATION_PARAMETERS");
			}
			$this->addInputPanel($inputPanel);
		}
		if (!is_null($this->controlPanel->id) && $this->controlPanel->isPlesk() && "" != $this->controlPanel->moduleSecretKey) {
			foreach ($this->module_input_controls as $control) {
				if ("modulePassword" == $control->name) {
					$control->required = false;
					break;
				}
			}
		}
		if (is_null(bgpc("command")) && ! is_null($this->controlPanel->id)) { // Check Panel Connection here in view only mode
            $this->checkPanelConnection($this->controlPanel);
		}
		if ($this->controlPanel->isPlesk() && !$this->controlPanel->isMasterPanel && SettingsClass::SSO_MODE_SIMPLIFIED == Settings()->sso_mode) {
			$this->addWarningMessage(UIFactory()->WarningMessage(blmsg("TRANS_CONTROL_PANEL_IN_SSO_SINGLE_PANEL_MODE", array(
				"TOPIC_LINK" => \Billing\getHelpContextUrl("sso_modes", 1158)
			))));
		}
		$this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;
	}

	public function initControl()
	{
		if (!is_null($panelID = $this->getValue("configGroupID"))) {
			$panel = Factory()->ControlPanel($panelID);
			if ($panel->isSyncable() && !is_null($panel->sync_error)) {
                $this->addErrorMessage(UIFactory()->ErrorMessage(
					blmsg("TRANS_SYNC_THIS_PANEL_ERROR_MSG", array("ERROR" => nl2br(billing_safetyhtml($panel->sync_error))))
				));
			}
		}
		parent::initControl();
	}

	public function processInput()
	{
		$modulePlainPassword = null;
		$configParams = array();
		foreach ($this->module_input_controls as $control) {
			switch ($control->name) {
			case "modulePassword":
				if ($control->wasChanged() || $this->getValue("modulePasswordWasChanged", false)) {
					$configParams[$control->name] = $control->value;
					$modulePlainPassword = $control->value;
					$this->setValue("modulePasswordWasChanged", true);
				} else {
					$configParams[$control->name] = ("" == $control->value) ? "" : Service()->CryptManager->decrypt($control->value, null, false);
				}
				break;
			case "api":
				$configParams[$control->name] = rtrim($control->value, "/");
				break;
			default:
				$configParams[$control->name] = $control->value;
			}
		}
		$configParams['_isMasterPanel'] = $this->controlPanel->isMasterPanel;
		$configParams['_modulePlainPassword'] = $modulePlainPassword;
		$configData = $this->controlPanel->getModule()->collectConfigData($configParams);

		$this->controlPanel->ConfigParams->clear();
		foreach ($this->module_input_controls as $control) {
			$value = $configData["data"][$control->name];
			switch ($control->name) {
			case "modulePassword":
				$this->controlPanel->modulePassword = $value;
				break;
			default:
				$this->controlPanel->ConfigParams[$control->name]->value = $value;
			}
			if (isset($configData["error"][$control->name])) {
				$control->setValidity(false);
			}
		}
		if (!empty($configData["error"])) {
			$this->addErrorMessage(UIFactory()->ErrorMessage($configData["error"]));
			return false;
		}
		$this->controlPanel->modulePlainPassword = $modulePlainPassword;
		try {
			Service()->ControlPanelManager->update($this->controlPanel);
            if(!is_null($this->controlPanel->_OLD_config_group_id) && $this->controlPanel->isSyncable() && ControlPanel::SYNC_STATUS_ERROR == $this->controlPanel->sync_status) {
                Service()->ControlPanelManager->sync($this->controlPanel);
            }
			if(ControlPanel::SYNC_STATUS_ERROR == $this->controlPanel->sync_status) {
				getCurrentSession()->Storage->warningMessages[] = blmsg(
					"TRANS_PANEL_UPDATED_WARNING_MSG",
					array("ERROR" => $this->controlPanel->sync_error)
				);
				$this->_skipWarnings = false;
			} else {
				getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_PANEL_UPDATED_SUCCESSFULLY_MSG");
			}
		} catch (ProductException $e) {
			$this->addErrorMessage(UIFactory()->ErrorMessage($e->getMessage()));
		}

		return !($this->hasErrors() || (!$this->_skipWarnings && $this->hasWarnings()));
	}

	private function checkPanelConnection(ControlPanel $controlPanel)
	{
        $warningMessages = array();
        try {
            Service()->ControlPanelManager->isPanelValid($controlPanel, $warningMessages);
        } catch(ProductException $e) {
            $this->addErrorMessage(UIFactory()->ErrorMessage($e->getMessage()));
            return false;
        }
        if (!empty($warningMessages)) {
            $this->addWarningMessage(UIFactory()->WarningMessage(nl2br(implode("\n\n", $warningMessages))));
        }
        return true;
	}

	private $controlPanel;
	private $module_input_controls;
	private $_skipWarnings = true;
}

class EditPanelConfig extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$config_group_id = bgpc("configGroupID");
		$parent_config_group_id = bgpc("parentConfigGroupID");
		$server_group_id = bgpc("serverGroupID");

		if (1 == bgpc("sync")) {
			try {
				$controlPanel = Factory()->ControlPanel($config_group_id);
				$controlPanel->sync_tasks |= ControlPanel::SYNC_NEED_UPDATE | ControlPanel::SYNC_NEED_REPAIR_SSO;
				Service()->ControlPanelManager->sync($controlPanel);
				if (ControlPanel::SYNC_NO_TASKS == $controlPanel->sync_tasks) {
					getCurrentSession()->Storage->infoMessages[] = blmsg(
						"TRANS_PANEL_SYNC_SUCCESS_MSG",
						array("NAME" => $controlPanel->name, "PANEL_ID" => $controlPanel->id)
					);
				} else {
					getCurrentSession()->Storage->errorMessages[] = blmsg(
						"TRANS_PANEL_SYNC_FAILURE_MSG",
						array("NAME" => $controlPanel->name, "PANEL_ID" => $controlPanel->id, "ERROR" => $controlPanel->sync_error)
					);
				}
			} catch (ProductException $e) {
				getCurrentSession()->Storage->errorMessages[] = blmsg(
					"TRANS_PANEL_SYNC_FAILURE_MSG",
					array("NAME" => $controlPanel->name, "PANEL_ID" => $controlPanel->id, "ERROR" => $e->getMessage())
				);
			}
			if ("ShowDashboard" == bgpc("redirectAction")) {
				redirect(action_url("dashboard", "ShowDashboard"));
			} else {
				redirect(action_url("servergroups", "ShowServerGroups"));
			}
		}
		if (1 == bgpc("syncAll")) {
			try {
				$controlPanel = Factory()->ControlPanel($config_group_id);
				Service()->ControlPanelManager->asyncAll($controlPanel);
				getCurrentSession()->Storage->infoMessages[] = blmsg(
					"TRANS_PANEL_SYNC_ALL_SUCCESS_MSG",
					array("NAME" => $controlPanel->name, "PANEL_ID" => $controlPanel->id)
				);
			} catch (ProductException $e) {
				getCurrentSession()->Storage->errorMessages[] = blmsg(
					"TRANS_PANEL_SYNC_ALL_FAILURE_MSG",
					array("NAME" => $controlPanel->name, "PANEL_ID" => $controlPanel->id, "ERROR" => $e->getMessage())
				);
			}
			redirect(action_url("servergroups", "ShowServerGroups"));
		}

		if ("ShowDashboard" == bgpc("redirectAction")) {
			$backUrl = action_url("dashboard", "ShowDashboard");
		} else if (PACKAGE_TYPE == "msp") {
			$backUrl = admin_action_url("configs", "system", "show");
		} else {
			$backUrl = action_url("servergroups", "ShowServerGroups");
		}
		$controller  = UIFactory()->PageController($backUrl);
		$controller->addPage($edit_page = new EditPanelConfigPage($controller));
		$edit_page->title = blmsg("TRANS_EDIT_PANEL_OPTIONS");
		$edit_page->parent_title = blmsg("TRANS_MANAGE_SYSTEM");
		$edit_page->parent_url = PACKAGE_TYPE == "msp" ? admin_action_url("configs", "system", "show") : action_url("servergroups", "ShowServerGroups");
		$controller->setValue("configGroupID", $config_group_id);
		$controller->setValue("parentConfigGroupID", $parent_config_group_id);
		$controller->setValue("serverGroupID", $server_group_id);
		$controller->processRequest();

		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}
