<?php
require_once("lib-action/action.php");
require_once("UI/Page.php");

use Billing\Database\Transaction;
use Billing\Object\Plan;
use Billing\Object\Store;
use Billing\Object\Customer;

class EditOrderFormSettingsPage extends UIPage
{

	const TAB_GENERAL = 'panel-general';
	const TAB_PAYMENT_OPTIONS = 'panel-payment';
	const FIELD_GENERAL_CODE = 'order_form_code';
	const FIELD_GENERAL_NAME = 'order_form_name';
	const FIELD_GENERAL_CURRENCY = 'currency_id';
	const FIELD_BANK_TRANSFER_GEATWAY_ID = 'hspc_bank_transfer_gateway_id';

	public function setVars($orderFormID, $languages, $countries)
	{
		if(is_null($orderFormID)) {
			$this->orderForm = Service()->StoreManager->create("", "", getCurrentSession()->getVendorId());
		} else {
			$this->orderForm = Factory()->Store($orderFormID);
		}
		$this->orderFormID = $orderFormID;
		$this->orderForm = Factory()->Store($this->orderFormID);
		$this->languages = array();
		foreach($languages as $key => $value) {
			$this->languages[] = array($key, $value);
		}

		$this->countries = array();
		foreach($countries as $key => $value) {
			$this->countries[] = array($key, $value);
		}
	}

	public function createControls()
	{
		require_once("UI/Tabs/UIOrderFormSettingsTabs.php");
		$tabs = new UIOrderFormSettingsTabs();
		$this->options = $tabs->setCurrent(bgpc("options"));
		$this->setScreenTabs($tabs);

		$domainTips = UIFactory()->TipsPanel("TRANS_ORDER_DOMAIN_OPTIONS_TIP", "TRANS_ORDER_DOMAIN_OPTIONS_DESC", "domains" == $this->options);
		$domainTips->id = "panel-domainTips";
		$this->addTipsPanel($domainTips);
		$this->addInputPanelToForm(self::TAB_GENERAL, NULL, $this->loadGeneralOptions(), "general" == $this->options);

		$this->addInputPanelToForm("panel-payment", NULL, $this->loadPaymentOptions(), "payment" == $this->options);

		$this->addInputPanelToForm("panel-domains", NULL, $this->loadDomainOptions(), "domains" == $this->options);

		$this->addInputPanelToForm("panel-taxes", NULL, $this->loadTaxesOptions(), "taxes" == $this->options);

        if (is_null($this->orderForm->vendor_id) && !getCurrentSession()->isReseller()) {
            $securityOptions = $this->loadSecurityOptions();
            $this->addInputPanelToForm("panel-security_email_banning", blmsg("TRANS_ORDER_EMAIL_BANNING_PANEL"), $securityOptions["email_banning"], "security" == $this->options);
            $this->addInputPanelToForm("panel-security_country_banning", blmsg("TRANS_ORDER_COUNTRY_BANNING_PANEL"), $securityOptions["country_banning"], "security" == $this->options);
        }

		$this->addInputPanelToForm("panel-license", NULL, $this->loadLicenseOptions(), "license" == $this->options);

		$this->addInputPanelToForm("panel-products", NULL, $this->loadProductsOptions(), "products" == $this->options, true);

		$this->getDefaultForm()->id = "settings_form";

		$optionControl = UIFactory()->InputHidden("options");
		$optionControl->value = $this->options;
		$this->getDefaultForm()->addHiddenControl($optionControl);
        $this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;
	}

    public function validateInput()
    {
    	$errorMessages = array();

        $customerCredentials = "panel-security_customer_credentials";
        $isValid = parent::validateInput();

        // special check of store name and code, if common validation was passed successfully
        $generalNameControl = $this->module_input_controls[self::TAB_GENERAL][self::FIELD_GENERAL_NAME];
        if ($generalNameControl->isValid() && !Service()->StoreManager->checkName($generalNameControl->value, $this->orderForm, getCurrentSession()->getVendorId(), $errorMessages)) {
            $this->module_input_controls[self::TAB_GENERAL][self::FIELD_GENERAL_NAME]->setValidity();
        }
        $generalCodeControl = $this->module_input_controls[self::TAB_GENERAL][self::FIELD_GENERAL_CODE];
        if ($generalCodeControl->isValid() && !Service()->StoreManager->checkCode($generalCodeControl->value, $this->orderForm, $errorMessages)) {
            $this->module_input_controls[self::TAB_GENERAL][self::FIELD_GENERAL_CODE]->setValidity();
        }

        // check, that currency of store accepted by selected bank transfer gateway
        $bankTransferGatewayIdControl = $this->module_input_controls[self::TAB_PAYMENT_OPTIONS][self::FIELD_BANK_TRANSFER_GEATWAY_ID];
        if ($bankTransferGatewayIdControl instanceof UIInputDropDown && "" != $bankTransferGatewayIdControl->value) {
        	$gateway = Factory()->PaymentGateway($bankTransferGatewayIdControl->value);
			$acceptedCurrencies = $gateway->acceptedCurrencies;
			$selectedCurrency = $this->module_input_controls[self::TAB_GENERAL][self::FIELD_GENERAL_CURRENCY]->value;
			if (!in_array(self::FIELD_BANK_TRANSFER_GEATWAY_ID, $bankTransferGatewayIdControl->getForm()->getDisabledControls()) && !empty($acceptedCurrencies) && !in_array($selectedCurrency, $acceptedCurrencies)) {
				$errorMessages[] = blmsg("TRANS_STORE_OVERVIEW_ERROR_NOT_SUPPORTER_CURRENCY_FOR_GATEWAY", array('CURRENCY' => $selectedCurrency, 'GATEWAY' => $gateway->moduleName));
				$bankTransferGatewayIdControl->setValidity();
			}
        }

        if (!empty($errorMessages)) {
        	foreach ($errorMessages as $errorMessage) {
        		$this->addErrorMessage(UIFactory()->ErrorMessage($errorMessage));
        	}
            $isValid = false;
        }
        return $isValid;
    }

	public function processInput()
	{
		try {
        	$transaction = new Transaction(Service()->DatabaseManager->getConnection());

			$this->saveGeneralOptions();
			$this->savePaymentOptions();
			$this->saveDomainOptions();
			$this->saveTaxesOptions();
			$this->saveSecurityOptions();
			$this->saveLicenseOptions();
			$this->saveProductsOptions();

			$transaction->commit();
			getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_SETTINGS_SAVED_SUCCESSFULLY");
		} catch (ProductException $e) {
			$this->addErrorMessage(UIFactory()->ErrorMessage($e->getMessage()));
		}

		if(!$this->hasWarnings() && !$this->hasErrors()) {
			if(is_null(bgpc("orderFormID")))
			{
				redirect_url(admin_action_url("orderforms", "store-overview", "index", array("id" => $this->orderForm->order_form_id)));
			}
			return true;
		}
		return false;

		return !$this->hasWarnings() && !$this->hasErrors();
	}

	protected function getSubmitButtons()
	{
		return array(
			UIFactory()->SubmitButton("ok", blmsg("TRANS_SAVE"), UIPageController::CMD_OK, "", "", "action"),
			UIFactory()->SubmitButton("cancel", blmsg("TRANS_CANCEL"), UIPageController::CMD_CANCEL)
		);
	}

	public function storeJS(UIJavaScriptStorage $storage)
	{
		parent::storeJS($storage);
		if ($storage->hasJS(get_class()))
			return;

		$customerPlansIDs = array();
		$productTypes = Plan::types();
		unset($productTypes[Plan::TYPE_RESELLER]);
		foreach(Factory()->PlanList(array("product_active" => 1, "product_is_addon" => 0, "product_type" => array_keys($productTypes))) as $plan) {
			$customerPlansIDs[] = $plan->id;
		}
		$customerPlansIDs = implode(",", $customerPlansIDs);

		$resellerPlansIDs = array();
		foreach(Factory()->PlanList(array("product_active" => 1, "product_is_addon" => 0, "product_type" => Plan::TYPE_RESELLER)) as $plan) {
			$resellerPlansIDs[] = $plan->id;
		}
		$resellerPlansIDs = implode(",", $resellerPlansIDs);
        
        $individualEnterpreneurId = Customer::TYPE_INDIVIDUAL_ENTERPRENEUR;
        $accountTypes = Customer::types();
        $individualEnterpreneurTitle = blmsg($accountTypes[$individualEnterpreneurId]);

		$js = "
		function toggleLicenseVariants()
		{
			var types = $('settings_form').use_terms_url;
			for(var i = 0; i < types.length; i++) {
				if('billing-id-use_terms_url_1' == types[i].id) {
					if(types[i].checked) {
						jQuery('#row-terms_url').show();
						jQuery('#row-terms_text').hide();
					} else {
						jQuery('#row-terms_url').hide();
						jQuery('#row-terms_text').show();
					}
					break;
				}
			}
		}

		function getCustomerPlans()
		{
			return new Array({$customerPlansIDs});;
		}

		function getResellerPlans()
		{
			return new Array({$resellerPlansIDs});
		}

		function showStorePlans(type)
		{
			if('".Store::TYPE_CUSTOMER."' == type) {
				showSelectablePlans(getCustomerPlans());
				hideSelectablePlans(getResellerPlans());
				hideSelectedPlans(getResellerPlans());
			} else {
				showSelectablePlans(getResellerPlans());
				hideSelectablePlans(getCustomerPlans());
				hideSelectedPlans(getCustomerPlans());
			}
			doubleListSelectorUpdateHiddenFields('products');
		}

		function showSelectablePlans(ids)
		{
			for(var i = 0; i < ids.length; i++) {
				if(jQuery('#products-hiddenFields > #products-' + ids[i]).val() == 0) {
					jQuery('#products-selectable-' + ids[i]).show();
				}
			}
		}

		function hideSelectablePlans(ids)
		{
			for(var i = 0; i < ids.length; i++) {
				jQuery('#products-selectable-' + ids[i]).hide();
			}
		}

		function hideSelectedPlans(ids)
		{
			for(var i = 0; i < ids.length; i++) {
				jQuery('#products-selected-' + ids[i]).hide();
			}
		}
        
        function updateAccountTypeList() 
        {
            if (jQuery('#billing-id-enable_individual_enterpreneur_type').is(':checked')) {
                if (0 == jQuery('#billing-id-default_account_type option[value=\'{$individualEnterpreneurId}\']').length) {
                    jQuery('#billing-id-default_account_type').append(\"<option value='{$individualEnterpreneurId}'>{$individualEnterpreneurTitle}</option>\");
                }
            } else {
                jQuery('#billing-id-default_account_type option[value=\'{$individualEnterpreneurId}\']').remove();
            }
        }
		";

		if(is_null($this->orderForm->id)) {
			$js .= "
			jQuery(document).ready(function() {
			    var orderFormType = (undefined == jQuery('#billing-id-order_form_type').val()) ? 'customer' : jQuery('#billing-id-order_form_type').val();
			    showStorePlans(orderFormType);
				jQuery('#products-MoveAllSelectable').click(function() {
					if('".Store::TYPE_CUSTOMER."' == jQuery('#billing-id-order_form_type').val()) {
						hideSelectedPlans(getResellerPlans());
					} else {
						hideSelectedPlans(getCustomerPlans());
					}
				});
				jQuery('#products-MoveAllSelected').click(function() {
					showStorePlans(jQuery('#billing-id-order_form_type').val());
				});
                updateAccountTypeList();
			});
			";
		} else {
            $js .= "
			jQuery(document).ready(function() {
                updateAccountTypeList();
            });
			";
        }

		$storage->addJS(get_class(), $js);
	}

	private function addInputPanelToForm($panelID, $title, $controls, $isVisible, $hideTitle = false)
	{
		$title = ("" != $title) ? $title : NULL;
		$inputPanel = UIFactory()->InputPanel($title, $hideTitle, $isVisible);
		$inputPanel->id = $panelID;
		foreach($controls as $id => $control) {
			$description = array(
				"id" => $id,
				"title"	=> $control["title"],
				"desc" => isset($control["desc"]) ? $control["desc"] : "",
				"type" => $control["type"],
				"required" => isset($control["required"]) ? $control["required"] : false,
				"read_only" => isset($control["read_only"]) ? $control["read_only"] : false,
				"disabled" => isset($control["disabled"]) ? $control["disabled"] : false,
				"onclick" => isset($control["onclick"]) ? $control["onclick"] : "",
				"onchange" => isset($control["onchange"]) ? $control["onchange"] : "",
				"items"	=> isset($control["items"]) ? $control["items"] : array(),
				"checker" => isset($control["checker"]) ? $control["checker"] : NULL,
				"extention" => isset($control["extention"]) ? $control["extention"] : NULL,
				"can_be_disabled" => isset($control["can_be_disabled"]) ? $control["can_be_disabled"] : NULL,
				"left_column_title" => isset($control["left_column_title"]) ? $control["left_column_title"] : NULL,
				"right_column_title" => isset($control["right_column_title"]) ? $control["right_column_title"] : NULL,
                "show_positions_buttons" => isset($control["show_positions_buttons"]) ? $control["show_positions_buttons"] : NULL,
			);
			$isControlVisible = (!isset($control['isVisible']) || true === $control['isVisible']);

			$inputControl = UIFactory()->InputConfig($description);
			$inputControl->value = $control["value"];
			$inputPanel->addInputControl($inputControl, $isControlVisible);
			$this->module_input_controls[$panelID][$id] = $inputControl;
		}
		$this->addInputPanel($inputPanel);
	}

	private function loadGeneralOptions()
	{
		$generalOptions = array();
		$generalOptions[self::FIELD_GENERAL_NAME] = array(
			"title" => blmsg("TRANS_ORDERFORM_NAME"),
			"desc" => blmsg("TRANS_ORDERFORM_NAME_DESC"),
			"value" => $this->orderForm->order_form_name,
			"type" => "t",
			"required" => true
		);

		$generalOptions[self::FIELD_GENERAL_CODE] = array(
			"title" => blmsg("TRANS_ORDERFORM_CODE"),
			"desc" => blmsg("TRANS_ORDERFORM_CODE_DESC"),
			"value" => $this->orderForm->order_form_code,
			"type" => "t",
			"required" => true
		);

		if (!getCurrentSession()->isReseller()) {
			$types = Store::getStoreTypes();
			$generalOptions["order_form_type"] = array(
				"title" => blmsg("TRANS_ONLINE_STORES_TYPE"),
				"desc" => blmsg("TRANS_ONLINE_STORES_TYPE_DESCRIPTION"),
				"value" => is_null($this->orderForm->id) ? Store::TYPE_CUSTOMER : blmsg($types[$this->orderForm->order_form_type]),
				"type" => is_null($this->orderForm->id) ? "s" : "d",
				"required" => is_null($this->orderForm->id),
				"items" => array(
					array(Store::TYPE_CUSTOMER, $types[Store::TYPE_CUSTOMER]),
					array(Store::TYPE_RESELLER, $types[Store::TYPE_RESELLER]),
				),
				"onchange" => "showStorePlans(this.value)"
			);
		}

		require_once("object_managers/PaymentGatewayManager.php");
		$currenciesList = array();
		foreach(Service()->PaymentGatewayManager->getAcceptedCurrencies() as $key => $value) {
			$currenciesList[] = array($key, $value);
		}
		$generalOptions["currency_id"] = array(
			"title" => blmsg("TRANS_CURRENCY"),
			"desc" => blmsg("TRANS_ORDERFORM_CURRENCY_DESC"),
			"value" => $this->orderForm->currency_id,
			"type" => "s",
			"read_only" => Service()->StoreManager->canChangeCurrency($this->orderForm) ? 0 : 1,
			"items" => $currenciesList
		);

		$generalOptions["language_id"] = array(
			"title" => blmsg("TRANS_LANGUAGE"),
			"desc" => blmsg("TRANS_ORDERFORM_LANGUAGE_DESC"),
			"value" => $this->orderForm->language_id,
			"type" => "s",
			"items" => $this->languages
		);

		$generalOptions["countries_id"] = array(
			"title" => blmsg("TRANS_COUNTRY"),
			"desc" => blmsg("TRANS_ORDERFORM_COUNTRY_DESC"),
			"value" => $this->orderForm->countries_id,
			"type" => "s",
			"items" => $this->countries
		);

		$generalOptions["enable_individual_enterpreneur_type"] = array(
			"title" => blmsg("TRANS_ENABLE_INDIVIDUAL_ENTERPRENEUR_TYPE_OPTION"),
			"desc" => blmsg("TRANS_ENABLE_INDIVIDUAL_ENTERPRENEUR_TYPE_DESC"),
			"value" => $this->orderForm->enable_individual_enterpreneur_type,
			"type" => "b",
            "onclick" => "updateAccountTypeList()"
		);
        
        $accountTypes = array();
		foreach(Customer::types() as $key => $value) {
            if (1 == $this->orderForm->enable_individual_enterpreneur_type || Customer::TYPE_INDIVIDUAL_ENTERPRENEUR != $key) {
                $accountTypes[] = array($key, $value);
            }
		}
        
        $generalOptions["default_account_type"] = array(
			"title" => blmsg("TRANS_ORDERFORM_DEFAULT_ACCOUNT_TYPE"),
			"desc" => blmsg("TRANS_ORDERFORM_DEFAULT_ACCOUNT_TYPE_DESC"),
			"value" => $this->orderForm->default_account_type,
			"type" => "s",
			"items" => $accountTypes
		);

		$generalOptions["is_default"] = array(
			"title" => blmsg("TRANS_STORE_DEFAULT"),
			"desc" => blmsg("TRANS_STORE_DEFAULT_DESC", array("DEFAULT_URL" => Service()->CompanyManager->getCompanyByVendorId($this->orderForm->vendor_id)->company_order_url)),
			"value" => $this->orderForm->is_default,
			"read_only" => $this->orderForm->is_default,
			"type" => "b"
		);

		$widgetTemplatesList = array();
		foreach(Service()->StoreManager->getWidgetTemplates() as $key => $value) {
			$widgetTemplatesList[] = array($key, $value);
		}
		$generalOptions["default_widget_template"] = array(
			"title" => blmsg("TRANS_STORE_DEFAULT_WIDGET_TEMPLATE"),
			"desc" => blmsg("TRANS_STORE_DEFAULT_WIDGET_TEMPLATE_DESC", array("URL" => Service()->CompanyManager->getCompanyByVendorId($this->orderForm->vendor_id)->company_order_url)),
			"value" => $this->orderForm->default_widget_template,
			"type" => "s",
			"items" => $widgetTemplatesList
		);

		$generalOptions["continue_shopping_url"] = array(
			"title" => blmsg("TRANS_ORDERFORM_CONTINUE_SHOPPING_URL"),
			"desc" => is_null($this->orderForm->id) ? blmsg("TRANS_ORDERFORM_CONTINUE_SHOPPING_URL_DISABLED_DESC") : blmsg("TRANS_ORDERFORM_CONTINUE_SHOPPING_URL_DESC"),
			"value" => isset($this->orderForm->OrderFormParams["continue_shopping_url"]) ?
					$this->orderForm->OrderFormParams["continue_shopping_url"]->order_form_param_value :
					Service()->StoreManager->getDefaultContinueShoppingUrl($this->orderForm->order_form_code, $this->orderForm->vendor_id),
			"type" => "t",
			"disabled" => is_null($this->orderForm->id)
		);

		return $generalOptions;
	}

	private function loadPaymentOptions()
	{
		$vendorId = getCurrentSession()->getVendorId();
		require_once("objects/PaymentGateway.php");
		// load primary gateways list
		$primaryGateways = Factory()->PaymentGatewayList(
			array(
				"payment_gateway_type" => array(PaymentGateway::TYPE_PRIMARY, PaymentGateway::TYPE_HSPC_CREDIT_CARD),
				"vendor_id"            => $vendorId
			)
		);
		$primaryGateways = DataObject::getMapOf($primaryGateways, "id", "payment_gateway_name");
		if ( false === asort($primaryGateways) ) {
        	    global $php_errormsg;
		    throw new Exception("Failed to sort primary gateways: {$php_errormsg}");
		}
		$primaryGatewaysList = array();
		foreach($primaryGateways as $key => $value) {
			$primaryGatewaysList[] = array($key, $value);
		}

		// load HSPC gateways list
		$hspcBankTransferGateways = Factory()->PaymentGatewayList(
			array(
				"payment_gateway_type" => PaymentGateway::TYPE_HSPC_BANK_TRANSFER,
				"vendor_id"            => $vendorId
			)
		);
		$hspcBankTransferGateways = DataObject::getMapOf($hspcBankTransferGateways, "id", "payment_gateway_name");
		if ( false === asort($hspcBankTransferGateways) ) {
        	    global $php_errormsg;
		    throw new Exception("Failed to sort HSPC Bank Transfer gateways: {$php_errormsg}");
		}
		$hspcBankTransferGatewaysList = array();
		foreach($hspcBankTransferGateways as $key => $value) {
			$hspcBankTransferGatewaysList[] = array($key, $value);
		}

		// load third party gateways list
		$thirdPartyGateways = Factory()->PaymentGatewayList(
			array(
				"payment_gateway_type" => array(PaymentGateway::TYPE_THIRD_PARTY, PaymentGateway::TYPE_HSPC_THIRD_PARTY),
				"vendor_id"            => $vendorId
			)
		);
		$thirdPartyGateways = DataObject::getMapOf($thirdPartyGateways, "id", "payment_gateway_name");
		if ( false === asort($thirdPartyGateways) ) {
        	    global $php_errormsg;
		    throw new Exception("Failed to sort third party gateways: {$php_errormsg}");
		}
		$thirdPartyGatewaysList = array();
		foreach($thirdPartyGateways as $key => $value) {
			$thirdPartyGatewaysList[] = array($key, $value);
		}

		$selectedThirdPartyGateways = array();
		foreach ($this->orderForm->ThirdPartyGatewaysList() as $gateway) {
			$selectedThirdPartyGateways[] = $gateway->id;
		}

		// configure payment options
		$paymentOptions = array();
		$paymentOptions["order_form_disable_charging"] = array(
			"title" => blmsg("TRANS_DISABLE_CHARGING"),
			"desc" => blmsg("TRANS_DISABLE_CHARGING_DESC"),
			"value" => $this->orderForm->order_form_disable_charging,
			"type" => "b"
		);
		if(0 != count($primaryGatewaysList)) {
			$paymentOptions["payment_gateway_id"] = array(
				"title" => blmsg("TRANS_PRIMARY_PAYMENT_GATEWAY"),
				"desc" => blmsg("TRANS_PRIMARY_GATEWAY_DESCRIPTION"),
				"value" => $this->orderForm->payment_gateway_id,
				"type" => "s",
				"items" => $primaryGatewaysList,
				"can_be_disabled" => true
			);
		} else {
			$paymentOptions["payment_gateway_id"] = array(
				"title" => blmsg("TRANS_PRIMARY_PAYMENT_GATEWAY"),
				"value" => blmsg("TRANS_ONLINE_STORE_NO_PRIMARY_GATEWAY_MSG"),
				"type" => "d"
			);
		}
		if(0 != count($hspcBankTransferGatewaysList)) {
			$paymentOptions["hspc_bank_transfer_gateway_id"] = array(
				"title" => blmsg("TRANS_HSPC_BANK_TRANSFER_GATEWAY"),
				"desc" => blmsg("TRANS_HSPC_BANK_TRANSFER_GATEWAY_DESCRIPTION"),
				"value" => $this->orderForm->hspc_bank_transfer_gateway_id,
				"type" => "s",
				"items" => $hspcBankTransferGatewaysList,
				"can_be_disabled" => true
			);
		} else {
			$paymentOptions["hspc_bank_transfer_gateway_id"] = array(
				"title" => blmsg("TRANS_HSPC_BANK_TRANSFER_GATEWAY"),
				"value" => blmsg("TRANS_ONLINE_STORE_NO_BANK_TRANSFER_PLUGIN_MSG"),
				"type" => "d"
			);
		}
		if(0 != count($thirdPartyGatewaysList)) {
			$paymentOptions["third_party_gateway_ids"] = array(
				"title" => blmsg("TRANS_THIRD_PARTY_GATEWAYS"),
				"desc" => blmsg("TRANS_THIRD_PARTY_GATEWAY_DESCRIPTION"),
				"value" => $selectedThirdPartyGateways,
				"type" => "bl",
				"items" => $thirdPartyGatewaysList
			);
		} else {
			$paymentOptions["third_party_gateway_ids"] = array(
				"title" => blmsg("TRANS_THIRD_PARTY_GATEWAYS"),
				"value" => blmsg("TRANS_ONLINE_STORE_NO_THIRD_PARTY_GATEWAY_MSG"),
				"type" => "d"
			);
		}

		foreach($this->_additionalPaymentOptions as $optionName => $optionParams) {
			$paymentOptions[$optionName] = array(
				"title" => blmsg($optionParams[0]),
				"desc" => blmsg($optionParams[1]),
				"value" => $this->orderForm->OrderFormParams[$optionName]->order_form_param_value,
				"type" => "b"
			);
		}

		return $paymentOptions;
	}

	private function loadDomainOptions()
	{
		$selectedOptions = unserialize($this->orderForm->OrderFormParams["dom_options"]->order_form_param_value);
		$domainOptions = array();
		foreach(Service()->StoreManager->getDomainOptions() as $optionName => $optionTitle) {
			$domainOptions[$optionName] = array(
				"title" => blmsg($optionTitle),
				"value" => in_array($optionName, $selectedOptions) ? 1 : 0,
				"type" => "b"
			);
		}

		return $domainOptions;
	}

	private function loadTaxesOptions()
	{
		// load tax zones
		$taxZonesList = array();
		$taxZonesList[] = array("", blmsg("TRANS_SHOW_NET_PRICE"));
		foreach( Factory()->TaxPolicyList(array("tax_zone_group_active" => 1, "vendor_id" => $this->vendorID )) as $taxZoneGroup) {
			foreach ($taxZoneGroup->taxZoneList($this->vendorID) as $taxZone) {
				if (TaxZone::TYPE_PERCENTAGE == $taxZone->tax_zone_amount_type) {
					$percent = BillingLocale::getPercentNumber(floatval($taxZone->tax_zone_amount));
				} else {
					if (is_null($this->orderForm->id)) {
						$percent = $taxZone->tax_zone_amount . " " . blmsg('TRANS_UNIT');
					} else {
						$percent = BillingLocale::getCurrency($taxZone->tax_zone_amount, $this->orderForm->currency_id);
					}
				}
				$taxZonesList[] = array(
					$taxZone->tax_zone_id,
					blmsg("TRANS_SHOW_GROSS_PRICE", array(
						"tax_group" => $taxZoneGroup->tax_zone_group_name,
						"country" => blmsg("TRANS_ISO_3166_".$taxZone->countries_iso_2),
						"percent" => $percent
					))
				);

			}
		}

		$taxesOptions = array();
		$taxesOptions["default_tax_zone_id"] = array(
			"title" => blmsg("TRANS_INCLUDE_DEFAULT_TAX_ZONE_OPTION"),
			"desc" => blmsg("TRANS_INCLUDE_DEFAULT_TAX_ZONE_OPTION_DESC"),
			"value" => $this->orderForm->default_tax_zone_id,
			"type" => "s",
			"items" => $taxZonesList
		);
		$taxesOptions["request_tax_vat_id"] = array(
			"title" => blmsg("TRANS_REQUEST_TAX_VAT_ID_OPTION"),
			"desc" => blmsg("TRANS_REQUEST_TAX_VAT_ID_DESC"),
			"value" => $this->orderForm->request_tax_vat_id,
			"type" => "b"
		);
		$taxesOptions["order_form_verify_vat"] = array(
			"title" => blmsg("TRANS_VERIFY_VAT_NUMBERS"),
			"desc" => blmsg("TRANS_VERIFY_VAT_NUMBERS_DESC"),
			"value" => $this->orderForm->order_form_verify_vat,
			"type" => "b"
		);

		return $taxesOptions;
	}

	private function loadSecurityOptions()
	{
        $securityOptions = array();

        if (is_null($this->orderForm->vendor_id)) {
            $orderFormParams = $this->orderForm->OrderFormParams;
            $securityOptions["email_banning"] = array(
                "email_banning" => array(
                    "title" => blmsg("TRANS_ORDER_EMAIL_BANNING"),
                    "desc" => blmsg("TRANS_ORDER_EMAIL_BANNING_DESC"),
                    "value" => $orderFormParams["email_banning"]->order_form_param_value,
                    "type" => "b",
                ),
                "email_banning_text" => array(
                    "title" => blmsg("TRANS_ORDER_EMAIL_BANNING_TEXT"),
                    "desc" => blmsg("TRANS_ORDER_EMAIL_BANNING_TEXT_DESC"),
                    "value" => $orderFormParams["email_banning_text"]->order_form_param_value,
                    "type" => "a",
                ),
            );
            $securityOptions["country_banning"] = array(
                "country_banning" => array(
                    "title" => blmsg("TRANS_ORDER_COUNTRY_BANNING"),
                    "desc" => blmsg("TRANS_ORDER_COUNTRY_BANNING_DESC"),
                    "value" => $orderFormParams["country_banning"]->order_form_param_value,
                    "type" => "b",
                ),
                "country_banning_text" => array(
                    "title" => blmsg("TRANS_ORDER_COUNTRY_BANNING_TEXT"),
                    "desc" => blmsg("TRANS_ORDER_COUNTRY_BANNING_TEXT_DESC"),
                    "value" => $orderFormParams["country_banning_text"]->order_form_param_value,
                    "type" => "a",
                ),
            );
        }
		return $securityOptions;
	}

	private function loadLicenseOptions()
	{
		$licenseOptions = array();
		$licenseOptions["use_terms_url"] = array(
			"title" => blmsg("TRANS_ORDER_FORM_TERMS_TEXT"),
			"value" => $this->orderForm->use_terms_url,
			"type" => "r",
			"onclick" => "toggleLicenseVariants()",
			"items" => array(
				array("1", blmsg("TRANS_ORDERFORM_TERMS_URL_FORWARD_CUSTOMERS")),
				array("0", blmsg("TRANS_ORDERFORM_TERMS_URL_NO"))
			)
		);

		$licenseOptions["terms_url"] = array(
			"title" => blmsg("TRANS_ORDERFORM_TERMS_URL_YES"),
			"value" => $this->orderForm->OrderFormParams["terms_url"]->order_form_param_value,
			"type" => "t",
			"isVisible" => isset($_REQUEST["use_terms_url"]) ? (1 == bgpc("use_terms_url")) : (1 == $this->orderForm->use_terms_url)
		);

		$licenseOptions["terms_text"] = array(
			"title" => blmsg("TRANS_EDIT_ORDER_FORM_TERMS_TEXT_OPTION"),
			"value" => Service()->StoreManager->getTermsText($this->orderForm),
			"type" => "w",
			"extention" => array(
				"elements" => "terms_text",
			),
			"isVisible" => isset($_REQUEST["use_terms_url"]) ? (0 == bgpc("use_terms_url")) : (0 == $this->orderForm->use_terms_url)
		);

		return $licenseOptions;
	}

	private function loadProductsOptions()
	{
		$productsOptions = array();

		$availableProducts = array();
		$reseller = getCurrentSession()->getReseller();
		$filter = array(
			"product_is_addon" => 0,
			"vendor_id" => $reseller ? $reseller->id : null
		);
		if(!is_null($this->orderForm->id)) {
			if(Store::TYPE_RESELLER == $this->orderForm->order_form_type) {
				$filter["product_type"] = Plan::TYPE_RESELLER;
			} else {
				$productTypes = Plan::types(!is_null($reseller));
				unset($productTypes[Plan::TYPE_RESELLER]);
				$filter["product_type"] = array_keys($productTypes);
			}
		}
		foreach(Factory()->PlanList($filter) as $product) {
            $title = billing_safetyhtml($product->product_name)." - ".blmsg(Plan::typeName($product->product_type));
            if (!$product->product_active) {
                $title .= " (".blmsg("TRANS_INACTIVE").")";
            }
			$availableProducts[] = array($product->product_id, $title);
		}


		$selectedProducts = array();
		foreach($this->orderForm->PlanList() as $product) {
			$selectedProducts[$product->product_id] = 1;
		}

		$productsOptions["products"] = array(
			"title" => blmsg("TRANS_ASSIGNED_PRODUCTS"),
			"desc" => blmsg("TRANS_ORDER_FORM_PRODUCT_SETTINGS_DESC"),
			"value" => $selectedProducts,
			"type" => "dsl",
			"items" => $availableProducts,
			"left_column_title" => blmsg("TRANS_ASSIGNED_PRODUCTS"),
			"right_column_title" => blmsg("TRANS_UNASSIGNED_PRODUCTS"),
            "show_positions_buttons" => true,
		);
		return $productsOptions;
	}

	private function saveGeneralOptions()
	{
		foreach ($this->module_input_controls["panel-general"] as $control) {
			$name = $control->name;
			switch($name) {
				case "continue_shopping_url" :
					if (
						"" == $control->value ||
						$control->value == Service()->StoreManager->getDefaultContinueShoppingUrl($this->orderForm->order_form_code, $this->orderForm->vendor_id) ||
						$control->value == Service()->StoreManager->getDefaultContinueShoppingUrl($this->orderForm->_OLD_order_form_code, $this->orderForm->vendor_id)
					) {
						unset($this->orderForm->OrderFormParams["continue_shopping_url"]);
					} else {
						$this->orderForm->OrderFormParams["continue_shopping_url"]->order_form_param_value = $control->value;
					}
					break;
				case "order_form_type" :
				    if (!getCurrentSession()->isReseller() && is_null($this->orderForm->id)) {
				        $this->orderForm->order_form_type = $control->value;
				    }
				    break;
				default:
					$this->orderForm->$name = $control->value;
			}
		}
		Service()->StoreManager->update($this->orderForm);
	}

	private function savePaymentOptions()
	{
		foreach ($this->module_input_controls["panel-payment"] as $control) {
			$name = $control->name;
			switch($name) {
				case "payment_gateway_id" :
				case "hspc_bank_transfer_gateway_id" :
					$this->orderForm->$name = (0 == bgpc('enable' . $name)) ? NULL : $control->value;
					break;
				case "third_party_gateway_ids" :
					$this->orderForm->thirdPartyGatewayIDs = is_array($control->value) ? $control->value : array();
					break;
				default :
					if (in_array($name, array_keys($this->_additionalPaymentOptions))) {
						$this->orderForm->OrderFormParams[$name]->order_form_param_value = $control->value;
					} else {
						$this->orderForm->$name = $control->value;
					}
			}
        }

		//One payment method must be enabled
		if (!$this->orderForm->isAvailablePaymentMethods()) {
			throw new ProductException(ProductException::E_OBJECT_HAS_INVALID_ATTRS, blmsg("TRANS_ORDERFORM_PAYMENT_METHODS_ERROR"));
		}

		Service()->StoreManager->update($this->orderForm);
	}

	private function saveDomainOptions()
	{
		$selectedOptions = array();
		foreach(Service()->StoreManager->getDomainOptions() as $optionName => $optionTitle) {
			if(isset($_REQUEST[$optionName])) {
				$selectedOptions[] = $optionName;
			}
		}

		$this->orderForm->OrderFormParams["dom_options"]->order_form_param_value = serialize($selectedOptions);
		Service()->StoreManager->update($this->orderForm);
	}

	private function saveTaxesOptions()
	{
		foreach ($this->module_input_controls["panel-taxes"] as $control) {
			$name = $control->name;
			switch($name) {
				case "default_tax_zone_id" :
					$this->orderForm->default_tax_zone_id = ((int)$control->value) ? $control->value : NULL;
					break;
				default :
					$this->orderForm->$name = $control->value;
					break;
			}
		}

		Service()->StoreManager->update($this->orderForm);
	}

	private function saveSecurityOptions()
	{
        $panels = array("panel-security_email_banning", "panel-security_country_banning");
		foreach($panels as $panelID) {
			foreach ($this->module_input_controls[$panelID] as $control) {
				$this->orderForm->OrderFormParams[$control->name]->order_form_param_value = $control->value;
			}
            if (!is_null($this->orderForm->vendor_id) && !getCurrentSession()->isReseller()) {
                $this->orderForm->OrderFormParams["email_banning"]->order_form_param_value = 0;
                $this->orderForm->OrderFormParams["country_banning"]->order_form_param_value = 0;
            }
		}
        Service()->StoreManager->update($this->orderForm);
	}

	private function saveLicenseOptions()
	{
		$this->orderForm->use_terms_url = bgpc("use_terms_url");
		if($this->orderForm->use_terms_url) {
			$termsURL = bgpc("terms_url");
			if("" == $termsURL) {
				$this->addErrorMessage(UIFactory()->ErrorMessage(blmsg("TRANS_ORDERFORM_TERMS_URL_REQUIRED_MSG")));
				return;
			}
			$this->orderForm->OrderFormParams["terms_url"]->order_form_param_value = $termsURL;
		} else {
			$this->orderForm->terms_text = bgpc("terms_text");
		}

		Service()->StoreManager->update($this->orderForm);
	}

	private function saveProductsOptions()
	{
		$products = array();
		foreach (bgpc("products") as $productId => $productIsEnabled) {
			if ($productIsEnabled && $this->orderForm->hasSameVendor(Factory()->Plan($productId))) {
				$products[] = $productId;
			}
		}
		Service()->StoreManager->updatePlans($this->orderForm, $products ? $products : array());
	}

	private $orderForm;
	private $languages;
	private $countries;
	private $options;
    private $_additionalPaymentOptions = array(
        "payment_type_check" => array("TRANS_ORDER_ENABLE_CHECK", "TRANS_ORDER_ENABLE_CHECK_DESC"),
        "payment_type_invoice" => array("TRANS_ORDER_ENABLE_INVOICE", "TRANS_ORDER_ENABLE_INVOICE_DESC"),
        "payment_type_draft" => array("TRANS_ORDER_ENABLE_BANK_DRAFT", "TRANS_ORDER_ENABLE_BANK_DRAFT_DESC"),
        "omit_zero_payments" => array("TRANS_STORE_OMIT_ZERO_PAYMENTS", "TRANS_STORE_OMIT_ZERO_PAYMENTS_DESC")
    );
}

class EditOrderFormSettings extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$orderFormID = bgpc("orderFormID");

		$parent_url = is_null($orderFormID) ? action_url("orderforms", "ShowOrderForms") :
			admin_action_url("orderforms", "store-overview", "index", array("id" => $orderFormID));

		$controller  = UIFactory()->PageController($parent_url);
		$controller->addPage($page = new EditOrderFormSettingsPage($controller));

		$this->mergeLanguageData();
		$this->mergeCountryData();
		$page->setVars($orderFormID, $this->smartyVars["languageList"], $this->smartyVars["countryList"]);

		$page->title = blmsg("TRANS_ONLINE_STORE_SETTINGS");
		$page->parent_title = is_null($orderFormID) ? blmsg("TRANS_ONLINE_STORES") : Factory()->Store($orderFormID)->order_form_name;
		$page->parent_url = $parent_url;

		$controller->processRequest();
		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}