<?php

use Billing\Object\Store;

class Orderforms_StoreOverviewController extends Billing_Zend_Controller_Action_AdminController
{
	public function init()
	{
		parent::init();
	}

	public function indexAction()
	{
		try {
            $this->_store = Factory()->Store($this->getRequest()->getParam('id'));
        } catch (ProductException $e) {
            $this->_redirectWithMessage($this->view->ActionUrl('orderforms', 'ShowOrderForms'), $e->getMessage(), "error");
        }

        $this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_ONLINE_STORE_OVERVIEW_TITLE', array('NAME' => billing_safetyhtml($this->_store->order_form_name))));

		$this->view->toolButtons = array(
			'settings' => array(
				'label' => 'TRANS_ONLINE_STORE_SETTINGS',
				'class' => 'sb-store-settings',
				'href' => $this->view->actionUrl('orderforms', 'EditOrderFormSettings', array('orderFormID' => $this->_store->id))
			),
			'go' => array(
				'label' => 'TRANS_ONLINE_STORES_GO_TO',
				'class' => 'sb-go-to-store',
				'separate' => true,
				'href' => Service()->CompanyManager->getCompanyByVendorId($this->_store->vendor_id)->company_order_url.$this->_store->order_form_code.".html",
				"target" => "_blank"
			),
			'remove' => array(
				'label' => 'TRANS_REMOVE',
				'class' => 'sb-remove',
				'separate' => true,
				'href' => $this->view->actionUrl('orderforms', 'RemoveOrderForm', array('orderFormID' => $this->_store->id))
			)
		);

		$this->view->storeDetails = new Billing_Zend_Form_Section();

		$this->view->storeDetails->addSubForm($this->_getPaymentOptions(), 'paymentOptions');
		$this->view->storeDetails->addSubForm($this->_getDomainOptions(), 'domainOptions');
		$this->view->storeDetails->addSubForm($this->_getTaxesOptions(), 'taxesOptions');

		$this->view->storeDetails->setDecorators(array(
			'FormElementsWithoutRender',
			array(
				'ViewScript',
				array(
					'leftColumn' => array('paymentOptions'),
					'rightColumn' => array('domainOptions', 'taxesOptions'),
					'viewScript' => 'partials/forms/two-columns.phtml'
				)
			),
			'Section'
		));

		$this->view->tabs = array(
			'design' => 'TRANS_ONLINE_STORE_DESIGN',
			'coupons' => 'TRANS_COUPONS'
		);

		$this->view->designForm = new Billing_Zend_Form_Section();
		$this->view->designForm->addElement('link', 'colorTheme', array(
			'label' => $this->_translator->translate('TRANS_ONLINE_STORE_COMMON_COLOR_THEME'),
			'value' => $this->_translator->translate('TRANS_ONLINE_STORE_CHANGE_THEME'),
			'href' => $this->view->actionUrl('orderforms', 'EditOrderFormTheme', array('orderFormID' => $this->_store->id))
		));
		$this->view->designForm->addElement('link', 'headerAndFooter', array(
			'label' => $this->_translator->translate('TRANS_ONLINE_STORE_CHECKOUT_SCREENS'),
			'value' => $this->_translator->translate('TRANS_ONLINE_STORE_EDIT_HEADER_AND_FOOTER'),
			'href' => $this->view->actionUrl('orderforms', 'EditOrderFormHeaderAndFooter', array('orderFormID' => $this->_store->id))
		));

		$this->view->designTabToolButtons = array(
			'add' => array(
				'label' => 'TRANS_ADD_WEBSITE_WIDGET',
				'class' => 'sb-add-website-widget',
				'href' => $this->view->actionUrl('websitewidgets', 'AddWebsiteWidget', array('orderFormID' => $this->_store->id))
			),
			'remove' => array(
				'label' => 'TRANS_REMOVE',
				'class' => 'sb-remove',
				'separate' => true,
				'onclick' => "if(0 == getSelectedWidgetsCount()) { alert('".$this->_translator->translate('TRANS_LIST_NO_SELECTED_ITEMS')."'); } else { jQuery('#widgets_form').submit(); return false; }"
			)
		);

		$this->view->standardStoreURL = Service()->CompanyManager->getCompanyByVendorId($this->_store->vendor_id)->company_order_url;

		$this->view->widgets = $this->_store->WebsiteWidgetList();

		$this->view->coupons = array();
		foreach(Factory()->CouponList(array("vendor_id" => $this->_store->vendor_id, "coupon_active" => 1, )) as $coupon) {
			if(!in_array($this->_store->id, $coupon->storeIDs)) {
				continue;
			}
			$this->view->coupons[] = $coupon;
		}

		if(!is_null($this->_store->hspc_bank_transfer_gateway_id)) {
			$paymentGateway = $this->_store->HSPCBankTransferGateway;
			$acceptedCurrencies = $paymentGateway->acceptedCurrencies;
			if(!empty($acceptedCurrencies) && !in_array($this->_store->currency_id, $acceptedCurrencies)) {
				$this->_status->addError($this->_translator->translate('TRANS_STORE_OVERVIEW_ERROR_NOT_SUPPORTER_CURRENCY_FOR_GATEWAY', array(
					'CURRENCY' => billing_safetyhtml($this->_store->currency_id),
					'GATEWAY' => billing_safetyhtml($paymentGateway->payment_gateway_name)
				)));
			}
		}
    }

    private function _getPaymentOptions()
	{
		$paymentOptions = new Billing_Zend_Form_Section(array(
			'legend' => $this->_translator->translate('TRANS_PAYMENT_OPTIONS'),
		));

		if(is_null($this->_store->default_tax_zone_id)) {
			$defaultTaxZone = $this->_translator->translate('TRANS_SHOW_NET_PRICE');
		} else {
			$taxZone = Factory()->TaxZone($this->_store->default_tax_zone_id);
			$defaultTaxZone = $this->_translator->translate('TRANS_SHOW_GROSS_PRICE', array(
                'tax_group' => $taxZone->getTaxZoneGroup()->tax_zone_group_name,
				'country' => $this->_translator->translate('TRANS_ISO_3166_'.$taxZone->countries_iso_2),
				'percent' => BillingLocale::getPercentNumber(floatval($taxZone->tax_zone_amount))
			));
		}

		$paymentOptions->addElement('SimpleText', 'defaultTaxZone', array(
			'label' => $this->_translator->translate('TRANS_INCLUDE_DEFAULT_TAX_ZONE_OPTION'),
			'value' => $defaultTaxZone
		));

		$paymentOptions->addElement('YesNoIndicator', 'orderFormDisableCharging', array(
			'label' => $this->_translator->translate('TRANS_DISABLE_CHARGING'),
			'value' => $this->_store->order_form_disable_charging
		));

		$paymentOptions->addElement('SimpleText', 'primaryPaymentGateway', array(
			'label' => $this->_translator->translate('TRANS_PRIMARY_PAYMENT_GATEWAY'),
			'value' => !is_null($this->_store->payment_gateway_id) ? $this->_store->PrimaryGateway->payment_gateway_name : "-"
		));

		$paymentOptions->addElement('SimpleText', 'hspcBankTransferGateway', array(
			'label' => $this->_translator->translate('TRANS_HSPC_BANK_TRANSFER_GATEWAY'),
			'value' => (!is_null($this->_store->hspc_bank_transfer_gateway_id)) ? $this->_store->HSPCBankTransferGateway->payment_gateway_name : "-"
		));

		$thirdPartyGateways = array();
		if(0 != count($this->_store->thirdPartyGatewayIDs)) {
			foreach($this->_store->thirdPartyGatewayIDs as $gatewayID) {
				$elementName = "thirdPartyGateway" . $gatewayID;
				$thirdPartyGateways[] = $elementName;
				$paymentOptions->addElement('SimpleText', $elementName, array(
					'renderStyle' => 'group',
					'label' => "",
					'value' => Factory()->PaymentGateway($gatewayID)->payment_gateway_name
				));
			}
		}
		if (!empty($thirdPartyGateways)) {
            $paymentOptions->addDisplayGroup($thirdPartyGateways, 'thirdPartyGatewaysList',
                array(
                'label' => $this->_translator->translate('TRANS_THIRD_PARTY_GATEWAYS'),
                'decorators' =>  array(
                    array('ViewScript', array(
                    'viewScript' => 'partials/forms/multiple-inputs.phtml'
                    ))
                )
                )
            );
        }

        $additionalPaymentOptions = array(
			"payment_type_check" => "TRANS_ORDER_ENABLE_CHECK",
			"payment_type_invoice" => "TRANS_ORDER_ENABLE_INVOICE",
			"payment_type_draft" => "TRANS_ORDER_ENABLE_BANK_DRAFT"
		);
        foreach($additionalPaymentOptions as $optionName => $optionTitle) {
        	$paymentOptions->addElement('YesNoIndicator', $optionName, array(
				'label' => $this->_translator->translate($optionTitle),
				'value' => $this->_store->OrderFormParams[$optionName]->order_form_param_value
			));
		}

		return $paymentOptions;
	}

	private function _getDomainOptions()
	{
		$domainOptions = new Billing_Zend_Form_Section(array(
			'legend' => $this->_translator->translate('TRANS_ORDER_DOMAIN_OPTIONS'),
		));

		$domainOptionParamsSelected = unserialize($this->_store->OrderFormParams["dom_options"]->order_form_param_value);

        $optionValues = array();
		foreach(Service()->StoreManager->getDomainOptions() as $optionName => $optionTitle) {
            $optionValues[$optionName] = in_array($optionName, $domainOptionParamsSelected);
			$domainOptions->addElement('YesNoIndicator', $optionName, array(
				'label' => $this->_translator->translate($optionTitle),
				'value' => $optionValues[$optionName]
			));
		}

        if (!$optionValues[Store::DOMAIN_TYPE_REG] && !$optionValues[Store::DOMAIN_TYPE_TRANSFER]) {
            $this->_status->addWarning($this->_translator->translate('TRANS_STORE_OVERVIEW_WARNING_DOMAIN_PLAN_IS_NOT_FOR_SALE'));
        }

		return $domainOptions;
	}

	private function _getTaxesOptions()
	{
		$taxesOptions = new Billing_Zend_Form_Section(array(
			'legend' => $this->_translator->translate('TRANS_ONLINE_STORE_TAXES'),
		));

		$taxesOptions->addElement('YesNoIndicator', 'requestTaxVatID', array(
			'label' => $this->_translator->translate('TRANS_REQUEST_TAX_VAT_ID_OPTION'),
			'value' => $this->_store->request_tax_vat_id
		));

		$taxesOptions->addElement('YesNoIndicator', 'orderFormVerifyVat', array(
			'label' => $this->_translator->translate('TRANS_VERIFY_VAT_NUMBERS'),
			'value' => $this->_store->order_form_verify_vat
		));

		return $taxesOptions;
	}

    private $_store = null;
}