<?php
require_once ("lib-action/action.php");
require_once ("lib-tk/toolkit.php");

use Billing\Object\Plan;
use Billing\Object\PlanVariantPrice;
use Billing\Object\Subscription;

class AddHistoricalPackage extends MB_Action
{
	public function execute()
	{
		$this->assignMode();
		$this->assignProductInfo();
		$this->assignClientInfo();
		$this->assignDateHandlers();
		$this->assignDefaults();

		$domainParams = array();
		if (!is_null($domainName = bgpc("domainName"))) {
			$domainParams["domainName"] = billing_safetyhtml($domainName);
		}
		if (!is_null($domainTLD = bgpc("domainTLD"))) {
			$domainParams["domainTLD"] = billing_safetyhtml($domainTLD);
		}
		$this->mergeFormState($domainParams);
		$this->addTemplate("packages/AddHistoricalPackage");
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

	protected function assignMode()
	{
			$this->smartyVars["mode"] = $mode = $_REQUEST["mode"];
			$this->smartyVars["historical"] =
			$this->historical =
				$mode == "2" ? true : false;

			$this->smartyVars["clientID"] = getParam("clientID");
			$this->smartyVars["orderFormID"] = getParam("orderFormID");
			$this->smartyVars["productID"] = getParam("productID");
			$this->smartyVars["productVariantID"] = getParam("productVariantID");
			$this->smartyVars["cycleID"] = getParam("cycleID");
	}

	protected function assignProductInfo()
	{
		$d = new DataSource(":products", $params = array(
			"productID" => $_REQUEST["productID"],
			"orderFormItems" => array(
				"orderFormID" => $_REQUEST["orderFormID"],
				"orderForm" => array(
				),
			),
			"productVariants" => array(
				"productVariantID" => $_REQUEST["productVariantID"],
				"TLD" => array(
				),
				"productVariantPrices" => array(
					"cycleID" => $_REQUEST["cycleID"],
					"cycle" => array(
					),
				),
			),
			"productAttributes" => array(
				"serverGroupID" => "*",
				//"groupBy" => array(
				//	"column" => "serverGroupID",
				//),
				"serverGroup" => array(
					"configGroups" => array(
						"column" => array(
							"configGroupID",
						),
						"configParams" => array(
							"column" => array(
								"configGroupID",
								"configParamValue",
							),
							"configParamName" => "moduleDisplayName",
							"comparisonOperator" => "=",
						),
					),
				),
			),
		));
		$this->e = $e = $d->getEssential();
		$vars = array(
			"product" => $this->product = $e[0],
			"orderForm" => $this->orderForm = $e[0]["orderFormItems"][0]["orderForm"][0],
			"productVariant" => $this->productVariant = $e[0]["productVariants"][0],
			"domainRequired" => $this->domainRequired = $e[0]["productDomainRequired"],
			"TLD" => ($this->TLD = isset($this->productVariant['TLD']) ? $this->productVariant['TLD'][0] : null),
			"productVariantPrices" => $this->productVariantPrices = $this->productVariant["productVariantPrices"],
			"cycle" => $this->cycle = $this->productVariantPrices[0]["cycle"][0],
			"serverGroupAttributes" => $this->serverGroupAttributes = $this->product["productAttributes"],
		);
		$this->productID = $this->product["productID"];
		$this->orderFormID = $this->orderForm["orderFormID"];
		$this->productVariantID = $this->productVariant["productVariantID"];
		$this->tldID = $this->TLD["tldID"];
		$this->cycleID = $this->cycle["cycleID"];

		$this->currencyID = $vars["currencyID"] = $this->orderForm["currencyID"];

		$this->clientID = (int) $_REQUEST["clientID"];

		$vars["cycle"]["cycleDisplayName"] = $this->cycle["cycleDisplayName"] = blmsg($this->cycle["cycleDisplayName"]);

		$productVariantPrices = array();
		foreach((array)$this->productVariantPrices as $price) {
			if($price["currencyID"] == $this->currencyID) $productVariantPrices[] = $price;
		}
		$priceT = new BillingTable($productVariantPrices);
		$priceT->setSource("variantPrices");
		$priceT->alterState(TABLE_STATE_INTERNAL);
		$priceT->setKey("productVariantPriceType");
		$priceT->setName("productVariantPriceAmount");
		$prices = $priceT->getOptions();

		$vars["prices"] = $prices;
        $preappliedCouponList = Factory()->Plan($this->productID)->preappliedCouponList();
        $coupon = reset($preappliedCouponList);
        if ($coupon && $coupon->canApply($this->clientID)) {
            $generalPrice = $vars["prices"][PlanVariantPrice::TYPENAME_GENERAL];
            $vars["prices"][PlanVariantPrice::TYPENAME_GENERAL] = $generalPrice -  Service()->CouponManager->getCouponAmount($generalPrice, $coupon);
        }

		$productT = new BillingTable(array($this->product));
		$productT->setSource("products");
		$productT->alterState(TABLE_STATE_INTERNAL);
		$this->product = $productT->getOne();
		$vars["productType"] = $this->type = $type = $this->product["productType"];

		if($type == "domain") {
			$this->smartyVars["tldID"] = $this->TLD["tldID"];
			$this->smartyVars["tldExtension"] = $this->TLD["tldExtension"];
		} else {
			// uniquize it by server group id
			$serverGroups = array();
			foreach((array)$this->serverGroupAttributes as $serverGroupAttribute) {
				$serverGroups[$serverGroupAttribute["serverGroupID"]] = $serverGroupAttribute;
			}
			$this->serverGroupAttributes = array_values($serverGroups);
			foreach (array_keys($serverGroups) as $serverGroupId) {
				$vars["showCredentialFields"][$serverGroupId] = false;
				$controlPanels = Factory()->PanelGroup($serverGroupId)->ControlPanelList();
				foreach ($controlPanels as $panel) {
					if ($panel->canSetSubscriptionCredentials()) {
						$vars["showCredentialFields"][$serverGroupId] = true;
						break;
					}
				}
				if (true === $vars["showCredentialFields"][$serverGroupId]) {
					break;
				}
			}

			$serverGroups = $panelsByServerGroupID = array();
			foreach((array)$this->serverGroupAttributes as $serverGroupAttribute) {
				$serverGroup = $serverGroupAttribute["serverGroup"][0];
				$serverGroupID = $serverGroup["serverGroupID"];
				$serverGroupName = $serverGroup["serverGroupName"];
				$serverGroups[$serverGroupID] = $serverGroupName;

				$panelsByServerGroupID[$serverGroupID] = array();
				foreach($serverGroup["configGroups"] as $configGroup) {
                    if (!isset($configGroup["configParams"])) {
                        continue;
                    }
					$serverID = $configGroup["configGroupID"];
					$serverName = $configGroup["configParams"][0]["configParamValue"];
					$panelsByServerGroupID[$serverGroupID][$serverID] = $serverName;
				}

			}

			$this->serverGroups = $vars["serverGroups"] = $serverGroups;
			$this->panelsByServerGroupID = $vars["panelsByServerGroupID"] = $panelsByServerGroupID;
		}

		// do we have an lek stored?
		$this->smartyVars["enableCharging"] = Service()->CryptManager->isEncryptionEnabled();
		$this->smartyVars["needKey"] = Service()->CryptManager->isEncryptionEnabled() && !Service()->CryptManager->isLekEnabled() ? true : false;

		$this->smartyVars += $vars;
	}

	protected function assignClientInfo()
	{
		$type = $this->smartyVars["productType"];
		$vars = array();

		$params = array(
			"clientID" => $this->clientID,
			"contacts" => array(
				"contactActive" => "true",
				"billingAccounts" => array(
					"billingAccountActive" => "true"
				),
			),
		);

		$domainParams = array();
		switch($type) {
			case "domain":
				$domainParams = array(
					"domainTLD" => $this->TLD["domainTLD"],
					"comparisonOperator" => "=",
				);
			case "hosting":
				$params += array(
					"packages" => array(
						"packageActive" => "true",
						"column" => array(
							"packageID",
							"clientID",
						),
						"domains" => array(
							"domainActive" => "true",
							"column" => array(
								"domainID",
								"domainSLD",
								"domainTLD",
							),
						) + $domainParams,
					),
				);
				break;
		}

		$d = new DataSource(":clients", $params);
		$e = $d->getEssential();

		require_once ("lib-billing/include/config/internal/mbapimap/billingAccounts.php");
		$billingAccounts = $this->primaryContact = array();
		array();
		foreach($e[0]["contacts"] as $contact) {
			if($contact["contactPrimary"]) {
				$this->primaryContact = $contact;
			}
            $store = Factory()->Store($this->orderFormID);
			foreach((array)$contact["billingAccounts"] as $billingAccount) {
                if (\Billing\Object\BillingAccount::TYPE_HSPC_BANK_TRANSFER == $billingAccount["billingAccountType"] && $store->hspc_bank_transfer_gateway_id != $billingAccount["paymentGatewayID"]) {
                    continue;
                }
				refBillingAccountColumnGenerator($billingAccount);
				$billingAccounts[$billingAccount["billingAccountID"]] = $billingAccount["billingAccountDisplayName"];
			}
		}
		$vars["clientInfo"] = $e[0] + $this->primaryContact;
		$vars["clientBillingAccounts"] = $billingAccounts;
		$vars["clientDomains"] = array();
		if(Plan::DOMAIN_NOT_NEEDED != $this->domainRequired) {
			foreach((array)$e[0]["packages"] as $package) {
                if (!isset($package['domains'])) {
                    continue;
                }
				foreach((array)$package["domains"] as $domain) {
					$vars["clientDomains"][$domain["domainID"]] = $domain["domainSLD"] . "." . $domain["domainTLD"];
				}
			}
		}

        if (count(Factory()->Plan($this->productID)->ParentPlanListForSupplementaryPlan())) {
            $vars["clientSubscriptions"] = array();
            foreach (Service()->SubscriptionManager->getSubscriptionsToAttachAsParent($this->clientID, $this->productID, $this->cycleID) as $subscription) {
                $vars["clientSubscriptions"][$subscription->id] = $subscription->displayName;
            }
        }

		$this->smartyVars += $vars;
	}

	private function assignDateHandlers()
	{
		if($this->product["productStopNumCycles"] != 1) {
			$this->formDefaults["packageNextRenewal"] = time();
			$this->setupDateSelector("packageNextRenewal");
		}
		$this->formDefaults["packageStartDate"] = time();
		$this->setupDateSelector("packageStartDate");

		if($this->type == "domain") {
			$this->formDefaults["registerDomain"] = time();
			$this->setupDateSelector("registerDomain", !$this->historical);
		} else {
			foreach($this->serverGroups as $serverGroupID => $serverGroupName) {
                $key = "sg" . $serverGroupID . "create";
				$this->formDefaults[$key] = time();
				$this->setupDateSelector("sg" . $serverGroupID . "create", !$this->historical);
                if(1 == bgpc("validationError")) {
                    $this->formDefaults[$key] = (1 == bgpc("validationError")) ? bgpc($key) : 1 ;
                }
			}
		}
	}

	private function assignDefaults()
	{
			$this->formDefaults["generalPrice"] = isset($this->smartyVars["prices"]["general"]) ? $this->smartyVars["prices"]["general"] : -1;
			$this->formDefaults["setupPrice"] = isset($this->smartyVars["prices"]["setup"]) ? $this->smartyVars["prices"]["setup"] : -1;
			$this->formDefaults["transferPrice"] = isset($this->smartyVars["prices"]["transfer"]) ? $this->smartyVars["prices"]["transfer"] : -1;
	}
}
