<?php

require_once("lib-action/action.php");
require_once("lib-mbapi/mbapi.php");
require_once("lib-billing/include/config/internal/cycles.php");

class AddPackageAddon extends MB_Action
{
	public $params = array("packageID");

	public function execute()
	{
		parent::execute();

        $subscriptionId = bgpc("packageID");
        $subscription = Factory()->Subscription($subscriptionId);
        $addonWithParentCycles = Service()->PackageAddonManager->selWithParentCycles();

        $rawPackageAddons = correctArray(grabResultsFromMBAPIArray(array(
			"mbapi" => array(
				"command" => "GetPackageAddons",
				"params" => array(
					"packageID" => $subscriptionId,
					"packageAddonActive" => 1,
				),
			),
		)));

        $packageAddons = array();
		foreach ($rawPackageAddons as $addon){
            $packageAddons[$addon["packageAddonProductID"]] = $addon;
		}

		$productsInfo = correctArray(grabSimpleMBAPIResults(dispatchMBAPI("
			<mbapi>
				<command>GetProducts</command>
				<params>
					<parentProductID>{$subscription->product_id}</parentProductID>
					<getProductAttributeData>1</getProductAttributeData>
					<getCouponData>1</getCouponData>
					<getAllOrderFormItemData>1</getAllOrderFormItemData>
					<getOrderFormItemData>1</getOrderFormItemData>
					<getOrderFormData>1</getOrderFormData>
					<getProductVariantData>1</getProductVariantData>
					<getProductVariantPriceData>1</getProductVariantPriceData>
					<getTLDData>1</getTLDData>
					<getCycleData>1</getCycleData>
				</params>
			</mbapi>", 1)));

		$addonGroupIDs = array();
		$productAddons = array();
		foreach ($productsInfo as $addon){
			$productAddons[$addon["productID"]] = $addon;
			$addonGroupIDs[] = $addon["productAddonGroupID"];
		}

		$rawProductAddonGroups = correctArray(grabResultsFromMBAPIArray(array(
			"mbapi" => array(
				"command" => "GetProductAddonGroups",
				"params" => array(
					"productAddonGroupID" => $addonGroupIDs,
				),
			),
		)));

		$productAddonGroups = array();
		foreach ($rawProductAddonGroups as $addonGroup){
            $productAddonGroups[$addonGroup["productAddonGroupID"]] = $addonGroup;
		}


		foreach ($packageAddons as $productID => $packageAddon){
			$groupID = $productAddons[$productID]["productAddonGroupID"];
			if ($productAddonGroups[$groupID]["productAddonGroupInputType"] != getAPITypeID("addoninputs", "checkbox")){
				unset($productAddonGroups[$groupID]);
			}
		}

        $resultAddons = array();
		foreach ($productsInfo as $addon){
			if (isset($packageAddons[$addon["productID"]]) || !isset($productAddonGroups[$addon["productAddonGroupID"]])
                || !isset($addon["productVariantPrices"])) {
                // addon of this plan is already purchased
                // OR addon from this (non multiple) addon group is already purchased
                // OR prices for this addon are not set
                continue;
			}

            foreach($addon["productVariantPrices"] as $price) {
                if ($price["currencyID"] != $subscription->Store->currency_id
                    || ($addonWithParentCycles && $price["cycleID"] != $subscription->cycle_id)
                    || (!$addonWithParentCycles && isCycleLessThan($subscription->cycle_id, $price["cycleID"]))) {
                    // OR addon has bigger cycle
                    // OR addon has different cycle and "Purchase addon with non parent cycles" feature is not available
                    // OR addon has bigger cycle and "Purchase addon with non parent cycles" feature is available
                    continue;
                }

                $id = $addon["productID"].'-'.$price["cycleID"];
                if (!isset($resultAddons[$addon["productAddonGroupID"]][$id])) {
                    $resultAddons[$addon["productAddonGroupID"]][$id] = array(
                        "productName" => $addon["productName"],
                        "productActive" => $addon["productActive"],
                        "cycleID" => $price["cycleID"],
                    );
                }

                $resultAddons[$addon["productAddonGroupID"]][$id][$price["productVariantPriceType"]] = $price;
                $this->formDefaults["{$id}-nextrenewal"] = getNewAddonNextRenewalDate($subscription, $price["cycleID"]);
                $this->setupDateSelector("{$id}-nextrenewal");
            }
		}

		$this->smartyVars["products"] = $resultAddons;
        $this->smartyVars["displayAddonCycles"] = !$addonWithParentCycles;
        $this->smartyVars["addonGroups"] = $productAddonGroups;
        $this->smartyVars["types"] = getTypes("products");
        $this->smartyVars["packageInfo"] = correctArray(grabFirstResultFromMBAPIArray(array(
            "mbapi" => array(
                "command" => "GetPackages",
                "params" => array(
                    "packageID" => $subscriptionId,
                ),
            ),
        )));
		$this->smartyVars["packageInfo"]["canTransfer"] = $subscription->canTransfer();
		$this->smartyVars["packageInfo"]["canUpgrade"] = $subscription->canUpgrade();
		$this->smartyVars["packageInfo"]["canDowngrade"] = $subscription->canDowngrade();
        $this->smartyVars["buttons"] = array(
            array('id' => 'billing-id-add', 'class' => 'action', 'name' => 'add', 'value' => "TRANS_ADD", 'type' => 'submit', 'label' => 'TRANS_ADD'),
        );
        $this->mergeFormState();

        $this->addTemplate("packages/AddPackageAddon");
	}
}
