<?php
require_once ("lib-action/action.php");

class ChangePackageClient extends MB_Action
{
	public function execute()
	{
		parent::execute();

        try {

            if (is_null($packageID = bgpc('packageID')) || !($packageInfo = oneByID("packages", $packageID))) {
                billing_put_error("Invalid package ID given.");
                redirect("packages.php", getActionID("ShowPackages"), array());
            }
            $clientID = bgpc('clientID');
            if (!getCurrentSession()->checkVendor(Factory()->Customer($clientID))) {
                throw new ProductException(ProductException::E_FORBIDDEN_OPERATION, "Access denied: Client with ID '{$clientID}'");
            }
            $clientInfo = oneByID("clients", $clientID, array("mergePrimaryContact" => 1,));
            $this->smartyVars["toClient"] = $clientInfo;
            $this->smartyVars["package"] = $packageInfo;
            $this->smartyVars["fromClient"] = oneByID("clients", $packageInfo["clientID"], array(
                "mergePrimaryContact" => 1,
            ));
            $this->smartyVars["selectableBillingAccounts"] = array('') + optionsFor("clientBillingAccounts", array("clientID"=>$clientID));
            $this->smartyVars["packageTypes"] = getTypes("products");
			$subscription = Factory()->Subscription($packageID);
			$this->smartyVars["package"]["canTransfer"] = $subscription->canTransfer();
			$this->smartyVars["package"]["canUpgrade"] = $subscription->canUpgrade();
			$this->smartyVars["package"]["canDowngrade"] = $subscription->canDowngrade();

			Service()->SubscriptionManager->checkTransferSubscription($subscription, $subscription->client_id, $clientID);
            $this->smartyVars["buttons"] = array(
                array('id' => 'billing-id-process-transfer', 'class' => 'action', 'name' => 'process-transfer', 'value' => "TRANS_CONFIRM_PACKAGE_TRANSFER", 'type' => 'submit', 'label' => 'TRANS_CONFIRM_PACKAGE_TRANSFER'),
                array('id' => 'billing-id-cancel', 'name' => 'cancel', 'value' => "TRANS_CANCEL", 'type' => 'button', 'label' => 'TRANS_CANCEL', 'onclick' => 'javascript: history.go(-1);'),
            );
            $this->addTemplate("packages/ChangePackageClient", $this->smartyVars);

        } catch(ProductException $e) {
            billing_put_error($e->getMessage());
            redirect("packages.php", getActionID("ShowPackageDetails"), array("packageID" => $packageID));
        }
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
