<?php
require_once("lib-action/action.php");

use Billing\Object\Plan;

class EditPackage extends MB_Action
{
	public $params = array("packageID");

	public function execute()
	{
		global $CONF;
		parent::execute();

		// fake exception handling for php4
		do {
			if($this->err($packageID = getParam("packageID"), "No package ID given.")) break;

			$packageQuery = new MBQuery(
				"GetPackages", "",
				array(
					"packageID" => $packageID,
					"getPackageAttributeData" => 1,
					"getPackageAddonData" => 1,
					"getDomainData" => 1,
				)
			);
			if(!$this->addErrorsFromResult($packageResult =& $packageQuery->getResult())) break;
			$packageTable =& $packageResult->getTableFrom(
				array("mbapi", "results", "packages", "package", 0),
				"packageID", "packageName"
			);
			dbg($packageResult);
			$package = $packageTable->getOne();
            $this->_subscription = Factory()->Subscription($packageID);
            if (Plan::TYPE_DOMAIN == $this->_subscription->package_type) {
                foreach($this->_subscription->DomainList() as $domain) {
                    $package["packageDomainID"][] = "{$domain->domain_sld}.{$domain->domain_tld}";
                }
            } else {
                $package["packageDomainID"] = $this->_subscription->primary_domain_id;
            }

			$packageAttributeTable =& $packageResult->getTableFrom(
				array("mbapi", "results", "packages", "package", "packageAttributes", "packageAttribute"),
				"packageAttributeName", "packageAttributeValue"
			);
			$packageAttributes = $packageAttributeTable->getTable();

			$packageAddons = $packageResult->getEssentialFrom(
				array("mbapi", "results", "packages", "package", "packageAddons", "packageAddon")
			);
			dbg($packageAddons);

			$package["canTransfer"] = $this->_subscription->canTransfer();
			$package["canUpgrade"] = $this->_subscription->canUpgrade();
			$package["canDowngrade"] = $this->_subscription->canDowngrade();

			$this->smartyVars["client"] = correctArray(grabFirstResultFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetClients",
					"params" => array(
						"clientID" => $package["clientID"],
						"mergePrimaryContact" => 1,
					),
				),
			)));

			/** AUTO GENERATE FORM DATA BEGIN **/
			$this->smartyVars["editForm"] = new MBAutoFormFields();
            $packageDomainIDFieldType = (Plan::TYPE_DOMAIN == $this->_subscription->package_type) ? "text" : "select" ;
			$formFields = array(

				"TRANS_PACKAGE_DETAILS" => array("titleRow","TRANS_PACKAGE_DETAILS"),
				"clientContactID" => array("select", "TRANS_BILL_TO_SPECIFIC_CONTACT"),
				"packageType" => array("text", "TRANS_PRODUCT_TYPE"),
				"packageName" => array("text", "TRANS_PRODUCT_NAME"),
				"packageQuantity" => array("text", "TRANS_PRODUCT_QUANTITY"),
				"packageDisableAccountStacking" => array("select", "TRANS_ACCOUNT_STACKING"),
				"packageDomainID" => array($packageDomainIDFieldType, "TRANS_ASSOCIATED_DOMAINS"),
				"packageComments" => array("textarea", "TRANS_PACKAGE_COMMENTS"),

				"TRANS_PACKAGE_CANCELLATION_OPTIONS" => array("titleRow","TRANS_PACKAGE_CANCELLATION_OPTIONS"),
				"packageCancellationType" => array("select", "TRANS_CANCELLATION_RULES"),

				"TRANS_USAGE_RENEWAL_DATES" => array("titleRow","TRANS_USAGE_RENEWAL_DATES"),
				"usageCycleID" => array("select", "TRANS_USAGE_CYCLE"),
				"usageCalculateOnly" => array("checkbox", "TRANS_USAGE_CALCULATE_ONLY"),
				"usageDateLastRenewal" => array("dateselect", "TRANS_USAGE_DATE_LAST_RENEWAL"),
				"usageDateNextRenewal" => array("dateselect", "TRANS_USAGE_DATE_NEXT_RENEWAL"),

				"TRANS_GENERAL_RENEWAL_DATES" => array("titleRow","TRANS_GENERAL_RENEWAL_DATES"),
				"cycleID" => array("select", "TRANS_BILLING_CYCLE"),
				"packageDateLastRenewal" => array("dateselect", "TRANS_PACKAGE_DATE_LAST_RENEWAL"),
				"packageDateNextRenewal" => array("dateselect", "TRANS_PACKAGE_DATE_NEXT_RENEWAL"),
			);

            if (!Service()->PackageAddonManager->selWithParentCycles()) {
                $formFields["updateAddonRenewalDate"] = array("checkbox", "TRANS_UPDATE_ADDON_RENEWAL_DATE");
                $package["updateAddonRenewalDate"] = 1;
            }

            $formFields = array_merge($formFields, array(
                "packageDateStopRenewal" => array("dateselect", "TRANS_PACKAGE_DATE_STOP_RENEWAL"),
                "packageManualRenewal" => array("checkbox", "TRANS_ALLOW_MANUAL_RENEWAL"),
                "packageRenewalNotification" => array("text", "TRANS_RENEWAL_NOTIFICATION"),

                "TRANS_SYSTEM_RULES_AND_PERMISSIONS" => array("titleRow","TRANS_SYSTEM_RULES_AND_PERMISSIONS"),
                "packageClientPermissions" => array("select", "TRANS_PACKAGE_CLIENT_PERMISSIONS"),
            ));

            $this->smartyVars["editForm"]->addFields($formFields);
			$this->smartyVars["editForm"]->setFieldData("usageDateNextRenewal", "params", array("startYear" => date("Y")));
			$this->smartyVars["editForm"]->setFieldData("packageDateNextRenewal", "params", array("startYear" => date("Y")));
			$this->smartyVars["editForm"]->setFieldData("packageDateStopRenewal", "params", array("startYear" => date("Y")));

			$this->smartyVars["editForm"]->addHiddenFields(array(
				"packageID" => $package["packageID"],
			));

            if (Plan::TYPE_DOMAIN != $this->_subscription->package_type) {
                $this->smartyVars["editForm"]->setFieldData("packageDomainID", "addDomainField", 1);
                $this->smartyVars["editForm"]->setFieldData("packageDomainID", "allowBlankChoice", 1);
            }

			$this->smartyVars["editForm"]->setFieldData("packageDisableAccountStacking", "helpTitle", blmsg('TRANS_WARNING'));
			$this->smartyVars["editForm"]->setFieldData("packageDisableAccountStacking", "helpText", blmsg('TRANS_SETTING_STACKING_WARNING'));

			$this->smartyVars["editForm"]->setFieldData("usageCycleID", "allowBlankChoice", 1);

			$packageTypes = blmsg_array(Plan::types($this->_isReseller()));
			$package["packageType"] = $packageTypes[$package["packageType"]];
			$this->smartyVars["editForm"]->addDataFromSource($package);

			$this->smartyVars["editForm"]->setDateOptional("packageDateStopRenewal");
			$this->smartyVars["editForm"]->setDateOptional("usageDateNextRenewal");

			$this->smartyVars["editForm"]->setShowOnly(array(
				"packageType",
				"packageDateLastRenewal",
				"usageDateLastRenewal",
			));

            if (Plan::TYPE_DOMAIN == $this->_subscription->package_type) {
                $this->smartyVars["editForm"]->setShowOnly(array("packageDomainID"));
            }

			$contacts = arrayFor("contacts", array(
				"clientID" => $package["clientID"],
			));

			$contactChoices = array("0" => "");
			foreach ($contacts as $cc){
				$contactChoices[$cc["clientContactID"]] = billing_get_full_name($cc["contactFirstName"], $cc["contactLastName"]);
			}

			$this->smartyVars["editForm"]->setSelectable("packageDisableAccountStacking", array(
				'1' => blmsg('TRANS_STACKING_PER_SERVER'),
				'0' => blmsg('TRANS_DISABLE_STACKING'),
			));
			$this->smartyVars["editForm"]->setSelectable("clientContactID", $contactChoices);

            if (Plan::TYPE_DOMAIN != $this->_subscription->package_type) {
			    $this->smartyVars["editForm"]->setSelectable("packageDomainID", $this->_getClientDomains());
            }
			$this->smartyVars["editForm"]->setSelectable("packageClientPermissions", getTypes("packagemanagement"));
			$this->smartyVars["editForm"]->setSelectable("cycleID", getCyclesArray());
			$this->smartyVars["editForm"]->setSelectable("usageCycleID", getCyclesArray());
			$this->smartyVars["editForm"]->setSelectable("packageCancellationType", getTypes("cancellation"));

			$this->smartyVars["editForm"] = $this->smartyVars["editForm"]->getData();

			/** AUTO GENERATED EDIT FORM DATA END **/

			$tmp = explode(";", $CONF["user"]["locale"]->localeArray["locale_pattern_currency"]);
			$tmp[0] = preg_replace("/^¤#0.00/", "¤#,##0.00", $tmp[0]);
			$tmp[1] = preg_replace("/^¤#0.00/", "¤#,##0.00", $tmp[1]);
			if (!$tmp[1])$tmp[1] = $tmp[0];
			$CONF["user"]["locale"]->localeArray["locale_pattern_currency"] = $tmp[0].";".$tmp[1];
 			$this->smartyVars["localeArray"] =  $CONF["user"]["locale"]->localeArray;

			$this->smartyVars['package'] = $package;
			$this->smartyVars['types'] = getTypes("products");
			$this->smartyVars['packageAttributes'] = $packageAttributes;

			$defaults = $package;

			$this->mergeFormState($defaults);
			$this->addTemplate("packages/EditPackage");
		} while (false);

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

    private function _getClientDomains()
    {
        $clientDomains = array();
        $subscriptions = Factory()->SubscriptionList(array('client_id' => $this->_subscription->client_id));
        foreach ($subscriptions as $subscription) {
            foreach ($subscription->DomainList() as $domain) {
                $clientDomains[$domain->domain_id] = $domain->name;
            }
        }
        return $clientDomains;
    }

    private $_subscription;
}
