<?php

require_once("lib-action/action.php");
require_once("lib-mbapi/mbapi.php");
require_once("lib-billing/include/config/internal/cycles.php");

use Billing\Object\PlanVariantPrice;

class EditPackageAddon extends MB_Action
{
	public $params = array("packageID", "packageAddonID");

	public function execute()
	{
		parent::execute();

        $subscriptionId = bgpc("packageID");
        $addonId = bgpc("packageAddonID");

        $subscription = Factory()->Subscription($subscriptionId);
        $addonGroupId = Factory()->PackageAddon($addonId)->AddonProduct->product_addon_group_id;
        $addonWithParentCycles = Service()->PackageAddonManager->selWithParentCycles();

        $rawPackageAddons = correctArray(grabResultsFromMBAPIArray(array(
            "mbapi" => array(
                "command" => "GetPackageAddons",
                "params" => array(
                    "packageID" => $subscriptionId,
                    "packageAddonActive" => 1,
                ),
            ),
        )));

        $packageAddons = array();
        foreach ($rawPackageAddons as $addon){
            $packageAddons[$addon["packageAddonProductID"]] = $addon;
        }

        $productsInfo = correctArray(grabSimpleMBAPIResults(dispatchMBAPI("
			<mbapi>
				<command>GetProducts</command>
				<params>
					<parentProductID>{$subscription->product_id}</parentProductID>
					<getProductAttributeData>1</getProductAttributeData>
					<getCouponData>1</getCouponData>
					<getAllOrderFormItemData>1</getAllOrderFormItemData>
					<getOrderFormItemData>1</getOrderFormItemData>
					<getOrderFormData>1</getOrderFormData>
					<getProductVariantData>1</getProductVariantData>
					<getProductVariantPriceData>1</getProductVariantPriceData>
					<getTLDData>1</getTLDData>
					<getCycleData>1</getCycleData>
				</params>
			</mbapi>", 1)));

        $resultAddons = array();
		foreach ($productsInfo as $addon){
            if ($addon["productAddonGroupID"] != $addonGroupId || !isset($addon["productVariantPrices"])) {
                // addon is from different addon group
                // OR prices for this addon are not set
                continue;
            }

            foreach($addon["productVariantPrices"] as $price) {
                if ($price["currencyID"] != $subscription->Store->currency_id
                    || ($addonWithParentCycles && $price["cycleID"] != $subscription->cycle_id)
                    || (!$addonWithParentCycles && isCycleLessThan($subscription->cycle_id, $price["cycleID"]))) {
                    // addon has wrong currency
                    // OR addon has different cycle and "Purchase addon with non parent cycles" feature is not available
                    // OR addon has bigger cycle and "Purchase addon with non parent cycles" feature is available
                    continue;
                }

                $id = $addon["productID"].'-'.$price["cycleID"];
                if (!isset($resultAddons[$id])) {
                    $resultAddons[$id] = array(
                        "productName" => $addon["productName"],
                        "productActive" => $addon["productActive"],
                        "cycleID" => $price["cycleID"],
                        "packageAddonDateNextRenewal" => getNewAddonNextRenewalDate($subscription, $price["cycleID"]),
                        "generalPrice" => 0,
                        "setupPrice" => 0,
                        "couponPrice" => 0,
                        "couponCycles" => 0,
                        "isSelected" => 0,
                    );

                    if (isset($packageAddons[$addon["productID"]]) && $packageAddons[$addon["productID"]]["cycleID"] == $price["cycleID"]) {
                        // addon with this cycle is already purchased - take price from it
                        $purchasedAddon = $packageAddons[$addon["productID"]];
                        $resultAddons[$id]["generalPrice"] = $purchasedAddon["packageAddonPrice"];
                        $resultAddons[$id]["setupPrice"] = $purchasedAddon["packageAddonSetup"];
                        $resultAddons[$id]["couponPrice"] = $purchasedAddon["couponPrice"];
                        $resultAddons[$id]["couponCycles"] = $purchasedAddon["couponNumCycles"];
                        $resultAddons[$id]["packageAddonID"] = $purchasedAddon["packageAddonID"];
                        $resultAddons[$id]["packageAddonDateLastRenewal"] = $purchasedAddon["packageAddonDateLastRenewal"];
                        $resultAddons[$id]["packageAddonDateNextRenewal"] = $purchasedAddon["packageAddonDateNextRenewal"];
                        $resultAddons[$id]["isSelected"] = 1;
                    }

                    $this->formDefaults["{$id}-nextrenewal"] = $resultAddons[$id]["packageAddonDateNextRenewal"];
                    $this->setupDateSelector("{$id}-nextrenewal");
                }

                if ($resultAddons[$id]["isSelected"]) {
                    // addon is purchased - no need to collect price data from plan
                    continue;
                }

                switch ($price["productVariantPriceType"]) {
                    case PlanVariantPrice::TYPE_GENERAL:
                        $resultAddons[$id]["generalPrice"] = $price["productVariantPriceAmount"];
                        $resultAddons[$id]["couponPrice"] = $price["productVariantPriceAmount"];
                        break;
                    case PlanVariantPrice::TYPE_SETUP:
                        $resultAddons[$id]["setupPrice"] = $price["productVariantPriceAmount"];
                        break;
                }
            }
		}

		$this->smartyVars["addons"] = $resultAddons;
        $this->smartyVars["displayAddonCycles"] = !$addonWithParentCycles;
        $this->smartyVars["addonGroup"] = correctArray(grabFirstResultFromMBAPIArray(array(
            "mbapi" => array(
                "command" => "GetProductAddonGroups",
                "params" => array(
                    "productAddonGroupID" => $addonGroupId,
                ),
            ),
        )));
        $this->smartyVars["types"] = getTypes("products");
        $this->smartyVars["packageInfo"] = correctArray(grabFirstResultFromMBAPIArray(array(
            "mbapi" => array(
                "command" => "GetPackages",
                "params" => array(
                    "packageID" => $subscriptionId,
                ),
            ),
        )));
        $this->smartyVars["packageInfo"]["canTransfer"] = $subscription->canTransfer();
        $this->smartyVars["packageInfo"]["canUpgrade"] = $subscription->canUpgrade();
        $this->smartyVars["packageInfo"]["canDowngrade"] = $subscription->canDowngrade();
        $this->mergeFormState();

		$this->addTemplate("packages/EditPackageAddon", $this->smartyVars);
	}
}
