<?php
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2005 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+


if(!$CONF || $_REQUEST["CONF"]) {
	if (!defined("MB_LIBPATH")){
		define("MB_LIBPATH", realpath(dirname(__FILE__)."/../../../.."));
		require_once(MB_LIBPATH."/lib-pkg/parsefunction.php");
	}
	require_once mb_path("/lib-tk/include/common.php");
}
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");
require_once mb_path("/lib-billing/include/config/internal/cycles.php");

use Billing\Object\PlanVariantPrice;

/**
 * This Action object collects information on all packages that match the given parameters, and
 * generates the display of that information.
 *
 * @author  Kris Bailey
 */
class EditPackageAddonResponse extends MB_Action {
	/**
	 * The constructor for a AddPackageAddon action object. This merely calls the parent constructor,
	 * and sets the string representation of this object. This method also sets up $this->params and
	 * $this->sortColumns for use in $this->getMBAPIQuery().
	 */
	function EditPackageAddonResponse() {
		global $CONF;
		parent::MB_Action();
		$this->stringRep = "AddPackageAddon";

		// define all possible params that we will accept
		$this->params = array
		(
			"packageID",
		);
	}

	/**
	 * Execute this action object, and display any output generated to the screen.
	 *
	 * @return  1 if no errors occurred during execution, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

        $packageID = bgpc("packageID");
        $selectedAddons = bgpc("addons", array());
        $productIDs = array_map(function($id) {return reset(explode('-', $id)); }, $selectedAddons);
        $cycleIDs = array_map(function($id) {return end(explode('-', $id)); }, $selectedAddons);
		$packageAddonIDs = bgpc("packageAddonIDs", array());
        $prices = bgpc("prices", array());

		if (!count($packageAddonIDs)){
			dbg("NO PRODUCT IDS GIVEN TO CREATE NEW ADDONS");
		} else {
            if (count($productIDs) != count(array_unique($productIDs))) {
                billing_put_error(blmsg("TRANS_NO_ADDONS_SAME_PLAN"));
                redirect("packages.php", getActionID("ShowPackageAddons"),  array("packageID" => $packageID));
            }

            $products = array_combine($productIDs, $cycleIDs);
			$mainPackageInfo = oneByID("packages", $packageID);
			$productsInfo = correctArray(grabResultsFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetProducts",
					"params" => array(
						"productID" => $productIDs,
						"getProductAttributeData" => 1,
						"getCouponData" => 1,
						"getAllOrderFormItemData" => 1,
						"getOrderFormItemData" => 1,
						"getOrderFormData" => 1,
						"getProductVariantData" => 1,
						"getProductVariantPriceData" => 1,
						"getTLDData" => 1,
						"getCycleData" => 1,
					),
				)
			)));
			
			$oldPackageAddons = correctArray(grabResultsFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetPackageAddons",
					"params" => array(
                        "packageID" => $subscriptionId,
                        "packageAddonActive" => 1,
					),
				),
			)));
			dbg("OLD ADDONS:");
			dbg($oldPackageAddons);
			$packageAddons = array();
			foreach ($oldPackageAddons as $addon){
				$packageAddons[$addon["packageAddonProductID"]] = $addon;
			}


			$addonGroupIDs = array();
			foreach ($productsInfo as $addon){
				$addonGroupIDs[] = $addon["productAddonGroupID"];
			}
			$addonGroupNames = optionsFor("productAddonGroups", array("productAddonGroupID" => $addonGroupIDs));

			$finalAddons = array();
			foreach ($productsInfo as $addon){
				dbg($addon);
                $cycleId = $products[$addon["productID"]];
                $priceId = $addon["productID"] . '-' . $cycleId;

				$vp = array();
				foreach ($addon["productVariantPrices"] as $price){
					if ($price["currencyID"] == $mainPackageInfo["currencyID"] && $price["cycleID"] == $cycleId){
						$vp[$price["productVariantPriceType"]] = $price;
					}
				}
				foreach ($addon["productVariants"] as $variant){
					if ($variant["productVariantID"] == $vp[PlanVariantPrice::TYPE_GENERAL]["productVariantID"]){
						$thisvariant = $variant;
					}
				}

				if (isset($packageAddons[$addon["productID"]]) && $packageAddons[$addon["productID"]]["cycleID"] == $cycleId) {
					$packageAddons[$addon["productID"]]["added"] = 1;
                    $purchasedAddon = $packageAddons[$addon["productID"]];
					dbg("old package addon of ID: {$purchasedAddon["packageAddonID"]} marked as NOT needing to be deleted.");

                    $dateSelector = $this->handleDateSelector("{$priceId}-nextrenewal", $purchasedAddon["packageAddonDateNextRenewal"]);
                    $packageAddonDateNextRenewal = min($dateSelector, $mainPackageInfo["packageDateNextRenewal"]);
                    $finalAddons[] = array(
						"packageAddonID" => $purchasedAddon["packageAddonID"],
						"packageAddonDisplayName" => $addonGroupNames[$addon["productAddonGroupID"]].": ".$addon["productName"],
						"packageAddonOriginalPrice" => BillingLocale::getNumberFromInput($prices[$priceId]["general"]),
						"packageAddonPrice" => BillingLocale::getNumberFromInput($prices[$priceId]["general"]),
						"packageAddonSetup" => BillingLocale::getNumberFromInput($prices[$priceId]["setup"]),
						"packageAddonPriceLock" => 1,
						"packageAddonActive" => 1,
						"packageAddonCycleDiscountGroupID" => $thisvariant["cycleDiscountGroupID"],
						"packageAddonDiscountable" => $addon["productDiscountable"],
						"packageAddonProductID" => $addon["productID"],
                        "cycleID" => $cycleId,
						"taxZoneGroupID" => $thisvariant["taxZoneGroupID"],
						"couponPrice" => BillingLocale::getNumberFromInput($prices[$priceId]["coupon"]),
						"couponNumCycles" => BillingLocale::getNumberFromInput($prices[$priceId]["couponcycles"]),
						"copyLedgerEntries" => 1,
                        "packageAddonDateLastRenewal" => $purchasedAddon["packageAddonDateLastRenewal"],
                        "packageAddonDateNextRenewal" => $packageAddonDateNextRenewal,
					);
				} else if ($packageAddons[$addon["productID"]] && !in_array($packageAddons[$addon["productID"]]["packageAddonID"], $packageAddonIDs)) {
                    // addon of this plan is already purchased but it was not displayed on edit page
                    // (maybe it has wrong cycle or it was detached from plan or sth);
                    // DO NOT add another one because existing addon will not be removed
                    billing_put_error(blmsg("TRANS_NO_ADDONS_SAME_PLAN"));
                    redirect("packages.php", getActionID("ShowPackageAddons"),  array("packageID" => $packageID));
                } else {
                    $dateSelector = $this->handleDateSelector("{$priceId}-nextrenewal", $mainPackageInfo["packageDateNextRenewal"]);
                    $packageAddonDateNextRenewal = min($dateSelector, $mainPackageInfo["packageDateNextRenewal"]);
                    $finalAddons[] = array(
						"packageAddonDisplayName" => $addonGroupNames[$addon["productAddonGroupID"]].": ".$addon["productName"],
						"packageAddonOriginalPrice" => BillingLocale::getNumberFromInput($prices[$priceId]["general"]),
						"packageAddonPrice" => BillingLocale::getNumberFromInput($prices[$priceId]["general"]),
						"packageAddonSetup" => BillingLocale::getNumberFromInput($prices[$priceId]["setup"]),
						"packageAddonPriceLock" => 1,
						"packageAddonActive" => 1,
						"packageAddonCycleDiscountGroupID" => $thisvariant["cycleDiscountGroupID"],
						"packageAddonDiscountable" => $addon["productDiscountable"],
						"packageAddonProductID" => $addon["productID"],
                        "cycleID" => $cycleId,
						"taxZoneGroupID" => $thisvariant["taxZoneGroupID"],
						"couponPrice" => BillingLocale::getNumberFromInput($prices[$priceId]["coupon"]),
						"couponNumCycles" => BillingLocale::getNumberFromInput($prices[$priceId]["couponcycles"]),
						"copyLedgerEntries" => 1,
                        "packageAddonDateLastRenewal" => getNewAddonLastRenewalDate($packageAddonDateNextRenewal, $cycleId, $mainPackageInfo["packageDateLastRenewal"]),
                        "packageAddonDateNextRenewal" => $packageAddonDateNextRenewal,
					);
				
				}
			}

			if (count($finalAddons)){
				$commandArray = array(
					"mbapi" => array(
						"command" => "SetPackages",
						"subCommand" => "update",
						"params" => array(
							"packageID" => $packageID,
							"packageAddons" => array(
                                "packageAddon" => $finalAddons,
							),
						),
					),
				);

				dispatchMBAPI(toXML($commandArray), 1);
			}

			// lets do the deactivating we need to do now:
			$deactivateAddonIDs = array();
			foreach ($packageAddons as $addon){
                if (!in_array($addon["packageAddonID"], $packageAddonIDs)) {
                    // this addon was not displayed on edit page
                    // (maybe it has wrong cycle or it was detached from plan or sth); DO NOT remove it
                    continue;
                }
				dbg("checking addon: {$addon["packageAddonID"]} to see if it needs to be deactivated:");
				if (!$addon["added"]){
					$commandArray = array(
						"mbapi" => array(
							"command" => "SetPackageAddonsGeneric",
							"subCommand" => "update",
							"params" => array(
								"packageAddonID" => $addon["packageAddonID"],
								"packageAddonActive" => 0,
							),
						),
					);

					dispatchMBAPI(toXML($commandArray), 1);
				}
			}		
		}

        redirect("packages.php", getActionID("ShowPackageAddons"),  array("packageID" => $packageID));
	}
} // EditPackageAddonResponse
?>
