<?php

require_once("lib-action/action.php");

use Billing\Object\Subscription;

class ShowPackages extends MB_Action
{
	public function execute()
	{
		parent::execute();

		if (1 == bgpc("sync")) {
			$this->processSync();
		}

		$page  = UIFactory()->Page();
		if ( !is_null( $client_id = bgpc("clientID") ) ) {
			$client = Factory()->Customer($client_id);
			$page->parent_title = billing_safetyhtml($client->PrimaryContact->contact_fullname);
			$page->parent_url = action_url("clients", "ShowClientDetails", array("clientID"	=> $client_id));
		} elseif ( !is_null( $product_id = bgpc("productID") ) ) {
			$product = Factory()->Plan( $product_id );
			$page->parent_title = $product->product_name;
			$page->parent_url = admin_action_url("products", "plan-overview", "index", array("id" => $product_id));
		} else {
			$page->parent_url = action_url("dashboard", "ShowDashboard");
		}

		if (bgpc('search')) {
			$page->title = blmsg("TRANS_SHOW_PACKAGES_SEARCH", array('SEARCH' => billing_safetyhtml(bgpc('search'))));
		} else {
			$page->title = blmsg("TRANS_SHOW_PACKAGES");
		}

		require_once("UI/List/UIPackageList.php");
		$page->addListPanel(new UIPackageList( $client_id, $product_id ));

		$this->smartyVars["page"] = $page;
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}

	private function processSync()
	{
		$subscriptionIDs = bgpc("id", array());
		if (empty($subscriptionIDs)) {
			return;
		}
		$errors = array();
		$leftSyncingSubscriptions = false;
		foreach ($subscriptionIDs as $subscriptionID) {
			try {
				$subscription = Factory()->Subscription($subscriptionID);
				if ($subscription->sync_tasks & Subscription::SYNC_CHECK_UPDATE_STATUS) {
					// check previous update operation status only
				} else {
					$subscription->sync_tasks |= Subscription::SYNC_NEED_REPAIR;
				}
				Service()->SubscriptionManager->sync($subscription);
				if (!is_null($subscription->sync_error)) {
					$errors[] = blmsg("TRANS_SYNC_SUBSCRIPTION_ERROR_MSG", array("SUBSCRIPTION_ID" => $subscription->id, "ERROR" => $subscription->sync_error));
				} else if (!$subscription->isSynced() && $subscription->isSyncable()) {
					$leftSyncingSubscriptions = true;
				}
			} catch (ProductException $e) {
				$errors[] = blmsg("TRANS_SYNC_SUBSCRIPTION_ERROR_MSG", array("SUBSCRIPTION_ID" => $subscriptionID, "ERROR" => $e->getMessage()));
			}
		}
		if (empty($errors)) {
			getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_SYNC_SUBSCRIPTIONS_SUCCESS_MSG");
			if ($leftSyncingSubscriptions) {
				billing_put_warning(blmsg("TRANS_SYNC_SUBSCRIPTIONS_PENDING_MSG"));
			}
		} else {
			getCurrentSession()->Storage->errorMessages[] = blmsg(
				"TRANS_SYNC_SUBSCRIPTIONS_FAILURE_MSG",
				array("ERROR" => billing_safetyhtml(implode("\n", $errors)))
			);
		}
	}
}
