<?php

require_once("admin/include/lib-action/packages/UpgradeDowngradeAction.php");

class UpgradePackage extends UpgradeDowngradeAction
{
	public function __construct()
	{
		$_REQUEST["packageID"] = $this->_getPackage()->id;
		parent::__construct();
		$this->params = array("packageID");
	}

	public function execute()
	{
		parent::execute();

		$this->smartyVars["feePriceType"] = getAPITypeID("price", "upgradefee");
        $addonWithParentCycles = Service()->PackageAddonManager->selWithParentCycles();

		$packageInfo = grabFirstResultFromMBAPIArray(array(
			"mbapi" => array(
				"command" => "GetPackages",
				"params" => array(
					"packageID" => (int)$_REQUEST["packageID"],
					"getPackageAttributeData" => 1,
					"getPackageAddonData" => 1,
				),
			)
		));

		$selectedAddonData = array();
        if (isset($packageInfo['packageAddons'])) {
    		foreach ($packageInfo["packageAddons"][0]["packageAddon"] as $addon) {
                $id = $addon["packageAddonProductID"].'-'.$addon["cycleID"];
    			$selectedAddonData[$id] = $addon;
    		}
        }


		$this->smartyVars["selectedAddons"] = $selectedAddonData;
		if (!$packageInfo["packageID"]){
			$this->addError("TRANS_ACTIONERROR", "Invalid package id.");
		} else {

			$this->smartyVars['packageInfo'] = $packageInfo;
			$this->smartyVars['packageStatuses'] = getStatuses("package");
			$this->smartyVars['types'] = getTypes("products");

			$this->smartyVars["cango"] = $this->canUpgradeDowngrade($packageInfo);

			$this->smartyVars["upgradeProducts"] = arrayFor(
				"productRelations",
				array(
					"subjectProductID" => $packageInfo["productID"],
					"productRelationType" => getAPITypeID("productrelations", "upgrade")
				)
			);

			$productIDs = array($packageInfo["productID"]);
			foreach ((array)$this->smartyVars["upgradeProducts"] as $relation){
				$productIDs[] = $relation["relatedProductID"];
			}

			dbg($productIDs);

			// at this point we have our package info, and we have our product ids.
			// we need to get all of our product info for each of those products
			// AND we need to get the available addons for those products.  once we
			// do that, we need to filter those addons so that they don't appear
			// if they don't have prices in the currency and cycle we are in.
			// this is complicated schtuff, so edit at your own peril...

			// first, get all of the products we need: (products has a datasource so we can use arrayFor)
			$rawProductsInfo = arrayFor("products", array(
				"productID" => $productIDs,
				"getProductVariantData" => 1,
				"getProductVariantPriceData" => 1,
				"getCycleData" => 1));

			dbg($rawProductsInfo);


			foreach ((array)$rawProductsInfo as $k=>$product){

				$rawProductsInfo[$k]["addonInfo"] = arrayFor("products", array(
					"parentProductID" => $product["productID"],
					"getProductVariantData" => 1,
					"getProductVariantPriceData" => 1,
					"getCycleData" => 1,
                    "getAddons" => 1
                ));

				// get the addon groups for these addons AND this product
				$groups = correctArray(grabResultsFromMBAPIArray(array(
					"mbapi" => array(
						"command" => "GetProductAddonGroups",
						"params" => array(
							"getGroupsByProductID" => $product["productID"],
						)
					)
				)));

				$rawProductsInfo[$k]["addonGroups"] = array();
				foreach ((array)$groups as $group){
					$rawProductsInfo[$k]["addonGroups"][$group["productAddonGroupID"]] = $group;
				}
			}


			$productsInfo = array(); // lets construct the product info, one row for EACH variant
			foreach ((array)$rawProductsInfo as $k=>$product){
				dbg("adding in for product: ".$product["productName"]);
				foreach ((array)$rawProductsInfo[$k]["productVariants"] as $variant){
					if ( 0 == $variant["productVariantActive"] ) {
						continue;
					}
					if ( getAPITypeID("products", "domain") == $packageInfo["packageType"] &&
						$packageInfo["productVariantID"] != $variant["productVariantID"]
					) {
						continue;
					}
					$newProduct = $product;
					$newProduct["productVariantID"] = $variant["productVariantID"];
					if ( count($rawProductsInfo[$k]["productVariants"]) > 1 ) {
						$newProduct["productVariantName"] = $variant["productVariantDisplayName"];
					}

					$productsInfo[] = $newProduct;
				}
			}

            $this->smartyVars["packageInfo"]["addonsAbsentInPlan"] = $this->_getAddonsAbsentInPlan($packageInfo, $productsInfo, $selectedAddonData);

			// this code loops through all of our products, AND their addons, and eliminates all prices
			// that don't match our cycle and currency.
			foreach ((array)$productsInfo as $k=>$product){
				$prices = array();
                $activeCycles = Factory()->Plan($product["productID"])->cycleIDs;
		 		if (isset($productsInfo[$k]["productVariantPrices"]) && in_array($packageInfo["cycleID"], $activeCycles)){
					foreach ($productsInfo[$k]["productVariantPrices"] as $price){
						dbg("checking price: ");
						dbg($price);
						dbg("against: ".$productsInfo[$k]["productVariantID"]);
						if ($price["productVariantID"]==$productsInfo[$k]["productVariantID"] && $price["currencyID"]==$packageInfo["currencyID"] && $price["cycleID"]==$packageInfo["cycleID"]){
							$prices[$price["productVariantPriceType"]] = $price;
							dbg("got in!");
						}
					}
				} else {
					dbg("variant price not set");
				}
				dbg("the prices we got for this prod are:");
				dbg($prices);
				$productsInfo[$k]["prices"] = $prices;
				if (count($productsInfo[$k]["prices"])){
					unset($productsInfo[$k]["productVariantPrices"]);
				} else {
					dbg("just unset for: ".$productsInfo[$k]["productName"]);
			 		unset($productsInfo[$k]);
				}

				if (is_array($productsInfo[$k]["addonInfo"])) {
					foreach ($productsInfo[$k]["addonInfo"] as $addon) {
                        $addonInfo = array();
                        $activeCycles = Factory()->Plan($addon["productID"])->cycleIDs;

                        if (isset($addon["productVariantPrices"])) {
                            foreach ($addon["productVariantPrices"] as $price) {
                                if (!in_array($price["cycleID"], $activeCycles) || $price["currencyID"] != $packageInfo["currencyID"]
                                    || ($addonWithParentCycles && $price["cycleID"] != $packageInfo["cycleID"])
                                    || (!$addonWithParentCycles && isCycleLessThan($packageInfo["cycleID"], $price["cycleID"]))) {
                                    // cycle is not active
                                    // OR addon has wrong currency
                                    // OR addon has different cycle and "Purchase addon with non parent cycles" feature is not available
                                    // OR addon has bigger cycle and "Purchase addon with non parent cycles" feature is available
                                    continue;
                                }

                                $id = $addon["productID"].'-'.$price["cycleID"];
                                if (!isset($addonInfo[$id])) {
                                    $addonInfo[$id] = array(
                                        "productID" => $addon["productID"],
                                        "productName" => $addon["productName"],
                                        "productActive" => $addon["productActive"],
                                        "cycleID" => $price["cycleID"],
                                    );
                                }
                                $addonInfo[$id]["prices"][$price["productVariantPriceType"]] = $price;
                            }

                            if (!$addonInfo) {
                                continue;
                            }

                            if (!isset($productsInfo[$k]["addons"][$addon["productAddonGroupID"]])) {
                                $productsInfo[$k]["addons"][$addon["productAddonGroupID"]] = array();
                            }
                            $productsInfo[$k]["addons"][$addon["productAddonGroupID"]] = array_merge($productsInfo[$k]["addons"][$addon["productAddonGroupID"]], $addonInfo);
                        }
					}
                    unset($productsInfo[$k]["addonInfo"]);
				}
			}

			$subscription = Factory()->Subscription($this->smartyVars["packageInfo"]["packageID"]);
			$this->smartyVars["packageInfo"]["canTransfer"] = $subscription->canTransfer();
			$this->smartyVars["packageInfo"]["canUpgrade"] = $subscription->canUpgrade();
			$this->smartyVars["packageInfo"]["canDowngrade"] = $subscription->canDowngrade();
			$this->smartyVars["upgradeProducts"] =& $productsInfo;
            $this->smartyVars["displayAddonCycles"] = !$addonWithParentCycles;

			$this->addTemplate("packages/UpgradePackage", $this->smartyVars);
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
