<?php

use Billing\Object\Subscription;

class Packages_ActionController extends Billing_Zend_Controller_Action_AdminController
{
    public function indexAction()
    {
        $subscriptionId = $this->_request->getParam('id');
        $this->_subscription = Factory()->Subscription($subscriptionId);
        $action = $this->_request->getParam('packAction');
        $this->_backUrl = $this->view->ActionUrl('packages', 'ShowPackageDetails', array("packageID" => $subscriptionId));

        try {
            switch ($action) {
                case 'suspend':
                case 'activate':
                case 'withdrawCancellation':
                case 'cancelUpgrade':
                case 'renew':
                case 'approveProvisioning':
                    call_user_func(array($this, '_'.$action));
                    break;
                case 'cancel':
                case 'cancelImmediately':
                    $this->_cancel('cancelImmediately' == $action);
                    break;
            }
        } catch(ProductException $e) {

        }
    }

    private function _suspend()
    {
        Service()->SubscriptionManager->suspend($this->_subscription, array('reason' => $this->_translator->translate('TRANS_SUBSCRIPTION_SUSPEND_REASON_MANUAL')));
        $this->_redirectWithMessage($this->_backUrl, $this->_translator->translate('TRANS_SUBSCRIPTION_OPERATION_SUSPEND_SUCCESS_MSG'));
    }

    private function _activate()
    {
        Service()->SubscriptionManager->activate($this->_subscription);
        $successMsg = ($this->_subscription->isNotProvisionedType()) ? 'TRANS_SUBSCRIPTION_OPERATION_ACTIVATE_WITHOUT_PROVISIONING_SUCCESS_MSG' :
            'TRANS_SUBSCRIPTION_OPERATION_ACTIVATE_SUCCESS_MSG';
        $this->_redirectWithMessage($this->_backUrl, $this->_translator->translate($successMsg));
    }

    private function _cancel($cancelImmediately)
    {
        $title = $cancelImmediately ? 'TRANS_SUBSCRIPTION_OPERATION_CANCEL_IMMEDIATELY' : 'TRANS_SUBSCRIPTION_OPERATION_CANCEL';
        $this->view->placeholder('pageTitle')->set($this->view->translate($title));

        $cancelForm = new Billing_Zend_Form_Final_PackageActionCancelForm(array('context' => array(
            'cancelLink' => $this->_backUrl,
            'subscription' =>  $this->_subscription,
            'cancelImmediately' => $cancelImmediately
        )));

        if ($this->_request->isPost() && $cancelForm->isValid($this->_request->getPost())) {
            try {
                $cancelForm->processForm();
                $successMsg  = ($cancelImmediately) ? $this->_translator->translate('TRANS_SUBSCRIPTION_OPERATION_CANCEL_IMMEDIATELY_SUCCESS_MSG')
                        : $this->_translator->translate('TRANS_SUBSCRIPTION_OPERATION_CANCEL_SUCCESS_MSG',
                                array("DATE" => billing_get_user_locale()->getLocaleDate($this->_subscription->package_date_next_renewal)));
                $this->_redirectWithMessage($this->_backUrl, $successMsg);
            } catch (ProductException $e) {
                $this->_status->addError($e->getMessage());
            }
        }
        $this->view->form = $cancelForm;
    }

    private function _withdrawCancellation()
    {
        $this->_subscription->package_pending_cancellation = Subscription::PENDING_CANCELLATION_TYPE_NO;
        $statusReason = $this->_translator->translate("TRANS_SUBSCRIPTION_CANCEL_REASON_WITHDRAW");
        $this->_subscription->setStatus($this->_subscription->package_status, $statusReason, $statusReason);
        Service()->SubscriptionManager->update($this->_subscription);
        $this->_redirectWithMessage($this->_backUrl, $this->_translator->translate('TRANS_SUBSCRIPTION_OPERATION_WITHDRAW_CANCELLATION_SUCCESS_MSG',
                array("DATE" => billing_get_user_locale()->getLocaleDate($this->_subscription->package_date_next_renewal))
        ));
    }

    private function _cancelUpgrade()
    {
        Service()->SubscriptionManager->cancelUpgrade($this->_subscription);
        $this->_redirectWithMessage($this->_backUrl, $this->_translator->translate('TRANS_SUBSCRIPTION_OPERATION_CANCEL_UPGRADE_DOWNGRADE_SUCCESS_MSG'));
    }

    private function _renew()
    {
        $this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_SUBSCRIPTION_OPERATION_RENEW'));

        $renewForm = new Billing_Zend_Form_Final_PackageActionRenewForm(array('context' => array(
            'cancelLink' => $this->_backUrl,
            'subscription' =>  $this->_subscription
        )));

        if ($this->_request->isPost() && $renewForm->isValid($this->_request->getPost())) {
            try {
                $renewForm->processForm();
                $this->_redirectWithMessage($this->_backUrl, $this->_translator->translate('TRANS_SUBSCRIPTION_OPERATION_RENEW_SUCCESS_MSG'));
            } catch (ProductException $e) {
                $this->_status->addError($e->getMessage());
            }
        }
        $this->view->form = $renewForm;
    }

    private function _approveProvisioning()
    {
        Service()->SubscriptionManager->approveProvisioning($this->_subscription);
        $this->_redirectWithMessage($this->_backUrl, $this->_translator->translate('TRANS_SUBSCRIPTION_OPERATION_APPROVE_PROVISIONING_SUCCESS_MSG'));
    }

    private $_subscription = null;
    private $_backUrl = null;
}