<?php
require_once("lib-action/action.php");

class AddTLDResponse extends MB_Action
{
    public $params = array(
        "tldName",
        "tldExtension",
        "tldCustomExtension",
        "tldCustomExtensionActive",
        "registrarID",
        "tldActive",
        "tldWhoisServer",
        "tldWhoisResp",
        "tldWhoisMatchRule",
        "tldAutoSearch",
        "tldTransferMinimumCycleID",
        "tldMinimumCycleID",
        "tldMaximumCycleID",
        "tldRenewType",
        "tldIsTransferable",
        "tldTransferType",
        "tldIDNSupport",
        "tldIDNType",
        "tldRemoteLockEnabled",
        "tldNotSupported"
    );

	public function execute()
    {
		parent::execute();
        if ('' == bgpc("tldWhoisResp")) {
            $_REQUEST["tldWhoisMatchRule"] = '';
        }
		$this->setReferralActionID(getActionID("AddTLD"));
		$this->setReferralActionFile("products.php");
		loadConfigs("domain");
		$tldNotSupported = 0;
		$_REQUEST["tldNotSupported"] = 0;
        if (1 == bgpc("tldActive") && 0 == bgpc("registrarID")) {
            getCurrentSession()->Storage->errorMessages[] = blmsg("TRANS_TLD_ACTIVE_PARAM_ERROR_MSG");
            $redirectParams = array();
            foreach($this->params as $param) {
                $redirectParams[$param] = $_REQUEST[$param];
            }

            redirect("products.php", "AddTLD", $redirectParams);
        }
        $customTLDExtension = ('1' == bgpc("tldCustomExtensionActive"));
		$success = $this->validateMultiInputs(
			array(
                "tldName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
                "tldExtension" => !$customTLDExtension ? INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED : '',
                "tldCustomExtensionActive" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
                "tldCustomExtension" => $customTLDExtension ? INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED : '',
                "registrarID" => INPUT_RESTRICTION_UNSIGNED_INT,
                "tldActive" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
                "tldWhoisServer" => INPUT_RESTRICTION_STRING,
                "tldWhoisResp" => INPUT_RESTRICTION_STRING,
                "tldWhoisMatchRule" => INPUT_RESTRICTION_STRING,
                "tldAutoSearch" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
                "tldTransferMinimumCycleID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
                "tldMinimumCycleID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
                "tldMaximumCycleID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
                "tldRenewType" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
                "tldIsTransferable" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
                "tldTransferType" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
                "tldIDNSupport" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
                "tldIDNType" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
                "tldRemoteLockEnabled" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
                "tldNotSupported" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED
			), array(), array(), 0, array("tldNotSupported" => $tldNotSupported)
        );

		if($success) {

            if (1 == bgpc("tldActive") && 0 == bgpc("registrarID")) {
                getCurrentSession()->Storage->errorMessages[] = blmsg("TRANS_TLD_ACTIVE_PARAM_ERROR_MSG");
                $this->validationRedirect(array(),array(),true);
            }
            $tld = Factory()->Tld();
            $tld->vendor_id = getCurrentSession()->getVendorId();
			$tld->registrar_id = (int)bgpc("registrarID");
			$tld->tld_extension = $customTLDExtension ? bgpc("tldCustomExtension") : bgpc("tldExtension");
			$tld->tld_name = bgpc("tldName");
			$tld->tld_whois_server = bgpc("tldWhoisServer");
			$tld->tld_whois_resp = bgpc("tldWhoisResp");
			$tld->tld_whois_match_rule = bgpc("tldWhoisMatchRule");
			$tld->tld_active = (int)bgpc("tldActive");
			$tld->tld_auto_search = (int)bgpc("tldAutoSearch");
			$tld->tld_transfer_minimum_cycle_id =(int) bgpc("tldTransferMinimumCycleID");
			$tld->tld_minimum_cycle_id = (int)bgpc("tldMinimumCycleID");
			$tld->tld_maximum_cycle_id = (int)bgpc("tldMaximumCycleID");
			$tld->tld_renew_type = (int)bgpc("tldRenewType");
			$tld->tld_is_transferable = (int)bgpc("tldIsTransferable");
			$tld->tld_transfer_type = (int)bgpc("tldTransferType");
			$tld->tld_idn_support = (int)bgpc("tldIDNSupport");
			$tld->tld_idn_type = (int)bgpc("tldIDNType");
			$tld->tld_remote_lock_enabled = (int)bgpc("tldRemoteLockEnabled");

            try {
                Service()->TldManager->update($tld);
                $tldID = $tld->id;
            } catch (ProductException $e) {
                billing_put_error($e->getMessage());
                $this->validationRedirect(array(), array(), true);
            }

			// upon new insert, create a default value for the sort order
			updateForKey(":TLDs", $tldID, array(
				"tldDisplayOrder" => $tldID,
			));
			$pd = new DataSource("products");
			$ps =& $pd->getSubject();
			$pt =& $ps->getTable();
			$productOptions = $pt->getOptions();
			if($productOptions) {
				$ptByType =& $pt->getAssociatedTablesBy("productType");
				$productIDs =
					  isset($ptByType[getTypeID("products", blmsg('TRANS_DOMAIN_PRODUCT'))])
					? $ptByType[getTypeID("products", blmsg('TRANS_DOMAIN_PRODUCT'))]->getKeys()
					: array();
				//print_r($productIDs);
				$productVariants = array();
				foreach($productIDs as $productID) {
					if (! getCurrentSession()->isReseller() && ! is_null(Factory()->Plan($productID)->vendor_id)) {
						continue;
					}
					if (getCurrentSession()->getVendorId() != Factory()->Plan($productID)->vendor_id) {
						continue;
					}
					$productVariant = array(
						"productID" => $productID,
						"productVariantActive" => 0,
						"productVariantDisplayName" => $tld->tld_name,
						"tldID" => $tldID,
					);
					$productVariants[] = $productVariant;
				}

				if($productVariants) {
					$setVariantQuery = new MBQuery(
						"SetProducts", "insert",
						array(
							"productVariants" => array(
								"productVariant" => $productVariants,
							),
						)
					);
					if(!$this->addErrorsFromResult($r =& $setVariantQuery->getResult())) {
						$this->addErrorTemplates();
						return $this->hasExecutedSuccessfully();
					}
				}
			}
			billing_put_info(blmsg("TRANS_ADD_TLD_SUCCESS_MSG", array("NAME" => bgpc('tldName'))));
			if(!$this->goBack()) redirect("products.php", getActionID("ShowTLDs"), array());
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}