<?php
require_once ("lib-action/action.php");
require_once ("lib-billing/include/accounting/Accounting.php");
require_once ("lib-billing/include/config/internal/cycles.php");

class EditLedgerGroupEntriesResponse extends MB_Action
{        
	public function execute()
    {
		parent::execute();
        
        $this->setReferralActionID(getActionID("EditLedgerGroupEntries"));
		$this->setReferralActionFile("products.php");
        $this->validateMultiInputs(
			array(
				"ledgerGroupName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
                "productVariantID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED
		));

        $productVariantID = (int)$_REQUEST["productVariantID"];
        if(!$productVariantID) {
            $this->addError('TRANS_ACTIONERROR', "No product variant ID given.");
        } else {
            $plan = Factory()->PlanVariant($productVariantID)->Plan;

            $ledgerGroupID = (int)$_REQUEST["ledgerGroupID"];
            if (!$productVariantID){
                $this->addError('TRANS_ACTIONERROR', "No Product Variant ID given.");
            } else {

                $variantInfo = oneByID("variants", $productVariantID);

                // lets try and get the info for this ledger group, and if we cant we will just make a new one
                $data = dispatchMBAPI("<mbapi><command>GetLedgerGroups</command><params><ledgerGroupID>".$ledgerGroupID."</ledgerGroupID></params></mbapi>", 1);
                if (!isset($data["mbapi"][0]["results"][0]["groups"][0]["group"][0])){
                    dbg("couldnt find group: ".$ledgerGroupID);
                    $ledgerGroupID = 0; // we cant find this group, so lets set our id to 0 to make a new one
                }

                if ($ledgerGroupID==0){ // if 0 then we will create a new one

                    $ledgerGroupName = secureInput(unsecureInput($_REQUEST["ledgerGroupName"]));
                    $query = "<mbapi><command>SetLedgerGroups</command><subCommand>insert</subCommand><params><ledgerGroupName>$ledgerGroupName</ledgerGroupName></params></mbapi>";
                    dbg("creating group:");
                    $result = dispatchMBAPI($query, 1);
                    $ledgerGroupID = $result["mbapi"][0]["header"][0]["lastInsertID"][0];
                    dbg($ledgerGroupID);

                    $ledgerEntries = $this->getRequestLedgerEntryData("ledgerGroup");

                    foreach ($ledgerEntries as $k=>$v){
                        $ledgerAccount = Factory()->LedgerAccount($v["ledgerAccountID"]);
                        if (!$ledgerAccount->hasSameVendor($plan)) {
                            throw new ForbiddenOperationException();
                        }
                        $query = "
                            <mbapi>
                                <command>SetProductLedgerEntries</command>
                                <subCommand>insert</subCommand>
                                <params>
                                    <productVariantID>$productVariantID</productVariantID>
                                    <ledgerGroupID>$ledgerGroupID</ledgerGroupID>
                                    <generalLedgerType>0</generalLedgerType>
                                    <journalType>general</journalType>
                                    <ledgerAccountID>".(int)$v["ledgerAccountID"]."</ledgerAccountID>
                                    <productLedgerDebit>".doubleval($v["generalLedgerDebitAmount"])."</productLedgerDebit>
                                    <productLedgerCredit>".doubleval($v["generalLedgerCreditAmount"])."</productLedgerCredit>
                                    <productLedgerComment>".secureInput(unsecureInput($v["generalLedgerDescription"]))."</productLedgerComment>
                                    <currencyID>".secureInput(unsecureInput($_REQUEST["currencyID"]))."</currencyID>
                                        <currencyIDs>".implode("", $this->_getCurrencies())."</currencyIDs>
                                        <cycleIDs>".implode("", $this->_getCycles())."</cycleIDs>
                                </params>
                            </mbapi>";

                        dbg($query);
                        $leresult = dispatchMBAPI($query, 1);
                        dbg("creating ledger entry:");
                        dbg($leresult);

                    }

                    if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
                        redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
                    } else {
                        redirect("products.php",getActionID("EditLedgerGroups"), array("productID"=>$variantInfo["productID"]));
                    }

                } else {

                    // deal with edit here...
                    $data = collapseArrayElements($data["mbapi"][0]["results"][0]["groups"][0]["group"]);
                    // fix collapsed ledger entries (should never happen since you can't have just one)
                    if (isset($data["ledgerEntries"]["entry"]["productVariantID"])){
                        $data["ledgerEntries"]["entry"] = array($data["ledgerEntries"]["entry"]);
                    }
                    // update our group name here:
                    $updateNameXML = "
                            <mbapi>
                                <command>SetLedgerGroups</command>
                                <subCommand>update</subCommand>
                                <params>
                                    <ledgerGroupID>$ledgerGroupID</ledgerGroupID>
                                    <ledgerGroupName>".secureInput(unsecureInput($_REQUEST["ledgerGroupName"]))."</ledgerGroupName>
                                </params>
                            </mbapi>";
                    $updateGroupResult = dispatchMBAPI($updateNameXML, 1);
                    // if we are successful...
                    if (isset($updateGroupResult["mbapi"][0]["header"][0]["success"][0]) && (int)$updateGroupResult["mbapi"][0]["header"][0]["success"][0]){

                        $productVariantID = $data["ledgerEntries"]["entry"][0]["productVariantID"];

                        // init the keyed existing entries array
                        $existingEntries = array();
                        foreach ($data["ledgerEntries"]["entry"] as $k=>$v){
                            $existingEntries[$v["productLedgerID"]] =& $data["ledgerEntries"]["entry"][$k];
                        }
                        dbg("Keyed Existing Ledger Entries:");
                        dbg($existingEntries);

                        // get new ledger entry data from request
                        $ledgerEntries = $this->getRequestLedgerEntryData("ledgerGroup");
                        if ($this->validateLedgerEntryData($ledgerEntries)){
                            dbg("Ledger entries to set:");
                            dbg($ledgerEntries);


                            //**********************************************
                            //*********** lets get the filter currencies and
                            //*********** cycles of the existing entries
                            //**********************************************
                            // get the first ledger id, we only need one
                            $productLedgerID = $data["ledgerEntries"]["entry"][0]["productLedgerID"];
                            // get that ledger entries data with its filter currencies and cycles
                            $query = "<mbapi><command>GetProductLedgerEntries</command><params><productLedgerID>".$productLedgerID."</productLedgerID><getProductLedgerCurrenyIDs>1</getProductLedgerCurrenyIDs><getProductLedgerCycleIDs>1</getProductLedgerCycleIDs></params></mbapi>";
                            $ledata = dispatchMBAPI($query, 1);
                            // init our arrays to hold the ones we need
                            $oldCycles = array();
                            $oldCurrencies = array();

                            // send debug to debug server
                            dbg("ledger data");
                            dbg($ledata);
                            // if we got results, should always...
                            if (isset($ledata["mbapi"][0]["results"][0]["productLedgerEntries"][0]["productLedgerEntry"][0])){
                                $ledata = $ledata["mbapi"][0]["results"][0]["productLedgerEntries"][0]["productLedgerEntry"][0];
                                $this->smartyVars["currencyID"] = $ledata["currencyID"][0];
                                // get the currencies
                                if (isset($ledata["currencies"][0]["currency"][0]["currencyID"])){
                                    foreach ($ledata["currencies"][0]["currency"] as $curr){
                                        $oldCurrencies[] = $curr["currencyID"][0];
                                    }
                                }
                                // get the cycles
                                if (isset($ledata["cycles"][0]["cycle"][0]["cycleID"])){
                                    foreach ($ledata["cycles"][0]["cycle"] as $cycle){
                                        $oldCycles[] = $cycle["cycleID"][0];
                                    }
                                }
                            }
                            //**********************************************
                            //*********** end getting filter currencies and
                            //*********** cycles
                            //**********************************************

                            // lets get the new currencies and cycles:
                            $filterCurrencies = (array)$_REQUEST["filterCurrencyIDs"]; // use these for new entries
                            $filterCycles = (array)$_REQUEST["filterCycleIDs"]; // use these for new entries
                            // init the remove and add cycles and currencies arrays
                            $removeCycles = array(); // nothing if there aren't differences
                            $removeCurrencies = array(); // "
                            if (count(array_diff($oldCurrencies, $filterCurrencies))>0 || count(array_diff($filterCurrencies, $oldCurrencies))>0){
                                dbg("currencies are different:");
                                dbg($oldCurrencies);
                                dbg($filterCurrencies);
                                $removeCurrencies = array_diff($oldCurrencies, $filterCurrencies); // remove these from each updated entry
                            }
                            if (count(array_diff($oldCycles, $filterCycles))>0 || count(array_diff($filterCycles, $oldCycles))>0){
                                dbg("cycles are different: ");
                                dbg($oldCycles);
                                dbg($filterCycles);
                                $removeCycles = array_diff($oldCycles, $filterCycles);
                            }

                            $addLedgerEntries = array(); // ledger entries we will have to create
                            $updateLedgerEntries = array(); // ledger entries we will have to update
                            $deleteLedgerEntries = array(); // ledger entries we will have to remove

                            // for each of the ledger entries we received, check if its an update
                            // or if its a new one, if its an update, remove the one we are updating from
                            // the existing entries array
                            foreach ($ledgerEntries as $le){
                                if ((int)$existingEntries[$le["productLedgerID"]]["productLedgerID"]){
                                    $updateLedgerEntries[] = $le;
                                    unset($existingEntries[$le["productLedgerID"]]);
                                } else {
                                    $addLedgerEntries[] = $le;
                                }
                            }
                            $deleteLedgerEntries = $existingEntries;

                            //**********************************************
                            //********** lets add the new ones:
                            //**********************************************
                            foreach ($addLedgerEntries as $k=>$v){
                                $ledgerAccount = Factory()->LedgerAccount($v["ledgerAccountID"]);
                                if (!$ledgerAccount->hasSameVendor($plan)) {
                                    throw new ForbiddenOperationException();
                                }
                                $query = "
                                    <mbapi>
                                        <command>SetProductLedgerEntries</command>
                                        <subCommand>insert</subCommand>
                                        <params>
                                            <productVariantID>$productVariantID</productVariantID>
                                            <ledgerGroupID>$ledgerGroupID</ledgerGroupID>
                                            <generalLedgerType>0</generalLedgerType>
                                            <journalType>general</journalType>
                                            <ledgerAccountID>".(int)$v["ledgerAccountID"]."</ledgerAccountID>
                                            <productLedgerDebit>".doubleval($v["generalLedgerDebitAmount"])."</productLedgerDebit>
                                            <productLedgerCredit>".doubleval($v["generalLedgerCreditAmount"])."</productLedgerCredit>
                                            <productLedgerComment>".secureInput(unsecureInput($v["generalLedgerDescription"]))."</productLedgerComment>
                                            <currencyID>".secureInput(unsecureInput($_REQUEST["currencyID"]))."</currencyID>
                                            <currencyIDs>".implode("", $this->_getCurrencies())."</currencyIDs>
                                            <cycleIDs>".implode("", $this->_getCycles())."</cycleIDs>
                                        </params>
                                    </mbapi>";
                                dbg($query);
                                $leresult = dispatchMBAPI($query, 1);
                                dbg("creating ledger entry:");
                                dbg($leresult);

                            }
                            //**********************************************
                            //********** lets update the ones we need to:
                            //**********************************************
                            foreach ($updateLedgerEntries as $k=>$v){
                                $query = "
                                    <mbapi>
                                        <command>SetProductLedgerEntries</command>
                                        <subCommand>update</subCommand>
                                        <params>
                                            <productLedgerID>".(int)$v["productLedgerID"]."</productLedgerID>
                                            <productVariantID>$productVariantID</productVariantID>
                                            <ledgerGroupID>$ledgerGroupID</ledgerGroupID>
                                            <generalLedgerType>0</generalLedgerType>
                                            <journalType>general</journalType>
                                            <ledgerAccountID>".(int)$v["ledgerAccountID"]."</ledgerAccountID>
                                            <productLedgerDebit>".doubleval($v["generalLedgerDebitAmount"])."</productLedgerDebit>
                                            <productLedgerCredit>".doubleval($v["generalLedgerCreditAmount"])."</productLedgerCredit>
                                            <productLedgerComment>".secureInput(unsecureInput($v["generalLedgerDescription"]))."</productLedgerComment>
                                            <currencyID>".secureInput(unsecureInput($_REQUEST["currencyID"]))."</currencyID>
                                            <currencyIDs>".implode("", $this->_getCurrencies())."</currencyIDs>
                                            <cycleIDs>".implode("", $this->_getCycles())."</cycleIDs>
                                                <removeCurrencyIDs>
                                                    ";
                                foreach ($removeCurrencies as $curr){
                                    $query .= "<currencyID>".secureInput(unsecureInput($curr))."</currencyID>";
                                }
                                $query .= "		</removeCurrencyIDs>
                                                <removeCycleIDs>";
                                foreach ($removeCycles as $cycle){
                                    $query .= "<cycleID>".(int)$cycle."</cycleID>";
                                }
                                $query .= "		</removeCycleIDs>
                                        </params>
                                    </mbapi>";
                                dbg($query);
                                $leresult = dispatchMBAPI($query, 1);
                                dbg("updating ledger entry:");
                                dbg($leresult);
                            }

                            //**********************************************
                            //********** lets update the ones we need to:
                            //**********************************************
                            foreach ($deleteLedgerEntries as $k=>$v){
                                $query = "
                                    <mbapi>
                                        <command>SetProductLedgerEntries</command>
                                        <subCommand>delete</subCommand>
                                        <params>
                                            <productLedgerID>".$v["productLedgerID"]."</productLedgerID>
                                        </params>
                                    </mbapi>";
                                dbg($query);
                                $leresult = dispatchMBAPI($query, 1);
                                dbg("deleted ledger entry");
                                dbg($leresult);
                            }

                            if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
                                redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
                            } else {
                                redirect("products.php",getActionID("EditLedgerGroups"), array("productID"=>$variantInfo["productID"]));
                            }
                        } else {
                            $this->addError('TRANS_ACTIONERROR', "Invalid Ledger Entries.");
                        }
                    } else {
                        $this->addError('TRANS_ACTIONERROR', "Unable to set group name.");
                    }

                }
            }
        }
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

    private function _getCurrencies()
    {
        $currencies = array();
        foreach ((array)$_REQUEST["currencies"] as $currencyId => $currencyIsEnabled){
            if ($currencyIsEnabled) {
                $currencies[] = "<currencyID>{$currencyId}</currencyID>";
            }
        }

        return $currencies;
    }

    private function _getCycles()
    {
        $cycles = array();
        foreach ((array)$_REQUEST["cycles"] as $cycleId => $cycleIsEnabled){
            if ($cycleIsEnabled) {
                $cycles[] = "<cycleID>{$cycleId}</cycleID>";
            }
        }

        return $cycles;
    }
}