<?php

require_once mb_path("/admin/include/lib-action/products/ShowProductDetailsAction.php");
require_once mb_path("/lib-billing/include/config/internal/emailactions.php");
require_once mb_path("/lib-tk/toolkit.php");


/**
 * @author  Kris Bailey
 */
class EditLedgerGroups extends MB_ProductAction {
	/**
	 * The constructor for a EditProductCycles action object. This merely calls the parent constructor,
	 * and sets the string representation of this object.
	 */
	function EditLedgerGroups() {
		parent::MB_ProductAction("ledgergroups");
		$this->stringRep = "EditLedgerGroups";
		$this->params = array("productID");
	}

	/**
	 * The main method of this class that executes the EditProductCycles action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		$productID = getParam("productID");
		if(!$productID) {
			$this->addError('TRANS_ACTIONERROR', "No product ID given.");
		} else {
			$q = new MBQuery("GetProducts", "", array("productID" => $productID, "getCycleGhostData" => 1));
			$r =& $q->getResult();
			if($r->getSuccess()) {
				$s =& $r->getSubjectFrom(
					array("mbapi", "results", "products", "product", 0),
					"productID", "productName"
				);

				$product = $s->getCollapsed();
				$types = $CONF["types"]["products"];

				$this->smartyVars['product'] = $product;
				$this->smartyVars['types'] = $types;

				$this->smartyVars["productVariants"] = arrayFor(":productVariants", array(
					"productID" => $productID,
				));

				$this->setupTypeButtons($product["productType"]);
				$this->smartyVars["productButtons"] = $this->productButtons->getButtons();

				$productVariantsData = arrayFor(":productVariants", array("productID" => $productID));
				for ($i = 0; $i < sizeof($productVariantsData); $i++) {
					if (!$productVariantsData[$i]['productVariantDisplayName']) {
						if ($tldId = $productVariantsData[$i]['tldID']) {
							$productVariantsData[$i]['productVariantDisplayName'] = Factory()->Tld($tldId)->tld_name;
						} else if ($variableValueId = $productVariantsData[$i]['productVariableValueID']) {
							$productVariantsData[$i]['productVariantDisplayName'] = Factory()->ProductVariableValue($variableValueId)->product_variable_value;
						} else {
							$productVariantsData[$i]['productVariantDisplayName'] = blmsg('TRANS_DEFAULT_PRODUCT_VARIANT_NAME');
						}
					}
				}
				$this->smartyVars["productVariants"] = $productVariantsData;

				$variantIDs = array();
				foreach ((array)$this->smartyVars["productVariants"] as $v){
					$variantIDs[] = $v["productVariantID"];
				}


				$q = new MBQuery("GetLedgerGroups", "", array("productVariantID" => $variantIDs));
				$query = $q->getXMLQuery();

				dbg("query: ".$query);

				$results = dispatchMBAPI($query, 1);
				$results = collapseArrayElements($results["mbapi"][0]["results"][0]["groups"][0]["group"]);
				if ((int)$results["productVariantID"]){
					$results = array($results);
				}
				$results = (array)$results;
				dbg("Results: ");
				dbg($results);
				dbg("");

				$groupsByVariant = array();
				foreach ($results as $k=>$row){
					$groupsByVariant[$row["productVariantID"]][] =& $results[$k];
				}

				foreach ($this->smartyVars["productVariants"] as $k=>$row){
					$this->smartyVars["productVariants"][$k]["ledgerGroups"] = $groupsByVariant[$row["productVariantID"]];
				}

				$this->addTemplate("widgets/products/EditLedgerGroups");

			}
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
?>
