<?php

require_once mb_path("/lib-tk/toolkit.php");

/**
 * This Action object takes information provided by the EditProductChildren action, applies that information to
 * records in the database, and redirects to the ShowProductDetails action for the given product.
 *
 * @author  Kris Bailey
 */
class EditProductAddonGroupsResponse extends MB_Action {
	/**
	 * The constructor for an EditProductChildrenResponse action object. This merely calls the parent constructor, 
	 * and sets the string representation of this object. It also sets up $this->params.
	 */
	function EditProductAddonGroupsResponse() {
		parent::MB_Action();
		$this->stringRep = "EditProductAddonGroupsResponse";
		$this->params = array(
			"productID",
			"productChildren",
		);
	}

	/**
	 * The main method of this class that executes the EditProductAddonGroupsResponse action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();
		
		$this->setReferralActionID(getActionID("EditProductAddonGroups"));
		$this->setReferralActionFile("products.php");
		$success = $this->validateMultiInputs(
			array(
				"productID" => INPUT_RESTRICTION_UNSIGNED_INT,
			),
			array(),
			array(),
			1
		);
		if($success) {
	
			$productID = ((int) $_REQUEST["productID"]);
			if(!$productID) {
				$this->addError('TRANS_ACTIONERROR', "No product ID given.");
			} else {
 				$params = array();
				$this->applyRequestTo(
					$params,
					array(
						'productAddonGroups',
					)
				);

				$q = new MBQuery(
					"GetProducts", "",
					array(
						"productID" => $productID,
						"getProductAddonGroupIDs" => 1,
					)
				);
				$r =& $q->getResult();
				
				$this->smartyVars["selectedAddonGroups"] = grabHTMLOpsFromArray(grabResultsFromMBAPIArray(array(
					"mbapi" => array(
						"command" => "GetProductAddonGroups",
						"params" => array(
							"getGroupsByProductID" => $productID,
							"productAddonGroupActive" => 1,
						),
					),
				)), "productAddonGroupID", "productAddonGroupName");

				$removeIDs = array("productAddonGroupID"=>array());
				$addIDs = array("productAddonGroupID"=>array());
				$newGroupIDs = (array)$_REQUEST["productAddonGroupIDs"];
				foreach ((array)$this->smartyVars["selectedAddonGroups"] as $id=>$name){
					if (!in_array($id, $newGroupIDs)){
						$removeIDs["productAddonGroupID"][] = $id;
					}
				}
				foreach ($newGroupIDs as $id){
					if (!$this->smartyVars["selectedAddonGroups"][$id]){
						$addIDs["productAddonGroupID"][] = $id;
					}
				}

				dbg(toXML(array(
					"mbapi" => array(
						"command" => "SetProducts",
						"subCommand" => "update",
						"params" => array(
							"productID" => $productID,
							"productAddonGroupIDs" => array(
								$addIDs,
							),
							"removeProductAddonGroupIDs" => array(
								$removeIDs,
							),
						),
					),
				)));

				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetProducts",
						"subCommand" => "update",
						"params" => array(
							"productID" => $productID,
							"productAddonGroupIDs" => array(
								$addIDs,
							),
							"removeProductAddonGroupIDs" => array(
								$removeIDs,
							),
						),
					),
				)),1);



				if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
					redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
				} else {
					redirect(
						"products.php", getActionID("EditProductAddonGroups"),
						array(
							"productID" => $productID,
							"loadActionFile" => "products",
							"loadActionID" => getActionID("EditProductAddonGroups"),
						)
					);
				}
			}

		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
?>
