<?php

require_once mb_path("/lib-tk/toolkit.php");

/**
 * This Action object takes information provided by the EditProductCycles action, applies that information to
 * records in the database, and redirects to the ShowProductDetails action for the given product.
 *
 * @author  Ben Woolley
 */
class EditProductCyclesResponse extends MB_Action {
	/**
	 * The constructor for an EditProductCyclesResponse action object. This merely calls the parent constructor, 
	 * and sets the string representation of this object. It also sets up $this->params.
	 */
	function EditProductCyclesResponse() {
		parent::MB_Action();
		$this->stringRep = "EditProductCyclesResponse";
		$this->params = array(
			"productID",
			"cycles",
		);
	}

	/**
	 * The main method of this class that executes the EditProductCyclesResponse action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		$this->setReferralActionID(getActionID("ShowProductDetails"));
		$this->setReferralActionFile("products.php");
		$success = $this->validateMultiInputs(
			array(
				"productID" => INPUT_RESTRICTION_UNSIGNED_INT,
			),
			array(),
			array(),
			0,
			array("loadActionFile"=>"products","loadActionID"=>getActionID("EditProductCycles"))
		);
		if($success) {

			$productID = ((int) $_REQUEST["productID"]);
			if(!$productID) {
				$this->addError('TRANS_ACTIONERROR', "No product ID given.");
			} else {
 				$params = array();
				$this->applyRequestTo(
					$params,
					array(
						'cycles',
					)
				);

				$q = new MBQuery(
					"GetProducts", "",
					array(
						"productID" => $productID,
						"getCycleGhostIDs" => 1,
					)
				);
				$r =& $q->getResult();
				
				$cycles =& $r->getSubjectFrom(
					array("mbapi", "results", "products", "product", "ghostCycles", "ghostCycle"),
					"cycleID"
				);
				$relParams = $this->getRelationParamsFromSubject($cycles, "cycleIDs", "cycleID", "cycles");
				$params += $relParams;

				if($relParams) {
					$setQuery = new MBQuery("SetProducts", "update", $params);
					$setResult =& $setQuery->getResult();
					$skipSuccess = false;
				} else {
					$skipSuccess = true;
				}
											
				if($skipSuccess || $setResult->getSuccess()) {
					if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
						redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
					} else {
						redirect(
							"products.php", getActionID("EditProductCycles"),
							array(
								"productID" => $productID,
								"loadActionFile" => "products",
								"loadActionID" => getActionID("EditProductCycles"),
							)
						);
					}
				}
			}

		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
?>
