<?php

require_once mb_path("/lib-tk/toolkit.php");

/**
 * This Action object takes information provided by the EditProductEmails action, applies that information to
 * records in the database, and redirects to the ShowProductDetails action for the given product.
 *
 * @author  Ben Woolley
 */
class EditProductEmailsResponse extends MB_Action {
	/**
	 * The constructor for an EditProductEmailsResponse action object. This merely calls the parent constructor, 
	 * and sets the string representation of this object. It also sets up $this->params.
	 */
	function EditProductEmailsResponse() {
		parent::MB_Action();
		$this->stringRep = "EditProductEmailsResponse";
		$this->params = array(
			"productID",
			"signupEmailActionGroup",
			"welcomeEmailActionGroup",
		);
	}

	/**
	 * The main method of this class that executes the EditProductEmailsResponse action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		$this->setReferralActionID(getActionID("ShowProductDetails"));
		$this->setReferralActionFile("products.php");
		$success = $this->validateMultiInputs(
			array(
				"productID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
				"welcomeEmailTemplateGroup" => INPUT_RESTRICTION_UNSIGNED_INT,
			),
			array(),
			array(),
			0,
			array("loadActionFile"=>"products","loadActionID"=>getActionID("EditProductEmails"))
		);
		if($success) {

			$productID = ((int) $_REQUEST["productID"]);
			if(!$productID) {
				$this->addError('TRANS_ACTIONERROR', "No product ID given.");
			} else {

				$params = NULL;
				//This will allow the template to be set to 0 if no template is needed.
				$templateID = ((int) $_REQUEST["welcomeEmailTemplateGroup"]);
				$params["emailTplGroupID"] = $templateID;
				$params["productID"] = $productID;

				if($params) {
					$q = new MBQuery("SetProducts", "update", $params);
					$r =& $q->getResult();		
				}

				if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
					redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
				} else {
					redirect(
						"products.php", getActionID("EditProductEmails"),
						array(
							"productID" => $productID,
							"loadActionFile" => "products",
							"loadActionID" => getActionID("EditProductEmails"),
						)
					);
				}

			}

		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
?>
