<?php

require_once mb_path("/lib-tk/toolkit.php");

/**
 * This Action object takes information provided by the EditProductServerGroups action, applies that information to
 * records in the database, and redirects to the ShowProductDetails action for the given product.
 *
 * @author  Ben Woolley
 */
class EditProductServerGroupsResponse extends MB_Action {
	/**
	 * The constructor for an EditProductServerGroupsResponse action object. This merely calls the parent constructor,
	 * and sets the string representation of this object. It also sets up $this->params.
	 */
	function EditProductServerGroupsResponse() {
		parent::MB_Action();
		$this->stringRep = "EditProductServerGroupsResponse";
		$this->params = array(
			"productID",
			"serverGroups",
		);
	}

	/**
	 * The main method of this class that executes the EditProductServerGroupsResponse action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		$this->setReferralActionID(getActionID("ShowProductDetails"));
		$this->setReferralActionFile("products.php");
		$success = $this->validateMultiInputs(
			array(
				"productID" => INPUT_RESTRICTION_UNSIGNED_INT,
			),
			array(),
			array(),
			0,
			array("loadActionFile"=>"products","loadActionID"=>getActionID("EditProductServerGroups"))
		);
		if($success) {

			$productID = ((int) $_REQUEST["productID"]);
			if(!$productID) {
				$this->addError('TRANS_ACTIONERROR', "No product ID given.");
			} else {
 				$params = array();
				$this->applyRequestTo(
					$params,
					array(
						'serverGroups',
					)
				);

				$q = new MBQuery(
					"GetProducts", "",
					array(
						"productID" => $productID,
						"getServerGroupGhostData" => 1,
					)
				);
				$r =& $q->getResult();

				$serverGroups =& $r->getSubjectFrom(
					array("mbapi", "results", "products", "product", "ghostServerGroups", "ghostServerGroup"),
					"serverGroupID"
				);
				$relParams = $this->getRelationParamsFromSubject($serverGroups, "serverGroupIDs", "serverGroupID", "serverGroups");
				if(isset($relParams["removeServerGroupIDs"])) {
					foreach($relParams["removeServerGroupIDs"]["serverGroupID"] as $serverGroupID) {
						foreach(Factory()->PanelGroup($serverGroupID)->ControlPanelList() as $controlPanel) {
							foreach($controlPanel->SubscriptionList() as $package) {
								if($package->product_id == $productID) {
									$this->addError(blmsg("TRANS_ERROR"), blmsg("TRANS_SERVER_GROUP_REMOVAL_ERROR_MSG"));
									$this->addErrorTemplates();
									return $this->hasExecutedSuccessfully();
								}
							}
						}
					}
				}

				$params += $relParams;

				if($relParams) {
					$setQuery = new MBQuery("SetProducts", "update", $params);
					$setResult =& $setQuery->getResult();
					$skipSuccess = false;
					getCurrentSession()->Storage->warningMessages[] = blmsg("TRANS_PRODUCT_SERVER_GROUP_CHANGE_WARNING_SET_PROPERTIES");
				} else {
					$skipSuccess = true;
				}
				if($skipSuccess || $setResult->getSuccess()) {
					if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
						redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
					} else {
						redirect(
							"products.php", getActionID("EditProductServerGroups"),
							array(
								"productID" => $productID,
								"loadActionFile" => "products",
								"loadActionID" => getActionID("EditProductServerGroups"),
							)
						);
					}
				}
			}

		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
?>
