<?php

require_once("admin/include/lib-action/products/ShowProductDetailsAction.php");
require_once("lib-tk/toolkit.php");
require_once("lib-billing/include/config/internal/datasources.php");

class EditProductTLDs extends MB_ProductAction
{
	public function __construct()
	{
		parent::MB_ProductAction("options");
		$this->stringRep = "EditProductTLDs";
		$this->params = array("productID");
	}

	public function execute()
	{
		parent::execute();

		do {
			if($this->err($productID = getParam("productID"), "No product ID given.")) break;
			$this->smartyVars["productID"] = $productID;

			$this->syncProductVariants($productID);

			require_once("object_managers/BillingRegistrarManager.php");
			$allowedTLDs = DataObject::getValues(Factory()->TldList(array("tld_active" => 1)), "tld_extension");
			foreach (Service()->BillingRegistrarManager->getEnabledRegistrars(getCurrentSession()->getVendorId()) as $registrar) {
				try {
					$allowedTLDs = array_merge($allowedTLDs, $registrar->getTLDs());
				} catch (ProductException $e) {
					// skip Registrar errors
				}
			}

			$selectedTLDs = $selectableTLDs = $nonConfiguredTLDs = array();
			$productVariants = Factory()->PlanVariantList(array("product_id" => $productID));
			foreach ($productVariants as $variant) {
				$tldExtention = $variant->Tld->tld_extension;
				$tldName = $variant->Tld->tld_name;
				if (1 == $variant->product_variant_active) {
					$selectedTLDs[$variant->id] = $tldName;
					if (!in_array($tldExtention, $allowedTLDs)) {
						$nonConfiguredTLDs[] = $tldName;
					}
				} elseif(in_array($tldExtention, $allowedTLDs)) {
				    $selectableTLDs[$variant->id] = $tldName;
				}
			}

			if (!empty($nonConfiguredTLDs)) {
				$this->smartyVars["errorMessages"][] = blmsg("TRANS_TLDS_IS_NOT_CONFIGURED", array("TLD_LIST" => implode(", ", $nonConfiguredTLDs)));
			}
			$this->smartyVars["selectedTLDs"] = $selectedTLDs;
			$this->smartyVars["selectableTLDs"] = $selectableTLDs;

			$product = grabFirstResultFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetProducts",
					"params" => array(
						"productID" => $productID
					)
				)
			));
			$this->smartyVars["product"] = $product;

			$this->setupTypeButtons($product["productType"]);
			$this->smartyVars["productButtons"] = $this->productButtons->getButtons();

			$this->mergeFormState();
			$this->addTemplate("widgets/products/EditProductTLDs");
		} while(false);

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
