<?php

require_once("lib-billing/include/config/internal/datasources.php");
require_once("lib-tk/toolkit.php");

class EditProductVariantPricesResponse extends MB_Action
{
	public $params = array("productID");

	public function execute()
	{
		parent::execute();

		do {

			switch($_REQUEST["updatePackages"]) {
				case "none":
					$updatePackages = 0;
					break;
				case "unlocked":
					$updatePackages = 1;
					break;
				case "all":
					$updatePackages = 2;
					break;
			}

			$this->setReferralActionID(getActionID("EditProductVariantPrices"));
			$this->setReferralActionFile("products.php");

			if($this->err($productID = $this->validateParam("productID", INPUT_RESTRICTION_POSITIVE_INT|INPUT_RESTRICTION_REQUIRED), "Did not get appropriate product ID")) break;
			if($this->err($productVariantID = $this->validateParam("productVariantID", INPUT_RESTRICTION_POSITIVE_INT|INPUT_RESTRICTION_REQUIRED), "Did not get appropriate product variant ID")) break;

			if (!getCurrentSession()->isValidToken()) {
				billing_put_error(blmsg("TRANS_ERROR_TOKEN"));
				redirect('products.php', getActionID("EditProductVariantPrices"), array("productVariantID" => $productVariantID));
			}

			$setTaxZoneXML	= "	<mbapi>
									<command>SetProducts</command>
									<subCommand>update</subCommand>
									<params>
										<productID>".$productID."</productID>
										<productVariants>
											<productVariant>
												<productVariantID>".$productVariantID."</productVariantID>
												<taxZoneGroupID>".(int)$_REQUEST["taxZoneGroupID"]."</taxZoneGroupID>
												<cycleDiscountGroupID>".(int)$_REQUEST["cycleDiscountGroupID"]."</cycleDiscountGroupID>
											</productVariant>
										</productVariants>
									</params>
								</mbapi>";
			$setTaxZone = dispatchMBAPI($setTaxZoneXML, 1);

			dbg("set cmd:");
			dbg($setTaxZoneXML);
			dbg("result of set:");
			dbg($setTaxZone);

			$byPrice = array();
			foreach($_REQUEST as $arg => $val) {
				$matches = array();
				if(preg_match("/^(price|id)_([A-Z]{3})_([0-9]+)_([0-9]+)$/", $arg, $matches)) {
					//print_r($matches);
					list($entirety, $field, $currencyID, $cycleID, $priceType) = $matches;
					if($field == "price") {
						$_REQUEST[$arg] = (float) BillingLocale::getNumberFromInput($val);
					}

					$this->validateParam($arg, $field == "id" ? INPUT_RESTRICTION_UNSIGNED_INT : INPUT_RESTRICTION_PRICE|INPUT_RESTRICTION_REQUIRED);
					$priceKey = serialize(array($currencyID, $cycleID, $priceType));
					if(!isset($byPrice[$priceKey])) $byPrice[$priceKey] = array();
					$byPrice[$priceKey][$field] = $_REQUEST[$arg];
				}
			}

			//print_r($_REQUEST);

			$productVariantPrices = array();
			foreach($byPrice as $priceKey => $price) {
				list($currencyID, $cycleID, $priceType) = unserialize($priceKey);

				$productVariantPrice = array(
					'productVariantPriceType' => $priceType,
					'productVariantPriceAmount' => (float) $price['price'],
					'productVariantID' => $productVariantID,
					'currencyID' => $currencyID,
					'cycleID' => $cycleID,
				);
				if($price['id']) {
					$productVariantPrice['productVariantPriceID'] = (int) $price['id'];
				}
				$productVariantPrices[] = $productVariantPrice;
			}
			$newProductVariantPrices = new BillingTable($productVariantPrices, "productVariantPriceID");
			//$newProductVariantPricesByID =& $newProductVariantPrices->getAssociatedTables();

			if(!$this->validationRedirect()) break;

			$d = new DataSource(
				"products",
				array(
					"productVariantID" => $productVariantID,
					"getProductVariantPriceData" => 1,
				)
			);

			if(!$s =& $this->getSubjectFromSource($d)) return false;
			$t =& $s->getTable();
			$product = $t->getOne();
			//$domainType = (int) $product["productType"] === (int) getTypeID("products", TRANS_DOMAIN_PRODUCT) ? true : false;

			$pvps =& $s->newFrom(array("productVariantPrices", "productVariantPrice"));

			$oldProductVariantPrices =& $pvps->getTable();
			$oldProductVariantPrices->setKey("productVariantPriceID");
			$oldProductVariantPricesByID = $oldProductVariantPrices->getAssociated();

			$updateProductPriceVariants = array();
			foreach($productVariantPrices as $productVariantPrice) {
				if(isset($productVariantPrice["productVariantPriceID"])) {
					$productVariantPriceID = $productVariantPrice["productVariantPriceID"];
					$oldPrice = (array) $oldProductVariantPricesByID[$productVariantPriceID];
					$oldPrice['productVariantPriceAmount'] = (float) $oldPrice['productVariantPriceAmount'];
					$diffPrice = array_diff_assoc($productVariantPrice, $oldPrice);
					if($diffPrice) {
						$updateProductPriceVariants[] = $diffPrice + array("productVariantPriceID" => $productVariantPriceID);
					}
				} else {
					$updateProductPriceVariants[] = $productVariantPrice;
				}
			}

			$oldProductVariantPriceIDs = $oldProductVariantPrices->getKeys();
			$newProductVariantPriceIDs = $newProductVariantPrices->getKeys();
			$deleteProductPriceVariantIDs = array_diff($oldProductVariantPriceIDs, $newProductVariantPriceIDs);

			if($updateProductPriceVariants || $deleteProductPriceVariantIDs) {
				$setParams = array(
					"productID" => $productID,
				);
				if($updateProductPriceVariants) {
					$setParams += array(
						"productVariants" => array(
							"productVariant" => array(
								"updatePackages" => $updatePackages,
								"productVariantID" => $productVariantID,
								"productVariantPrices" => array(
									"productVariantPrice" => $updateProductPriceVariants,
								),
							),
						),
					);
				}
				if($deleteProductPriceVariantIDs) {
					$setParams += array(
						"removeProductVariantPriceIDs" => array(
							"productVariantPriceID" => $deleteProductPriceVariantIDs,
						),
					);
				}
			} else {
				$setParams = false;
			}

			if($setParams) {
				$updateQuery = new MBQuery("SetProducts", "update", $setParams);

				if(!$this->addErrorsFromResult($updateResult =& $updateQuery->getResult())) break;
			}
			if(!$this->goBack()) {
				if ((int)$_REQUEST["singleVariant"]){
					redirect(
						"products.php", getActionID("EditProductVariantPrices"),
						array(
							"productID" => $productID,
							"productVariantID" => (int)$_REQUEST["productVariantID"],
							"productVariantActive" => 1,
							"pricesSaved" => 1,
							//"productVariantID" => $productVariantID,
						)
					);
				} else {
					redirect(
						"products.php", getActionID("ShowProductVariants"),
						array(
							"productID" => $productID,
							"productVariantActive" => 1,
							//"productVariantID" => $productVariantID,
						)
					);
				}
			}
		} while(false);
	}
}
