<?php

use Billing\Object\Plan;
use Billing\Object\PlanVariantPrice;
use Billing\Object\Reseller;

use Billing\Manager\SupplementaryPlanGroupManager;

class Products_PlanOverviewController extends Billing_Zend_Controller_Action_AdminController
{

	public function init()
	{
		parent::init();
	}

	public function indexAction()
	{
        try {
            if (!is_null($id = $this->getRequest()->getParam('id'))) {
                $plan = Factory()->Plan($id);
            } elseif(!is_null($guid = $this->getRequest()->getParam('productGUID'))) {
                $plan = Factory()->Plan($guid, 'guid');
            } elseif(!is_null($externalId = $this->getRequest()->getParam('planExternalID'))) {
                $plan = Factory()->Plan($externalId, 'external_id');
            } else {
                throw new ProductException(ProductException::E_OBJECT_NOT_FOUND, $this->_translator->translate("TRANS_NO_PLAN_ID_OR_GUID"));
            }
        } catch (ProductException $e) {
            $this->_redirectWithMessage($this->view->ActionUrl('products', 'ShowProducts'), $e->getMessage(), "error");
        }

		$this->view->plan = $this->_plan = $plan;
        $this->_problems['messages'][$this->_getProblemTitle()] = $this->_problems['messages'][$this->_getWarningTitle()] = array();
		$this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_PLAN_OVERVIEW_TITLE', array('NAME' => billing_safetyhtml($plan->product_name))));

		Service()->PlanManager->isPlanValidForStores($plan, $plan->orderFormIDs, $this->_problems['messages'][$this->_getProblemTitle()], $this->_problems['messages'][$this->_getWarningTitle()], true);


		$activeProductVariants = array();
		foreach ($plan->PlanVariantList() as $productVariant) {
			if ($productVariant->product_variant_active) {
				$activeProductVariants[$productVariant->id] = $productVariant;
			}
		}

		$activeCycles = array();
		foreach ($plan->cycleIDs as $cycleId) {
			$activeCycles[$cycleId] = true;
		}

		$this->view->type = 'TRANS_PRODUCT_TYPE_NAME_' . $plan->product_type;
		$this->view->subscribers = $plan->getPackagesCount();

		$activeCurrencies = array();
		foreach (Service()->PlanManager->getPricesData($plan->id) as $cycleId => $variants) {
			if (!isset($activeCycles[$cycleId])) {
				continue;
			}
			foreach ($variants as $variantId => $currencies) {
				if (!isset($activeProductVariants[$variantId])) {
					continue;
				}
				foreach ($currencies as $code => $prices) {
					if (isset($prices[PlanVariantPrice::TYPE_GENERAL])) {
						$activeCurrencies[$code] = true;
					}
				}
			}
		}
		$stores = $plan->StoreList();
		$actualCurrencies = array();
		$missedCurrencies = array();
		foreach ($stores as $store) {
			$actualCurrencies[$store->currency_id] = true;
			if (!isset($activeCurrencies[$store->currency_id]) && !isset($missedCurrencies[$store->currency_id])) {
				$missedCurrencies[$store->currency_id] = true;
			}
		}
		$this->view->stores = $stores;

		if (in_array($plan->product_type, array(Plan::TYPE_HOSTING, Plan::TYPE_RESELLER))) {
			$serverGroupIds = $plan->serverGroupIDs;

            if (sizeof($serverGroupIds) == 1) {
				$serverGroupId = $serverGroupIds[0];
				$attributes = array();
				$planAttributesList = $plan->ProductAttributesList(array('server_group_id' => $serverGroupId));
				if ($planAttributes = array_shift($planAttributesList)) {
					$displayableAttributes = array(
						'domainDiskSpace',
						'domainMaxTraffic',
						'domainMaxSites',
						'domainMaxMailbox',
						'domainMaxMaillists',
						'domainMaxMYSQLDB',
                        'domainMaxMSSQLDB',
                        'limit-pleskweb.domains_number',
                        'limit-disc_space',
                        'limit-pleskweb.traffic',
                        'limit-pleskweb.mysql.db_number'
					);
                    if (Plan::TYPE_RESELLER == $plan->product_type) {
                        array_unshift($displayableAttributes, "max_cl");
                    }
					foreach ($displayableAttributes as $name) {
						if (isset($planAttributes[$name]) && $planAttributes[$name]->product_attribute_value) {
							$item = array('value' => $planAttributes[$name]->product_attribute_value);
                            $pleskUnit = $name . 'UNIT';
                            $poaUnit = str_replace(array('limit-pleskweb.', 'limit-'), 'unit-', $name).'.unit';
							if (isset($planAttributes[$pleskUnit])) {
								$item['unit'] = $planAttributes[$pleskUnit]->product_attribute_value;
							} else if (isset($planAttributes[$poaUnit])) {
								$item['unit'] = $planAttributes[$poaUnit]->product_attribute_value;
							}
							$attributes[$name] = $item;
						}
					}
				}
				$this->view->attributes = $attributes;
				$this->view->attributesCount = sizeof($planAttributes);
				$this->view->displayAttributes = sizeof($attributes) ? true : false;
			} else {
                $this->view->displayAttributes = false;
			}

		} else {
			$this->view->displayAttributes = false;
		}

		if ($plan->isAddon()) {
			$this->view->parents = $plan->ParentList();
		} else {
			// Addons
			$addons = $plan->ChildrenList();
			usort($addons, array($this, '_sortAddons'));
			$this->view->addons = $addons;

		}

		$allPrices = array();
		$cyclePrices = array();
		$productVariantList = $plan->PlanVariantList();
		foreach ($productVariantList as $variant) {
			foreach ($plan->CycleList() as $cycle) {
				$tmp = $variant->PlanVariantPriceList(
					array(
						'cycle_id' => $cycle->id,
						'product_variant_price_type' => PlanVariantPrice::TYPE_GENERAL
					)
				);

				$variantPrices = array();
				foreach ($tmp as $price) {
					if (isset($actualCurrencies[$price->currency_id])) {
						$variantPrices[] = $price;
					}
				}
				if ($variantPrices) {
					$allPrices = array_merge($allPrices, $variantPrices);
					foreach ($variantPrices as $price) {
						$allPrices[$price->currency_id][$price->product_variant_id][$price->cycle_id] = $price;
					}
					if (!isset($cyclePrices[$cycle->cycle_display_name])) {
						$cyclePrices[$cycle->cycle_display_name] = $variantPrices;
					}
				}
			}
		}
		$this->view->prices = $cyclePrices;
		$this->view->displayPrices = $variant && !sizeof($productVariantList) ? true : false;

		if (!$plan->isAddon()) {
			// generate list of purchase links for this plan
			$links = array();
			$priceIdTemplate = '%priceId%';
			foreach ($stores as $store) {
				$linkTemplate = Service()->StoreManager->getBuyUrl($store->order_form_code, $store->vendor_id, $plan->id, $priceIdTemplate);
                $currencyPrices = $allPrices[$store->currency_id];
                if (!is_null($currencyPrices)) {
					foreach ($currencyPrices as $variantId => $variantPrices) {
						if (!Factory()->PlanVariant($variantId)->product_variant_active) {
							continue;
						}
						foreach ($variantPrices as $cycleId => $price) {
							$links[] = array(
								'currency' => $store->currency_id,
								'variant' => $plan->isDomainType() ?
									Factory()->PlanVariant($variantId)->Tld->tld_name :
									Factory()->PlanVariant($variantId)->product_variant_display_name,
								'cycle'=> Factory()->Cycle($cycleId)->cycle_display_name,
								'link' => str_replace(urlencode($priceIdTemplate), $price->id, $linkTemplate)
							);
						}
					}
				}
			}
			$this->view->links = $links;
			$this->view->isShowVariantForLinks = count($productVariantList) > 1 ? true : false;
			$this->view->isDomainPlan = $plan->isDomainType();

            // generate list of purchase links for this plan
            $supplementaries = array();

            foreach($plan->SupplementaryPlanGroupList() as $group) {
                $supplementaries[] = array(
                    "type" => SupplementaryPlanGroupManager::SUPPLEMENTARY_GROUP,
                    "value" => array(
                        "id" => $group->id,
                        "name" => $group->name,
                    ),
                );
                foreach ($group->SupplementaryPlans as $key => $value) {
                    $supplementaries[] = array(
                        "type" => SupplementaryPlanGroupManager::SUPPLEMENTARY_PLAN,
                        "value" => array(
                            "id" => $key,
                            "name" => Factory()->Plan($key)->product_name,
                            "groupId" => $group->id,
                            "setByDefault" => $value->set_by_default,
                        ),
                    );
                }
            }
            $this->view->supplementaries = $supplementaries;
            $this->view->token = getCurrentSession()->getToken();


            $this->view->hasOtherSupplementaries = (0 != count(Service()->PlanManager->getPlansToCopySupplementaries($plan)));
		}

		if ($variant && $discount = $variant->CycleDiscountGroup) {
			$this->view->discount = $discount;
		}

		$this->view->coupons = $plan->CouponList();
		$this->view->preappliedCouponIDs = array();
		foreach($plan->preappliedCouponList() as $coupon) {
			$this->view->preappliedCouponIDs[] = $coupon->id;
		}

		require_once 'UI/Page/UISelectProductTypePage.php';
		$toolButtons = array(
			'settings' => array(
				'label' => $plan->isAddon() ? 'TRANS_ADDON_OVERVIEW_TOOLBAR_SETTINGS' : 'TRANS_PLAN_OVERVIEW_TOOLBAR_SETTINGS',
				'class' => 'sb-plan-settings',
				'href' => $this->view->adminActionUrl('products', 'plan-settings', 'edit', array('id' => $plan->id))
			),
			'ledger' => array(
				'label' => 'TRANS_PLAN_OVERVIEW_TOOLBAR_LEDGER',
				'class' => 'sb-plan-ledger',
				'href' => $this->view->actionUrl('products', 'EditLedgerGroups', array('productID' => $plan->id)),
				'separate' => true
			),
			'sync' => array(
				'label' => 'TRANS_SYNC_CMD',
				'class' => 'sb-refresh',
				'href' => $this->view->actionUrl('products', 'ShowProducts', array("sync" => 1, 'id[]' => $plan->id, "redirectAction" => "ShowProductDetails")),
				'separate' => true
			),
			'clone' => array(
				'label' => 'TRANS_PLAN_OVERVIEW_TOOLBAR_CLONE',
				'class' => 'sb-clone',
				'href' => $this->view->adminActionUrl('products', 'plan-settings', 'clone', array('id' => $plan->id)),
				'separate' => false
			),
		);

		if(0 == $plan->getPackagesCount()) {
			$toolButtons['remove'] = array(
				'label' => 'TRANS_PLAN_OVERVIEW_TOOLBAR_REMOVE',
				'class' => 'sb-remove',
				'href' => $this->view->actionUrl('products', 'RemoveProduct', array('productID' => $plan->id)),
				'separate' => true
			);
		}

		if (($reseller = getCurrentSession()->getReseller()) && !$reseller->hasPermission(\Billing\Object\Reseller::PERMISSION_ACCOUNTING)) {
			unset($toolButtons['ledger']);
		}
        $this->_compareCycles();

		$this->view->problems = $this->_problems;
		$this->view->toolButtons = $toolButtons;

		$this->view->isAddon = $plan->isAddon();

		if ($plan->isSyncable() && !is_null($plan->sync_error)) {
			$transKey = $plan->isAddon() ? "TRANS_SYNC_THIS_PLAN_ADDON_ERROR_MSG" : "TRANS_SYNC_THIS_PLAN_ERROR_MSG";
			$this->_status->addError(blmsg($transKey, array("ERROR" => nl2br(billing_safetyhtml($plan->sync_error)))));
		}
    }

    private function _compareCycles()
    {
        $addonWithParentCycles = Service()->PackageAddonManager->selWithParentCycles();

        if (!$this->_plan->isAddon()) {
            foreach ($this->_plan->addonIDs as $addonId) {

                $addon = Factory()->Plan($addonId);

                $addonCycles = array();
                foreach ($addon->CycleList() as $addonCycle) {
                    $addonCycles[$addonCycle->cycle_id] = $addonCycle;
                }

                if (!$addonWithParentCycles) {
                    // sort addon cycles ASC
                    uasort($addonCycles, function($a, $b) {return $b->isLessThan($a);});
                }

                foreach($this->_plan->CycleList() as $planCycle) {
                    if ($addonWithParentCycles && isset($addonCycles[$planCycle->cycle_id])) {
                        // it's OK - addon should be sold with parent cycle and corresponding cycle is found
                        continue;
                    }
                    if (!$addonWithParentCycles && count($addonCycles) && !$planCycle->isLessThan(reset($addonCycles))) {
                        // it's OK - addon can be sold with different cycle and it has cycles not bigger than plan cycle
                        continue;
                    }
                    $this->_problems['messages']["TRANS_PLAN_ADDONS_SOME_CYCLES_NOT_COMPARE_PLAN"][] = $this->_translator->translate("TRANS_PLAN_ADDON_CYCLE_NOT_DEFINE_ADDON_MSG",
                        array(
                            "CYCLE" => $this->_translator->translate($planCycle->cycle_display_name),
                            "ADDON" => $addon->product_name
                        )
                    );
                }
            }
        } else {
            $addonCycles = array();
            foreach ($this->_plan->CycleList() as $addonCycle) {
                $addonCycles[$addonCycle->cycle_id] = $addonCycle;
            }

            if (!$addonWithParentCycles) {
                // sort addon cycles ASC
                uasort($addonCycles, function($a, $b) {return $b->isLessThan($a);});
            }

            foreach($this->_plan->ParentList() as $plan) {
                foreach ($plan->CycleList() as $planCycle) {
                    if ($addonWithParentCycles && isset($addonCycles[$planCycle->cycle_id])) {
                        // it's OK - addon should be sold with parent cycle and corresponding cycle is found
                        continue;
                    }
                    if (!$addonWithParentCycles && count($addonCycles) && !$planCycle->isLessThan(reset($addonCycles))) {
                        // it's OK - addon can be sold with different cycle and it has cycles not bigger that plan cycle
                        continue;
                    }
                    $this->_problems['messages']["TRANS_PLAN_ADDONS_SOME_CYCLES_NOT_COMPARE_ADDON"][] = $this->_translator->translate("TRANS_PLAN_ADDON_CYCLE_NOT_DEFINE_PLAN_MSG",
                        array(
                            "CYCLE" => $this->_translator->translate($planCycle->cycle_display_name),
                            "PLAN" => $plan->product_name
                        )
                    );
                }
            }
        }
    }

    private function _getProblemTitle()
    {
        return ($this->_plan->isAddon()) ? 'TRANS_ADDON_OVERVIEW_OVERVIEW_PROBLEM' : 'TRANS_PLAN_OVERVIEW_OVERVIEW_PROBLEM';
    }

    private function _getWarningTitle()
    {
        return ($this->_plan->isAddon()) ? 'TRANS_ADDON_OVERVIEW_OVERVIEW_PROBLEM_NON_CRITICAL' : 'TRANS_PLAN_OVERVIEW_OVERVIEW_PROBLEM_NON_CRITICAL';
    }

    private function _sortAddons($a, $b)
    {
    	if ($a->product_addon_group_id > $b->product_addon_group_id) {
    		return 1;
    	} else if ($a->product_addon_group_id < $b->product_addon_group_id) {
    		return -1;
    	} else {
    		return 0;
    	}
    }

    private $_plan = null;
    private $_problems = null;
}