<?php

require_once("lib-action/action.php");

use Billing\Manager\SecurityManager;
use Billing\Object\Session;

class ForwardToResellerInterface extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$resellerID = (int)$_REQUEST["resellerID"];

		if(!notZero($resellerID)) {
			throw new Exception("Missing parameter 'resellerID'");
		}

		if(!getCurrentSession()->isAdmin()) {
			throw new ProductException(ProductException::E_FORBIDDEN_OPERATION, "Must be an admin to process.");
		}

		$reseller = Factory()->Reseller($resellerID);
		$resellerUsername = Factory()->Admin($reseller->admin_id)->admin_username;

		Service()->SecurityManager->pushCurrentActionIntoLog(
			SecurityManager::USER_TYPE_RESELLER,
			$resellerID,
			$resellerUsername,
			"TRANS_ACTION_LOG_REASON_LOGIN",
			blmsg(
				"TRANS_ACTION_LOG_COMMENT_ADMIN_IMPERSONATED_RESELLER",
				array(
					"ADMIN_ID" => getCurrentSession()->user_id,
					"RESELLER_ID" => $resellerID,
					"USERNAME" => $resellerUsername,
				),
				Service()->SecurityManager->getActionLogLocale()
			)
		);

		$session = Factory()->Session();
		$session->user_id = $reseller->admin_id;
		$session->user_type = Session::USER_TYPE_ADMIN;
		$session->real_user_id = getCurrentSession()->real_user_id;
		$session->real_user_type = getCurrentSession()->real_user_type;
		$session->session_active = 1;
		$session->update();

        getCurrentSession()->Storage->impResellerSessions[$session->user_id] = $session->id;

        redirect_url(admin_action_url("dashboard", "home", "index", array("impResellerId" => $session->user_id)));

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}