<?php

require_once("lib-action/action.php");

use \Billing\Object\Reseller;

class ShowResellers extends MB_Action
{
	public function execute()
	{
		parent::execute();

		if (1 == bgpc("sync")) {
			$this->processSync();
		}

		$page = UIFactory()->Page();
		$page->title = blmsg("TRANS_RESELLERS");
		$page->parent_url = admin_action_url("configs", "system", "show");

		require_once("UI/List/UIResellerList.php");
		$page->addListPanel(new UIResellerList());

		$this->smartyVars["page"] = $page;
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}

	private function processSync()
	{
		$resellerIds = bgpc("id", array());
		if (empty($resellerIds)) {
			return;
		}
		$errors = array();
		foreach ($resellerIds as $resellerId) {
			try {
				$reseller = Factory()->Reseller($resellerId);
                if ($reseller->sync_tasks & Reseller::SYNC_CHECK_UPDATE_STATUS) {
					// check previous update operation status only
				} else {
				    $reseller->sync_tasks |= Reseller::SYNC_NEED_UPDATE | Reseller::SYNC_NEED_REPAIR_SSO;
                }
				Service()->ResellerManager->sync($reseller);
				if (!is_null($reseller->sync_error)) {
					$errors[] = blmsg("TRANS_SYNC_RESELLER_ERROR_MSG", array("RESELLER_ID" => $reseller->id, "ERROR" => $reseller->sync_error));
				}
			} catch (ProductException $e) {
				$errors[] = blmsg("TRANS_SYNC_RESELLER_ERROR_MSG", array("RESELLER_ID" => $resellerId, "ERROR" => $e->getMessage()));
			}
		}
		if (empty($errors)) {
			billing_put_info(blmsg("TRANS_SYNC_RESELLERS_SUCCESS_MSG"));
		} else {
			billing_put_error(blmsg("TRANS_SYNC_RESELLERS_FAILURE_MSG", array("ERROR" => billing_safetyhtml(implode("\n", $errors)))));
		}
	}
}
