<?php

require_once("lib-action/action.php");
require_once("UI/Page.php");

use Billing\TaskManager\Client as TaskManagerClient;
use Billing\TaskManager\Struct\Task;

class EditActionLogSettingsPage extends UIPage
{
	public function __construct($controller)
	{
		parent::__construct($controller);
		$this->form = $this->getDefaultForm();
	}

	public function createControls()
	{
		$this->title = blmsg("TRANS_ACTION_LOG_SETTINGS");
		$this->parent_title = blmsg("TRANS_ACTION_LOG");
		$this->parent_url = action_url("security", "ShowActionLog");

		require_once("UI/Tabs/UIActionLogTabs.php");
		$tabs = new UIActionLogTabs();
		$tabs->setCurrent("settings");
		$this->setScreenTabs($tabs);

		$this->form->addInputPanel(
			$general_panel = UIFactory()->InputPanel(blmsg("TRANS_ACTION_LOG_SETTINGS_PANEL"))
		);

		$input_rotation_interval = UIFactory()->InputString(
			"security_ActionLogRotationInterval",
			blmsg("TRANS_ACTION_LOG_ROTATION_INTERVAL"),
			true,
			\Billing\Checker::P_UINT_NOT_NULL,
			blmsg("TRANS_ACTION_LOG_ROTATION_INTERVAL_DESC")
		);
		$input_rotation_interval->value = $this->getValue("security_ActionLogRotationInterval", Settings()->security_ActionLogRotationInterval);
		$general_panel->addInputControl($input_rotation_interval);
		$this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;
	}

	public function processInput()
	{
		try {
			$rotationInterval = $this->getValue("security_ActionLogRotationInterval");
			$existingTask = TaskManagerClient::getPeriodicTask(TaskManagerClient::TASK_PROCESS_ACTION_LOG);
			$task = new Task($existingTask->name, $existingTask->input);
			$task->period = "P{$rotationInterval}M";
			TaskManagerClient::update($existingTask->id, $task);
			Settings()->security_ActionLogRotationInterval = $rotationInterval;
		} catch (\ProductException $e) {
			$this->addErrorMessage(UIFactory()->ErrorMessage($e->getMessage()));
			return false;
		}
		billing_put_info(blmsg("TRANS_ACTION_LOG_SETTINGS_SAVED_OK"));
		redirect(admin_action_url('configs', 'system', 'show'));
		return true;
	}

	public function processCancel()
	{
		redirect(admin_action_url('configs', 'system', 'show'));
	}
}

class EditActionLogSettings extends MB_Action
{
	public function execute()
	{
		parent::execute();
		$controller = UIFactory()->PageController(action_url("security", "EditActionLogSettings"));
		$controller->addPage(new EditActionLogSettingsPage($controller));
		$controller->processRequest();

		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}
