<?php

require_once("lib-action/action.php");
require_once("UI/Page.php");

use Billing\Checker;

class AuthSettingsPage extends UIPage
{
	public function __construct($controller)
	{
		parent::__construct($controller);
		$this->form = $this->getDefaultForm();
	}

	public function createControls()
	{

		$this->title = blmsg("TRANS_MANAGE_AUTH_SECURITY_SETTINGS");
		$this->parent_title = blmsg("TRANS_MANAGE_SYSTEM");
		$this->parent_url = admin_action_url("configs", "system", "show");

		$this->form->addInputPanel($general_panel = UIFactory()->InputPanel());

        // Username minimal length
        $input_username_minimal_length = UIFactory()->InputOptionalString(
            "security_UsernameMinimalLength",
            blmsg("TRANS_USERNAME_MINIMAL_LENGTH"),
            true,
            Checker::P_UINT_NOT_NULL,
            blmsg("TRANS_USERNAME_MINIMAL_LENGTH_DESC")
        );
        $input_username_minimal_length->offValue = "0";
        $input_username_minimal_length->optionTitle = blmsg("TRANS_USERNAME_DISABLE_MINIMAL_LENGTH");
        $input_username_minimal_length->value = $this->getValue("security_UsernameMinimalLength");
        $general_panel->addInputControl($input_username_minimal_length);

        // Password minimal length
        $input_pass_minimal_length = UIFactory()->InputOptionalString(
            "security_PasswordMinimalLength",
            blmsg("TRANS_PASSWORD_MINIMAL_LENGTH"),
            true,
            Checker::P_UINT_NOT_NULL,
            blmsg("TRANS_PASSWORD_MINIMAL_LENGTH_DESC")
        );
        $input_pass_minimal_length->offValue = "0";
        $input_pass_minimal_length->optionTitle = blmsg("TRANS_PASSWORD_DISABLE_MINIMAL_LENGTH");
        $input_pass_minimal_length->value = $this->getValue("security_PasswordMinimalLength");
        $general_panel->addInputControl($input_pass_minimal_length);

        // Password alphanumeric enable
        $input_pass_alphanumeric_enable = UIFactory()->InputCheckBox(
            "security_PasswordAlphanumericEnable",
            blmsg("TRANS_PASSWORD_ALPHANUMERIC_ENABLE"),
            false,
            NULL,
            blmsg("TRANS_PASSWORD_ALPHANUMERIC_ENABLE_DESC")
        );
        $input_pass_alphanumeric_enable->value = $this->getValue("security_PasswordAlphanumericEnable");
        $general_panel->addInputControl($input_pass_alphanumeric_enable);

		// Password history saving
		$input_pass_history_saving = UIFactory()->InputOptionalString(
			"security_PasswordHistorySaving",
			blmsg("TRANS_PASSWORD_HISTORY_SAVING"),
			true,
			Checker::P_UINT_NOT_NULL,
			blmsg("TRANS_PASSWORD_HISTORY_SAVING_DESC")
		);
		$input_pass_history_saving->offValue = "0";
		$input_pass_history_saving->optionTitle = blmsg("TRANS_PASSWORD_DISABLE_HISTORY_SAVING");
		$input_pass_history_saving->value = $this->getValue("security_PasswordHistorySaving");
		$general_panel->addInputControl($input_pass_history_saving);


		// Password expiration interval in days
		$input_pass_expiration_interval = UIFactory()->InputOptionalString(
			"security_PasswordExpirationInterval",
			blmsg("TRANS_PASSWORD_EXPIRATION_INTERVAL"),
			true,
			Checker::P_UINT_NOT_NULL,
			blmsg("TRANS_PASSWORD_EXPIRATION_INTERVAL_DESC")
		);
		$input_pass_expiration_interval->offValue = "0";
		$input_pass_expiration_interval->optionTitle = blmsg("TRANS_NEVER_EXPIRE_PASSWORD");
		$input_pass_expiration_interval->value = $this->getValue("security_PasswordExpirationInterval");
		$general_panel->addInputControl($input_pass_expiration_interval);

		// Number of the login failure attempts
		$input_login_failure_attempts = UIFactory()->InputOptionalString(
			"security_LoginFailureAttempts",
			blmsg("TRANS_LOGIN_FAILURE_ATTEMPTS"),
			true,
			Checker::P_UINT_NOT_NULL,
			blmsg("TRANS_LOGIN_FAILURE_ATTEMPTS_DESC")
		);
		$input_login_failure_attempts->offValue = "0";
		$input_login_failure_attempts->optionTitle = blmsg("TRANS_TURN_OFF_LOGIN_ATTEMPTS");
		$input_login_failure_attempts->value = $this->getValue("security_LoginFailureAttempts");
		$general_panel->addInputControl($input_login_failure_attempts);

		// Locking interval in minutes
		$input_login_locking_interval = UIFactory()->InputString(
			"security_LoginLockingInterval",
			blmsg("TRANS_LOGIN_LOCKING_INTERVAL"),
			true,
			Checker::P_UINT_GREATER_OR_EQUAL_ONE,
			blmsg("TRANS_LOGIN_LOCKING_INTERVAL_DESC")
		);
		$input_login_locking_interval->value = $this->getValue("security_LoginLockingInterval");
		$general_panel->addInputControl($input_login_locking_interval);

		// Notify Admin about user's account was locked.
		$input_notify_admin_about_locking = UIFactory()->InputCheckBox(
			"security_NotifyAdminAboutAccountLocking",
			blmsg("TRANS_NOTIFY_ADMIN_ABOUT_ACCOUNT_WAS_LOCKED"),
			false,
			NULL,
			blmsg("TRANS_NOTIFY_ADMIN_ABOUT_ACCOUNT_WAS_LOCKED_DESC")
		);
		$input_notify_admin_about_locking->value = $this->getValue("security_NotifyAdminAboutAccountLocking");
		$general_panel->addInputControl($input_notify_admin_about_locking);

		// Notify the abused account about he was locked.
		$input_notify_abused_account_about_locking = UIFactory()->InputCheckBox(
			"security_NotifyAbusedAccountAboutLocking",
			blmsg("TRANS_NOTIFY_ABUSED_ACCOUNT_ABOUT_ACCOUNT_WAS_LOCKED"),
			false,
			NULL,
			blmsg("TRANS_NOTIFY_ABUSED_ACCOUNT_ABOUT_ACCOUNT_WAS_LOCKED_DESC")
		);
		$input_notify_abused_account_about_locking->value = $this->getValue("security_NotifyAbusedAccountAboutLocking");
		$general_panel->addInputControl($input_notify_abused_account_about_locking);

		// Allow IP address changes during one session
		$inputDoNotCheckIp = UIFactory()->InputCheckBox(
			"session_doNotCheckIp",
			blmsg("TRANS_SESSION_DO_NOT_CHECK_IP_OPTION"),
			false,
			null,
			blmsg("TRANS_SESSION_DO_NOT_CHECK_IP_OPTION_DESC")
		);
		$inputDoNotCheckIp->value = $this->getValue("session_doNotCheckIp");
		$general_panel->addInputControl($inputDoNotCheckIp);

		$this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;
	}

	public function processInput()
	{
		$passwordHistorySavingOldValue = Settings()->security_PasswordHistorySaving;

		foreach ($this->params as $param_name) {
			Settings()->$param_name = $this->getValue($param_name);
		}

		Service()->UserPasswordHistoryManager->refreshPasswordsHistory($passwordHistorySavingOldValue);
		billing_put_info(blmsg("TRANS_UPDATE_AUTH_SECURITY_SETTINGS_SUCCESS_MSG"));
		return true;
	}

	public function getValue($param_name, $def_value = NULL)
	{
		$value_from_db = Settings()->$param_name;
		$def_value = is_null($value_from_db) ? $def_value : $value_from_db;

		return parent::getValue($param_name, $def_value);
	}

	private $form;
	private $params = array(
        "security_UsernameMinimalLength",
        "security_PasswordAlphanumericEnable",
        "security_PasswordMinimalLength",
		"security_PasswordHistorySaving",
		"security_PasswordExpirationInterval",
		"security_LoginFailureAttempts",
		"security_LoginLockingInterval",
		"security_NotifyAdminAboutAccountLocking",
		"security_NotifyAbusedAccountAboutLocking",
		"session_doNotCheckIp",
	);
}

class EditAuthSettings extends MB_Action
{
	public function execute()
	{
		parent::execute();
		$controller  = UIFactory()->PageController(admin_action_url("configs", "system", "show"));
		$controller->addPage(new AuthSettingsPage($controller));
		$controller->processRequest();

		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}
