<?php

require_once("lib-action/action.php");
require_once("lib-mbapi/mbapi.php");
require_once("lib-tk/include/xml/xml.php");

class CheckServerLoad extends MB_Action
{
	public $params = array("configGroupID");

	public function execute()
	{
		parent::execute();

		$serverGroupId = (int)$_REQUEST["serverGroupID"];
		$controlPanelId = (int)$_REQUEST["configGroupID"];

		$serverLoadData = array();

		try {
			if(!$serverGroupId) {
				throw new Exception("No ServerGroup ID given.");
			}
			if ($controlPanelId) {
				$controlPanel = Factory()->ControlPanel($controlPanelId);
				$serverLoadData[$controlPanelId] = $this->_getServerLoadData($controlPanel);
			} else {
				$serverGroup = Factory()->PanelGroup($serverGroupId);
				foreach ($serverGroup->ControlPanelList() as $controlPanel) {
					$serverLoadData[$controlPanel->id] = $this->_getServerLoadData($controlPanel);
				}
			}

			$smartyVars = array();
			$smartyVars["panelCount"] = count($serverLoadData);
			$smartyVars["serverLoadData"] = $serverLoadData;

			$this->addTemplate("widgets/checkServerLoad", $smartyVars);
		} catch (ProductException $exception) {
			$this->addOperationErrors($exception);
			$this->addErrorTemplates();
		}

		return $this->hasExecutedSuccessfully();
	}

	/**
	 * Retrieve server load data of Panel,
	 * corresponding given ControlPanel
	 *
	 * @param	ControlPanel	$controlPanel
	 */
	private function _getServerLoadData(ControlPanel $controlPanel)
	{
		require_once('ModuleTool.php');
		$panel = ModuleTool::getModuleInstance(
			"panel",
			$controlPanel->module_name,
			array(
				"subCommand" => "panelDummy",
				"configGroupID" => $controlPanel->id
			)
		);
		$serverLoadData = array();
		if($panel->capabilities[PANEL_GETSERVERLOAD]) {
			$panel->panelGetServerLoad();
			$serverLoadData = $panel->arrayData;
		} else {
			if($panel->capabilities[PANEL_GETTESTS]){
				$panel->panelGetTest();
				$serverLoadData = $panel->arrayData;
			} else {
				$serverLoadData = "not capable";
			}
		}
		return $serverLoadData;
	}
}