<?php

require_once("lib-action/action.php");
require_once("lib-mbapi/mbapi.php");
require_once("lib-mbapi/include/modules/module.php");

class ShowServerGroupsReport extends MB_Action
{
	public $params = array("serverGroupID", "serverGroupName");
	public $sortColumns = array("serverGroupID", "serverGroupName");

	public function execute()
	{
		parent::execute();

		$activePanelsOnly = (int) $_REQUEST["activePanelsOnly"];
		if($activePanelsOnly) {
			$query = $this->getMBAPIQuery("GetServerGroups", "", array("getPanels" => 1, "moduleEnabled" => 1));
		} else {
			$query = $this->getMBAPIQuery("GetServerGroups", "", array("getPanels" => 1));
		}
		$data = dispatchMBAPI($query, 1);

		$results =& $data["mbapi"][0]["results"][0];

		//Get the server names
		$query = $this->getMBAPIQuery("GetConfigs", "", array("configParamName" => "moduleDisplayName", "getFormAttributes" => 1));
		$data = dispatchMBAPI($query, 1);

		if($data["mbapi"][0]["header"][0]["numResults"][0]>0) {

			$serverNames = collapseArrayElements($data["mbapi"][0]["results"][0]);

			if(is_array($serverNames["configs"]["config"][0])) {
				foreach($serverNames["configs"]["config"] as $key => $value) {
					//Check to see if this one is active
					if($activePanelsOnly) {
						$moduleEnabled = NULL;
						$query = $this->getMBAPIQuery("GetConfigs", "", array("configGroupID" => $value["configGroupID"], "configParamName" => "moduleEnabled"));
						$data = correctArray(dispatchMBAPI($query, 1));
						if((array) $data["mbapi"][0]["header"][0]["success"]) {
							$moduleEnabled = (int) $data["mbapi"][0]["results"][0]["configs"][0]["configParamValue"];
						}

						if($moduleEnabled) {
							$temp[$value["configGroupID"]] = $value;
						}
					}
				}
				$serverNames = $temp;
				$temp = NULL;
			} else {
				$serverNames[$serverNames["configs"]["config"]["configGroupID"]] = $serverNames["configs"]["config"];
			}
		}

		//Get the max accounts for each panel
		$query = $this->getMBAPIQuery("GetConfigs", "", array("configParamName" => "panelMaxAccounts", "getFormAttributes" => 1));
		$data = dispatchMBAPI($query, 1);
        
        if($data["mbapi"][0]["header"][0]["numResults"][0]>0) {
			$maxAccounts = correctArray($data["mbapi"][0]["results"][0]["configs"][0]["config"]);
            foreach($maxAccounts as $value) {
                    $serverNames[$value["configGroupID"]]["panelMaxAccounts"] = (int) $value["configParamValue"];
            }		
        }

		//Get the current number of accounts per panel
		$query = $this->getMBAPIQuery("GetConfigs", "", array("configParamName" => "panelCurrentAccounts", "getFormAttributes" => 1));
		$data = dispatchMBAPI($query, 1);

		if($data["mbapi"][0]["header"][0]["numResults"][0]>0) {
            $maxAccounts = correctArray($data["mbapi"][0]["results"][0]["configs"][0]["config"]);            
			foreach($maxAccounts as $value) {
				$packageCountByProductName = NULL;
				if((int) $value["configGroupID"]) {
					$d = new DataSource(
						":packageAttributes",
						array(
							"configGroupID" =>  $value["configGroupID"],
							"aggregate" => array("function" => "count"),
							"groupBy" => array("column" => "packageID"),
						)
					);
					$packageIDs = $d->getKeysFor(new DataSource("packages"));
					$d = new DataSource(
						":packages",
						array(
							"packageID" => $packageIDs,
							"aggregate" => array("function" => "count"),
							"groupBy" => array("column" => "productID"),
							"packageStatus" => "none",
						)
					);
					$t =& $d->getTable();
					$t->setKey("productID");
					$t->setName("packageID.count");
					$productIDs = $t->getKeys();
					$packageCountByProductID = $t->getOptions();
					$productNames = optionsFor(":products", array("productID" => $productIDs, "productActive" => TRUE));
					$packageCountByProductName = array();
					foreach($productNames as $productID => $productName) {
						$packageCountByProductName[$productName] = $packageCountByProductID[$productID];
					}
				}
				$serverNames[$value["configGroupID"]]["panelCurrentAccounts"] = (int) $value["configParamValue"];
				if($packageCountByProductName) {
					$serverNames[$value["configGroupID"]]["panelCurrentPackages"] = $packageCountByProductName;
				}
			}
		}

		if($this->addMBAPIErrors($data) === 0) {
			// Get list of files in modules/panel to build panel list
			// Only if a Panel isn't associated yet, otherwise, must keep the same Panel.
			$panels = getAvailableModules("panel");
			asort($panels);
			foreach ($panels as $panelClass => $panelFile) {
				require_once mb_path("/lib-mbapi/include/modules/panel/".secureInput($panelFile));
				$moduleInfo = call_user_func(array($panelClass, "getModuleInfo"));
				$p = new $panelClass(array("subCommand" => "getCapabilities"));;
				$capabilities = $p->capabilities;

				//The smarty var name panels will not work
				$this->smartyVars["controlPanels"][$panelClass] = $moduleInfo["displayName"];
				$this->smartyVars["capabilities"][$panelClass] = $capabilities;

			}

			$this->smartyVars["paginationLinks"] = $results["paginationLinks"][0];
			$this->smartyVars = collapseArrayElements(protectArray($this->smartyVars));
			// collapse the outer array first and properly format the sub array if 1 item or more than 1
			if(is_array($results["serverGroups"][0]["serverGroup"])) {
				foreach($results["serverGroups"][0]["serverGroup"] as $key => $value) {
					$panels = $value["panels"][0]["panel"];
					$configGroups[$key] = collapseArrayElements($value);
					if(count($panels)>1) {
						$value = collapseArrayElements(arrayParentToChild($panels,"panels"));
						$configGroups[$key]["panels"] = $value;
						$configGroups[$key]["panelType"] = $value[0]["configGroupTransKey"];
						$configGroups[$key]["canGetServerLoad"] = $this->smartyVars["canGetServerLoad"][strtolower($value[0]["configGroupTransKey"])];

					} else {
						$panels = collapseArrayElements($panels);
						unset($configGroups[$key]["panels"]);
						if($panels) {
							$configGroups[$key]["panels"][0] = $panels;
							$configGroups[$key]["panelType"] = $panels["configGroupTransKey"];
							$configGroups[$key]["capabilities"] = $this->smartyVars["capabilities"][strtolower($panels["configGroupTransKey"])];
						}
					}
				}
			}

			//Drop inactive panels
			foreach($configGroups as $key => $value) {
				if(is_array($value["panels"])) {
					foreach($value["panels"] as $id => $v) {
						if(!$serverNames[$v["configGroupID"]]["configParamID"]) {
							unset($configGroups[$key]["panels"][$id]);
						}
					}
				}
			}

			if(count($serverNames)>1) {
				$this->smartyVars["serverNames"] = $serverNames;
			} else {
				$this->smartyVars["serverNames"][0] = $serverNames;
			}

			$this->smartyVars["serverGroups"] = $configGroups;
			$this->smartyVars["activePanelsOnly"] = $activePanelsOnly;


			$this->mergeSystemNavData();
			$this->addTemplate("servers/ShowServerGroupsReport", $this->smartyVars, 1);
		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}