<?php

require_once("lib-action/action.php");

use Billing\Checker;

class AddIPLogResponse extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$srcMinIp = bgpc("ip");
		$srcMaxIp = bgpc("ipMax");
		$minIP = trim($srcMinIp);
		$maxIP =  trim($srcMaxIp);
		$configGroupID = bgpc('configGroupID');

		if(!Checker::ipAddress($minIP)) {
			getCurrentSession()->Storage->errorMessages[] = blmsg("TRANS_IP_RANGE_IP_WRONG_FORMAT_MSG", array("IP" => htmlspecialchars($minIP)));
			redirect("servers.php", "AddIPLog", array(
				'configGroupID' => $configGroupID,
				'minIp' => $srcMinIp,
				'maxIp' => $srcMaxIp
			));
		}

		if("" != $maxIP) {
			if(!Checker::ipAddress($maxIP)) {
				getCurrentSession()->Storage->errorMessages[] = blmsg("TRANS_IP_RANGE_IP_WRONG_FORMAT_MSG", array("IP" => htmlspecialchars($maxIP)));
				redirect("servers.php", "AddIPLog", array(
					'configGroupID' => $configGroupID,
					'minIp' => $srcMinIp,
					'maxIp' => $srcMaxIp
				));
			} else if(!Service()->IpAddressManager->hasSameVersion($minIP, $maxIP)) {
				getCurrentSession()->Storage->errorMessages[] = blmsg("TRANS_IP_RANGE_DIFFERENT_VERSIONS_MSG", array("MIN_IP" => htmlspecialchars($minIP), "MAX_IP" => htmlspecialchars($maxIP)));
				redirect("servers.php", "AddIPLog", array(
					'configGroupID' => $configGroupID,
					'minIp' => $srcMinIp,
					'maxIp' => $srcMaxIp
				));
			}
		} else {
			$maxIP = $minIP;
		}

		$range = Service()->IpAddressManager->getRange($minIP, $maxIP);
		$isWasAdded = false;
		$isWasSkipped = false;
		foreach($range as $ip) {
			if (Factory()->IPLogList(array('ip' => $ip))) {
				$isWasSkipped = true;
				continue;
			}
			$ipLog = Factory()->IPLog();
			$ipLog->ip = $ip;
			$ipLog->config_group_id =  $configGroupID;
			$ipLog->ip_version = Service()->IpAddressManager->getVersion($ip);
			$ipLog->update();
			$isWasAdded = true;
		}
		if (!$isWasAdded) {
			getCurrentSession()->Storage->errorMessages[] = blmsg("TRANS_IP_RANGE_IP_DUPLICATED_MSG");
			redirect("servers.php", "AddIPLog", array(
				'configGroupID' => $configGroupID,
				'minIp' => $srcMinIp,
				'maxIp' => $srcMaxIp
			));
		}
		if ($isWasSkipped) {
			getCurrentSession()->Storage->warningMessages[] = blmsg("TRANS_IP_RANGE_IP_SKIPPED_MSG");
		} else {
			getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_IP_RANGE_IP_ADDED_MSG");
		}
		redirect("servers.php", "ShowIPLog", array("configGroupID" => $configGroupID));

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}