<?php

use Billing\Manager\CryptManager;
use Billing\Manager\SecurityManager;

/**
 * modernbill-admin specific implementation of operations required by ProcessSSOAPIRequest
 */
class AdminSSOAPI
{
    /**
     * Check admin's credentials.
     *
     * @param string $name User name
     * @param string $password User's password.
     * @return String User name identifier (login name).
     */
	public function checkCredentials($name, $password)
	{
       return !is_null($adminId = $this->getAdminID($name, $password)) ? $adminId : NULL;

    }

    /**
     * Check audit for admin
     *
     * @param string $name User name
     * @param string $loginAttemptState state of current login attempt
     * @return string status of audit.
     *
     *
     */
	public function checkAudit($name, $loginAttemptState)
	{
        $ip_address = bgpc("request_origin");
        require_once("lib-tk/include/sso/SSOSettings.php");
        $has_true_creds = (SecurityManager::LOGIN_ATTEMPT_STATE_SUCCESS == $loginAttemptState) ? true : false;
        $admin = reset((Factory()->AdminList( array('admin_username' => $name))));
        if ($admin) {
            Service()->SecurityManager->registerAdminAccess($name, $ip_address, $has_true_creds);
            return Service()->SecurityManager->isAdminAccessAllowed($name, $ip_address) ? SecurityManager::AUDIT_STATE_SUCCESS : SecurityManager::AUDIT_STATE_FAILED;
        }
        $contact = reset((Factory()->ContactList( array('contact_username' => $name))));
        if ($contact) {
            $customer = $contact->Customer;
            if ($customer->isReseller()) {
                Service()->SecurityManager->registerAdminAccess($name, $ip_address, $has_true_creds);
                return Service()->SecurityManager->isAdminAccessAllowed($name, $ip_address) ? SecurityManager::AUDIT_STATE_SUCCESS : SecurityManager::AUDIT_STATE_FAILED;
            } else {
                Service()->SecurityManager->registerContactAccess($name, $ip_address, $has_true_creds);
                return Service()->SecurityManager->isContactAccessAllowed($name, $ip_address) ? SecurityManager::AUDIT_STATE_SUCCESS : SecurityManager::AUDIT_STATE_FAILED;
            }
        }
        return SecurityManager::AUDIT_STATE_SUCCESS;
    }

    /**
     * Close session by admin's user name.
     *
     * @param string $userName
     */
	public function closeSessionByName($userName)
	{
        $qh = new QueryMaster('select');
        $qh->addTables('admins');
        $qh->addColumns('admin_id');
        $qh->addAndClause('admin_username', '=', $userName);
        foreach ($qh->getResults() as $row) {
            $adminId = $row['admin_id'];

            $h = new QueryMaster('update');
            $h->addTables('sessions');
            $h->addParam('session_active', 0, 'i');
            $h->addAndClause('user_id', '=', $adminId, 'i');
            $h->addAndClause('user_type', '=', 2, 'i');
            $h->execute();
        }
    }

	private function getAdminID($name, $password)
    {
        try	{
            $admin = Factory()->Admin($name, 'admin_username');
        } catch (ProductException $e) {
            if ($e->getCode() == ProductException::E_OBJECT_NOT_FOUND)
                return NULL;

            throw $e;
        }

		if (Service()->CryptManager->checkPassword($password, $admin->admin_temp_password, CryptManager::TYPE_CRYPT)) {
            $admin->plainPassword = $password;
			$admin->admin_temp_password = NULL;
			Service()->AdminManager->update($admin);
			return $admin->id;
		}

		if (Service()->CryptManager->checkPassword($password, $admin->admin_password, $admin->admin_password_type)) {
			if ("" != $admin->admin_temp_password) {
				$admin->admin_temp_password = NULL;
				Service()->AdminManager->update($admin);
			}
			return $admin->id;
		}
		return NULL;
	}
}

