<?php

use Billing\Object\Plan;
use Billing\Manager\SupplementaryPlanGroupManager;

class Supplementary_IndexController extends Billing_Zend_Controller_Action_AdminController
{
    public function attachAction()
    {
        $backUrl = $this->view->adminActionUrl("products", "plan-overview", "index", array("id" => $this->getRequest()->getParam('planId')));
        $plan = Factory()->Plan($this->getRequest()->getParam('planId'));
        $this->_checkVendor($plan);
        $this->view->placeholder('pagePath')->set(array(
            'TRANS_PLAN_OVERVIEW_TITLE' => $this->_translator->translate('TRANS_PLAN_OVERVIEW_TITLE', array('NAME' => billing_safetyhtml($plan->product_name)))
        ));
        $form = new Billing_Zend_Form_Final_Supplementary_Attach(array('context' =>
            array( 'plan' => $plan, 'cancelLink' => $backUrl )
        ));
        if ($this->_request->isPost()) {
            try {
                if ($form->isValid($this->_request->getPost())) {
                    $form->processForm();
                    $this->_redirectWithMessage($backUrl, $this->_translator->translate('TRANS_SUPPLEMENTARIES_SUCCESSFULLY_ATTACHED'));
                }
            } catch (ProductException $e) {
                $this->_status->addError($e->getMessage());
            }
        }
        $this->view->form = $form;
    }

    public function editAction()
    {
        $supplementaryPlanGroup = Factory()->SupplementaryPlanGroup($this->getRequest()->getParam('id'));
        $plan = $supplementaryPlanGroup->Plan;
        $this->_checkVendor($plan);
        $backUrl = $this->view->adminActionUrl("products", "plan-overview", "index", array("id" => $plan->id));

        $this->view->placeholder('pagePath')->set(array(
            'TRANS_PLAN_OVERVIEW_TITLE' => $this->_translator->translate('TRANS_PLAN_OVERVIEW_TITLE', array('NAME' => billing_safetyhtml($plan->product_name)))
        ));
        $this->view->placeholder('pageTitle')->set($this->_translator->translate('TRANS_PLAN_SUPPLEMENTARY_PLAN_GROUP_EDIT', array("NAME" => $supplementaryPlanGroup->name)));
        $form = new Billing_Zend_Form_Final_Supplementary_Edit(array('context' =>
            array( 'supplementaryPlanGroup' => $supplementaryPlanGroup, 'cancelLink' => $backUrl )
        ));
        if ($this->_request->isPost()) {
            try {
                if ($form->isValid($this->_request->getPost())) {
                    $form->processForm();
                    $this->_redirectWithMessage($backUrl, $this->_translator->translate('TRANS_SUPPLEMENTARY_PLAN_GROUP_SUCCESSFULLY_CHANGED'));
                }
            } catch (ProductException $e) {
                $this->_status->addError($e->getMessage());
            }
        }
        $this->view->form = $form;

    }

    public function detachAction()
    {
        $backUrl = $this->view->adminActionUrl("products", "plan-overview", "index", array("id" => $this->getRequest()->getParam('planId')));
        $plan = Factory()->Plan($this->getRequest()->getParam('planId'));
        $this->_checkVendor($plan);
        $groupIds = $planIds = array();
        foreach($this->getRequest()->getParam('id') as $value) {
            if (false !== strpos($value, SupplementaryPlanGroupManager::SUPPLEMENTARY_GROUP."-")) {
                $groupId = str_replace( SupplementaryPlanGroupManager::SUPPLEMENTARY_GROUP."-", "", $value);
                $this->_checkVendor(Factory()->SupplementaryPlanGroup($groupId)->Plan);
                $groupIds[] = $groupId;
            }
            if (false !== strpos($value, SupplementaryPlanGroupManager::SUPPLEMENTARY_PLAN."-")) {
                $planId = str_replace( SupplementaryPlanGroupManager::SUPPLEMENTARY_PLAN."-", "", $value);
                $this->_checkVendor(Factory()->Plan($planId));
                $planIds[] = $planId;
            }
        }
        if (!getCurrentSession()->isValidToken()) {
            $this->_redirectWithMessage($backUrl, $this->_translator->translate('TRANS_ERROR_TOKEN'), Billing_Zend_Controller_Action_Status::STATUS_ERROR);
        }
        Service()->PlanManager->detachSupplementaries($plan, $groupIds, $planIds);
        $this->_redirectWithMessage($backUrl, $this->_translator->translate('TRANS_SUPPLEMENTARIES_SUCCESSFULLY_DETACHED'));
    }

    public function sortAction()
    {
        $planId = $this->getRequest()->getParam('planId');
        $groupId = $this->getRequest()->getParam('groupId');
        if (!is_null($planId) && !is_null($groupId)) {

            $supplementaryPlanGroup = Factory()->SupplementaryPlanGroup($groupId);
            $this->_checkVendor($supplementaryPlanGroup->Plan);
            Service()->SupplementaryPlanGroupManager->sortPlan(Factory()->Plan($planId), $supplementaryPlanGroup, $this->getRequest()->getParam('direct'));
            $this->_redirectWithMessage($this->view->adminActionUrl("products", "plan-overview", "index", array("id" => $supplementaryPlanGroup->product_id)), $this->_translator->translate('TRANS_SUPPLEMENTARY_PLANS_SUCCESSFULLY_SORTED'));

        } elseif (is_null($planId) && !is_null($groupId)) {

            $supplementaryPlanGroup = Factory()->SupplementaryPlanGroup($groupId);
            $this->_checkVendor($supplementaryPlanGroup->Plan);
            Service()->SupplementaryPlanGroupManager->sortGroup($supplementaryPlanGroup, $this->getRequest()->getParam('direct'));
            $this->_redirectWithMessage($this->view->adminActionUrl("products", "plan-overview", "index", array("id" => $supplementaryPlanGroup->product_id)), $this->_translator->translate('TRANS_SUPPLEMENTARY_GROUPS_SUCCESSFULLY_SORTED'));
        }
        $this->_redirectWithMessage($this->view->actionUrl("products", "ShowProducts"), $this->_translator->translate('TRANS_SUPPLEMENTARIES_SORT_MISSING_REQUIRED_PARAM_ERROR'), Billing_Zend_Controller_Action_Status::STATUS_ERROR);
    }

    public function copyAction()
    {
        $backUrl = $this->view->adminActionUrl("products", "plan-overview", "index", array("id" => $this->getRequest()->getParam('planId')));
        $plan = Factory()->Plan($this->getRequest()->getParam('planId'));
        $this->_checkVendor($plan);
        $this->view->placeholder('pagePath')->set(array(
            'TRANS_PLAN_OVERVIEW_TITLE' => $this->_translator->translate('TRANS_PLAN_OVERVIEW_TITLE', array('NAME' => billing_safetyhtml($plan->product_name)))
        ));
        $form = new Billing_Zend_Form_Final_Supplementary_Copy(array('context' =>
            array( 'plan' => $plan, 'cancelLink' => $backUrl )
        ));
        if ($this->_request->isPost()) {
            try {
                if ($form->isValid($this->_request->getPost())) {
                    $form->processForm();
                    $this->_redirectWithMessage($backUrl, $this->_translator->translate('TRANS_SUPPLEMENTARIES_SUCCESSFULLY_ATTACHED'));
                }
            } catch (ProductException $e) {
                $this->_status->addError($e->getMessage());
            }
        }
        $this->view->form = $form;
    }

    public function getAction()
    {
        if($this->_request->isXmlHttpRequest()) {
            $this->_helper->layout->setLayout('ajax');
        }
        $selectedPlan = Factory()->Plan($this->getRequest()->getParam('selectedPlanId'));
        $currentPlan = Factory()->Plan($this->getRequest()->getParam('planId'));
        if (!getCurrentSession()->checkVendor($selectedPlan) || !getCurrentSession()->checkVendor($currentPlan)) {
            header('HTTP/1.0 403 Forbidden');
            exit;
        }

        // generate list of purchase links for this plan
        $supplementaries = array();

        $currentPlanIds[] = $currentPlan->id;
        foreach($currentPlan->SupplementaryPlanGroupList() as $group) {
            foreach ($group->SupplementaryPlans as $key => $value) {
                $currentPlanIds[] = $key;
            }
        }

        foreach($selectedPlan->SupplementaryPlanGroupList() as $group) {


            $supplementaryPlans = array();
            $hasSupplementaryPlans = false;
            foreach ($group->SupplementaryPlans as $key => $value) {
                if (!in_array($key, $currentPlanIds)) {
                    $hasSupplementaryPlans = true;
                }
                $supplementaryPlans[] = array(
                    "type" => SupplementaryPlanGroupManager::SUPPLEMENTARY_PLAN,
                    "value" => array(
                        "id" => $key,
                        "name" => Factory()->Plan($key)->product_name,
                        "groupId" => $group->id,
                    ),
                );
            }
            if (!$hasSupplementaryPlans) {
                continue;
            }
            $supplementaries[] = array(
                "type" => SupplementaryPlanGroupManager::SUPPLEMENTARY_GROUP,
                "value" => array(
                    "id" => $group->id,
                    "name" => $group->name,
                ),
            );
            $supplementaries = array_merge($supplementaries, $supplementaryPlans);


        }
        $this->view->supplementaries = $supplementaries;
        $this->view->currentPlanIds = $currentPlanIds;
    }
    
    public function editPropertiesAction() {
        $supplementaryPlanGroup = Factory()->SupplementaryPlanGroup($this->getRequest()->getParam('groupId'));
        $plan = $supplementaryPlanGroup->Plan;
        $this->_checkVendor($plan);
        $backUrl = $this->view->adminActionUrl("products", "plan-overview", "index", array("id" => $plan->id));
        
        $supplementaryPlanId = $this->getRequest()->getParam('planId');
        $supplementaryPlan = Factory()->Plan($supplementaryPlanId);
        $this->_checkVendor($supplementaryPlan);
        
        $supplementaryPlanProperties = $supplementaryPlanGroup->SupplementaryPlans[$supplementaryPlanId];
        $this->view->placeholder('pagePath')->set(array(
            'TRANS_PLAN_OVERVIEW_TITLE' => $this->_translator->translate('TRANS_PLAN_OVERVIEW_TITLE', array('NAME' => billing_safetyhtml($plan->product_name)))
        ));
        $this->view->placeholder('pageTitle')->set($this->_translator->translate('TRANS_PLAN_SUPPLEMENTARY_PLAN_EDIT_PROPERTIES', 
                array("NAME" => $supplementaryPlan->product_name)));
        
        $form = new Billing_Zend_Form_Final_Supplementary_EditProperties(array('context' =>
            array( 'supplementaryPlanProperties' => $supplementaryPlanProperties, 'cancelLink' => $backUrl )
        ));
        if ($this->_request->isPost() && $form->isValid($this->_request->getPost())) {
            try {
                $form->processForm();
                $this->_redirectWithMessage($backUrl, $this->_translator->translate('TRANS_PLAN_SUPPLEMENTARY_PLAN_EDIT_PROPERTIES_SUCCESS'));
            } catch (ProductException $e) {
                $this->_status->addError($e->getMessage());
            }
        }
        $this->view->form = $form;
    }

    private function _checkVendor(Plan $plan)
    {
        if (!getCurrentSession()->checkVendor($plan)) {
            $this->_redirectWithMessage($this->view->actionUrl("products", "ShowProducts"), $this->_translator->translate('TRANS_ACCESS_DENIED'), Billing_Zend_Controller_Action_Status::STATUS_ERROR);
        }
    }
}