<?php
use \Billing\TaskManager\Client as TaskManagerClient;
use \Billing\TaskManager\Struct\Task as Task;

class Tasks_TaskController extends Billing_Zend_Controller_Action_AdminController
{
    public function init()
    {
        parent::init();
        $this->_backUrl = $this->_getBackUrl();
        $this->_task = $this->_getTaskById();
    }

    public function runallAction()
    {
        $backUrl = ('CheckRunPaymentQueue' == $this->getRequest()->getParam('redirectAction')) ? $this->view->ActionUrl('paymentqueue', 'CheckRunPaymentQueue') : $this->_backUrl;
        try {
            TaskManagerClient::runAllTask();
        } catch (Exception $e) {
            $this->_redirectWithMessage($backUrl, blmsg("TRANS_TASK_MANAGER_EXCEPTION", array('EXCEPTION' => $e->getMessage())), "error");
        }

        $this->_redirectWithMessage($backUrl, blmsg("TRANS_TASK_RUN_ALL_TASK_SUCCESSFULLY"), "info");
    }

    public function runAction()
    {
        $this->_isTaskExists();

        try {
            TaskManagerClient::runTask($this->_task->id);
        } catch (Exception $e) {
            $this->_redirectWithMessage($this->_backUrl, blmsg("TRANS_TASK_MANAGER_EXCEPTION", array('EXCEPTION' => $e->getMessage())), "error");
        }

        $this->_redirectWithMessage($this->_backUrl, blmsg("TRANS_TASK_RUN_TASK_SUCCESSFULLY"), "info");
    }

    public function cancelAction()
    {
        $this->_isTaskExists();

        try {
            TaskManagerClient::cancelTasks(array('id' => $this->_task->id), Task::STATUS_CANCELLED, blmsg("TRANS_TASK_CANCELLED_BY_USER", array('USER' => Factory()->Admin(getCurrentSession()->user_id)->admin_username)));
        } catch (Exception $e) {
            $this->_redirectWithMessage($this->_backUrl, blmsg("TRANS_TASK_MANAGER_EXCEPTION", array('EXCEPTION' => $e->getMessage())), "error");
        }

        $this->_redirectWithMessage($this->_backUrl, blmsg("TRANS_TASK_CANCEL_TASK_SUCCESSFULLY"), "info");
    }

    public function showAction()
    {
        $this->_isTaskExists();
        $this->_addToolButtons();
        $this->view->placeholder('pageTitle')->set($this->_translator->translate('TRANS_TASK_PAGE_SHOW_TITLE', array("TASKNAME" => $this->_task->name)));

		$form = new Billing_Zend_Form_Final_TaskForm(
            array('context' =>
                array(
                    'task' => $this->_task,
                    'readonly' => true,
                    'cancelLink' => $this->_getBackUrl()
                )
            )
        );
        $this->view->form = $form;
    }

    public function editAction()
    {
        $this->_isTaskExists();
        $this->view->placeholder('pageTitle')->set($this->_translator->translate('TRANS_TASK_PAGE_EDIT_TITLE', array("TASKNAME" => $this->_task->name)));
        $backUrl = $this->view->AdminActionUrl('tasks', 'task', 'show', array('taskId' => (int)$this->_request->getParam('taskId')));
        $form = new Billing_Zend_Form_Final_TaskForm(
            array('context' =>
                array(
                    'task' => $this->_task,
                    'readonly' => false,
                    'cancelLink' => $backUrl
                )
            )
        );
		if ($this->_request->isPost()) {
			try {
				if ($form->isValid($this->_request->getPost())) {
                    $form->process();
                    $this->_redirectWithMessage($backUrl, $this->_translator->translate('TRANS_TASK_SAVED_SUCCESSFULLY'));
                }
			} catch (ProductException $e) {
    			$this->_status->addError($e->getMessage());
			}
		}
        $this->view->form = $form;
    }

    private function _getBackUrl()
    {
        if (1 == $this->_request->getParam('isPeriodicalTaskList') || ($this->_task && !is_null($this->_task->period))) {

            return $this->view->ActionUrl('tasks', 'TaskList', array('isPeriodicalTaskList' => 1));
        }

        return $this->view->ActionUrl('tasks', 'TaskList');
    }

    private function _addToolButtons()
    {
        if (!is_null($this->_task->period) || key_exists($this->_task->applicationStatus, Task::statusesAllowRunning())) {
            $this->view->toolButtons = array(
                'settings' => array(
                    'label' => 'TRANS_TASK_BUTTON_EDIT',
                    'class' => 'sb-edit',
                    'href' => $this->view->adminActionUrl('tasks', 'task', 'edit', array('taskId' => $this->_task->id))
                )
            );
        }
    }

    private function _getTaskById()
    {
        try {
            return reset(
                TaskManagerClient::getTaskList(
                    array(
                        'id' => (int)$this->_request->getParam('taskId'),
                        'attributes' => array('resellerId' => getCurrentSession()->getVendorId())
                    ),
                    array(
                        'attributes' => true,
                        'logs' => true
                    )
                )->tasks
            );
        } catch (Exception $e) {
            $this->_redirectWithMessage($this->_backUrl, blmsg("TRANS_TASK_MANAGER_EXCEPTION", array('EXCEPTION' => $e->getMessage())), "error");
        }
    }

    private function _isTaskExists()
    {
        if (false === $this->_task) {
            $this->_redirectWithMessage($this->_backUrl, blmsg("TRANS_TASK_NOT_EXISTS"), "error");
        }
    }

    private $_task = null;
}