<?php

require_once ("lib-action/action.php");

class ShowTodos extends MB_Action
{
	public function __construct()
    {
		parent::__construct();
		$this->params = array(
			"todoID",
			"productID",
			"todoTitle",
			"todoDesc",
			"userID",
			"todoStatusID",
			"todoDateDue",
			"todoDateCreated",
			"todoAssignedAdminID",
			"todoIsPublic"
        );
		$this->sortColumns = array(
			"todoStatusID",
			"todoTitle",
			"userID",
			"todoDateDue",
			"todoAssignedAdminID",
			"todoIsPublic
		");
	}

	public function execute()
    {
		parent::execute();

		$query = $this->getMBAPIQuery("GetTodos", "",
				array("todoDateDueGreatest" => time(),
					"getAdminData" => 1,
					"getAssignedData" => 1));
		$data = dispatchMBAPI($query, 1);
		$this->handleGroup( "pastdue", $data );

		$this->smartyVars["rowCount"]["pastdue"] = $this->getMaxCount( "pastdue" );


		// get todo's that are a day old
		$query = $this->getMBAPIQuery("GetTodos", "",
				array("todoDateDueLeast" => time(),
				"todoDateDueGreatest" => time() + 86400,
				"getAdminData" => 1,
				"getAssignedData" => 1));
		$data = dispatchMBAPI($query, 1);
		$this->handleGroup( "near", $data );

		$this->smartyVars["rowCount"]["near"] = $this->getMaxCount( "near" );

		// get todo's for the next 6 days (1 week forward)
		$query = $this->getMBAPIQuery("GetTodos", "",
				array( "todoDateDueLeast" => time() + 86400,
				"todoDateDueGreatest" => time() + 604800,
				"getAdminData" => 1,
				"getAssignedData" => 1));
		$data = dispatchMBAPI($query, 1);
		$this->handleGroup("week", $data);

		$this->smartyVars["rowCount"]["week"] = $this->getMaxCount( "week" );

		// defs
		$this->smartyVars["todostatus"] = getStatuses("todo");
		$this->smartyVars["todostatusheaders"] = array(
            "pastdue" => blmsg('TRANS_DUE'),
            "near" => blmsg('TRANS_24HOURS'),
            "week" => blmsg('TRANS_1WEEK')
        );

		$data = arrayFor("admins", array("adminActive" => 1));

		if (is_array( $data ))
		foreach( $data as $admin ) {
			$this->smartyVars["adminsList"][ $admin["adminID"] ] = $admin["adminLastName"].", ".$admin["adminFirstName"];
		}

		if (!$this->_err) {
			$this->smartyVars = correctArray( $this->smartyVars );

			$this->addTemplate("todos/ShowTodos");
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

	public function handleGroup($group, &$data )
    {
		global $CONF;

		if ($this->addMBAPIErrors($data) === 0) {
			$results = correctArray($data["mbapi"][0]["results"][0]);

			if (isset($results["todos"][0]) && is_array($results["todos"][0])) {
				$this->smartyVars["todos"][$group] = array();

				if (is_array( $results["todos"] ))
				foreach($results["todos"] as $item) {
					$key=null;

					if ($item["todoAssignedAdminID"] == $CONF["user"]["id"]) {
						// my todo tasks
						$key = "mine";
					} elseif (($item["userID"] == $CONF["user"]["id"]) && ((int)$item["todoAssignedAdminID"] > 0)) {
						// tasks that I am the author of
						$key = "authored";
					} elseif (($item["todoIsPublic"] == 1) || ( ($item["userID"] == $CONF["user"]["id"]) && !$item["todoAssignedAdminID"]) ) {
						// public todo tasks
						$key = "public";
					}

					if (!is_null( $key )) {
						if (isset( $item["assigned"][0] ) && is_array( $item["assigned"][0] )) {
							$item["assigned"] = $item["assigned"][0]["assigned"];
						}
						$this->smartyVars["todos"][ $group ][ $key ][] = $item;
					}

				}
			}
		} else {
			$this->_err=1;
		}
		unset( $query, $data, $key, $item, $group );
		$this->smartyVars["paginationLinks"] = $results["paginationLinks"][0];
	}

	public function getMaxCount( $group )
    {
        if (!isset($this->smartyVars["todos"][$group])) {
            return 1;
        }
		$mine = isset($this->smartyVars["todos"][$group]["mine"]) ? count($this->smartyVars["todos"][$group]["mine"]) : 0;
		$auth = isset($this->smartyVars["todos"][$group]["authored"]) ? count($this->smartyVars["todos"][$group]["authored"]) : 0;
		$pub = isset($this->smartyVars["todos"][$group]["public"]) ? count($this->smartyVars["todos"][$group]["public"]) : 0;

		if ($mine > $auth && $mine > $pub)
			return $mine;

		if ($auth > $mine && $auth > $pub)
			return $auth;

		if (!notZero( $pub ))
			return 1;
		return $pub;
	}
}