<?php
require_once("lib-action/action.php");
require_once("UI/Page.php");

class EditWebsiteWidgetPage extends UIPage
{
	public function createControls()
	{
		$websiteWidget = Factory()->WebsiteWidget(bgpc("websiteWidgetID"));

		$inputPanel = UIFactory()->InputPanel();
		$this->addInputPanel($inputPanel);

		$websiteWidgetNameInput = UIFactory()->InputString("websiteWidgetName", blmsg("TRANS_WEBSITE_WIDGET_NAME_OPTION"), true);
		$websiteWidgetNameInput->value = $websiteWidget->website_widget_name;
		$inputPanel->addInputControl($websiteWidgetNameInput);
		$this->input_controls["websiteWidgetName"] = $websiteWidgetNameInput;

		$codeInput = UIFactory()->InputString("code", blmsg("TRANS_WEBSITE_WIDGET_CODE"), true);
		$codeInput->value = $websiteWidget->code;
		$inputPanel->addInputControl($codeInput);
		$this->input_controls["code"] = $codeInput;

		$continueShoppingUrlInput = UIFactory()->InputString("continue_shopping_url", blmsg("TRANS_WEBSITE_WIDGET_CONTINUE_SHOPPING_URL"));
		$continueShoppingUrlInput->value = $websiteWidget->continue_shopping_url;
		$continueShoppingUrlInput->hint = blmsg("TRANS_WEBSITE_WIDGET_CONTINUE_SHOPPING_URL_DESC");
		$inputPanel->addInputControl($continueShoppingUrlInput);
		$this->input_controls["continue_shopping_url"] = $continueShoppingUrlInput;

		$websiteWidgetTplInput = UIFactory()->InputDropDown("websiteWidgetTpl", blmsg("TRANS_WEBSITE_WIDGET_TEMPLATE_OPTION"), true);
		$websiteWidgetTplInput->value = $websiteWidget->website_widget_tpl;
		foreach(Service()->StoreManager->getWidgetTemplates() as $name => $title) {
			$websiteWidgetTplInput->addOption($name, $title);
		}
		$inputPanel->addInputControl($websiteWidgetTplInput);
		$this->input_controls["websiteWidgetTpl"] = $websiteWidgetTplInput;

		$productsList = Factory()->Store(bgpc("orderFormID"))->PlanList();
		if(count($productsList) > 0) {
			$productsInput = UIFactory()->InputDoubleListSelector("products-group", blmsg("TRANS_WEBSITE_WIDGET_PRODUCTS_OPTION"));
			foreach ($websiteWidget->planIDs as $planId) {
				$productsInput->value[$planId] = 1;
			}
			foreach($productsList as $product) {
				$productsInput->addOption($product->product_id, billing_safetyhtml($product->product_name));
			}
			$inputPanel->addInputControl($productsInput);
			$this->input_controls["products-group"] = $productsInput;
		} else {
			$this->addWarningMessage(UIFactory()->WarningMessage(blmsg("TRANS_NO_PRODUCTS_TIED_TO_ORDER_FORM")));
		}
		$this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;
	}

	public function processInput()
	{
		try {
			$code = $this->input_controls["code"]->value;
			$websiteWidgetID = bgpc("websiteWidgetID");
			foreach(Factory()->WebsiteWidgetList(array("code" => $code)) as $websiteWidget) {
				if($websiteWidget->website_widget_id != $websiteWidgetID) {
					$this->addErrorMessage(UIFactory()->ErrorMessage(blmsg("TRANS_WEBSITE_WIDGET_CODE_ALREADY_EXISTS_MSG", array("CODE" => $code))));
					return false;
				}
			}

			$websiteWidget = Factory()->WebsiteWidget($websiteWidgetID);
			$websiteWidget->website_widget_name = $this->input_controls["websiteWidgetName"]->value;
			$websiteWidget->website_widget_tpl = $this->input_controls["websiteWidgetTpl"]->value;
			$websiteWidget->order_form_id = (int)bgpc("orderFormID");
			$websiteWidget->code = $this->input_controls["code"]->value;
			$websiteWidget->continue_shopping_url = $this->input_controls["continue_shopping_url"]->value;

			$planIds = array();
			foreach((array)$this->input_controls["products-group"]->value as $planId => $planIsEnabled) {
				if ($planIsEnabled) {
					$planIds[] = $planId;
				}
			}
			$websiteWidget->planIDs = $planIds;

			$websiteWidget->update();

			getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_SETTINGS_SAVED_SUCCESSFULLY");
		} catch (ProductException $e) {
			$this->addErrorMessage(UIFactory()->ErrorMessage($e->getMessage()));
		}

		return !$this->hasWarnings() && !$this->hasErrors();
	}

	protected function getSubmitButtons()
	{
		return array(UIFactory()->SubmitButton("ok", blmsg("TRANS_SAVE"), UIPageController::CMD_OK, "", "", "action"));
	}
}

class EditWebsiteWidget extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$websiteWidgetID = bgpc("websiteWidgetID");
		if ( is_null($websiteWidgetID) ) {
			throw new Exception("Missing parameter 'websiteWidgetID'");
		}

		$websiteWidget = Factory()->WebsiteWidget($websiteWidgetID);
		$orderForm = Factory()->Store($websiteWidget->order_form_id);

		$orderFormPageURL = admin_action_url("orderforms", "store-overview", "index", array("id" => $orderForm->order_form_id));
		$controller  = UIFactory()->PageController($orderFormPageURL);
		$controller->addPage($edit_page = new EditWebsiteWidgetPage($controller));

		$edit_page->title = blmsg("TRANS_EDIT_WEBSITE_WIDGET_FOR_STORE", array("store_name" => billing_safetyhtml($orderForm->order_form_name)));
		$edit_page->parent_title = billing_safetyhtml($orderForm->order_form_name);
		$edit_page->parent_url = $orderFormPageURL;

		$controller->processRequest();
		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}