<?php

class Affiliate_IndexController extends Billing_Zend_Controller_Action_ClientController
{
	public function joinAction()
	{
		$backUrl = action_url('dashboard', 'ShowDashboard');
		$contact = Factory()->Contact(getCurrentSession()->user_id); 
        if ($contact->Customer->isReseller()) {
            $this->_redirectWithMessage($backUrl, blmsg('TRANS_AFFILIATE_NOT_FOR_RESELLER_ACTION_ERROR'), Billing_Zend_Controller_Action_Status::STATUS_ERROR);
        }
        
        if (!$contact->Customer->is_affiliate) {
            try {
                Service()->CustomerManager->joinToAffiliatePrograms($contact->Customer);
            } catch(ProductException $e) {
                $this->_redirectWithMessage($backUrl, $e->getMessage(), Billing_Zend_Controller_Action_Status::STATUS_ERROR);
            }
            $this->_redirectWithMessage($backUrl, blmsg('TRANS_AFFILIATE_PROGRAM_CUSTOMER_SUCCESSFULLY_JOINED'));
        }
        $this->_redirectWithMessage($backUrl, blmsg('TRANS_AFFILIATE_PROGRAM_CUSTOMER_ALREADY_JOINED'), Billing_Zend_Controller_Action_Status::STATUS_ERROR);
	}
	
	public function payoutAction()
	{
		$backUrl = action_url('affiliate', 'ShowAffiliatePrograms');
		$contact = Factory()->Contact(getCurrentSession()->user_id);
        if ($contact->Customer->isReseller()) {
            $this->_redirectWithMessage(action_url('dashboard', 'ShowDashboard'), blmsg('TRANS_AFFILIATE_NOT_FOR_RESELLER_ACTION_ERROR'), Billing_Zend_Controller_Action_Status::STATUS_ERROR);
        }
        
		$affiliatePrograms = $contact->Customer->getAffiliatePrograms(true);
		$ids = $this->getRequest()->getParam('id');
		if (is_null($ids)) {
			$ids = array();
			foreach ($affiliatePrograms as $affiliateProgram) {
				$ids[] = $affiliateProgram->id;
			}
		}
		if (empty($ids)) {
			$this->_redirectWithMessage($backUrl, blmsg('TRANS_AFFILIATE_PROGRAMS_REQUEST_PAYOUT_EMPTY_MSG'), Billing_Zend_Controller_Action_Status::STATUS_WARNING);
		}
		$allowPayoutIds = array();
		foreach ($affiliatePrograms as $affiliateProgram) {
			if (in_array($affiliateProgram->id, $ids)) {
				$allowPayoutIds[] = $affiliateProgram->id;
			}
		}
		if (!empty($allowPayoutIds)) {
			try {
				Service()->AffiliateProgramManager->requestPayout($contact->Customer->id, $allowPayoutIds);
			} catch(ProductException $e) {
				$this->_redirectWithMessage($backUrl, blmsg('TRANS_AFFILIATE_PROGRAMS_REQUEST_PAYOUT_ERROR_MSG'), Billing_Zend_Controller_Action_Status::STATUS_ERROR);
			}
			billing_put_info(blmsg('TRANS_AFFILIATE_PROGRAMS_REQUEST_PAYOUT_SUCCESS_MSG'));
		}
		$disallowPayoutIds = array_diff($ids, $allowPayoutIds);
		if (!empty($disallowPayoutIds)) {
			$skipped = array();
			foreach ($disallowPayoutIds as $id) {
				$skipped[] = Factory()->AffiliateProgram($id)->name;
			}
			billing_put_warning(blmsg("TRANS_AFFILIATE_PROGRAMS_REQUEST_PAYOUT_WARNING_MSG", array("PROGRAMS" => implode('<br>', $skipped))));
		}
		$this->_redirect($backUrl);
	}
}