<?php

require_once("lib-action/action.php");

class ShowDashboard extends MB_Action
{
	public function execute()
	{
        parent::execute();
		$contact = Factory()->Contact(getCurrentSession()->user_id);
        $customer = $contact->Customer;
        if ($customer->isReseller()) {
            redirect(action_url("dashboard", "ShowDashboard", array(), "admin"), null, array(), array(), "admin");
        }
		$clientID = $customer->id;
		$this->mergeClientData();
		
		// ---------------------------------------------------
		// Get DUE invoices ...
		// ---------------------------------------------------
		$this->smartyVars["invoices"] = array(); // init
		$params = array(
			"clientID" => $clientID,
			"getDueInvoicesOnly" => 1,
			"getInvoiceLineItemsData" => 0
		);
		require_once("lib-mbapi/mbapi.php");
		$query = getMBAPIQuery("GetInvoices", "",  $params, "ASC", "invoiceID", 0, 5);
		$data = dispatchMBAPI($query, 1);
		$results =& $data["mbapi"][0]["results"][0];
		if($this->addMBAPIErrors($data) === 0 && !empty($results["invoices"][0]["invoice"])) {
			$this->smartyVars["invoices"] = collapseArrayElements(protectArray($results["invoices"][0]["invoice"]));
		}

		// ---------------------------------------------------
		// Get ACTIVE packages ...
		// ---------------------------------------------------
		$suppressFeePackages = bgpc('showFees') ? 0 : 1;

		$this->smartyVars["packages"] = correctArray(grabResultsFromMBAPIArray(array(
			"mbapi" => array(
				"command" => "GetPackages",
				"params" => array(
					"clientID" => $clientID,
					"getBillablePackagesOnly" => 1,
					"getDomainData" => 1,
					"suppressFeePackages" => $suppressFeePackages,
					"getPackageAttributeData" => 1,
				),
			),
		)));

		foreach ($this->smartyVars["packages"] as &$package) {
			foreach ($package['packageAttributes'] as $property) {
				if ($property['packageAttributeName'] == Billing\Object\Subscription::ADDITIONAL_NAME) {
					$package['additionalName'] = $property['packageAttributeValue'];
					break;
				}
			}
		}
        unset($package);

		foreach ($this->smartyVars["packages"] as &$package) {
			foreach ($package['packageAttributes'] as $property) {
				if ($property['packageAttributeName'] == Billing\Object\Subscription::ADDITIONAL_NAME) {
					$package['additionalName'] = $property['packageAttributeValue'];
					break;
				}
			}
		}
        unset($package);

		$this->smartyVars["packageStatuses"] = getStatuses("package");
		require_once("lib-billing/include/config/internal/cycles.php");
		$this->smartyVars["cycles"] = getCyclesArray();
        $this->smartyVars["overuses"] = Service()->CustomerManager->getTotalOveruse(Factory()->Customer($clientID));

		$uir_packages = Factory()->SubscriptionList(array(
			"client_id" => $clientID,
			"package_status" => getAPIStatusID("package", "user_input_required"),
		));
		if (0 < count($uir_packages))
		{
			$messages = array(blmsg("TRANS_UIR_YOU_HAVE_PACKAGES_THAT_REQUIRE_ADDITIONAL_DATA"));
			foreach ($uir_packages as $package)
			{
				$domain = is_null($package->PrimaryDomain)
					? ""
					: " ({$package->PrimaryDomain->domain_sld}.{$package->PrimaryDomain->domain_tld})";

				$message = blmsg("TRANS_UIR_PACKAGE_NAME", array(
					"packageID" => $package->package_id,
					"packageName" => $package->package_name . $domain,
				));
				$message = "<a href=\"".action_url("packages", "ShowClientServiceInputForm", array("packageID" => $package->package_id))."\">{$message}</a>";
				$messages[] = $message;
			}
			$this->smartyVars["messages"] = UIFactory()->WarningMessage(join("<BR/>", $messages))->getHTML();
		}
		
		$this->smartyVars["isAffiliate"] = $contact->Customer->is_affiliate;
        $this->smartyVars["affiliatePrograms"] = $contact->Customer->getAffiliatePrograms();
        if ($contact->Customer->is_affiliate) {
			$affiliateStatistics = Service()->AffiliateProgramManager->getStatistics($contact->Customer->vendor_id, $contact->Customer->id);
			$affiliateBalance = array();
			foreach ($affiliateStatistics['commission'] as $currency => $value) {
				$affiliateBalance[$currency] = BillingLocale::getCurrency($value, $currency);
			}
			$this->smartyVars["affiliateBalance"] = $affiliateBalance;
			$affiliateBalanceAboveThreshold = array();
			foreach ($affiliateStatistics['commissionToPayout'] as $currency => $value) {
				$affiliateBalanceAboveThreshold[$currency] = (float)$value > 0 ? BillingLocale::getCurrency($value, $currency) : false;
			}
			$this->smartyVars["affiliateBalanceAboveThreshold"] = $affiliateBalanceAboveThreshold;
		} else {
			$this->smartyVars["affiliateBalance"] = array();
			$this->smartyVars["affiliateBalanceAboveThreshold"] = array();
		}
		
		$creditBalance = array();
		foreach (Service()->CustomerManager->getCreditBalance($contact->Customer) as $currency => $value) {
			$creditBalance[] = BillingLocale::getCurrency($value, $currency);
		}
		$this->smartyVars["creditBalance"] = $creditBalance;
		
		$dueBalance = array();
		foreach (Service()->CustomerManager->getDueBalance($contact->Customer) as $currency => $value) {
			$dueBalance[] = BillingLocale::getCurrency($value, $currency);
		}
		$this->smartyVars["dueBalance"] = $dueBalance;
		
		$closestInvoice = null;
		foreach ($contact->Customer->getOutstandingInvoices() as $invoice) {
			if (is_null($closestInvoice) || $closestInvoice->invoice_date_due > $invoice->invoice_date_due) {
				$closestInvoice = $invoice;
			}
		}
		$this->smartyVars["closestInvoiceDueDate"] = is_null($closestInvoice) ? false : BillingLocale::getDateShort($closestInvoice->invoice_date_due);
		$this->smartyVars["closestInvoiceDueDateInPast"] = is_null($closestInvoice) || (0 < \Billing\compareDate($closestInvoice->invoice_date_due)) ? false : true;
		
		$this->hasExecutedSuccessfully = ((int) $this->addTemplate("dashboard/ShowDashboard", $this->smartyVars));

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
