<?php

require_once("lib-action/action.php");
require_once("lib-mbapi/mbapi.php");

/**
 * This action takes the information provided by the client-side ShowHelpdeskTicketDetails action, and applies it to the database,
 * thus editing a helpdesk ticket in the system. If the provided helpdesk ticket ID does not exist, an error is
 * thrown. Provided no errors occur, once the helpdesk ticket is edited, this action redirects to the
 * ShowHelpdeskTicketDetails action, where the new changes can be viewed.
 *
 * @author  Vic Fryzel
 * @author  Andy Christ
 */
class EditHelpdeskTicketResponse extends MB_Action
{
	/**
	 * The constructor for an EditHelpdeskTicketResponse action object. This calls the parent constructor,
	 * sets the string representation of this object and sets up $this->params.
	 */
	function EditHelpdeskTicketResponse() {
		parent::MB_Action();
		$this->stringRep = "EditHelpdeskTicketResponse";
		$this->params = array("hdTicketID", "hdTicketSubject", "hdTicketBody",
			"hdTicketDateCreated", "hdTicketDateModified", "hdTicketPriorityID",
			"hdTicketStatusID");
	}

	/**
	 * The main method of this class that executes the EditHelpdeskTicketResponse action, and returns
	 * success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		$ticketID = ((int) $_REQUEST["hdTicketID"]);
		if(!$ticketID) {
			$this->addError('TRANS_ACTIONERROR', "No helpdesk ticket ID given.");
		} else {

			$ticket = $this->getMBAPIData("GetHelpdeskTickets", "helpdeskTickets", "helpdeskTicket",
				array("hdTicketID" => $ticketID, "getHelpdeskDepartmentData" => 1,
					"getHelpdeskBucketData" => 1));

			if($ticket === false || !count($ticket)) {
				$this->addError('TRANS_ACTIONERROR', "Helpdesk ticket not found.");
			} else {
				//This line is causing a seg fault
				//$ticket =& $ticket[0][0];
				$ticket = $ticket[0][0];

				$data = array();
				if(count($_REQUEST["hdDepartmentIDs"])) {
					$depts = array();
					if(is_array($ticket["hdDepartmentIDs"][0]["hdDepartmentID"])) {
						$depts = $ticket["hdDepartmentIDs"][0]["hdDepartmentID"];
					}
					$data["hdDepartmentIDs"] = getAddIDXML("hdDepartmentID", $depts, $_REQUEST["hdDepartmentIDs"]);
					$data["removeHDDepartmentIDs"] = getRemoveIDXML("hdDepartmentID", $depts,
						$_REQUEST["hdDepartmentIDs"]);
				}
				if(count($_REQUEST["hdBucketIDs"])) {
					$buckets = array();
					if(is_array($ticket["hdBucketIDs"][0]["hdBucketID"])) {
						$buckets = $ticket["hdBucketIDs"][0]["hdBucketID"];
					}
					$data["hdBucketIDs"] = getAddIDXML("hdBucketID", $buckets, $_REQUEST["hdBucketIDs"]);
					$data["removeHDBucketIDs"] = getRemoveIDXML("hdBucketID", $buckets, $_REQUEST["hdBucketIDs"]);
				}

				if($_REQUEST["hdTicketResponse"]) {
					$data["helpdeskTicketResponses"] = "<helpdeskTicketResponse>\n"
						."<userID>".$CONF["user"]["id"]."</userID>"
						."<userType>1</userType>"
						."<hdTicketRespBody>".$_REQUEST["hdTicketResponse"]."</hdTicketRespBody>\n"
						."<hdTicketRespDateCreated>".time()."</hdTicketRespDateCreated>\n"
						."<hdTicketRespDateModified>".time()."</hdTicketRespDateModified>\n"
						."</helpdeskTicketResponse>\n";
				}

				// Generate the query
				$query = $this->getMBAPIQuery("SetHelpdeskTickets", "update", $data);
				$data = dispatchMBAPI($query, 1);

				if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {
					redirect("helpdesk.php", "ShowHelpdeskTicketDetails", array("hdTicketID" => $ticketID));
				}
			}
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}