<?php

use \Billing\Manager\CryptManager;
use \Billing\Manager\PaymentGatewayManager;
use \Billing\ModuleTool;
use \Billing\Object\BillingAccount;
use \Billing\Object\Contact;
use \Billing\Object\PaymentGateway;
use \Billing\Object\Store;

require_once("lib-action/action.php");

class PayInvoices extends MB_Action
{
	public function execute()
    {
		parent::execute();
        $this->smartyVars["encryption"] = Service()->CryptManager->canDecryptData();
        $this->smartyVars["accountIds"] = array();
        $this->smartyVars["accountsWithShowNumber"] = array();
        $this->smartyVars["billingAccountCC"] = BillingAccount::TYPE_CREDIT_CARD;
        $this->smartyVars["billingAccountEcheck"] = BillingAccount::TYPE_ECHECK;
        $this->smartyVars["billingBankTransfer"] = BillingAccount::TYPE_HSPC_BANK_TRANSFER;

        $this->_contact = Factory()->Contact(getCurrentSession()->user_id);

		for ($i=date("Y"); $i<date("Y")+21; $i++){
			$this->smartyVars["cardYears"][$i] = $i;
		}
		$this->smartyVars["cardTypes"] = blmsg_array(PaymentGateway::card_types());
		$this->smartyVars["payTotal"] = 0;

		// ---------------------------------------------------
		// Get DUE invoices ...
		// ---------------------------------------------------
        $this->smartyVars["invoices"] = array(); // init
		$this->smartyVars["invoiceGroups"] = array();
		$params = array(
			"clientID" => $this->_contact->client_id,
			"getDueInvoicesOnly" => 1,
		);
		if ((int)$_REQUEST["invoiceID"]){
			$params["invoiceID"] = $this->smartyVars["invoiceID"] = (int)$_REQUEST["invoiceID"];
		}
		$query = getMBAPIQuery("GetInvoices", "",  $params, "ASC", "invoiceID", 0);
		$data = dispatchMBAPI($query, 1);
		if ( 0 !== $this->addMBAPIErrors($data) ) {
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

		// set master invoices data array
		$results = $data["mbapi"][0]["results"][0];
		$this->smartyVars["invoices"] = collapseArrayElements(protectArray($results["invoices"][0]["invoice"]));

		if (empty($this->smartyVars["invoices"])) {
			if (isset($params["invoiceID"])) {
				throw new ProductException(
					ProductException::E_FAIL_OPERATION,
					blmsg("TRANS_PAYMENT_INVOICE_ALREADY_PAYED_ERROR_MSG", array("INVOICE_ID" => $params["invoiceID"]))
				);
			} else {
				throw new ProductException(ProductException::E_FAIL_OPERATION, blmsg("TRANS_PAYMENT_NO_INVOICES_TO_PAY_ERROR_MSG"));
			}
		}

		// build invoice line item data
		if($this->smartyVars["invoices"]) {
			for ($i = 0; $i < count($this->smartyVars["invoices"]); $i++){
				$this->smartyVars["invoices"][$i]["invoiceAmountDue"] = $this->smartyVars["invoices"][$i]["invoiceAmount"]-$this->smartyVars["invoices"][$i]["invoiceAmountPaid"];
				$this->smartyVars["invoiceGroups"][$this->smartyVars["invoices"][$i]["orderFormID"]]["invoices"][] =& $this->smartyVars["invoices"][$i];
				$thisdue = $this->smartyVars["invoices"][$i]["invoiceAmount"]-$this->smartyVars["invoices"][$i]["invoiceAmountPaid"];
				if ($thisdue<0)$thisdue = 0;
				$this->smartyVars["invoiceGroups"][$this->smartyVars["invoices"][$i]["orderFormID"]]["payTotal"] += $thisdue;
			}
		}

		$order_form_ids = array_keys($this->smartyVars["invoiceGroups"]);
		$order_forms = Factory()->StoreList(array("order_form_id" => $order_form_ids));
		foreach ( $order_forms as $order_form ) {
			if ( !is_null($order_form->PrimaryGateway) ) {
				$this->smartyVars["invoiceGroups"][$order_form->id]["hasPrimary"] = 1;
				$this->smartyVars["invoiceGroups"][$order_form->id]["cardTypes"] = $order_form->PrimaryGateway->acceptedCardTypes;
                $this->_canBePaid = true;
            }
			if (!is_null($order_form->HSPCBankTransferGateway) && Service()->CryptManager->isEncryptionEnabled()) {
				$this->smartyVars["invoiceGroups"][$order_form->id]["hasHSPCBankTransfer"] = 1;
                $billingAccount = Factory()->BillingAccount();
                $billingAccount->payment_gateway_id = $order_form->HSPCBankTransferGateway->payment_gateway_id;
                $this->smartyVars['invoiceGroups'][$order_form->id]['bankTransferFields']['bt'] = $this->_getBankTransferFields($billingAccount);
                $this->_canBePaid = true;
			}
            $this->_setBillingAccounts($order_form);
			$this->smartyVars["invoiceGroups"][$order_form->id]["secondary"] = $order_form->ThirdPartyGatewaysList();
		}
		$this->smartyVars["secondaryGatewayNames"] = array();
		foreach ((array)$this->smartyVars["invoices"] as $k=>$invoice){
			if ( !isset($this->smartyVars["invoiceGroups"][$invoice["orderFormID"]]["secondary"]) ) {
				continue;
			}
			$secondaries = $this->smartyVars["invoiceGroups"][$invoice["orderFormID"]]["secondary"];
			foreach ( $secondaries as $secondary_gateway ) {
				try {
					if (!Service()->PaymentGatewayManager->isOperationSupported($secondary_gateway, PaymentGatewayManager::OPERATION_CHARGE)) {
						continue;
					}
					$billingAccount = reset(Factory()->BillingAccountList(array(
						"billing_account_type" => BillingAccount::TYPE_THIRD_PARTY_GATEWAY,
						"client_contact_id" => getCurrentSession()->user_id,
						"payment_gateway_id" => $secondary_gateway->id,
					)));
					if (false === $billingAccount) {
						$billingAccount = Factory()->BillingAccount();
						$billingAccount->billing_account_type = BillingAccount::TYPE_THIRD_PARTY_GATEWAY;
						$billingAccount->client_contact_id = getCurrentSession()->user_id;
						$billingAccount->payment_gateway_id = $secondary_gateway->id;
						$billingAccount->update();
					}
					list($chargeRequest) = Service()->PaymentManager->getChargeRequests(Factory()->Invoice($invoice["invoiceID"]), $billingAccount, true);
					$this->smartyVars["invoices"][$k]["secondary"][$secondary_gateway->moduleName][] = $chargeRequest;
                    $this->_canBePaid = true;
				} catch (ProductException $e) {
					dbg($e->getMessage());
					// skip gateway
				}
			}
			if ($this->smartyVars["invoices"] && $this->smartyVars["invoices"][$k] && is_array($this->smartyVars["invoices"][$k])){
				$this->smartyVars["secondaryGatewayNames"][$invoice["orderFormID"]] = array_keys((array)$this->smartyVars["invoices"][$k]["secondary"]);
			}
		}
        if (!$this->_canBePaid) {
            $this->smartyVars["warningMessages"][] = blmsg('TRANS_BILLING_ACCOUNT_NOT_ATTACHED_TO_STORE');
        }
        $this->mergeFormState();
		$this->hasExecutedSuccessfully = ((int) $this->addTemplate("invoices/PayInvoices", $this->smartyVars));

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

    private function _setBillingAccounts(Store $store)
    {
		$billingAccounts = array();
		if ($this->_contact->contact_primary) {
			foreach ($this->_contact->Customer->ContactList() as $contact) {
				$billingAccounts = array_merge($billingAccounts, $contact->BillingAccountList(array('billing_account_active' => 1)));
			}
		} else {
			$billingAccounts = $this->_contact->BillingAccountList(array('billing_account_active' => 1));
		}

        $this->smartyVars['invoiceGroups'][$store->id]['billingAccounts'] = $this->smartyVars['invoiceGroups'][$store->id]['billingAccountTypes'] = array();
        foreach ($billingAccounts as $account) {
            switch ($account->billing_account_type) {
			case BillingAccount::TYPE_HSPC_BANK_TRANSFER:
				if (!Service()->CryptManager->isEncryptionEnabled() ||
                    null === $store->HSPCBankTransferGateway ||
                    $account->payment_gateway_id != $store->HSPCBankTransferGateway->payment_gateway_id
                ) {
					break;
				}
                $this->smartyVars['accountIds'][] = $account->id;
				$this->smartyVars['invoiceGroups'][$store->id]['billingAccounts'][$account->id] = $account->getAccountName();
				$this->smartyVars['invoiceGroups'][$store->id]['billingAccountTypes'][$account->id] = $account->billing_account_type;
                if ('' == $account->_OLD_billing_bank_account_num) {
                    $this->smartyVars['accountsWithShowNumber'][] = $account->id;
                    $this->smartyVars['invoiceGroups'][$store->id]['bankTransferFields'][$account->id] = $this->_getBankTransferFields($account);
                }
                $this->_canBePaid = true;
                break;
			case BillingAccount::TYPE_CREDIT_CARD:
			case BillingAccount::TYPE_ECHECK:
                if (null === $store->PrimaryGateway) {
                    break;
                }
                $this->smartyVars['accountIds'][] = $account->id;
				$this->smartyVars['invoiceGroups'][$store->id]['billingAccounts'][$account->id] = $account->getAccountName();
				$this->smartyVars['invoiceGroups'][$store->id]['billingAccountTypes'][$account->id] = $account->billing_account_type;
                if (!Service()->CryptManager->canDecryptData() ||
                    BillingAccount::TYPE_CREDIT_CARD == $account->billing_account_type && '' == $account->billing_card_num ||
                    BillingAccount::TYPE_ECHECK == $account->billing_account_type && '' == $account->billing_bank_account_num
                ) {
                    $this->smartyVars['accountsWithShowNumber'][] = $account->id;
                }
                $this->_canBePaid = true;
				break;
            }
        }
    }

    private function _getBankTransferFields(BillingAccount $billingAccount)
    {
        $plugin = ModuleTool::getModuleInstance('gateway', 'HSPCBankTransfer', array('gateway_id' => $billingAccount->payment_gateway_id));
        $adapterInputFields = (null === $billingAccount->billing_account_id) ?
            $plugin->getPaymentMethodAdapter($billingAccount)->getInputFields(true) :
            $plugin->getPaymentMethodAdapter($billingAccount)->getInputFields(false, true)
        ;
        $bankTransferFields = array();
        foreach ($adapterInputFields as $fieldName => $fieldOpts) {
            $bankTransferFields[] = array(
                'name' => $fieldName,
                'label' => blmsg($fieldOpts[0]),
                'value' => blmsg($fieldOpts[1]),
                'required' => true,
                'description' => blmsg($fieldOpts[2]),
            );
        }
        return $bankTransferFields;
    }

    private $_contact = null;
    private $_canBePaid = false;
}
