<?php

require_once("lib-action/action.php");

use Billing\Manager\PaymentGatewayManager;
use Billing\Object\BillingAccount;
use Billing\Object\PaymentGateway;
use Billing\Object\Plan;

class AddBillingAccount extends AddAction
{
	public function __construct()
	{
		global $CONF;
		$clientContactID = $CONF["user"]["clientContactID"];
		$_REQUEST["clientContactID"] = $clientContactID;

		$customer = Factory()->Contact(getCurrentSession()->user_id)->Customer;

		if (!$this->_allowedBillingAccountTypes = Service()->BillingAccountManager->getAllowedBillingAccountTypes($customer)) {
			$this->addError('TRANS_ACTIONERROR', blmsg("TRANS_NO_ALLOWED_BILLING_TYPES_FOR_ADD"));
			$this->addErrorTemplates();
			exit;
		}

		$this->d = new DataSource("billingAccounts", array("clientContactID" => $clientContactID));
		$description = $this->d->getDescription();
		$description["names"]["file"] = "profile";
		$description["display"]["addTemplate"] = "profile/AddBillingAccount";
		$this->d->setDescription($description);
		parent::AddAction();
	}

	function addHandler()
	{

		global $CONF;
		$clientID = $CONF["user"]["id"];
		$contact = oneByID(":contacts", $CONF["user"]["clientContactID"]);
		$this->smartyVars['isPrimary'] = $contact['contactPrimary'];
		$this->smartyVars['client'] = oneByID(":clients", $CONF['user']['id']);
		if (!getCurrentSession()->isReseller() && $contact['contactPrimary']) {
			$this->smartyVars['contacts'] = array();
			foreach (Factory()->Customer($clientID)->ContactList() as $contact){
				$this->smartyVars['contacts'][$contact->id] = $contact->contact_fullname;
			}
		}
		loadConfigs("payment");

		$this->smartyVars["billingAccountTypes"] = $this->_allowedBillingAccountTypes;
		// remove account types that the admin wants to restrict from the client.
		foreach( (array)$this->smartyVars["billingAccountTypes"] as $k => $v ) {
			if ($CONF["payment"]["clientSidePaymentOptions"][ getAPITypeName( "billingAccount", $k ) ] != 1) {
				unset( $this->smartyVars["billingAccountTypes"][ $k ] );
			}
		}	unset( $k, $v );

        $accountTypeKeys = array_keys($this->smartyVars["billingAccountTypes"]);
        $bankKeys = array_intersect(
            array(
                BillingAccount::TYPE_BANK_DRAFT,
                BillingAccount::TYPE_ECHECK,
                BillingAccount::TYPE_WIRE,
            ), $accountTypeKeys
        );
        $this->smartyVars["showCC"] = in_array(BillingAccount::TYPE_CREDIT_CARD, $accountTypeKeys) ? 1 : 0;
        $this->smartyVars["showBank"] = empty($bankKeys) ? 0 : 1 ;
		$this->smartyVars["bankAccountTypes"] = getTypes("bankAccount");
		$this->smartyVars["cardExtraTypes"] = getTypes("cardExtraIDs");
        $this->smartyVars["cardTypes"] = blmsg_array(PaymentGateway::card_types());

        $this->smartyVars["doubleListSelectorOptions"] = array();
        foreach(Factory()->SubscriptionList(array('client_id' => $clientID, 'package_active' => 1)) as $subscription) {
            if (Plan::TYPE_FEE != $subscription->package_type) {
                $this->smartyVars["doubleListSelectorOptions"][$subscription->id] = array(
                    'id' => $subscription->id,
                    'title' => $subscription->displayName
                );
            }
        }

		$client = Factory()->Customer($clientID);

		$bt_gateways = Factory()->PaymentGatewayList(array("payment_gateway_type" => PaymentGateway::TYPE_HSPC_BANK_TRANSFER, 'vendor_id' => $client->vendor_id));
		if (0 < count($bt_gateways) && Service()->PaymentGatewayManager->isHSPCAvailable()) {
			$fake_billing_account = Factory()->BillingAccount();
			foreach ($bt_gateways as $bt_gateway) {
				$plugin = ModuleTool::getModuleInstance("gateway", "HSPCBankTransfer", array("gateway_id" => $bt_gateway->id));
				$fake_billing_account->payment_gateway_id = $bt_gateway->id;
				$adapter_input_fields = $plugin->getPaymentMethodAdapter($fake_billing_account)->getInputFields(true);
				$input_fields = array();
				foreach ($adapter_input_fields as $field_name => $field_opts) {
					$input_fields[] = array(
						"field_name" => "{$field_name}_{$bt_gateway->id}",
						"field_title" => $field_opts[0],
						"field_desc" => $field_opts[2],
					);
				}
				$this->smartyVars["ddbp_accounts"][] = array(
					"gateway_id" => $bt_gateway->id,
					"gateway_name" => $bt_gateway->payment_gateway_name,
					"gateway_desc" => $bt_gateway->payment_gateway_description,
					"fields" => $input_fields,
				);
			}
			$this->smartyVars["selected_ddbp_gateway_id"] = bgpc("selected_ddbp_gateway_id");
		}

        $this->smartyVars["ccAccount"] = BillingAccount::TYPE_CREDIT_CARD;
        $this->smartyVars["manualAccount"] = BillingAccount::TYPE_MANUAL_PAYMENT;
        $this->smartyVars["thirdPartyAccount"] = BillingAccount::TYPE_THIRD_PARTY_GATEWAY;
        $this->smartyVars["bankTransferAccount"] = BillingAccount::TYPE_HSPC_BANK_TRANSFER;

        asort($this->smartyVars["doubleListSelectorOptions"]);
        $this->smartyVars["doubleListSelectorAttribs"] = array('id' => 'subscriptions');
        $GLOBALS["tpl"]->assign_by_ref("doubleListSelector", new Billing_Zend_View_Helper_DoubleListSelector());

        return true;
	}

	private $_allowedBillingAccountTypes;
}
