<?php

require_once("lib-action/action.php");

use Billing\Manager\CryptManager;
use Billing\Object\Contact;

class AddClientContactResponse extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$current_contact = Factory()->Contact(getCurrentSession()->user_id);

		if ( 0 == $current_contact->contact_primary ) {
			$this->addError('TRANS_ACTIONERROR', "Only the primary contact can add contacts.");
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

		$this->setReferralActionID(getActionID("AddClientContact"));
		$this->setReferralActionFile("profile.php");

        $this->_validateZip();
        $this->_validateAndTransformCodeState();

        $validate = array(
			"contactFirstName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactLastName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactEmail" => array(INPUT_RESTRICTION_EMAIL|INPUT_RESTRICTION_REQUIRED|INPUT_RESTRICTION_NOTINCOL, array("source" => "contacts", "col" => "contactEmail")),
			"contactEmailFormat" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
			"contactCompany" => INPUT_RESTRICTION_STRING,
			"contactAddress1" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactAddress2" => INPUT_RESTRICTION_STRING,
			"contactCity" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactState" => INPUT_RESTRICTION_STRING,
			"contactZip" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"countriesID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
			"languageID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
			"contactPhone1" => INPUT_RESTRICTION_PHONE|INPUT_RESTRICTION_REQUIRED,
			"contactPhone2" => INPUT_RESTRICTION_PHONE,
			"contactFax" => INPUT_RESTRICTION_PHONE,
			"contactMobilePhone" => INPUT_RESTRICTION_PHONE,
			"emailActionIDs" => INPUT_RESTRICTION_ARRAY,
			"contactUseSpecifiedEmailRulesOnly" => INPUT_RESTRICTION_BOOL,
            "contactCounty" => INPUT_RESTRICTION_STRING,
            "contactUsername" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactPassword" => array(INPUT_RESTRICTION_PASSWORDCONFIRM|INPUT_RESTRICTION_REQUIRED, array("confirm" => "contactPasswordConfirm")),
		);

		if (!Service()->ContactManager->isEmailUnique(bgpc("contactEmail"))) {
			$this->invalidParams[] = "contactEmail-" . base64_encode(INPUT_RESTRICTION_NOTINCOL);
		}
        if (!Service()->ContactManager->isUsernameUnique(bgpc("contactUsername"))) {
            $this->invalidParams[] = "contactUsername-" . base64_encode(INPUT_RESTRICTION_NOTINCOL);
        }

        $contact = Factory()->Contact();
        $this->_collectContact($contact, $current_contact);
        if (count($violations = Service()->ContactManager->validate($contact))) {
            $this->_addInvalidParams($violations, $validate);
        }

		$this->validateMultiInputs($validate);
        if (count($violations)) {
            $this->validationRedirect(array(), array(), true);
        }

		try {
			Service()->ContactManager->update($contact);

			if ( "" != ($redirectURL = bgpc("redirectURL")) && "" != ($redirectAction = bgpc("redirectAction")) )
				redirect($redirectURL."?action=".getActionID($redirectAction)."&".bgpc("redirectArgs"));

			redirect("profile.php", getActionID("ShowClientContacts"), array("clientID" => $current_contact->client_id));
		} catch ( ProductException $e ) {
			$this->addOperationErrors($e);
			$this->addErrorTemplates();
        }
		return $this->hasExecutedSuccessfully();
	}

    private function _collectContact(Contact $contact, Contact $currentContact)
    {
		$contact->client_id = $currentContact->client_id;
		$contact->contact_primary = 0;
		foreach (Contact::$attrsDescriptor as $attr_name => $attr_desc) {
			switch ($attr_name) {
            case "client_id":
            case "contact_primary":
                continue;
            case "contact_password":
                $contact->plainPassword = bgpc('contactPassword');
                continue;
            default:
                if ( false === ($param_name = getDBXMLCol($attr_name)) || is_null($param_value = bgpc($param_name)) ) {
                    continue;
                }
                $contact->$attr_name = $param_value;
			}
		}
		if (!is_null($emailActionIDs = bgpc("emailActionIDs"))) {
			$contact->emailActionIDs = $emailActionIDs;
		}
    }
}
