<?php

require_once("lib-action/action.php");

use Billing\Manager\CryptManager;
use Billing\Object\Contact;

class EditClientContactResponse extends MB_Action
{
	public function execute()
	{
		global $CONF;
		parent::execute();

		$currentUser = oneByID("contacts", $CONF["user"]["clientContactID"]);
		$clientContactID = bgpc('clientContactID');
        if ('' == $clientContactID) {
            $clientContactID = getCurrentSession()->user_id;
        }
		$requestedClientContact = oneByID('contacts', $clientContactID);

		if (!$currentUser["contactPrimary"] && $requestedClientContact['clientContactID'] != $currentUser['clientContactID']) {
			$this->addError('TRANS_ACTIONERROR', "Only the primary contact can add/edit others' contacts.");
		} elseif (!notZero($clientContactID)) {
			$this->addError('TRANS_ACTIONERROR', "Client Contact ID is not given.");
		} elseif ($currentUser['clientID'] != $requestedClientContact['clientID']) {
			$this->addError('TRANS_ACTIONERROR', "Requested client Contact ID does not belong to logged in client.");
	    } else {
			$contact = Factory()->Contact($clientContactID);
			$this->setReferralActionID(getActionID("EditClientContact"));
			$this->setReferralActionFile("profile.php");
            
             $contactFields = array(
                "contact_active",
                "contact_address_1",
                "contact_address_2",
                "contact_city",
                "contact_state",
                "contact_county",
                "contact_zip",
                "language_id",
                "contact_phone1",
                "contact_phone2",
                "contact_fax",
                "contact_mobile_phone",
                "contact_email_format"
            );
            
			$postedValues = array(
				"clientContactID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
				"contactAddress1" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
				"contactAddress2" => INPUT_RESTRICTION_STRING,
				"contactCity" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
				"contactState" => INPUT_RESTRICTION_STRING,
				"contactZip" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
				"languageID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
				"contactPhone1" => INPUT_RESTRICTION_PHONE|INPUT_RESTRICTION_REQUIRED,
				"contactPhone2" => INPUT_RESTRICTION_PHONE,
				"contactFax" => INPUT_RESTRICTION_PHONE,
				"contactMobilePhone" => INPUT_RESTRICTION_PHONE,
                "contactEmailFormat" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
			);
            
            foreach(Factory()->ConfigGroup(0)->getChild("userSettings")->getChild("display")->ConfigParams as $config) {
                if (0 == $config->value)
                    continue;

                switch($config->name) {
                case "editUserName":
                    if ( 1 == $requestedClientContact['contactPrimary'] ) {
                        $postedValues["contactUsername"] = INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED;
                    }
                    if (Service()->ContactManager->isUsernameChangeable(Factory()->Contact($clientContactID)) && !is_null($contactUsername = bgpc("contactUsername"))) {
                        if (!Service()->ContactManager->isUsernameUnique($contactUsername, $contact->id)) {
                            $this->invalidParams[] = "contactUsername-" . base64_encode(INPUT_RESTRICTION_NOTINCOL);
                        }
                        $postedValues["contactUsername"] = INPUT_RESTRICTION_REQUIRED;
                        $contactFields[] = "contact_username";
                    }
                    break;
                case "editFirstName":
                    $postedValues["contactFirstName"] = INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED;
                    $contactFields[] = "contact_first_name";
                    break;
                case "editLastName":
                    $postedValues["contactLastName"] = INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED;
                    $contactFields[] = "contact_last_name";
                    break;
                case "editEmail":
                    if (!is_null($contactEmail = bgpc("contactEmail")) && !Service()->ContactManager->isEmailUnique($contactEmail, $contact->id)) {
                        $this->invalidParams[] = "contactEmail-" . base64_encode(INPUT_RESTRICTION_NOTINCOL);
                    }
                    $postedValues["contactEmail"] = INPUT_RESTRICTION_EMAIL|INPUT_RESTRICTION_REQUIRED;
                    $contactFields[] = "contact_email";
                    break;
                case "editCompanyName":
                    $postedValues["contactCompany"] = INPUT_RESTRICTION_STRING;
                    $contactFields[] = "contact_company";
                    break;
                case "editCountry":
                    $postedValues["countriesID"] = INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED;
                    $contactFields[] = "countries_id";
                    break;
                case "editEmailActions":
                    $postedValues["contactUseSpecifiedEmailRulesOnly"] = INPUT_RESTRICTION_BOOL;
                    $contactFields[] = "contact_use_specified_email_rules_only";
                    $contactFields[] = "emailActionIDs";
                    break;
                }
            }

            if ("" != bgpc("contactPassword")) {
                $postedValues["contactPassword"] = array(
                    INPUT_RESTRICTION_PASSWORDCONFIRM|INPUT_RESTRICTION_REQUIRED,
                    array("confirm" => "contactPasswordConfirm")
                );
                $contactFields[] = "contact_password";
            }

            if ("" != bgpc("countriesID")) {
                $this->_validateZip();
                $this->_validateAndTransformCodeState();
            } else {
                $zipValidator = new Billing_Zend_Validate_Zip($contact->countries_id);
                if (!$zipValidator->isValid(bgpc("contactZip"))) {
                    $this->invalidParams[] = 'contactZip-'.base64_encode(blmsg("TRANS_ERROR_ZIP"));
                }
                $stateValidator = new Billing_Zend_Validate_State($contact->countries_id);
                if (!$stateValidator->isValid(bgpc("contactState"))) {
                    $this->invalidParams[] = 'contactState-'.base64_encode(blmsg("TRANS_ERROR_STATE"));
                }
                $_REQUEST["contactState"] = Service()->ContactManager->getStateCode(bgpc("contactState"), $contact->countries_id);
            }

            $this->_updateContact($contact, $contactFields);
            if (count($violations = Service()->ContactManager->validate($contact))) {
                $this->_addInvalidParams($violations, $postedValues);
            }

            $this->validateMultiInputs($postedValues, array(), array(), 1);
            if (count($violations)) {
                $this->validationRedirect(array(), array(), true);
            }

            try {
                Service()->ContactManager->update($contact);
            } catch (ProductException $ex) {
                billing_put_error($ex->getMessage());
                $this->validationRedirect(array(), array(), true);
            }

            if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
                redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
            } elseif( 1 == Factory()->Contact(getCurrentSession()->user_id)->contact_primary ) {
                redirect("profile.php", getActionID("ShowClientContacts"));
            } else {
                redirect("dashboard.php", getActionID("ShowDashboard"));
            }

        }

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

    private function _updateContact(Contact $contact, $contactFields)
    {
        foreach ($contactFields as $attrName) {
            if ("emailActionIDs" == $attrName) {
                $contact->emailActionIDs = (array)bgpc("emailActionIDs");
                continue;
            }
            if ( false === ($paramName = getDBXMLCol($attrName)) || is_null($paramValue = bgpc($paramName)) ) {
                continue;
            }
            if ("contact_password" == $attrName) {
                $contact->plainPassword = $paramValue;
                continue;
            }
            $contact->$attrName = trim($paramValue);
        }
    }
}
