<?php

require_once("lib-billing/include/config/internal/statuses.php");
require_once("lib-tk/include/config/internal/statuses.php");
require_once("lib-tk/include/crypt/crypt.php");

use Billing\Manager\CryptManager;

/**
 * modernbill-client specific implementation of operations required by ProcessSSOAPIRequest
 */
class ClientSSOAPI
{
    /**
     * Chceck contact's credentials.
     *
     * @param string $name User name
     * @param string $password User's password. If this parameter is empty then only check that user name is correct.
     * @return String User name identifier (login name).
     */
	public function checkCredentials($name, $password)
	{
        return !is_null($contactId = $this->getContactID($name, $password)) ? $contactId : NULL;
    }

    public function checkAudit($name, $loginAttemptState)
    {
        throw new ProductException(ProductException::E_FAIL_OPERATION, __METHOD__." is implemented only for admin application.");
    }

    /**
     * Close session by client contact's user name.
     *
     * @param string $userName
     */
    public function closeSessionByName($userName)
	{
        $qh = new QueryMaster('select');
        $qh->addTables('client_contacts');
        $qh->addColumns('client_contact_id');
        $qh->addOrClause('contact_username', '=', $userName);
        $qh->addOrClause('contact_email', '=', $userName);
        foreach ($qh->getResults() as $row) {
            $userId = $row['client_contact_id'];

            $h = new QueryMaster('update');
            $h->addTables('sessions');
            $h->addParam('session_active', 0, 'i');
            $h->addAndClause('user_id', '=', $userId, 'i');
            $h->addAndClause('user_type', '=', 1, 'i');
            $h->execute();
        }
    }

    public function getContactID($name, $password)
	{
    	try	{
    		$contact = Factory()->Contact($name, 'contact_username');
    	}
		catch (ProductException $e)	{
			if ($e->getCode() == ProductException::E_OBJECT_NOT_FOUND)
				return NULL;

			throw $e;
		}

        if (Service()->CryptManager->checkPassword($password, $contact->contact_temp_password, CryptManager::TYPE_CRYPT)) {
            $contact->plainPassword = $password;
			$contact->contact_password = $contact->contact_temp_password;
			$contact->contact_temp_password = NULL;
			Service()->ContactManager->update($contact);
			return $contact->id;
		}
		if (Service()->CryptManager->checkPassword($password, $contact->contact_password, $contact->contact_password_type)) {
            if ("" != $contact->contact_temp_password) {
				$contact->contact_temp_password = NULL;
				Service()->ContactManager->update($contact);
			}
			return $contact->id;
		}
		return NULL;
    }
}
