<?php

/**
 * This file maintains all information having to do with certs. The order of the
 * $CONF["certs"] array in this file is very important, do not change it, as the
 * array keys are considered the cert type IDs. Note that a cert ID of 0 is possible.
 *
 * Format: $CONF["certs"][ ID ] = array( ProductCode , ProductName )
 *
 * @author  Michael Fountain
 */

$CONF["certs"] = array();
$CONF["certs"][0] = array ("QuickSSL", blmsg("TRANS_QUICKSSL"));
$CONF["certs"][1] = array ("QuickSSLPremium", blmsg("TRANS_QUICKSSLPREMIUM"));
$CONF["certs"][2] = array ("TrueBizID", blmsg("TRANS_TRUEBIZID"));
$CONF["certs"][3] = array ("RapidSSL", blmsg("TRANS_RAPIDSSL"));


/**
 * This function gets the ID of a cert by searching for the given ProductCode. If the given
 * ProductCode is found in the certs list, this function returns the ID, however if the given
 * ProductCode is not found, this function returns false. Note that a cert ID of 0 is possible.
 *
 * @param  name  The cert ProductCode to search for, case-insensitive.
 * @return  The cert ID of the given cert ProductCode, or false if the given ProductCode was not found.
 */
function getCertID($name) {
	global $CONF;
	$retval = false;
	$c = count($CONF["certs"]);
	for($i=0; $i<$c; $i++) {
		if(strcasecmp($CONF["certs"][$i][0], $name) === 0) {
			$retval = $i;
			break;
		}
	}
	return $retval;
}

/**
 * This function gets the ProductCode of a cert by searching for the given cert ID. If the given
 * ID is found in the certs list, this function returns the ProductCode associated with that ID,
 * however if the given ID is not found, this function returns false. Note that a cert ID of 0 is
 * possible.
 *
 * @param  id  The cert ID to search for.
 * @return  The ProductCode of the given cert ID, or false if the given ID was not found.
 */
function getCertProductCode($id) {
	global $CONF;
	$retval = false;
	if($id < count($CONF["certs"])) {
		$retval = $CONF["certs"][$id][0];
	}
	return $retval;
}

/**
 * This function gets the ProductName of a cert by searching for the given cert ID. If the given
 * ID is found in the certs list, this function returns the ProductName associated with that ID,
 * however if the given ID is not found, this function returns false. Note that a cert ID of 0 is
 * possible.
 *
 * @param  id  The cert ID to search for.
 * @return  The ProductName of the given cert ID, or false if the given ID was not found.
 */
function getCertName($id) {
	global $CONF;
	$retval = false;
	if($id < count($CONF["certs"])) {
		$retval = $CONF["certs"][$id][1];
	}
	return $retval;
}
?>