<?php

/*
cycles
cycle_groups
cycle_group_rel
packages.cycle_id
product_cycle_group_rel
suspend_cycles
*/

/**
 * This file maintains all information having to do with billing cycles. The order of the
 * $CONF["cycles"] array in this file is very important, do not change it, as the
 * array keys are used to associate billing cycle with their delta timestamps.
 *
 * billingCycleData => array (
 *     [0] => ### (Seconds)
 *     [1] => ### (Minutes)
 *     [2] => ### (Hours)
 *     [3] => ### (Days)
 *     [4] => ### (Months)
 *     [5] => ### (Years)
 *  )
 *
 * @author  Michael Fountain
 */

// hourly billing cycles: 1 to 48 hours
$CONF["cycles"] = array();

function populateCONFCycles(){
	global $CONF;
	if (!isset($CONF["cycles"][1]) || !is_array($CONF["cycles"][1])) {
		// get all billing cycles
		//$d = new datasource("cycles", array("cycleGroups"=>array()));
		$tempcycles = arrayFor(":cycles"); //, array("cycleGroups"=>array())); //$d->getEssential();
		// put all billing cycles info $CONF[""] cycles by cycleID with same format as old way
		dbg($tempcycles);
		foreach ($tempcycles as $id => $cycle) {
			if ($cycle["cycleNumSeconds"]>0){
				$operativeUnitValue = $cycle["cycleNumSeconds"];
			}
			if ($tempcycles[$id]["cycleNumMinutes"]>0){
				$operativeUnitValue = $cycle["cycleNumMinutes"];
			}
			if ($tempcycles[$id]["cycleNumHours"]>0){
				$operativeUnitValue = $cycle["cycleNumHours"];
			}
			if ($tempcycles[$id]["cycleNumDays"]>0){
				$operativeUnitValue = $cycle["cycleNumDays"];
			}
			if ($tempcycles[$id]["cycleNumMonths"]>0){
				$operativeUnitValue = $cycle["cycleNumMonths"];
			}
			if ($tempcycles[$id]["cycleNumYears"]>0){
				$operativeUnitValue = $cycle["cycleNumYears"];
			}
			$CONF["cycles"][$cycle["cycleID"]] = array(
				isset($cycle["cycleGroups"][0]["cycleGroupInternalName"]) ? $cycle["cycleGroups"][0]["cycleGroupInternalName"] : null, // group internal name
				$operativeUnitValue, // the number of x units we are using
				$cycle["cycleDisplayName"], // TRANS_ name of cycle
				array(
					$cycle["cycleNumSeconds"],
					$cycle["cycleNumMinutes"],
					$cycle["cycleNumHours"],
					$cycle["cycleNumDays"],
					$cycle["cycleNumMonths"],
					$cycle["cycleNumYears"]
				) // mktime array
			);
		}
	} // END IF $CONF["cycles"]
} // populateCONFCycles




/**
 * Get all integer IDs & constant values of the time cycle with the given internal name.
 *
 * @param  type  The type of time cycle to lookup (i.e. "monthly", or "yearly", or "" for all)
 * @return  The integer ID & constant value of the time cycle as an array.
 */
function getCyclesArray($type="") {
	global $CONF;
    populateCONFCycles();
	$retval = array();
	$type = strval($type);
	foreach($CONF["cycles"] as $key => $value) {
        if( $type == $value[0] || !$type) {
		    $retval[$key] = blmsg($value[2]) ;
		}
	}
    ksort($retval);
	return $retval;
}

/**
 * Get the integer ID of the time cycle with the given internal name and multiplier value.
 * If the given internal name & multiplier values are not found, this function returns false.
 *
 * @param  type  The type of time cycle to lookup (i.e. "monthly", or "yearly")
 * @param  multiplier  The value of the time cycle multiplier
 * @return  The integer ID of the time cycle, or false if the name was not found.
 */
function getCycleID($type, $multiplier) {
	global $CONF;
    populateCONFCycles();
	$retval = false;
	$type = strval($type);
	$multiplier = ((int) $multiplier);
	foreach($CONF["cycles"] as $key => $value) {
		if( preg_match('/' . preg_quote($type) . '/',$value[0]) && ($value[1] == $multiplier) ) {
			$retval = $key;
			break;
		}
	}
	return $retval;
}

/**
 * Returns an array of time cycle ids of a certain type.
 *
 * @param  type  The type of time cycle ids to return.
 * @return  An array of time cycle ids of the given type, or empty array if the type was not found.
 */
function getCycleIDs($type) {
    global $CONF;
    populateCONFCycles();
    $type = strval($type);
    $retval = array();
    foreach($CONF["cycles"] as $key => $value) {
        if ( preg_match('/' . preg_quote($type) . '/', $value[0]) ) {
            $retval[] = $key;
        }
    }
    print_r($retval);
    return $retval;
}

/**
 * Returns an array of all time cycle ids.
 *
 * @return  An array of all time cycle ids.
 */
function getAllCycleIDs() {
    global $CONF;
    populateCONFCycles();
    $retval = array();
    foreach($CONF["cycles"] as $key => $value) {
        $retval[] = $key;
    }
    return $retval;
}

/**
 * Get the string constant name of the time cycle associated with the given integer ID.
 * If the given ID is not found, then this function returns false.
 *
 * @param  id  The id of the time cycle constant to lookup
 * @return  The string constant name associated with the given ID, or false if the ID was not found
 */
function getCycleName($id) {
	global $CONF;
    populateCONFCycles();
	$id = ((int) $id);
	$retval = false;
	if(is_array($CONF["cycles"][$id])) {
		$retval = $CONF["cycles"][$id][2];
	}
	return $retval;
}

/**
 * Get the time cycle data of the time cycle associated with the given integer ID.
 * If the given ID is not found, then this function returns false.
 *
 * @param  id  The id of the time cycle data to lookup
 * @return  The array of time cycle data associated with the given ID, or false if the ID was not found
 */
function getCycleData($id) {
	global $CONF;
    populateCONFCycles();
	$id = ((int) $id);
	$retval = false;
	if(is_array($CONF["cycles"][$id])) {
		$retval = $CONF["cycles"][$id][3];
	}
	return $retval;
}

/**
 * Check if length of cycle id1 is less than length of cycle id2
 * @param int $id1
 * @param int $id2
 * @return bool
 */
function isCycleLessThan($id1, $id2) {
    $timestamp = time();
    return getNextBillingTimestamp($timestamp, $id1) <  getNextBillingTimestamp($timestamp, $id2);
}

/*
 * Get allowed renewal dates shift - 8 days for each year
 * @param int $id
 * @return int
 */
function getAllowedRenewalDateShift($id) {
    $billingCycleData = getCycleData($id);

    $delta = 8 * $billingCycleData[5];
    if ($billingCycleData[4] || $billingCycleData[3] || $billingCycleData[2]
        || $billingCycleData[1] || $billingCycleData[0]
    ) {
        $delta += 8;
    }

    return $delta;
}

/**
 * Returns next renewal timestamp in unix seconds. This will be the NEXT time this package renews.
 *
 * @param  timestamp  The current renewal timestamp of this package.
 * @param  cycleID  The time cycle id of this package.
 * @return  the next time timestamp for this package or 0 if not valid.
 */
function getNextBillingTimestamp($timestamp, $cycleID, $count = 1) {
    $timeData = localtime(((int) $timestamp), 1);

    $billingCycleData = getCycleData($cycleID);
    $retval = 0;


    if(is_array($billingCycleData)) {
        $secs   = ((int) $timeData["tm_sec"]) + (int)$billingCycleData[0] * $count;
        $mins   = ((int) $timeData["tm_min"]) + (int)$billingCycleData[1] * $count;
        $hours  = ((int) $timeData["tm_hour"]) + (int)$billingCycleData[2] * $count;
        $days   = ((int) $timeData["tm_mday"]) + (int) $billingCycleData[3] * $count;
        $months = ((int) $timeData["tm_mon"]) + (int)$billingCycleData[4] * $count + 1; // localtime counts 0 as January
        $years  = 1900 + ((int) $timeData["tm_year"])  + (int)$billingCycleData[5] * $count;

        $retval = mktime($hours, $mins, $secs, $months, $days, $years);
    }

    return $retval;
}


/**
 * Returns last renewal timestamp in unix seconds. This will be the LAST time this package renewed.
 * (unless they switched packages from a package with a different payment cycle in the last cycle)
 *
 * @param  timestamp  The current renewal timestamp of this package.
 * @param  cycleID  The time cycle id of this package.
 * @return  the next time timestamp for this package or 0 if not valid.
 */
function getLastBillingTimestamp($timestamp, $cycleID) {
    $timeData = localtime(((int) $timestamp), 1);

    $billingCycleData = getCycleData($cycleID);
    $retval = 0;

    if(is_array($billingCycleData)) {
        $secs   = ((int) $timeData["tm_sec"])   - ((int) $billingCycleData[0]);
        $mins   = ((int) $timeData["tm_min"])   - ((int) $billingCycleData[1]);
        $hours  = ((int) $timeData["tm_hour"])  - ((int) $billingCycleData[2]);
        $days   = ((int) $timeData["tm_mday"])  - ((int) $billingCycleData[3]);
        $months = ((int) $timeData["tm_mon"])   + 1 - ((int) $billingCycleData[4]); // localtime counts 0 as January
        $years  = 1900 + ((int) $timeData["tm_year"])  - ((int) $billingCycleData[5]);

        $retval = mktime($hours, $mins, $secs, $months, $days, $years);
    }
    return $retval;
}

/**
 * Get expiration date that will be applied to subscription when renewal process is completed.
 * For subscriptions with ability to renew manually.
 *
 * @param $subscription
 * @return timestamp
 */
function getNextStopRenewalDate($subscription) {
    $lastRenewal = getLastBillingTimestamp($subscription->package_date_next_renewal, $subscription->cycle_id);
    while ($lastRenewal < $subscription->package_date_stop_renewal) {
        $lastRenewal = getNextBillingTimestamp($lastRenewal, $subscription->cycle_id);
    }
    $offset = $subscription->isDomainType() ? 20 : 10;
    return getNextBillingTimestamp($lastRenewal, $subscription->cycle_id) - $offset;
}

/**
 * Get expiration date that will be applied to subscription when renewal invoice is generated (grace period is included).
 * This temporary expiration date will be replaced by actual value when renewal process is completed.
 * For subscriptions with ability to renew manually or with smart renewal custom property.
 *
 * @param $subscription
 * @param $renewalPoint (is equal to last renewal date in case single cycle renewal)
 * @return timestamp
 */
function getExpirationDateOnRenewal($subscription, $renewalPoint)
{
    if ($subscription->package_manual_renewal) {
        return strtotime('+1 month', $subscription->package_date_stop_renewal);
    }

    $expirationDate = strtotime('+1 month', $renewalPoint);
    $offset = $subscription->isDomainType() ? 20 : 10;
    return $expirationDate - $offset;
}

/**
 * Get next renewal date for addon purchased on subscription upgrade.
 * E.g.: 1-month addon is purchased for 1-year subscription on Mar, 3;
 * subscription's last renewal date is Jan, 1.
 * Therefore, next renewal date will be Apr, 1
 *
 * @param $subscription
 * @param $cycleID (id of addon's cycle)
 * @return timestamp
 */
function getNewAddonNextRenewalDate($subscription, $cycleID)
{
    $timestamp = time();
    $nextRenewal = getNextBillingTimestamp($subscription->package_date_last_renewal, $cycleID);

    while ($timestamp > $nextRenewal) {
        $nextRenewal = getNextBillingTimestamp($nextRenewal, $cycleID);
    }
    return min($nextRenewal, $subscription->package_date_next_renewal);
}

/**
 * Get last renewal date for addon purchased on subscription upgrade.
 * E.g.: 1-month addon is purchased for 1-year subscription on Mar, 3;
 * subscription's last renewal date is Jan, 1.
 * Therefore, last renewal date for addon will be Mar, 1
 *
 * @param $timestamp (addon's next renewal date)
 * @param $cycleID (id of addon's cycle)
 * @param $subscriptionLastRenewalDate
 * @return timestamp
 */
function getNewAddonLastRenewalDate($timestamp, $cycleID, $subscriptionLastRenewalDate)
{
    $lastRenewal = $subscriptionLastRenewalDate;
    while ($timestamp > $lastRenewal) {
        $lastRenewal = getNextBillingTimestamp($lastRenewal, $cycleID);
    }

    $lastRenewal = getLastBillingTimestamp($lastRenewal, $cycleID);
    return max($lastRenewal, $subscriptionLastRenewalDate);
}
?>
