<?php

require_once("lib-mbapi/include/modules/gateway/gateway.php");

$className = "WorldPayDirect";
$GLOBALS["moduleInfo"][$className] = array(
    "type" => "gateway",
    "status" => PaymentGateway::STATUS_STABLE,
    "version" => "1.0.0",
    "gatewaytype" => "1",
    "displayName" => "WorldPay Direct",
    "author" => "Parallels",
    "capabilities" => array(
		GATEWAY_AUTHORIZE,
		GATEWAY_CHARGE,
		GATEWAY_PRIOR_AUTH_CHARGE,
		GATEWAY_REFUND,
		GATEWAY_PARTIAL_REFUND,
		GATEWAY_VOID,
		GATEWAY_IPV6_COMPLIANCE,
		GATEWAY_3D_SECURE,
	),
	"countryCodes" => array("GB"),
);

class WorldPayDirect extends Gateway
{
	public function getModuleInfo()
	{
		return $GLOBALS["moduleInfo"]["WorldPayDirect"];
	}

	public function __construct($input)
	{
		parent::Gateway($input);
		$this->paymentURL = (0 == $this->config["Testing"]) ?
			"https://secure.worldpay.com/jsp/merchant/xml/paymentService.jsp" :
			"https://secure-test.worldpay.com/jsp/merchant/xml/paymentService.jsp"
		;
	}

	public function addDefaultConfigParams()
	{
		return array(
			array(NULL, NULL, "merchantCode", "", "", "TRANS_WORLDPAY_MERCHANT_CODE", "TRANS_WORLDPAY_MERCHANT_CODE_DESC", "t", 1, 1, 0, NULL),
			array(NULL, NULL, "xmlPassword", "", "", "TRANS_WORLDPAY_XML_PASSWORD", "TRANS_WORLDPAY_XML_PASSWORD_DESC", "p", 1, 1, 0, NULL),
			array(NULL, NULL, "Testing", "1", "", "TRANS_GATEWAY_TESTING", "TRANS_GATEWAY_TESTING_DESC", "b", 1, 4, 0, NULL),
			array(NULL, NULL, "card_accept_0", "1", "", "TRANS_ACCEPT_VISA", "TRANS_ACCEPT_VISA_DESC", "b", 1, 8, 0, NULL),
			array(NULL, NULL, "card_accept_1", "1", "", "TRANS_ACCEPT_MASTERCARD", "TRANS_ACCEPT_MASTERCARD_DESC", "b", 1, 9, 0, NULL),
			array(NULL, NULL, "card_accept_2", "1", "", "TRANS_ACCEPT_AMEX", "TRANS_ACCEPT_AMEX_DESC", "b", 1, 10, 0, NULL),
			array(NULL, NULL, "card_accept_3", "1", "", "TRANS_ACCEPT_DISCOVER", "TRANS_ACCEPT_DISCOVER_DESC", "b", 1, 11, 0, NULL),
			array(NULL, NULL, "card_accept_4", "1", "", "TRANS_ACCEPT_JCB", "TRANS_ACCEPT_JCB_DESC", "b", 1, 12, 0, NULL),
			array(NULL, NULL, "card_accept_6", "1", "", "TRANS_ACCEPT_DINERS", "TRANS_ACCEPT_DINERS_DESC", "b", 1, 14, 0, NULL),
			array(NULL, NULL, "card_accept_7", "1", "", "TRANS_ACCEPT_SOLO", "TRANS_ACCEPT_SOLO_DESC", "b", 1, 15, 0, NULL),
			array(NULL, NULL, "card_accept_8", "1", "", "TRANS_ACCEPT_MAESTRO", "TRANS_ACCEPT_MAESTRO_DESC", "b", 1, 16, 0, NULL),
			array(NULL, NULL, "card_accept_9", "1", "", "TRANS_ACCEPT_DELTA", "TRANS_ACCEPT_DELTA_DESC", "b", 1, 17, 0, NULL),
			array(NULL, NULL, "card_accept_11", "1", "", "TRANS_ACCEPT_ELECTRON", "TRANS_ACCEPT_ELECTRON_DESC", "b", 1, 19, 0, NULL),
		);
	}

	public function execute()
	{
		try {
			switch ($subCommand = $this->params["subCommand"][0]) {
			case "auth":
				return $this->_auth();
			case "charge":
				$this->_auth();
				if (GATEWAY_SUCCESS != $this->status) {
					return $this->createOutputXML();
				}
				if ('AUTHORISED' == $this->_paymentStatus) {
					return  $this->_capture();
				}
				return $this->createOutputXML();
			case "finalize":
				return  $this->_capture();
			case "refund":
				return  $this->_refund();
			case "void":
				return  $this->_void();
			default:
				throw new \Exception("Invalid gateway action requested: {$subCommand}");
			}
		} catch (\Exception $e) {
			$this->status = GATEWAY_ERROR;
			$this->message = $e->getMessage();
			return $this->createOutputXML();
		}
	}

	public function allowAutomaticPayments()
	{
		return 0;
	}

	private function _auth()
	{
		$processingParams = $this->_getPaymentTransaction()->processingParams;
		dbg($processingParams);
		if (isset($processingParams->backRedirectParams)) {
			parse_str($processingParams->backRedirectParams, $backRedirectParams);
		}
		if (isset($backRedirectParams) && isset($backRedirectParams['PaRes'])) {
			// handle 3D secure response
			parse_str($backRedirectParams['MD'], $submitOrderParams);
			parse_str($submitOrderParams['cookies'], $cookies);
			$submitOrderParams['paResponse'] = $backRedirectParams['PaRes'];
			$response = $this->_doRequest($this->_createSubmitOrderRequest($submitOrderParams), $cookies);

			if (isset($response->reply->orderStatus->error)) {
				throw new \Exception((string)$response->reply->orderStatus->error);
			}
		} else {
			// send charge request
			$cookies = array();
			$response = $this->_doRequest($this->_createSubmitOrderRequest(), $cookies);
			$this->transactionID = $this->_getOrderCode();

			if (isset($response->reply->orderStatus->error)) {
				throw new \Exception((string)$response->reply->orderStatus->error);
			}

			if (isset($response->reply->orderStatus->requestInfo)) {
				// handle 3D secure request
				$this->status = GATEWAY_PROCESSING;
				$transitParams = array(
					'echoData' => (string)$response->reply->orderStatus->echoData,
					'cardNumber' => $this->params['billingCardNum'][0],
					'cookies' => http_build_query($cookies),
				);
				if (isset($this->params['cardCVV2']) && '' != $this->params['cardCVV2'][0]) {
					$transitParams['cvc'] = $this->params['cardCVV2'][0];
				}
				$processingParams->redirectUrl = (string)$response->reply->orderStatus->requestInfo->request3DSecure->issuerURL;
				$processingParams->redirectParams = http_build_query(array(
					'PaReq' => (string)$response->reply->orderStatus->requestInfo->request3DSecure->paRequest,
					'TermUrl' => $processingParams->backRedirectUrl,
					'MD' => http_build_query($transitParams),
				));
				dbg($processingParams);
				return $this->createOutputXML();
			}
		}

		if (isset($response->reply->orderStatus->payment) && isset($response->reply->orderStatus->payment->lastEvent)) {
			switch ($this->_paymentStatus = (string)$response->reply->orderStatus->payment->lastEvent) {
			case 'AUTHORISED':
			case 'CAPTURED':
			case 'SETTLED':
				if (isset($response->reply->orderStatus->payment->AuthorisationId)) {
					$this->authorizationCode = $response->reply->orderStatus->payment->AuthorisationId['id'];
				}
				if (isset($response->reply->orderStatus->payment->AVSResultCode)) {
					$this->AVSCode = $response->reply->orderStatus->payment->AVSResultCode['description'];
				}
				$this->status = GATEWAY_SUCCESS;
				break;
			default:
				$this->status = GATEWAY_DECLINED;
				if (isset($response->reply->orderStatus->payment->ISO8583ReturnCode)) {
					$this->message = $response->reply->orderStatus->payment->ISO8583ReturnCode['description'];
				}
			}
			return $this->createOutputXML();
		}

		throw new \Exception('Unrecognized gateway reply.');
	}

	private function _capture()
	{
		$response = $this->_doRequest($this->_createCaptureOrderRequest());

		if (isset($response->reply->error)) {
			throw new \Exception((string)$response->reply->error);
		}

		if (isset($response->reply->ok->captureReceived)) {
			$this->status = GATEWAY_SUCCESS;
			return $this->createOutputXML();
		}

		throw new \Exception('Unrecognized gateway reply.');
	}


	private function _refund()
	{
		$response = $this->_doRequest($this->_createRefundOrderRequest());

		if (isset($response->reply->error)) {
			throw new \Exception((string)$response->reply->error);
		}

		if (isset($response->reply->ok->refundReceived)) {
			$this->status = GATEWAY_SUCCESS;
			return $this->createOutputXML();
		}

		throw new \Exception('Unrecognized gateway reply.');
	}

	private function _void()
	{
		$response = $this->_doRequest($this->_createCancelOrderRequest());

		if (isset($response->reply->error)) {
			throw new \Exception((string)$response->reply->error);
		}

		if (isset($response->reply->ok->cancelReceived)) {
			$this->status = GATEWAY_SUCCESS;
			return $this->createOutputXML();
		}

		throw new \Exception('Unrecognized gateway reply.');
	}

	private function _createCancelOrderRequest()
	{
		$request = $this->_createRequest();
		$orderModification = $request->addChild('modify')->addChild('orderModification');
		$orderModification->addAttribute('orderCode', $this->_getOrderCode());
		$orderModification->addChild('cancel');
		return $request;
	}

	private function _createCaptureOrderRequest()
	{
		$request = $this->_createRequest();
		$orderModification = $request->addChild('modify')->addChild('orderModification');
		$orderModification->addAttribute('orderCode', $this->_getOrderCode());
		$this->_addAmount($orderModification->addChild('capture'));
		return $request;
	}

	private function _createRefundOrderRequest()
	{
		$request = $this->_createRequest();
		$orderModification = $request->addChild('modify')->addChild('orderModification');
		$orderModification->addAttribute('orderCode', $this->_getOrderCode());
		$this->_addAmount($orderModification->addChild('refund'));
		return $request;
	}

	private function _createSubmitOrderRequest($params = array())
	{
		$request = $this->_createRequest();

		$order = $request->addChild('submit')->addChild('order');
		$order->addAttribute('orderCode', $this->_getOrderCode());
		$order->description = $this->params['paymentDescription'][0];

		$this->_addAmount($order);

		$paymentDetails = $order->addChild('paymentDetails');
		$paymentMethod = $paymentDetails->addChild($this->_getPaymentMethodCode());
		$paymentMethod->cardNumber = isset($params['cardNumber']) ? $params['cardNumber'] : $this->params['billingCardNum'][0];
		$expiryDate = $paymentMethod->addChild('expiryDate')->addChild('date');
		$expiryDate->addAttribute('month', $this->params['billingCardExpMonth'][0]);
		$expiryDate->addAttribute('year', $this->params['billingCardExpYear'][0]);
		$paymentMethod->cardHolderName = $this->params['billingCardNameOnCard'][0];
		if (isset($params['cvc']) || null !== $this->_getCvv2()) {
			$paymentMethod->cvc = isset($params['cvc']) ? $params['cvc'] : $this->_getCvv2();
		} else if (isset($this->params['billingCardIssueNum']) && '' != $this->params['billingCardIssueNum'][0]) {
			$paymentMethod->issueNumber = $this->params['billingCardIssueNum'][0];
		} else if (isset($this->params['billingCardStartDate']) && '' != $this->params['billingCardStartDate'][0]) {
			$paymentMethod->startDate = $this->params['billingCardStartDate'][0];
		}
		$session = $paymentDetails->addChild('session');
		$session->addAttribute('shopperIPAddress', $this->_getShopperIpAddress());
		$session->addAttribute('id', $this->_getShopperSessionId());
		if (isset($params['paResponse'])) {
			$paymentDetails->info3DSecure->paResponse = $params['paResponse'];
		}

		if (isset($this->params['contactEmail']) && '' != $this->params['contactEmail'][0]) {
			$order->shopper->shopperEmailAddress = $this->params['contactEmail'][0];
		}
		$order->shopper->browser->acceptHeader = $this->_getShopperAcceptHeader();
		$order->shopper->browser->userAgentHeader = $this->_getShopperUserAgentHeader();
		if (isset($params['echoData'])) {
			$order->echoData = $params['echoData'];
		}
		return $request;
	}

	private function _createRequest()
	{
		$request = new \SimpleXMLElement(
			'<?xml version="1.0" encoding="utf-8"?>
			<!DOCTYPE paymentService PUBLIC "-//WorldPay/DTD WorldPay PaymentService v1//EN" "http://dtd.worldpay.com/paymentService_v1.dtd">
			<paymentService/>'
		);
		$request->addAttribute('version', '1.4');
		$request->addAttribute('merchantCode', $this->config["merchantCode"]);
		return $request;
	}

	private function _doRequest(\SimpleXMLElement $request, array &$cookies = array())
	{
		$this->_getClient()->setRawData($request->asXML(), 'text/xml');
		$this->_getClient()->setCookie($cookies);
		$httpResponse = $this->_getClient()->request();
		dbg($this->_getClient()->getLastRequest());
		dbg($httpResponse->asString());
		if (!$httpResponse->isSuccessful()) {
			throw new \Exception("HTTP error: HTTP/{$httpResponse->getVersion()} {$httpResponse->getStatus()} {$httpResponse->getMessage()}");
		}
		$response = new \SimpleXMLElement($httpResponse->getBody());
		if (isset($response->reply->error)) {
			throw new \Exception((string)$response->reply->error);
		}
		foreach ($this->_getClient()->getCookieJar()->getAllCookies() as $cookie) {
			$cookies[$cookie->getName()] = $cookie->getValue();
		}
		dbg($cookies);
		return $response;
	}

	private function _addAmount(\SimpleXMLElement $request)
	{
		$amount = $request->addChild('amount');
		$currency = Factory()->Currency($this->params['currencyID'][0]);
		$amount->addAttribute('currencyCode', $currency->id);
		$amount->addAttribute('exponent', $currency->exponent);
		$amount->addAttribute('value', (int)((float)$this->params['paymentAmount'][0] * pow(10, $currency->exponent)));
	}

	private function _getClient()
	{
		if (is_null($this->_client)) {
			$this->_client = new \Zend_Http_Client($this->paymentURL);
			$this->_client->setMethod(\Zend_Http_Client::POST);
			$this->_client->setAuth($this->config['merchantCode'], $this->config['xmlPassword']);
			$this->_client->setCookieJar();
		}
		return $this->_client;
	}

	private function _getOrderCode()
	{
		return $this->params['paymentTransactionId'][0]."__".Settings()->core_instanceId;
	}

	private function _getCvv2()
	{
		return isset($this->params['cardCVV2']) && '' != $this->params['cardCVV2'][0] ? $this->params['cardCVV2'][0] : null;
	}

    private function _getPaymentMethodCode()
	{
		if (!isset($this->_paymentMethodCodes[$this->params['billingCardType'][0]])) {
			throw new \Exception("Invalid card type: {$this->params['billingCardType'][0]}");
		}
		return $this->_paymentMethodCodes[$this->params['billingCardType'][0]];
	}

	private $_client;
	private $_paymentStatus;
	private $_paymentMethodCodes = array(
		PaymentGateway::TYPE_CARD_VISA => 'VISA-SSL',
		PaymentGateway::TYPE_CARD_MASTERCARD => 'ECMC-SSL',
		PaymentGateway::TYPE_CARD_AMEX => 'AMEX-SSL',
		PaymentGateway::TYPE_CARD_DISCOVER => 'DISCOVER-SSL',
		PaymentGateway::TYPE_CARD_JCB => 'JCB-SSL',
		PaymentGateway::TYPE_CARD_DINERS => 'DINERS-SSL',
		PaymentGateway::TYPE_CARD_SOLO => 'SOLO_GB-SSL',
		PaymentGateway::TYPE_CARD_MAESTRO => 'MAESTRO-SSL',
		PaymentGateway::TYPE_CARD_DELTA => 'VISA-SSL',
		PaymentGateway::TYPE_CARD_ELECTRON => 'VISA-SSL',
	);
}
