<?php
/***************************************************************************
 *   Copyright (C) 2011 by Vasily M. Stashko                               *
 *   vstashko@mail.ru                                                      *
 ***************************************************************************/
/* $Id$ */

	final class RegRuModule
	{
		const BASE_URL = 'https://api.reg.ru/api/regru2/';
		
		protected static $instance;	// object instance
		
		protected static $curl;
		protected static $username;
		protected static $password;
		protected static $errorText;
		
		private function __construct()	{ /* ... */ }
		private function __clone()		{ /* ... */ }
		
		public static function me($username, $password)
		{
			if (is_null(self::$instance)) {
				self::$instance = new RegRuModule;
			}
			
			self::$username = $username;
			self::$password = $password;
			
			return self::$instance;
		}
		
		public function getErrorText()
		{
			return self::$errorText;
		}
		
		public function domainCreate($parameters)
		{
			return $this->execCurl('domain/create', $parameters);
		}
		
		public function domainTransfer($parameters)
		{
			return $this->execCurl('domain/transfer', $parameters);
		}
		
		public function domainRenew($parameters)
		{
			$parameters['servtype'] = 'srv_hosting_ispmgr';
			
			return $this->execCurl('service/renew', $parameters);
		}
		
		protected function getCurl()
		{
			if (is_null(self::$curl)) {
				self::$curl = curl_init();
			}
			
			return self::$curl;
		}
		
		protected function getData()
		{
			return array(
				'username'		=> self::$username,
				'password'		=> self::$password,
				'output_format'	=> 'json',
				'comment'		=> 'Reg.ru module for CBM',
				'fail_if_no_money'	=> 0
			);
		}
		
		protected function execCurl($function, $parameters)
		{
			$curl = $this->getCurl();
			
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($curl, CURLOPT_POST, true);
			curl_setopt($curl, CURLOPT_URL, self::BASE_URL.$function);
			curl_setopt($curl, CURLOPT_POSTFIELDS, array_merge($parameters, $this->getData()));
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
			
			$answer = json_decode(curl_exec($curl));
			$result = $answer->result == 'success' ? true : false;
			
			if ($result) {
				self::$errorText = '';
			} else {
				self::$errorText = $answer->error_text ? $answer->error_text : curl_error($curl);
				
				if (isset($answer->error_params->error_detail)) {
					$errors		= array();
					$errors[]	= $answer->error_code.': '.$answer->error_text;
					
					foreach ($answer->error_params->error_detail as $field => $error) {
						$errors[] = $field.': '.$error->error_text;
					}
					
					self::$errorText = $errors;
				}
			}
			
			return $result;
		}
	}
?>
