<?php
/***************************************************************************
 *   Copyright (C) 2011 by Vasily M. Stashko                               *
 *   vstashko@mail.ru                                                      *
 ***************************************************************************/
/* $Id$ */

	final class RegRuValidate
	{
		const RUSSIAN_PATTERN				= '/^([а-яА-ЯёЁ0-9a-zA-Z"\.\, _+\-\:\s(\)(\/)]+)$/';
		const ENGLISH_PATTERN				= '/^[0-9a-zA-Z"\.\,_+\-\s]+$/';
		const COMPANY_PATTERN				= '/^[^!*+\'"\$\\\\\/\t\s]+(\s[^!*+\'"\$\\\\\/\t]+){1,}$/';
		const PHONE_PATTERN					= '/(\s*\+[0-9]{1,3} [0-9]{1,5} [0-9]+\s*(\n)*)/';
		const PHONE_INT_PATTERN				= '/^\\+[1-9]\\d{0,2}\\.\\d{5,12}$/';
		const MAIL_PATTERN					= '/([a-zA-Z0-9\!\#\$\%\&\'\*\+\-\/\=\?\^\_\`\{\|\}\~]+(\.[a-zA-Z0-9\!\#\$\%\&\'\*\+\-\/\=\?\^\_\`\{\|\}\~]+)*@[a-zA-Z0-9][\w\.-]*[a-zA-Z0-9]\.[a-zA-Z][a-zA-Z\.]*[a-zA-Z](\n)*)/';
		const NUMBER_PATTERN				= '/^\d+$/';
		const DATE_PATTERN					= '/^(\d{2})\.(\d{2})\.(\d{4})$/';
		const PASSPORT_PATTERN				= '/^\d{2} \d{2} \d{6} выдан[\s\d\S]+ \d{2}\.\d{2}\.\d{4}$/';
		const COUNTRY_PATTERN				= '/^[A-Z]+$/';
		const PERSON_PATTERN				= '/^[A-Z][a-z]+ [A-Z] [A-Z][a-z]+$/';
		const PERSON_R_PATTERN				= '/^[А-Я][а-я]+ [А-Я][а-я]+ [А-Я][а-я]+$/u';
		const RSELNEXUS_APP_PURPOSE_PATTERN	= '/^P[1-5]$/';
		const RSELNEXUS_CATEGORY_PATTERN	= '/^C(11|12|21|31|32)$/';
		const ENGLISH_UP_PATTERN			= '/^[A-Z][-A-Za-z ]+$/';
		
		protected static $names = array(
			'inn'					=> 'inn',
			'phone'					=> 'phone',
			'phone_int'				=> 'phoneInternational',
			'mail'					=> 'mail',
			'english'				=> 'english',
			'russian'				=> 'russian',
			'number'				=> 'number',
			'date'					=> 'date',
			'birth_date'			=> 'birthDate',
			'passport'				=> 'passport',
			'country'				=> 'country',
			'person'				=> 'person',
			'person_r'				=> 'personR',
			'RselnexusAppPurpose'	=> 'RselnexusAppPurpose',
			'RselnexusCategory'		=> 'RselnexusCategory',
			'company'				=> 'company',
			'english-up'			=> 'englishUp',
		);
		
		protected $country = null;
		
		public function validate($type, $value)
		{
			if (isset(self::$names[$type])) {
				$method = self::$names[$type];
				return $this->$method($value);
			}
			
			return true;
		}
		
		public function birthDate($value)
		{
			if (
				preg_match(self::DATE_PATTERN, $value, $matches)
				&& checkdate($matches[2], $matches[1], $matches[3])
			)  {
				if ($this->country != 'RU') {
					return true;
				}
				
				$age = $this->getAgeByBirthDate($matches);
				
				if ($age < 14 || $age > 100) {
					return false;
				}
				
				return true;
			}
			
			return false;
		}
		
		public function date($value)
		{
			if (preg_match(self::DATE_PATTERN, $value, $matches)) {
				return checkdate($matches[2], $matches[1], $matches[3]);
			}
			
			return false;
		}
		
		public function person($value)
		{
			if ($this->country != 'RU') {
				return true;
			}
			
			return preg_match(self::PERSON_PATTERN, $value);
		}
		
		public function personR($value)
		{
			if ($this->country != 'RU') {
				return true;
			}
			
			return preg_match(self::PERSON_R_PATTERN, $value);
		}
		
		public function passport($value)
		{
			if ($this->country != 'RU') {
				return true;
			}
			
			return preg_match(self::PASSPORT_PATTERN, $value);
		}
		
		public function RselnexusCategory($value)
		{
			return preg_match(self::RSELNEXUS_CATEGORY_PATTERN, $value);
		}
		
		public function RselnexusAppPurpose($value)
		{
			return preg_match(self::RSELNEXUS_APP_PURPOSE_PATTERN, $value);
		}
		
		public function number($value)
		{
			return preg_match(self::NUMBER_PATTERN, $value);
		}
		
		public function russian($value)
		{
			return preg_match(self::RUSSIAN_PATTERN, $value);
		}
		
		public function english($value)
		{
			return preg_match(self::ENGLISH_PATTERN, $value);
		}
		
		public function englishUp($value)
		{
			return preg_match(self::ENGLISH_UP_PATTERN, $value);
		}
		
		public function company($value)
		{
			if (preg_match(self::ENGLISH_PATTERN, $value))
				return preg_match(self::COMPANY_PATTERN, $value);
			
			return false;
		}
		
		public function phone($values)
		{
			return $this->validateManyValuesInOneField(self::PHONE_PATTERN, $values);
		}
		
		public function phoneInternational($value)
		{
			return preg_match(self::PHONE_INT_PATTERN, $value);
		}
		
		public function mail($values)
		{
			return $this->validateManyValuesInOneField(self::MAIL_PATTERN, $values);
		}
		
		public function country($value)
		{
			if (preg_match(self::COUNTRY_PATTERN, $value)) {
				$this->country = $value;
				
				return true;
			}
			
			return false;
		}
		
		public function inn($inn)
		{
            if (preg_match('/\D/', $inn))
				return false;
			
			$inn = (string)$inn;
			$len = strlen($inn);
			
			if ($len === 10) {
				return
					$inn[9] === (string)(
						(
							(
								2*$inn[0] + 4*$inn[1] + 10*$inn[2] + 
								3*$inn[3] + 5*$inn[4] +  9*$inn[5] + 
								4*$inn[6] + 6*$inn[7] +  8*$inn[8]
							) % 11
						) % 10
					);
			} elseif ($len === 12) {
				$num10 = (string)(
					(
						(
							7*$inn[0]  + 2*$inn[1] + 4*$inn[2] +
							10*$inn[3] + 3*$inn[4] + 5*$inn[5] + 
							9*$inn[6]  + 4*$inn[7] + 6*$inn[8] +
							8*$inn[9]
						) % 11
					) % 10
				);
				
				$num11 = (string)(
					(
						(
							3*$inn[0] +  7*$inn[1] + 2*$inn[2] +
							4*$inn[3] + 10*$inn[4] + 3*$inn[5] +
							5*$inn[6] +  9*$inn[7] + 4*$inn[8] +
							6*$inn[9] +  8*$inn[10]
						) % 11
					) % 10
				);
				
				return $inn[11] === $num11 && $inn[10] === $num10;
			}
			
			return false;
		}
		
		protected function getAgeByBirthDate(Array $matches)
		{
			$actualDate = time();
			
			$actualYear		= (integer)date('Y', $actualDate);
			$actualMonth	= (integer)date('n', $actualDate);
			$actualDay		= (integer)date('j', $actualDate);
			
			$result = $actualYear - $matches[3];
			
			if (
				$actualMonth < $matches[2]
				|| (
					$actualMonth == $matches[2]
					&& $actualDay < $matches[1]
				)
			) {
					--$result;
			}
			
			return $result;
		}

		public function validateManyValuesInOneField($regexp, $values)
		{
			$todoBeEmpty = $values;
			if (preg_match_all($regexp, $values, $arrayValues)) {
				if (isset($arrayValues[1])) {
					$todoBeEmpty = str_replace($arrayValues[1], '', $todoBeEmpty);
					$todoBeEmpty = trim($todoBeEmpty);
					return empty($todoBeEmpty);
				}
			} 
			return false;
		}
	}



?>
