<?php
use Billing\Checker;

class RuCenterChecker extends Checker {
    const MINUS18YEARS = '-18years';

    const P_DATE_DDMMYYYY = '^(31|[123]0|[012][1-9])+\.(0[1-9]|1[0-2])+\.([12][0-9]{3})+$';
    const P_ENGLISH_CHARACTERS = '^[a-zA-Z\-. ]+$';

    const P_TWO_WORDS = '^\w+\s+\w+.*$';
    const P_FULL_ORG_NAME = '^[^\s#]+\s+[^\s#]+[^#]*$'; // minimum of two words, do not contain hash symbol (#)
    const P_NOT_EMPTY = '^\w+.*$';
    const P_DIGITS = '^\d*$';

    public static function phone($phone)
    {
        self::_setInternalEncoding('utf8');
        if (!parent::phone($phone) || !(0 === mb_strpos($phone, '+'))) {

            return false;
        }

        return true;
    }

    public static function checkRegexp($pattern, $value)
	{
        self::_setInternalEncoding('utf8');

		return (0 < mb_ereg($pattern, $value));
	}

    public static function checkMaxDate($maxDate, $value)
    {
        self::_setInternalEncoding('utf8');
        $date = strtotime(mb_ereg_replace(self::P_DATE_DDMMYYYY, "\\2/\\1/\\3", $value));
        if (!$date || $date > $maxDate) {

            return false;
        }

        return true;
    }

    public static function checkMaxLength($length, $value)
    {
        self::_setInternalEncoding('utf8');
        if (mb_strlen($value) > $length) {

            return false;
        }

        return true;
    }

    private static function _setInternalEncoding($encoding = 'utf8')
    {
        if ($encoding !== self::$_encoding) {
            self::$_encoding = $encoding;
            mb_internal_encoding($encoding);
            mb_regex_encoding($encoding);
        }
    }

    private static $_encoding = null;
}