<?php
require_once('lib-mbapi/include/modules/registrar/registrar.php');
require_once('lib-mbapi/include/modules/registrar/core/regru/RegRuModule.php');

use Billing\Object\Contact;
use Billing\Object\Tld;

$className = 'regru';
$GLOBALS['moduleInfo'][$className] = array(
    'type'			=> 'registrar',               //module type, MUST be registrar
    'status'		=> Registrar::STATUS_STABLE,  //module status, can be Registrar::STATUS_STABLE or Registrar::STATUS_TESTING
    'version'       => '1.0.0',
    'displayName'	=> 'Reg.ru',                  //module name visible in CBM
    'author'        => 'Reg.ru',
    'countryCodes' => array('RU'),
);

class regru extends Registrar
{
	const REGRU_ORG_TYPE		= 1;
	const REGRU_INDIVIDUAL_TYPE	= 2;

	private static $contractNames = array(
		self::REGRU_ORG_TYPE		=> 'TRANS_REGISTRAR_REGRU_ORG_TYPE',
		self::REGRU_INDIVIDUAL_TYPE	=> 'TRANS_REGISTRAR_REGRU_INDIVIDUAL_TYPE',
	);

	private static $translitTlds = array(
		'tj', 'com.ua', 'kiev.ua', 'pp.ua', 'com', 'net', 'org', 'biz', 'name', 'info', 'mobi', 'uk', 'cc', 'tv', 'ws', 'bz', 'me', 'us'
			);
    /**
     * Returns the module display. This includes version number and display name.
     * @return    An array of arrays containing all the display info for the module.
     */
    public function getModuleInfo()
    {
        return $GLOBALS['moduleInfo']['regru'];
    }

	/**
	 * Construct a new regru object. This function merely sets up capabilities after calling the
	 * constructor for Registrar.
	 */
	public function __construct($input)
	{
		mb_internal_encoding('UTF8');
		mb_regex_encoding('UTF8');

		// quick fix to enable IDN support across all functions
		if (isset($this->input['domainSLD']) && trim($this->input['domainSLD']) != '') {
			$this->input['domainSLD'] = $this->getPUNYCode($input);
		}

		$this->capabilities[REGISTRAR_REGISTERDOMAIN]					= 1;
		$this->capabilities[REGISTRAR_TRANSFERDOMAIN]					= 1;
		$this->capabilities[REGISTRAR_RENEWDOMAIN]						= 1;
		$this->capabilities[REGISTRAR_GETDOMAINEXTENDEDINFO]			= 0;
		$this->capabilities[REGISTRAR_CHECKDOMAINAVAILABILITY]			= 0;
		$this->capabilities[REGISTRAR_CHECKDOMAINTRANSFERAVAILABILITY]	= 0;

		//Domain Management options
		$this->capabilities[REGISTRAR_GETDOMAINCONTACTDATA]				= 0;
		$this->capabilities[REGISTRAR_SETDOMAINCONTACTDATA]				= 0;
		$this->capabilities[REGISTRAR_GETDOMAINRENEWTYPEOPTIONS]		= 0;
		$this->capabilities[REGISTRAR_SETDOMAINRENEWTYPEOPTIONS]		= 0;
		$this->capabilities[REGISTRAR_GETDOMAINREGISTRARLOCKOPTIONS]	= 0;
		$this->capabilities[REGISTRAR_SETDOMAINREGISTRARLOCKOPTIONS]	= 0;
		$this->capabilities[REGISTRAR_GETDOMAINACCESSPASSWORD]			= 0;
		$this->capabilities[REGISTRAR_SETDOMAINACCESSPASSWORD]			= 0;
		$this->capabilities[REGISTRAR_GETDOMAINDNS]						= 0;
		$this->capabilities[REGISTRAR_SETDOMAINDNS]						= 0;
		$this->capabilities[REGISTRAR_GETDOMAINRENEWDATE]				= 0;
		$this->capabilities[REGISTRAR_LIVERENEW]						= 0;

		$this->capabilities[REGISTRAR_CHECKACCOUNTBALANCE]				= 0;
		$this->capabilities[REGISTRAR_GETTLDPRICING]					= 0;
		$this->capabilities[REGISTRAR_GETTRANSFERSTATUS]				= 0;
		$this->capabilities[REGISTRAR_GETTLDLIST]						= 0;

		$this->capabilities[REGISTRAR_TLDS] = array(
			'ru', 'su',
			'tj',
			'com.ua', 'kiev.ua',
			//'pp.ua',
			//'kz',
			'com', 'net', 'org', 'biz', 'name', 'info', 'mobi', 'uk', 'cc', 'tv', 'ws', 'bz', 'me',
			'us',
//			'in','co.in','firm.in','gen.in','ind.in',
//			'net.in','org.in','me','us','ws','es','com.es','org.es','nom.es',
//			'com.mx','bz','com.bz','net.bz',
		);

		parent::__construct($input);

		$this->capabilities[REGISTRAR_UPLOADDOCUMENTS]					= 0;
	}

    // this function registers a new domain
    public function registerDomain()
    {
        /**
         * $this->input - an array of a domain regisrtation subscription attributes. Includes the following:
         * domainSLD, domainTLD, domainYears, domainNameserver1, domainNameserver2, domainNameserver3, domainNameserver4,
         * registrant info (registrantFirstName, registrantLastName, registrantOrganizationName,
         * registrantJobTitle, registrantAddress1, registrantAddress2, registrantCity, registrantState,
         * registrantPostalCode, registrantCountry, registrantEmail, registrantPhone, registrantFax etc
         */

        /**
         * Make an appropriate request to registrar (use Curl, Soap etc)
         * using $this->UID and $this->PW ('moduleUsername','modulePassword')
         * $registrarResponse = callToRegistrar(prepareRequest($this->input));
         * initialize the arrayData variable (what params function will returns)
         * $this->arrayData = array();
         */

        /**
         * Registrar respnse processing:
         * self::ACTION_STATUS_COMPLETED - a corresponsing Business Manager event gets 'completed', the subscription gets 'active'
         * self::ACTION_STATUS_WAITING - the event still has the 'new' status, the subscription get the 'work in progress' status. The event will be executed again in 5 minutes
         * self::ACTION_STATUS_ERROR - the event gets the 'error' status, the subscription gets 'failed'
         */
		$xml = @simplexml_load_file(dirname(__FILE__).DIRECTORY_SEPARATOR.get_class($this).'.xml');

		if (!$xml) {
			$this->addError(blmsg('TRANS_REGISTRARERROR'), get_class($this).'.xml not found or parse error');
			return $this->getRegistrarReturnXML();
		}

		$form = $xml->
			xpath("/configuration/createForms/createForm/tld[@name='".$this->input['domainTLD']."']/..");

		if (count($form) != 1) {
			$this->addError(blmsg('TRANS_REGISTRARERROR'), get_class($this).'.xml structure error');
			return $this->getRegistrarReturnXML();
		}

		$form = reset($form);

		if (isset($this->input['contract_type'])) {
			$contractType = $this->input['contract_type'];
		}

		$parameterList = array();

		foreach ($form->field as $fieldItem) {

			if (isset($contractType)) {
				$itemContractType = (integer)$fieldItem['contractType'];

				if (!empty($itemContractType) && $itemContractType != $contractType) {
					continue;
				}
			}

			$requestKey	= (string)$fieldItem['id'];
			$inputKey	= $requestKey;

			$required	= (string)$fieldItem['required'] == 'true' ? true : false;
			if ($required && empty($this->input[$inputKey])) {
				// error
				$registrarResponse['errors'][] = 'Fatal error. Required field '.$requestKey.' is empty.';
				continue;
			}

			$parameterList[$requestKey] = $this->input[$inputKey];
		}

		$parameterList['domain_name']	= $this->input['domainSLD'].'.'.$this->input['domainTLD'];
		$parameterList['period']		= $this->input['domainYears'];
		$parameterList['ns0']			= $this->input['domainNameserver1'];
		$parameterList['ns1']			= $this->input['domainNameserver2'];
		$parameterList['ns2']			= $this->input['domainNameserver3'];
		$parameterList['ns3']			= $this->input['domainNameserver4'];

                $this->execOutput['nsPassed'] = array();
                for ($i=1; $i<=4; $i++) {
                    if ($this->input["domainNameserver$i"]) {
                        $this->execOutput['nsPassed'][] = $this->input["domainNameserver$i"];
                    }
                }

		if ($this->input['domainTLD'] == 'ru' || $this->input['domainTLD'] == 'su') {
			$parameterList['period'] = 1;
		}

		$regRuModule = RegRuModule::me($this->UID, $this->PW);
		$result = $regRuModule->
			domainCreate($parameterList);

		if ($result) {
			$this->arrayData = 'success';
			$this->commandStatus = self::ACTION_STATUS_COMPLETED;
		} else {
			$this->arrayData	= 'error';
			$error				= $regRuModule->getErrorText();

			if (is_array($error)) {
				foreach ($error as $errorMessage) {
					$this->addError(blmsg('TRANS_REGISTRARERROR'), $errorMessage);
				}
			} else {
				$this->addError(blmsg('TRANS_REGISTRARERROR'), $error);
			}

			$this->commandStatus = self::ACTION_STATUS_ERROR;
		}

        // the function transfer the $this->arrayData to xml and writes the data to the subscription attributes
        return $this->getRegistrarReturnXML();
    }

    /**
     * This finction is used to set the status of a domain name registration, transfer, or renewal
     * @return true/false (true if the event is completed, otherwise - false)
     */
    public function isDone()
    {
        if (self::ACTION_STATUS_WAITING == $this->commandStatus) {
            return false;
        }
        return true;
    }

	// this function performs a domain transfer
	public function transferDomain()
	{
		/**
		 *  The function is similar to the registerDomain function, the difference is the request structure
		 *	and the using of the $this->input[domainTransferKey] parameter that defines the transferability of the domain
		 */
		$xml = @simplexml_load_file(dirname(__FILE__).DIRECTORY_SEPARATOR.get_class($this).'.xml');

		if (!$xml) {
			$this->addError(blmsg('TRANS_REGISTRARERROR'), get_class($this).'.xml not found or parse error');
			return $this->getRegistrarReturnXML();
		}

		$form = $xml->
			xpath("/configuration/createForms/createForm/tld[@name='".$this->input['domainTLD']."']/..");

		if (count($form) != 1) {
			$this->addError(blmsg('TRANS_REGISTRARERROR'), get_class($this).'.xml structure error');
			return $this->getRegistrarReturnXML();
		}

		$form = reset($form);

		if (isset($this->input['contract_type'])) {
			$contractType = $this->input['contract_type'];
		}

		$parameterList = array();

		foreach ($form->field as $fieldItem) {

			if (isset($contractType)) {
				$itemContractType = (integer)$fieldItem['contractType'];

				if (!empty($itemContractType) && $itemContractType != $contractType) {
					continue;
				}
			}

			$requestKey	= (string)$fieldItem['id'];
			$inputKey	= $requestKey;

			$required	= (string)$fieldItem['required'] == 'true' ? true : false;
			if ($required && empty($this->input[$inputKey])) {
				// error
				$registrarResponse['errors'][] = 'Fatal error. Required field '.$requestKey.' is empty.';
				continue;
			}

			$parameterList[$requestKey] = $this->input[$inputKey];
		}

		$parameterList['domain_name']	= $this->input['domainSLD'].'.'.$this->input['domainTLD'];
		$parameterList['period']		= $this->input['domainYears'];
		$parameterList['ns0']			= $this->input['domainNameserver1'];
		$parameterList['ns1']			= $this->input['domainNameserver2'];
		$parameterList['ns2']			= $this->input['domainNameserver3'];
		$parameterList['ns3']			= $this->input['domainNameserver4'];

		if ($this->input['domainTLD'] == 'ru' || $this->input['domainTLD'] == 'su') {
			$parameterList['period'] = 0;
		}

		if (in_array($this->input['domainTLD'], self::$translitTlds)) {
			$parameterList['authinfo'] = $this->input['authinfo'];
		}

		$regRuModule = RegRuModule::me($this->UID, $this->PW);
		$result = $regRuModule->domainTransfer($parameterList);

		if ($result) {
			$this->arrayData = 'success';
			$this->commandStatus = self::ACTION_STATUS_COMPLETED;
		} else {
			$this->arrayData	= 'error';
			$error				= $regRuModule->getErrorText();

			if (is_array($error)) {
				foreach ($error as $errorMessage) {
					$this->addError(blmsg('TRANS_REGISTRARERROR'), $errorMessage);
				}
			} else {
				$this->addError(blmsg('TRANS_REGISTRARERROR'), $error);
			}

			$this->commandStatus = self::ACTION_STATUS_ERROR;
		}

        // the function transfer the $this->arrayData to xml and writes the data to the subscription attributes
        return $this->getRegistrarReturnXML();
	}

	// this function renews a  domain
	public function renewDomain()
	{
		// The function is similar to the registerDomain function, the difference is the request structure.
		// Самурай без меча подобен самураю с мечом, но только без меча.

		$parameterList = array();

		$parameterList['domain_name']	= $this->input['domainSLD'].'.'.$this->input['domainTLD'];
		$parameterList['period']		= $this->input['domainYears'];

		$regRuModule = RegRuModule::me($this->UID, $this->PW);
		$result = $regRuModule->domainRenew($parameterList);

		if ($result) {
			$this->arrayData		= 'success';
			$this->commandStatus	= self::ACTION_STATUS_COMPLETED;
		} else {
			$this->arrayData	= 'error';
			$error				= $regRuModule->getErrorText();

			if (is_array($error)) {
				foreach ($error as $errorMessage) {
					$this->addError(blmsg('TRANS_REGISTRARERROR'), $errorMessage);
				}
			} else {
				$this->addError(blmsg('TRANS_REGISTRARERROR'), $error);
			}

			$this->commandStatus = self::ACTION_STATUS_ERROR;
		}

        // the function transfer the $this->arrayData to xml and writes the data to the subscription attributes
        return $this->getRegistrarReturnXML();
	}

	protected function getDefaultConfigOptions()
	{
		return array(
			'TRANS_REGISTRAR_MODULE_SETTINGS' => array(
				array(
					'moduleUsername',
					't',
					1,
					NULL,
					blmsg('TRANS_REGISTRAR_REGRU_LOGIN'),
					NULL,
					NULL
				),
				array(
					'modulePassword',
					'p',
					1,
					NULL,
					blmsg('TRANS_REGISTRAR_REGRU_PASSWORD'),
					NULL,
					NULL
				),
			),
			'TRANS_REGISTRAR_NS' => array(
				array('domainNameserver1', 't', 1, 'ns1.reg.ru', 'TRANS_REGISTRAR_NS1', NULL, NULL, 'TRANS_REGISTRAR_NS_DESC'),
				array('domainNameserver2', 't', 1, 'ns2.reg.ru', 'TRANS_REGISTRAR_NS2', NULL, NULL, 'TRANS_REGISTRAR_NS_DESC'),
				array('domainNameserver3', 't', 0, NULL, 'TRANS_REGISTRAR_NS3', NULL, NULL, 'TRANS_REGISTRAR_NS_DESC'),
				array('domainNameserver4', 't', 0, NULL, 'TRANS_REGISTRAR_NS4', NULL, NULL, 'TRANS_REGISTRAR_NS_DESC'),
			)
		);
		}

	/** Get form field descriptor for domain contact information (filled in Store and in Admin CP before domain buying)
	 * param string $action - (register, transfer)
	 * param string $domain_sld
	 * param string $domain_tld
	 */
	public function getRegistrantParams($action, $domain_sld, Tld $tld, Contact $contact = NULL, $withTLDParams = false)
	{
        $domain_tld = $tld->tld_extension;
		$xml = @simplexml_load_file(dirname(__FILE__).DIRECTORY_SEPARATOR.get_class($this).'.xml');
		if (!$xml) {
			return parent::getRegistrantParams($action, $domain_sld, $tld, $contact, $withTLDParams);
        }

		$form = $xml->xpath("/configuration/forms/form/tld[@name='".$domain_tld."']/..");

		if (count($form) != 1) {
			$form = $xml->xpath("/configuration/forms/form/tld[@name='default']/..");

			if (count($form) != 1) {
				return parent::getRegistrantParams(
					$action, $domain_sld, $tld, $contact, $withTLDParams
				);
			}
		}
        $client_id = is_null($contact) ? NULL : $contact->client_id;
        $priorSubscription =  is_null($client_id) ? NULL : $this->getPriorSubscription($this->input["configGroupID"], $client_id);
        $form = reset($form);
		$inputs = array();

		$countries = array();
		foreach (Service()->CountryManager->getCountries() as $code => $name) {
			$countries[] = array($code, $name);
		}

		$contractList = array();
		foreach ($form->contractType as $contractTypeItem) {
			$showList = array();
			foreach ($contractTypeItem->show as $showItem) {
				$showList[] = (string)$showItem['id'];
			}
			$contractList[(integer)$contractTypeItem['id']] = $showList;
		}

		// this is workaround!!! empty($contact->contact_company) always true
		$companyName = '';
		if (!is_null($contact))
			$companyName = $contact->contact_company;
		// //////////////////////////////////////////////////////////////////

        $contractType = empty($companyName) ? self::REGRU_INDIVIDUAL_TYPE : self::REGRU_ORG_TYPE;
        if (!is_null($priorSubscription) && isset($priorSubscription->PackageAttributes["contractType"])) {
            $contractType = $priorSubscription->PackageAttributes["contractType"]->value;
        }
		if (count($contractList) > 1) {
			$item = array(
				'id'			=> 'contractType',
				'type'			=> 's',
				'title'			=> blmsg('TRANS_TYPE'),
				'default'		=> $contractType,
			);

			$items = array();

			foreach ($contractList as $contractListKey => $contractListItem) {
				$items[] = array(
					$contractListKey,
					blmsg(self::$contractNames[$contractListKey]),
					$contractListItem,
					null
				);
			}

			$item['items']	= $items;

            $item["read_only"] = !is_null($priorSubscription) ? 1 : 0 ;
			$inputs[] = $item;
		}

		foreach ($form->input as $inputItem) {
			$default		= '';
			$defaultString	= trim((string)$inputItem['default']);

			if (!empty($defaultString)) {
				$defaultKeyList = explode(' ', $defaultString);

				if (!(empty($defaultKeyList) || is_null($contact))) {
					$defaultList = array();

					foreach ($defaultKeyList as $key) {
						$defaultList[] = $contact->$key;
					}
					$default = implode(' ', $defaultList);
				}

				if (in_array($domain_tld, self::$translitTlds)) {
					require_once('lib-mbapi/include/modules/registrar/core/regru/RegRuReplace.php');
					$default = RegRuReplace::toTranslit($default);
				}
			}

			$description		= '';
			$itemDescription	= blmsg((string)$inputItem->description);
			$itemExample		= blmsg((string)$inputItem->example);

			if (!empty($itemDescription))
				$description .= $itemDescription;
			if (!empty($itemExample))
				$description .= '<br/><em><strong>'
					.blmsg('TRANS_REGISTRAR_REGRU_EXAMPLE')
					.':</strong> '
					.$itemExample
					.'</em>';

			$required = (string)$inputItem['required'] == 'true' ? true : false;
            $id = reset($inputItem["id"]);
            if (!is_null($priorSubscription) && isset($priorSubscription->PackageAttributes[$id])) {
                $default = $priorSubscription->PackageAttributes[$id]->value;
            }

			$item = array(
				'id'			=> (string)$inputItem['id'],
				'type'			=> (string)$inputItem['type'],
				'title'			=> blmsg((string)$inputItem->title),
				'description'	=> $description,
				'required'		=> $required,
				'hidden'		=> isset($contractList[$contractType]) && !in_array((string)$inputItem['id'], $contractList[$contractType]),
				'default'		=> $default,
			);

			if ('x' == $item['type']) {
				$item['default'] = '<p align="center"><strong>'.$description.'</strong></p>';
			} elseif ('s' == $item['type']) {
				$items = array();

				foreach ($inputItem->item as $itemItem) {
					$show = array();

					foreach ($itemItem->show as $showItem) {
						$show[] = (string)$showItem['id'];
					}

					if (empty($show))
						$show = null;

					$items[] = array((string)$itemItem['value'], blmsg((string)$itemItem['title']), $show, null);
				}

				$item['items']	= $items;
			} elseif ('country' == $item['type']) {
                $default = is_null($contact) ? 'RU' : $contact->Country->countries_iso_2;
                if (!is_null($priorSubscription) && isset($priorSubscription->PackageAttributes['country'])) {
                    $default = $priorSubscription->PackageAttributes['country']->value;
                }
				$item['default']	= $default;
				$item['type']		= 's';
				$item['items']		= $countries;
			}
            $item["read_only"] = !is_null($priorSubscription) ? 1 : 0 ;
			$inputs[] = $item;
		}

		if (
			'transfer' == $action
			&& $this->input['domainTLD'] != 'ru'
			&& $this->input['domainTLD'] != 'su'
		) {
            $inputs[] = array(
            	'id'			=> 'authinfo',
            	'title'			=> blmsg('TRANS_DOMAIN_TRANSFER_AUTHCODE_TITLE'),
            	'description'	=> blmsg('TRANS_DOMAIN_TRANSFER_AUTHCODE_DESC'),
            	'type'			=> 't',
            	'required'		=> 1,
                'read_only'     => !is_null($priorSubscription) ? 1 : 0,
            );
		}

		$registrantParams = array(
			array(
				'title'			=> (string)$form['title'],
				'description'	=> (string)$form['description'],
				'inputs'		=> $inputs,
			)
		);

		return $registrantParams;
	}

	public function collectRegistrantParams($action, $domain_sld, Tld $tld, Contact $contact = NULL)
	{
        $domain_tld = $tld->tld_extension;
        $collectedParams = parent::collectRegistrantParams($action, $domain_sld, $tld, $contact);
        unset($collectedParams['errors_list']);
        $priorSubscription = !is_null($contact) ? $this->getPriorSubscription($this->input["configGroupID"], $contact->client_id) : null ;
        if (!is_null($priorSubscription)) {
            $collectedParams["data"] = DataObject::getMapOf($priorSubscription->PackageAttributes, "package_attribute_name", "package_attribute_value");
        }
		$xml = @simplexml_load_file(dirname(__FILE__).DIRECTORY_SEPARATOR.get_class($this).'.xml');
		if (!$xml) {
			throw new ProductException(ProductException::E_FAIL_OPERATION, get_class($this).'.xml not found or parse error');
		}

		$form = $xml->xpath("/configuration/createForms/createForm/tld[@name='".$domain_tld."']/..");
        if (count($form) != 1) {
			throw new ProductException(ProductException::E_FAIL_OPERATION, get_class($this).'.xml structure error');
		}

		$form = reset($form);
        $contractType = isset($collectedParams['data']['contractType']) ? $collectedParams['data']['contractType'] : null;

		$messageList = array();
		require_once('lib-mbapi/include/modules/registrar/core/regru/RegRuValidate.php');
		require_once('lib-mbapi/include/modules/registrar/core/regru/RegRuReplace.php');
		$regRuValidate = new RegRuValidate;
        $inputs = $this->_getInputs($xml, $domain_tld, $contractType);


		foreach ($form->field as $fieldItem) {
			$itemContractType	= (integer)$fieldItem['contractType'];
			$requestKey			= (string)$fieldItem['id'];
            $noExclude			= isset($fieldItem['noExclude']) ? $fieldItem['noExclude'] : 0 ;
			$inputKey			= $requestKey;

			if (isset($contractType) && $itemContractType && ($contractType != $itemContractType)) {
				if (!$noExclude) {
                    unset($collectedParams['data'][$inputKey]);
                }
				continue;
			}

			$itemRequired	= (string)$fieldItem['required'] == 'true' ? true : false;
			$type			= (string)$fieldItem['type'];
			$min			= (integer)$fieldItem['min'];
			$max			= (integer)$fieldItem['max'];

			$required		= ($itemRequired && empty($collectedParams['data'][$inputKey]) && $inputs[(string)$fieldItem['id']]['required']);
			$message = null;
			if ($required) {
				$message = blmsg('TRANS_REGISTRAR_REGRU_ERROR_MISSED', array("FIELD_TITLE" => $inputs[(string)$fieldItem['id']]['title']));
			} else {
				if (
					$value = RegRuReplace::replace(
						$type,
						$collectedParams['data'][$inputKey]
					)
				) {
					$collectedParams['data'][$inputKey] = $value;
				}
				if (!empty($collectedParams['data'][$inputKey])) {
					if (
						$min
						&& mb_strlen($collectedParams['data'][$inputKey]) < $min
					) {
						$message = blmsg('TRANS_REGISTRAR_REGRU_ERROR_MIN', array("VALUE" => $min, "FIELD_TITLE" => $inputs[(string)$fieldItem['id']]['title']));
					} elseif (
						$max
						&& mb_strlen($collectedParams['data'][$inputKey]) > $max
					) {
						$message = blmsg('TRANS_REGISTRAR_REGRU_ERROR_MAX', array("VALUE" => $max, "FIELD_TITLE" => $inputs[(string)$fieldItem['id']]['title']));
					} elseif ($type) {
						if (!$regRuValidate->validate($type, $collectedParams['data'][$inputKey])) {
							$message = blmsg('TRANS_REGISTRAR_REGRU_ERROR_WRONG', array("FIELD_TITLE" => $inputs[(string)$fieldItem['id']]['title']));
						}
					}
				}
			}

			if (!empty($message))
				$messageList[$inputKey] = $message;
		}

		foreach ($collectedParams['form_descriptor'] as &$params) {
			foreach ($params['inputs'] as &$input) {
//				if (isset($collectedParams['data'][$input['id']]))
//					$input['default'] = $collectedParams['data'][$input['id']];

				if (isset($messageList[$input['id']])) {
					$input['error_msg'] =
					$collectedParams['errors_list'][$input['id']] =
						$messageList[$input['id']];
				} elseif (
					isset($input['error_msg'])
					&& isset($collectedParams['errors_list'][$input['id']])
				) {
					unset($input['error_msg'], $collectedParams['errors_list'][$input['id']]);
				}
			}
		}

		return $collectedParams;
	}

    private function _getInputs(SimpleXMLElement $xml, $domainTld, $contractType)
    {
        $inputs = array();
        $form = $xml->xpath("/configuration/forms/form/tld[@name='".$domainTld."']/..");
        if (count($form) != 1) {
            $form = $xml->xpath("/configuration/forms/form/tld[@name='default']/..");
        }
        $form = reset($form);
        $currentContractInputs = array();
        if (!is_null($contractType)) {
            foreach ($form->contractType as $contractTypeItem) {
                if (reset($contractTypeItem["id"]) != $contractType) {
                    continue;
                }
                foreach ($contractTypeItem->show as $showItem) {
                    $currentContractInputs[] = (string)$showItem['id'];
                }
            }
        }
        foreach ($form->input as $inputItem) {
            $inputId = (string)$inputItem['id'];
            $inputs[$inputId]['title'] = blmsg((string)$inputItem->title);
            $inputs[$inputId]['required'] = false;
            if ((empty($currentContractInputs) || in_array($inputId, $currentContractInputs)) && (string)$inputItem['required'] == 'true') {
                $inputs[$inputId]['required'] = true;
            }

        }
        return $inputs;

    }
}
