<?php

require_once ("lib-mbapi/include/modules/sslmodule/sslmodule.php");
use Billing\Module\Exception;

class EnomSSL extends SSLModule
{
	const CONTACT_ADMIN		= "Admin";
	const CONTACT_TECHNICAL	= "Tech";
	const CONTACT_BILLING	= "Billing";
	const CERT_PARAMS		= "Cert";

	// Services supported by EnomSSL

	const GEOTRUST_QUICKSSL				= "GeoTrust-QuickSSL";
	const GEOTRUST_QUICKSSL_PREMIUM		= "GeoTrust-QuickSSL-Premium";
	const GEOTRUST_TRUEBIZID			= "GeoTrust-TrueBizID";
	const GEOTRUST_TRUEBIZID_WILDCARD	= "GeoTrust-TrueBizID-Wildcard";
	const GEOTRUST_TRUEBIZID_EV			= "GeoTrust-TrueBizID-EV";
	const RAPIDSSL_RAPIDSSL				= "RapidSSL-RapidSSL";
	const VERISIGN_SECURE_SITE			= "VeriSign-Secure-Site";
	const VERISIGN_SECURE_SITE_PRO		= "VeriSign-Secure-Site-Pro";
	const VERISIGN_SECURE_SITE_EV		= "VeriSign-Secure-Site-EV";
	const VERISIGN_SECURE_SITE_PRO_EV	= "VeriSign-Secure-Site-Pro-EV";
	const COMODO_ESSENTIAL				= "Comodo-Essential";
	const COMODO_INSTANT				= "Comodo-Instant";

	public function getModuleInfo()
	{
		return array(
			"type"			=> "sslmodule",
			"version"		=> "0.0.1",
			"displayName"	=> "eNom",
			"author"		=> "Parallels",
		);
	}

	public function __construct($input)
	{
		parent::__construct($input);

		$this->capabilities = array(
			SSLModule::CAPABILITY_ADD,
			SSLModule::CAPABILITY_GETISSUEDCERT,
			SSLModule::CAPABILITY_GETCSR,
			SSLModule::CAPABILITY_PRODUCT_PARAMS,
			SSLModule::CAPABILITY_USER_PARAMS,
		);

		/* other capabilites possible for other cert providers
		const CAPABILITY_DELETE = "capability_delete";
		const CAPABILITY_RENEWCERT = "capability_renewcert";

		These capabilities will define how the application interacts with the certificate provider
		*/

	}

	public function getCertificates()
	{
		$certificates = array();
		$config = $this->getConfig();
		foreach (array_keys($this->services) as $service) {
			$certificates[$service] = isset($config[$service]) ? $config[$service] : false;
		}
		return $certificates;
	}

	public function getPeriod($certificate)
	{
		return isset($this->services[$certificate]) ? $this->services[$certificate]["period"] : 0;
	}

	public function getCertificateAttributeName()
	{
		return 'enomssl_service';
	}

	public function getPeriodAttributeName()
	{
		return 'enomssl_service_years';
	}

	public function cryptedConfigParams()
	{
		return array(
			"api_user_id",
			"api_user_pwd",
		);
	}

	/*
	public function cryptedUserParams()
	{
		return array_merge(
			parent::cryptedUserParams(),
			array(
				// Plugin's parameters to be crypted
			)
		);
	}
	*/

	public function getParamGroupsMap()
	{
		return array_merge($this->getContactGroupMap(), array(self::CERT_PARAMS => "TRANS_ENOMSSL_CERT_PARAMS"));
	}

	protected function drawConfigParams(UIForm $ui_form, array $config)
	{

		/* This defines how the module is configured in the application.  This section should
			contain any piece of data needed to connect with the cert provider.  These will
			will be used to build a form the admin will use to configure their account
			settings given to them by the cert provider.

			CIT_TEXT		creates an input box
			CIT_TEXT_AREA		creates a text area input box
			CIT_RADIO		creates a radio group
			CIT_SELECT		creates a drop down list
			CIT_BOOL		creates a true/false radio group
			CIT_PASSWORD		creates a password input box

			variable $config will contain any existing settings
		*/

		$ui_form->addInputPanel(
			$general_panel = UIFactory()->InputPanel(blmsg("TRANS_SSL_PLUGIN_CONFIG"))
		);
		$general_panel->addInputControl(UIFactory()->InputConfig(array(
			"id"		=> "api_url",
			"type"		=> SSLModule::CIT_TEXT,
			"default"	=> isset($config["api_url"])
				? $config["api_url"]
				: "http://reseller.enom.com/interface.asp",
			"required"	=> true,
			"title"		=> "TRANS_ENOMSSL_API_URL",
		)));
		$general_panel->addInputControl(UIFactory()->InputConfig(array(
			"id"		=> "api_user_id",
			"type"		=> SSLModule::CIT_TEXT,
			"default"	=> isset($config["api_user_id"])
				? $config["api_user_id"]
				: "",
			"required"	=> true,
			"title"		=> "TRANS_ENOMSSL_API_USER_ID",
		)));
		$general_panel->addInputControl(UIFactory()->InputConfig(array(
			"id"		=> "api_user_pwd",
			"type"		=> SSLModule::CIT_PASSWORD,
			"default"	=> isset($config["api_user_pwd"])
				? $config["api_user_pwd"]
				: "",
			"required"	=> true,
			"title"		=> "TRANS_ENOMSSL_API_USER_PWD",
		)));
		$this->drawContact($ui_form, self::CONTACT_ADMIN, $config);

		if (! empty($config["api_url"]) && ! empty($config["api_user_id"]) && ! empty($config["api_user_pwd"]))
		{
			// Credentials for access to provider is configured, try to get current available ballance
			try {
				$resp_it = $this->remoteCall($config, array("command" => "GetBalance"));
				$resp = $resp_it->getValuesMap();
			} catch (Exception $e) {
				$resp = array(
					"ErrCount" => 1,
					"errors" => $e->getMessage(),
				);
			}

			if (0 < $resp["ErrCount"])
			{
				$ui_form->addMessage(UIFactory()->ErrorMessage($resp["errors"]));
				return;
			}

			// Account Info
			$ui_form->addInputPanel(
				$panel = UIFactory()->InputPanel(blmsg("TRANS_ENOMSSL_ACCOUNT_INFO_FROM_REMOTE_TITLE"))
			);
			$panel->addInputControl(UIFactory()->InputConfig(array(
				"id"		=> "acc_info_balance",
				"type"		=> SSLModule::CIT_TEXT,
				"title"		=> blmsg("TRANS_ENOMSSL_ACCOUNT_INFO_BALANCE"),
				"default"	=> BillingLocale::getCurrency(str_replace(",","",$resp["Balance"]), "USD"),
				"read_only"	=> true,
			)));
			$panel->addInputControl(UIFactory()->InputConfig(array(
				"id"		=> "acc_info_available_balance",
				"type"		=> SSLModule::CIT_TEXT,
				"title"		=> blmsg("TRANS_ENOMSSL_ACCOUNT_INFO_AVAILABLE_BALANCE"),
				"default"	=> BillingLocale::getCurrency(str_replace(",","",$resp["AvailableBalance"]), "USD"),
				"read_only"	=> true,
			)));

			// Prices
			$ui_form->addSubmitButton(UIFactory()->SubmitButton("reload-prices", blmsg("TRANS_ENOMSSL_RELOAD_PRICE_FROM_REMOTE"), "reload", "", "", "action"));
			$ui_form->addInputPanel(
				$panel = UIFactory()->InputPanel(blmsg("TRANS_ENOMSSL_PRICE_LIST"))
			);
			foreach ($this->services as $service_name => $service)
			{
				$panel->addInputControl(UIFactory()->InputConfig(array(
					"id"		=> $service_name,
					"type"		=> SSLModule::CIT_TEXT,
					"title"		=> $service_name,
					"default"	=> isset($config[$service_name])
						? BillingLocale::getCurrency(str_replace(",","",$config[$service_name]), "USD")
						: blmsg("TRANS_NONE"),
					"read_only"	=> true,
				)));
			}
		}

	}

	protected function getCollectedConfigParams(UIForm $ui_form, array $config)
	{

		if ("reload" == bgpc("command"))
		{
			/*
			Pull config data down from the SSL Cert provider.  This data can be
			specific to this reseller, such as prices.

			Return an array of items matching the avalable services listed at the
			top of this file.  Ex:

				['GeoTrust-QuickSSL-Premium'] => [329.99]
				['GeoTrust-TrueBizID'] => [429.99]

			*/

			if (! isset($config["api_url"]) || ! isset($config["api_user_id"]) || ! isset($config["api_user_pwd"]))
				return array();

			$price = array();

			foreach ($this->services as $service_name => $service)
			{
				$resp_it = $this->remoteCall($config, array(
					"command" => "PE_GetResellerPrice",
					"ProductType" => $service["ID"],
				));
				$resp = $resp_it->getValuesMap("productprice/*");

				$price[$service_name] = (0 < $resp["ErrCount"])
					? ""
					: $resp["price"];
			}

			return $price;
		}

		$collected_values = array();
		$ui_form->storeValues($collected_values);

		$result = array(
			"api_url"		=> $collected_values["api_url"],
			"api_user_id"	=> $collected_values["api_user_id"],
			"api_user_pwd"	=> $collected_values["api_user_pwd"],
		);
		$result = array_merge(
			$result,
			$this->collectContact($collected_values, self::CONTACT_ADMIN)
		);
		return $result;
	}

	protected function drawProductParams(UIForm $ui_form, array $config, array $product)
	{
		/*
			Where Product specific parameters are defined

		*/

		// Prepare Service items
		$service_items = array(array("", ""));
		foreach (array_keys($this->services) as $service)
		{
			$service_items[] = array(
				$service,
				$service.(isset($config[$service]) ? " (".billing_get_user_locale()->getLocaleCurrency($config[$service],"USD").")" : "")
			);
		}

		// Draw the panel with UI controls
		$ui_form->addInputPanel(
			$panel = UIFactory()->InputPanel(blmsg("TRANS_ENOMSSL_PRODUCT_CONFIG"))
		);

		// Add choosing a Service provider
		$panel->addInputControl(UIFactory()->InputConfig(array(
			"id"		=> "enomssl_service",
			"type"		=> SSLModule::CIT_SELECT,
			"default"	=> $product["enomssl_service"],
			"title"		=> "TRANS_ENOMSSL_SERVICE_PROVIDER",
			"required"	=> true,
			"items"		=> $service_items,
		)));

		// Add choosing a number of years to be used for purchasing of the
		// selected Service Provider
		if (isset($product["enomssl_service"]))
		{
			// Prepare Service Years Num items
			$years_items = array(
				array("", ""),
				array("1", blmsg("TRANS_1YEAR")),
			);
			for ($i = 2; $i <= $this->services[$product["enomssl_service"]]["period"]; $i++)
				$years_items[] = array($i, blmsg("TRANS_{$i}YEARS"));

			$panel->addInputControl(UIFactory()->InputConfig(array(
				"id"		=> "enomssl_service_years",
				"type"		=> SSLModule::CIT_SELECT,
				"default"	=> $product["enomssl_service_years"],
				"title"		=> "TRANS_ENOMSSL_SERVICE_YEARS_NUM",
				"required"	=> true,
				"items"		=> $years_items,
			)));
		}
	}

	protected function getCollectedProductParams(UIForm $ui_form, array $config, array $product)
	{
		$collected_values = array();
		$ui_form->storeValues($collected_values);


		return array(
			"enomssl_service"		=> $collected_values["enomssl_service"],
			"enomssl_service_years"	=> $collected_values["enomssl_service_years"],
		);
	}

	protected function drawUserParams(UIForm $ui_form, array $config, array $attributes)
	{
		$this->drawCertParams($ui_form, $config, $attributes);

		if (1 != $attributes["CertConfigured"] && $this->isStandAloneCert())
		{
			$this->drawContact($ui_form, self::CONTACT_ADMIN, $config, true);
			$this->drawContact($ui_form, self::CONTACT_TECHNICAL, $config, true);
			$this->drawContact($ui_form, self::CONTACT_BILLING, $config, true);
		}
	}


	/**
	 * getUserParamsDescription
	 *
	 * get fields decription of the user input required params
	 *
	 * @param array $config module config
	 * @param array $attributes module attributes
	 * @return array $userParams
	 *
	 */
	protected function getUserParamsDescriptor(array $config, array $attributes)
	{
		$userParams[self::CERT_PARAMS] = $this->getCertParams($config, $attributes);

		if (1 != $attributes["CertConfigured"] && $this->isStandAloneCert()) {
			$userParams[self::CONTACT_ADMIN] = $this->getContactParams(self::CONTACT_ADMIN, $config, true);
			$userParams[self::CONTACT_TECHNICAL] = $this->getContactParams(self::CONTACT_TECHNICAL, $config, true);
			$userParams[self::CONTACT_BILLING] = $this->getContactParams(self::CONTACT_BILLING, $config, true);
		}

		return $userParams;
	}

	protected function getCertParams(array $config, array $attributes)
	{

		/*
			This is used to display information about a Certificate within the
			application.
		*/

		$controls = array();
		if (1 == $attributes["CertPurchased"]) {
			return $controls;
        }
		if ($this->isStandAloneCert()) {
			if (! isset($attributes["CSR"]) || false === openssl_csr_get_subject($attributes["CSR"])) {
				$controls[] = array(
					"id"		=> "CSR",
					"type"		=> SSLModule::CIT_TEXT_AREA,
					"title"		=> blmsg("TRANS_ENOMSSL_CERT_PARAM_CSR"),
					"required"	=> true,
				);
			}
		}
		elseif (! isset($attributes["CSR"])) {
			$domain = $this->getDomainOpts();
			$controls[] = array(
				"id"		=> "SubDomain",
				"type"		=> SSLModule::CIT_TEXT,
				"title"		=> blmsg("TRANS_ENOMSSL_CERT_PARAM_SUBDOMAIN"),
				"desc"		=> blmsg("TRANS_ENOMSSL_INPUT_SUBDOMAIN_DESC"),
				"default"	=> empty($domain["SubDomain"]) ? "www" : $domain["SubDomain"],
				"extention" => ".{$domain['SLD']}.{$domain['TLD']}",
			);

		}

		if (! isset($attributes["WebServerType"])) {
			list($default_web_server_type, $web_server_types) = self::webServerTypes($attributes);
			$converted_web_server_types = array();
			foreach ($web_server_types as $key => $name)
				$converted_web_server_types[] = array($key, $name);

			$controls[] = array(
				"id"		=> "WebServerType",
				"type"		=> SSLModule::CIT_SELECT,
				"title"		=> blmsg("TRANS_ENOMSSL_CERT_PARAM_WEBSERVERTYPE"),
				"required"	=> true,
				"default"	=> $default_web_server_type,
				"items"		=> $converted_web_server_types,
			);
		}
		if (1 == $attributes["CertConfigured"] && ! isset($attributes["ApproverEmail"])) {
			$approver_emails = $this->getApproverEmails($config, $attributes);
			$approver_emails_items = array();
			foreach ($approver_emails as $email)
				$approver_emails_items[] = array($email, $email);
			$controls[] = array(
				"id"		=> "ApproverEmail",
				"type"		=> SSLModule::CIT_SELECT,
				"title"		=> blmsg("TRANS_ENOMSSL_CERT_PARAM_APPROVEREMAIL"),
				"required"	=> true,
				"items"		=> $approver_emails_items,
			);
		}
        if (1 == $attributes["CertConfigured"] && ! isset($attributes["address"])) {
            switch ($attributes[$this->getCertificateAttributeName()]) {
                case self::COMODO_ESSENTIAL:
                case self::COMODO_INSTANT:
                    $params = array(
                        array('CSRAddress1', SSLModule::CIT_TEXT, "TRANS_ENOMSSL_CERT_PARAM_ORGANIZATION_ADDRESS1", "TRANS_ENOMSSL_CERT_PARAM_ORGANIZATION_ADDRESS1_DESC" ),
                        array('CSRLocality', SSLModule::CIT_TEXT, "TRANS_ENOMSSL_CERT_PARAM_ORGANIZATION_CITY", "TRANS_ENOMSSL_CERT_PARAM_ORGANIZATION_CITY_DESC"),
                        array('CSRPostalCode', SSLModule::CIT_TEXT, "TRANS_ENOMSSL_CERT_PARAM_ORGANIZATION_POSTAL_CODE", "TRANS_ENOMSSL_CERT_PARAM_ORGANIZATION_POSTAL_CODE_DESC"),
                    );
                    foreach($params as $param) {
                        $controls[] = array(
                            "id"		=> $param[0],
                            "type"		=> $param[1],
                            "title"		=> blmsg($param[2]),
                            "desc"		=> blmsg($param[3]),
                            "required"	=> true,
                        );
                    }

                    break;
                default:
                    break;
            }

        }
		return $controls;
	}

	protected function getContactParams($contact_type, $data, $replace_from_admin)
	{
		/*
			This is used to display information about a Certificate within the
			application specific to the customer making the purchase.
		*/

		// Prepare 'Country' input items
		$countries = $this->getCountries();
		$countries_as_items = array(array("", ""));

		foreach ($countries as $country_code => $country_name)
			$countries_as_items[] = array($country_code, $country_name);

		// Prepare 'States' input items
		$states = $this->getUSStates();
		$states_as_items = array(array("", ""));

		foreach ($states as $state_code => $state_name)
			$states_as_items[] = array($state_code, $state_name);

		$contact_fields = array(
			"FName" => array(
				"title" => "TRANS_ENOMSSL_CONTACT_FNAME",
				"type" => SSLModule::CIT_TEXT,
				"required" => true,
				"checker" => "/.{1,60}/",
			),
			"LName" => array(
				"title" => "TRANS_ENOMSSL_CONTACT_LNAME",
				"type" => SSLModule::CIT_TEXT,
				"required" => true,
				"checker" => "/.{1,60}/",
			),
			"OrgName" => array(
				"title" => "TRANS_ENOMSSL_CONTACT_ORGNAME",
				"type" => SSLModule::CIT_TEXT,
				"required" => true,
				"checker" => "/.{1,60}/",
			),
			"JobTitle" => array(
				"title" => "TRANS_ENOMSSL_CONTACT_JOBTITLE",
				"type" => SSLModule::CIT_TEXT,
				"required" => true,
				"checker" => "/.{1,60}/",
			),
			"Address1" => array(
				"title" => "TRANS_ENOMSSL_CONTACT_ADDRESS1",
				"type" => SSLModule::CIT_TEXT,
				"required" => true,
				"checker" => "/.{1,60}/",
			),
			"Address2" => array(
				"title" => "TRANS_ENOMSSL_CONTACT_ADDRESS2",
				"type" => SSLModule::CIT_TEXT,
				"checker" => "/.{0,60}/",
			),
			"City" => array(
				"title" => "TRANS_ENOMSSL_CONTACT_CITY",
				"type" => SSLModule::CIT_TEXT,
				"required" => true,
				"checker" => "/.{1,60}/",
			),
			"StateProvinceChoise" => array(
				"title" => "TRANS_ENOMSSL_CONTACT_STATEPROVINCECHOISE",
				"desc" => "TRANS_ENOMSSL_CONTACT_STATEPROVINCECHOISE_DESC",
				"type" => SSLModule::CIT_SELECT,
				"default" => "P",
				"items" => array(
					array("P", blmsg("TRANS_ENOMSSL_CONTACT_PROVINCE")),
					array("S", blmsg("TRANS_ENOMSSL_CONTACT_STATE")),
				),
			),
			"Province" => array(
				"title" => "TRANS_ENOMSSL_CONTACT_PROVINCE",
				"type" => SSLModule::CIT_TEXT,
			),
			"State" => array(
				"title" => "TRANS_ENOMSSL_CONTACT_STATE",
				"type" => SSLModule::CIT_SELECT,
				"items" => $states_as_items,
			),
			"PostalCode" => array(
				"title" => "TRANS_ENOMSSL_CONTACT_POSTALCODE",
				"type" => SSLModule::CIT_TEXT,
				"required" => true,
				"checker" => "/\d{1,16}/",
			),
			"Country" => array(
				"title" => "TRANS_ENOMSSL_CONTACT_COUNTRY",
				"type" => SSLModule::CIT_SELECT,
				"required" => true,
				"items" => $countries_as_items,
			),
			"Phone" => array(
				"title" => "TRANS_ENOMSSL_CONTACT_PHONE",
				"desc" => "TRANS_ENOMSSL_CONTACT_PHONE_DESC",
				"type" => SSLModule::CIT_TEXT,
				"required" => true,
				"checker" => "/\+\d+\.\d+/",
			),
			"PhoneExt" => array(
				"title" => "TRANS_ENOMSSL_CONTACT_PHONEEXT",
				"type" => SSLModule::CIT_TEXT,
				"checker" => "/\d*/",
			),
			"Fax" => array(
				"title" => "TRANS_ENOMSSL_CONTACT_FAX",
				"type" => SSLModule::CIT_TEXT,
				"checker" => "/(^$)|(\+\d+\.\d+)/",
			),
			"EmailAddress" => array(
				"title" => "TRANS_ENOMSSL_CONTACT_EMAILADDRESS",
				"type" => SSLModule::CIT_TEXT,
				"required" => true,
				"checker" => "/.+@.+\..+/",
			),
		);

		$defaults_from_type = $replace_from_admin
			? self::CONTACT_ADMIN
			: $contact_type;

		foreach (array_keys($contact_fields) as $name)
		{
			$contact_fields[$name]["default"] = array_key_exists($defaults_from_type.$name, $data)
				? $data[$defaults_from_type.$name]
				: "";
		}

		return $contact_fields;
	}

	protected function getCollectedUserParamsFromUIForm(UIForm $ui_form, array $config, array $attributes)
	{
		$collected_values = array();
		$ui_form->storeValues($collected_values);
		return $this->getCollectedUserParams($config, $attributes, $collected_values);
	}

	// alias of getCollectedUserParams()
	protected function getCollectedUserParamsFromArray( array $config, array $attributes, array $collected_values)
	{
		return $this->getCollectedUserParams($config, $attributes, $collected_values);
	}

	/********************
	 * Plugin's actions *
	 ********************/

	protected function pluginAdd(array $config, array $attributes)
	{
		$data = array();

		try
		{
			$order_id = $attributes["OrderID"];
			$cert_id = $attributes["CertID"];

			if (! isset($order_id))
			{
				//
				// Try to purchase the service
				//
				$resp_it = $this->remoteCall($config, array(
					"command" => "PurchaseServices",
					"Service" => "Certificate-".$attributes["enomssl_service"],
					"NumYears" => $attributes["enomssl_service_years"],
				));
				$resp = $resp_it->getValuesMap();

				if (0 < $resp["ErrCount"])
				{
					return array(
						"status" => SSLModule::ACTION_STATUS_ERROR,
						"status_reason" => "TRANS_ENOMSSL_STATUS_REASON__REMOTE_ERRORS",
						"error" => $resp["errors"],
					);
				}
				$order_id = $data["OrderID"] = $resp["orderid"];
				$cert_id = $data["CertID"] = $resp["certid"];
			}

			if (1 != $attributes["CertConfigured"])
			{
				//
				// Try to configure certificate
				//
				$contacts = array();
				foreach (array(self::CONTACT_ADMIN, self::CONTACT_TECHNICAL, self::CONTACT_BILLING) as $contact_type)
				{
					if (! $this->isStandAloneCert())
					{
						$attributes = array_merge(
							$attributes,
							$this->collectContact($config, $contact_type, true)
						);
					}
					$contacts = array_merge(
						$contacts,
						array(
							$contact_type."FName"			=> $attributes[$contact_type."FName"],
							$contact_type."LName"			=> $attributes[$contact_type."LName"],
							$contact_type."OrgName"			=> $attributes[$contact_type."OrgName"],
							$contact_type."JobTitle"		=> $attributes[$contact_type."JobTitle"],
							$contact_type."Address1"		=> $attributes[$contact_type."Address1"],
							$contact_type."Address2"		=> $attributes[$contact_type."Address2"],
							$contact_type."City"			=> $attributes[$contact_type."City"],
							$contact_type."State"			=> $attributes[$contact_type."State"],
							$contact_type."Province"		=> $attributes[$contact_type."Province"],
							$contact_type."StateProvinceChoise" => $attributes[$contact_type."StateProvinceChoise"],
							$contact_type."PostalCode"		=> $attributes[$contact_type."PostalCode"],
							$contact_type."Country"			=> $attributes[$contact_type."Country"],
							$contact_type."Phone"			=> $attributes[$contact_type."Phone"],
							$contact_type."PhoneExt"		=> $attributes[$contact_type."PhoneExt"],
							$contact_type."Fax"				=> $attributes[$contact_type."Fax"],
							$contact_type."EmailAddress"	=> $attributes[$contact_type."EmailAddress"],
						)
					);
				}
				$resp_it = $this->remoteCall($config, array_merge(
					array(
						"command" => "CertConfigureCert",
						"CertID" => $cert_id,
						"CSR" => str_replace(
							array(
								"-----BEGIN CERTIFICATE REQUEST-----",
								"-----END CERTIFICATE REQUEST-----",
								"\n", "\r",
							), "", $attributes["CSR"]
						),
						"WebServerType" => $attributes["WebServerType"],
					),
					$contacts
				));
				$resp = $resp_it->getValuesMap();

				if (0 < $resp["ErrCount"])
				{
					return array(
						"status" => SSLModule::ACTION_STATUS_ERROR,
						"status_reason" => "TRANS_ENOMSSL_STATUS_REASON__REMOTE_ERRORS",
						"error" => $resp["errors"],
						"data" => $data,
					);
				}
				$data["CertConfigured"] = 1;

				// Waiting for other required data from an additional User Input
				return array(
					"status" => SSLModule::ACTION_STATUS_PENDING,
					"status_reason" => "TRANS_ENOMSSL_STATUS_REASON__WAITING_FOR_ADDITIONAL_DATA",
					"data" => $data,
				);
			}

			if (1 != $attributes["CertPurchased"]) {
				//
				// Try to purchase the configured certificate
				//
				$csr_details = $this->getCSRDetails($config, array_merge($attributes, $data));
                $purchaseParams = array(
                    "command"			=> "CertPurchaseCert",
                    "CertID"			=> $cert_id,
                    "ApproverEmail"		=> $attributes["ApproverEmail"],
                    "CSROrganization"		=> $csr_details["Organization"],
                    "CSROrganizationUnit"	=> $csr_details["OrganizationUnit"],
                    "CSRAddress1"			=> $csr_details["Address1"],
                    "CSRAddress2"			=> $csr_details["Address2"],
                    "CSRAddress3"			=> $csr_details["Address3"],
                    "CSRLocality"			=> $csr_details["Locality"],
                    "CSRStateProvince"		=> $csr_details["State"],
                    "CSRPostalCode"			=> $csr_details["PostalCode"],
                    "CSRCountry"			=> $csr_details["Country"],
                );

                switch ($attributes[$this->getCertificateAttributeName()]) {
                    case self::COMODO_ESSENTIAL:
                    case self::COMODO_INSTANT:
                        $purchaseParams["CSRAddress1"] = $attributes["CSRAddress1"];
                        $purchaseParams["CSRLocality"] = $attributes["CSRLocality"];
                        $purchaseParams["CSRPostalCode"] = $attributes["CSRPostalCode"];
                        break;
                    default:
                        break;
                }

				$resp_it = $this->remoteCall($config, $purchaseParams);
				$resp = $resp_it->getValuesMap();

				if (0 < $resp["ErrCount"])
				{
					return array(
						"status" => SSLModule::ACTION_STATUS_ERROR,
						"status_reason" => "TRANS_ENOMSSL_STATUS_REASON__REMOTE_ERRORS",
						"error" => $resp["errors"],
						"data" => $data,
					);
				}

				$data["CertPurchased"] = 1;
			}

			$result = $this->pluginGetIssuedCert($config, array_merge($attributes, $data));
			$result["data"] = isset($result["data"]) ? array_merge($data, $result["data"]) : $data;
			return $result;
		}
		catch (ProductException $e)
		{
			return array(
				"status" => SSLModule::ACTION_STATUS_ERROR,
				"status_reason" => "TRANS_ENOMSSL_STATUS_REASON__INTERNAL_ERRORS",
				"error" => $e->getMessage(),
				"data" => $data,
			);
		}
		return array(
			"status" => SSLModule::ACTION_STATUS_COMPLETED,
			"data" => $data,
		);
	}

	protected function pluginRenew(array $config, array $attributes)
	{
		$data = array();

		try
		{
			if (1 == $attributes["CertPurchased"]	&& 1 == $attributes["CertConfigured"]	&& isset($attributes["SSLCertificate"])) {
				// save current SSL certificate settings
				$data["lastOrderID"] = $attributes["OrderID"];
				$data["lastCertID"] = $attributes["CertID"];
				$data["lastSSLCertificate"] = $attributes["SSLCertificate"];
				$data["lastCSR"] = $attributes["CSR"];
				$data["lastPVT"] = $attributes["PVT"];
				$data["CertPurchased"] = $attributes["CertPurchased"] = NULL;
				$data["CertConfigured"] = $attributes["CertConfigured"] = NULL;
				$data["CertID"] = $attributes["CertID"] = NULL;
				$data["OrderID"] = $attributes["OrderID"] = NULL;
				$data["SSLCertificate"] = $attributes["SSLCertificate"] = NULL;

				$csr_data = $this->generateCSR();
				$data["CSR"] = $attributes["CSR"] = $csr_data["csr"];
				$data["PVT"] = $attributes["PVT"] = $csr_data["pvt"];
			}
			$resp = $this->pluginAdd($config, $attributes);

		}
		catch (ProductException $e)
		{
			$this->pushErrorToAdminsToDo($error = $e->getMessage());
			return array(
				"execution_status" => self::STATUS_FAILURE,
				"error" => $error,
			);
		}

		$resp["data"] = isset($resp["data"]) ? array_merge($data, $resp["data"]) : $data;

		return $resp;
	}

	protected function pluginGetIssuedCert(array $config, array $attributes)
	{
		if (1 != $attributes["CertPurchased"] || ! isset($attributes["CertID"]))
		{
			return array(
				"status" => SSLModule::ACTION_STATUS_ERROR,
				"status_reason" => "TRANS_ENOMSSL_STATUS_REASON__INTERNAL_ERRORS",
				"error" => "SSL Ceritificate is not configured or purchased yet",
			);
		}
		$data = array(
			"CSR" => $attributes["CSR"],
			"PVT" => $attributes["PVT"],
			"SSLCertificate" => $attributes["SSLCertificate"],
		);

		try
		{
			if (isset($attributes["SSLCertificate"]))
			{
				return array(
					"status" => SSLModule::ACTION_STATUS_COMPLETED,
					"data" => $data,
				);
			}

			$resp = $this->getCertDetails($config, $attributes);

			switch ((int)$resp["CertStatusID"])
			{
				case 2: // Processing
					return array(
						"status" => SSLModule::ACTION_STATUS_PENDING,
						"status_reason" => "TRANS_ENOMSSL_STATUS_REASON__CERT_INPROGRESS",
					);
				case 4: // Issued
					$data["SSLCertificate"] = $resp["SSLCertificate"];
					return array(
						"status" => SSLModule::ACTION_STATUS_COMPLETED,
						"data" => $data,
					);
				case 6: // Rejected by Customer
					return array(
						"status" => SSLModule::ACTION_STATUS_ERROR,
						"status_reason" => "TRANS_ENOMSSL_STATUS_REASON__CERT_REJECTED_BY_CUSTOMER",
						"error" => "SSL Certificate processing was rejected by Customer",
						"data" => array("CertPurchased" => 0),
					);
				case 9: // Approval E-Mail Sent
					return array(
						"status" => SSLModule::ACTION_STATUS_PENDING,
						"status_reason" => "TRANS_ENOMSSL_STATUS_REASON__CERT_APPROVAL_EMAIL_SENT",
					);
				default:
					return array(
						"status" => SSLModule::ACTION_STATUS_ERROR,
						"status_reason" => "TRANS_ENOMSSL_STATUS_REASON__CERT_UNKNOWN_STATUS",
						"error" => "Enom SSL Service returns unknown status '{$resp['CertStatusID']}': {$resp['CertStatus']}",
					);
			}
		}
		catch (ProductException $e)
		{
			return array(
				"status" => SSLModule::ACTION_STATUS_ERROR,
				"status_reason" => "TRANS_ENOMSSL_STATUS_REASON__INTERNAL_ERRORS",
				"error" => $e->getMessage(),
				"data" => $data,
			);
		}
	}

	protected function pluginGetCSR(array $config, array $attributes)
	{
		if (! isset($attributes["CSR"]) && ! isset($attributes["CertID"]))
		{
			return array(
				"status" => SSLModule::ACTION_STATUS_ERROR,
				"status_reason" => "TRANS_ENOMSSL_STATUS_REASON__INTERNAL_ERRORS",
				"error" => "No CSR defined",
				"data" => array("CSR" => ""),
			);
		}

		try
		{
			if (isset($attributes["CSR"]))
			{
				return array(
					"status" => SSLModule::ACTION_STATUS_COMPLETED,
					"data" => array("CSR" => $attributes["CSR"]),
				);
			}

			$resp = $this->getCertDetails($config, $attributes);
			return array(
				"status" => SSLModule::ACTION_STATUS_COMPLETED,
				"data" => array(
					"CSR" => (1 == $resp["CertStatusID"] ? "" : $resp["CSR"]),
				),
			);
		}
		catch (ProductException $e)
		{
			return array(
				"status" => SSLModule::ACTION_STATUS_ERROR,
				"status_reason" => "TRANS_ENOMSSL_STATUS_REASON__INTERNAL_ERRORS",
				"error" => $e->getMessage(),
			);
		}
	}


	protected function pluginDelete(array $config, array $attributes)
	{

	}

	protected function pluginCheckAttributes(array $config, array $attributes, $action)
	{
		$errors = array();

		switch ($action)
		{
			case "add":
				if (! isset($attributes["CSR"])) {
					$errors["CSR"] = "No CSR defined";
				} elseif(false === openssl_csr_get_subject($attributes["CSR"])) {
					$errors["CSR"] = "Unknown CSR key";
				}
				if (1 == $attributes["CertConfigured"] && ! isset($attributes["ApproverEmail"]))
					$errors["ApproverEmail"] = "No approver's EMail defined";
				if (1 != $attributes["CertConfigured"])
				{
					if (! isset($attributes["WebServerType"]))
						$errors["WebServerType"] = "No web server type defined";

					if ($this->isStandAloneCert())
					{
						if (! isset($attributes["AdminFName"]))
							$errors["AdminFName"] = "Admin contact is not defined";
						if (! isset($attributes["TechFName"]))
							$errors["TechFName"] = "Technical contact is not defined";
						if (! isset($attributes["BillingFName"]))
							$errors["BillingFName"] = "Billing contact is not defined";
					}
				}
				break;
			case "delete":
				break;
			case "getCSR":
				break;
			case "getIssuedCert":
				break;
		}
		return $errors;
	}

	private function drawContact(UIForm $ui_form, $contact_type, array $data, $replace_from_admin = false)
	{
		$contact_group_map = $this->getContactGroupMap();
		$ui_form->addInputPanel(
			$contact_panel = UIFactory()->InputPanel(blmsg($contact_group_map[$contact_type]))
		);

		$contact_fields = $this->getContactParams($contact_type, $data, $replace_from_admin);
		foreach (array_keys($contact_fields) as $name)
		{
			$contact_panel->addInputControl(UIFactory()->InputConfig(array(
				"id"		=> $contact_type.$name,
				"type"		=> $contact_fields[$name]["type"],
				"title"		=> $contact_fields[$name]["title"],
				"desc"		=> $contact_fields[$name]["desc"],
				"required"	=> $contact_fields[$name]["required"],
				"items"		=> $contact_fields[$name]["items"],
				"checker"	=> $contact_fields[$name]["checker"],
				"default"	=> $contact_fields[$name]["default"],
			)));
		}
	}

	private function collectContact(array $params, $contact_type, $replace_from_admin = false)
	{
		$contact_params = array(
			"FName",
			"LName",
			"OrgName",
			"JobTitle",
			"Address1",
			"Address2",
			"City",
			"StateProvinceChoise",
			"Province",
			"State",
			"PostalCode",
			"Country",
			"Phone",
			"PhoneExt",
			"Fax",
			"EmailAddress",
		);

		$collected = array();
		foreach ($contact_params as $param_name)
			$collected[$contact_type.$param_name] = $params[($replace_from_admin ? self::CONTACT_ADMIN : $contact_type).$param_name];

		return $collected;
	}

	private function drawCertParams($ui_form, array $config, array $attributes)
	{
		if (1 == $attributes["CertPurchased"])
			return;

		$certParams = $this->getCertParams($config, $attributes);

		$controls = array();
		foreach ($certParams as $param) {
			$controls[] = UIFactory()->InputConfig($param);
		}

		if (0 < count($controls))
		{
			$ui_form->addInputPanel(
				$panel = UIFactory()->InputPanel(blmsg("TRANS_ENOMSSL_CERT_PARAMS"))
			);
			foreach ($controls as $control)
				$panel->addInputControl($control);
		}
	}

	public static function webServerTypes($attributes)
	{
		if (0 < preg_match("/GeoTrust|RapidSSL|VeriSign/", $attributes["enomssl_service"])) {

			return array(
					28,		// default Plesk
					array(
						"1" => "Apache + MOD SSL",
						"2" => "Apache + Raven",
						"3" => "Apache + SSLeay",
						"4" => "C2Net Stronghold",
						"7" => "IBM HTTP",
						"8" => "iPlanet Server 4.1",
						"9" => "Lotus Domino Go 4.6.2.51",
						"10" => "Lotus Domino Go 4.6.2.6+",
						"11" => "Lotus Domino 4.6+",
						"12" => "Microsoft IIS 4.0",
						"13" => "Microsoft IIS 5.0",
						"14" => "Netscape Enterprise/FastTrack",
						"17" => "Zeus v3+",
						"18" => "Other",
						"20" => "Apache + OpenSSL",
						"21" => "Apache 2",
						"22" => "Apache + ApacheSSL",
						"23" => "Cobalt Series",
						"24" => "Cpanel",
						"25" => "Ensim",
						"26" => "Hsphere",
						"27" => "Ipswitch",
						"28" => "Plesk",
						"29" => "Jakart-Tomcat",
						"30" => "WebLogic (all versions)",
						"31" => "O'Reilly WebSite Professional",
						"32" => "WebStar",
						"33" => "Microsoft IIS 6.0",
					),
				);
		}

		return array(
					1030,	// default Plesk
					array(
						"1000" => "Other",
						"1001" => "AOL",
						"1002" => "Apache/ModSSL",
						"1003" => "Apache-SSL (Ben-SSL => not Stronghold)",
						"1004" => "C2Net Stronghold",
						"1005" => "Cobalt Raq",
						"1006" => "Covalent Server Software",
						"1007" => "IBM HTTP Server",
						"1008" => "IBM Internet Connection Server",
						"1009" => "iPlanet",
						"1010" => "Java Web Server (Javasoft / Sun)",
						"1011" => "Lotus Domino",
						"1012" => "Lotus Domino Go!",
						"1013" => "Microsoft IIS 1.x to 4.x",
						"1014" => "Microsoft IIS 5.x and later",
						"1015" => "Netscape Enterprise Server",
						"1016" => "Netscape FastTrack",
						"1017" => "Novell Web Server",
						"1018" => "Oracle",
						"1019" => "Quid Pro Quo",
						"1020" => "R3 SSL Server",
						"1021" => "Raven SSL",
						"1022" => "RedHat Linux",
						"1023" => "SAP Web Application Server",
						"1024" => "Tomcat",
						"1025" => "Website Professional",
						"1026" => "WebStar 4.x and later",
						"1027" => "WebTen (from Tenon)",
						"1028" => "Zeus Web Server",
						"1029" => "Ensim",
						"1030" => "Plesk",
						"1031" => "WHM/cPanel",
						"1032" => "H-Sphere",
					),
				);
	}

	private function collectCertParams(array $params, array $attributes)
	{
		if (1 == $attributes["CertPurchased"])
			return array();

		$collected = array();

		if (! $this->isStandAloneCert() && ! isset($attributes["CSR"]))
		{
			$collected["SubDomain"] = $params["SubDomain"];

			// Generate CSR key
			$domain_opts = $this->getDomainOpts();
			$domain_opts["SubDomain"] = $params["SubDomain"];
			$csr_data = $this->generateCSR($this->getDomainName($domain_opts));

			$collected["CSR"] = $csr_data["csr"];
			$collected["PVT"] = $csr_data["pvt"];
			$collected["CertConfigured"] = 0;
		}
		$param_names = array(
			"CSR",
			"PVT",
			"WebServerType",
			"ApproverEmail",
			"CSRAddress1",
			"CSRLocality",
			"CSRStateProvince",
			"CSRPostalCode",
		);
		foreach ($param_names as $name)
			if (isset($params[$name]) && ! isset($collected[$name])) $collected[$name] = $params[$name];

		return $collected;
	}

	private function getCertDetails(array $config, array $attributes)
	{
		$resp_it = $this->remoteCall($config, array(
			"command" => "CertGetCertDetail",
			"CertID" => $attributes["CertID"],
		));
		$resp = $resp_it->getValuesMap();

		if (0 < $resp["ErrCount"])
			throw new Exception($resp["errors"]);

		return $resp_it->getValuesMap("CertGetCertDetail/*");
	}

	private function parseCSR(array $config, array $attributes)
	{
		if (! is_null($this->csr_details))
			return $this->csr_details;

		$csr_resp_it = $this->remoteCall(
			$config,
			array(
				"command" => "CertParseCSR",
				"CertID" => $attributes["CertID"],
				"CSR" => str_replace(
					array(
						"-----BEGIN CERTIFICATE REQUEST-----",
						"-----END CERTIFICATE REQUEST-----",
						"\n", "\r",
					), "", $attributes["CSR"]
				),
			)
		);
		$csr_resp = $csr_resp_it->getValuesMap();

		if (0 < $csr_resp["ErrCount"])
			throw new Exception($csr_resp["errors"]);

		return $this->csr_details = $csr_resp_it->getValuesMap("CertParseCSR/*");
	}

	private function getCSRDetails(array $config, array $attributes)
	{
		$csr_details = $this->parseCSR($config, $attributes);

		$csr_details_map = array(
			"CSROrganization"		=> "Organization",
			"CSROrganizationUnit"	=> "OrganizationUnit",
			"CSRAddress1"			=> "Address1",
			"CSRAddress2"			=> "Address2",
			"CSRAddress3"			=> "Address3",
			"CSRLocality"			=> "Locality",
			"CSRStateProvince"		=> "State",
			"CSRPostalCode"			=> "PostalCode",
			"CSRCountry"			=> "Country",
		);
		foreach ($csr_details_map as $attr_name => $csr_item_right)
			if (isset($attributes[$attr_name])) $csr_details[$csr_item_right] = $attributes[$attr_name];

		return $csr_details;
	}

	private function getApproverEmails(array $config, array $attributes)
	{
		$emails_resp_it = $this->remoteCall(
			$config,
			array(
				"command" => "CertGetApproverEmail",
				"CertID" => $attributes["CertID"],
				"Domain" => $this->getDomainName(),
			)
		);
		$emails_resp = $emails_resp_it->getValuesMap();

		if (0 < $emails_resp["ErrCount"])
			throw new Exception($emails_resp["errors"]);

		return $emails_resp_it->getValuesSet("CertGetApproverEMail/Approver/ApproverEmail");
	}

	private function remoteCall(array $config, array $args)
	{

		/*
			This is where the remote calls to the SSL Cert provider take place.  Add
			code specific to your API here to match the avalable parameters to the
			call being made to the requirements of the target system.  These calls
			can utilize any comunications technology needed.
		*/

		$args = array_merge(
			array(
				"uid"			=> $config["api_user_id"],
				"pw"			=> $config["api_user_pwd"],
				"DebugMode"		=> "on",
				"ResponseType"	=> "XML",
			),
			$args
		);

		$remote_url = "{$config['api_url']}?".http_build_query($args);

		$this->debug("EnomSSL URL:");
		$this->debug($remote_url);

		$this->debug("EnomSSL Response:");
		$this->debug($result_xml = file_get_contents($remote_url));

		if ("" == $result_xml){
			throw new Exception(blmsg("TRANS_ENOMSSL_REMOTE_CALL_UNKNOWN_ERROR_MSG"));
		}
		require_once("XML/Parser.php");
		$result_it = XMLParser::parse($result_xml);
		return $result_it;
	}

	private function getContactGroupMap()
	{
		return array(
			self::CONTACT_ADMIN => "TRANS_ENOMSSL_CONTACT_ADMIN",
			self::CONTACT_TECHNICAL => "TRANS_ENOMSSL_CONTACT_TECHNICAL",
			self::CONTACT_BILLING => "TRANS_ENOMSSL_CONTACT_BILLING",
		);
	}

	private function getCollectedUserParams( array $config, array $attributes, array $collected_values)
	{
		$result = $this->collectCertParams($collected_values, $attributes);

		if (1 != $attributes["CertConfigured"] && $this->isStandAloneCert())
		{
			$result = array_merge(
				$result,
				$this->collectContact($collected_values, self::CONTACT_ADMIN),
				$this->collectContact($collected_values, self::CONTACT_TECHNICAL),
				$this->collectContact($collected_values, self::CONTACT_BILLING)
			);
		}
		return $result;
	}

	private $csr_details;
	private $services = array(
		self::GEOTRUST_QUICKSSL				=> array("ID" => 26, "period" => "2"),
		self::GEOTRUST_QUICKSSL_PREMIUM		=> array("ID" => 20, "period" => "2"),
		self::GEOTRUST_TRUEBIZID			=> array("ID" => 21, "period" => "2"),
		self::GEOTRUST_TRUEBIZID_WILDCARD	=> array("ID" => 27, "period" => "2"),
		self::GEOTRUST_TRUEBIZID_EV			=> array("ID" => 24, "period" => "2"),
		self::RAPIDSSL_RAPIDSSL				=> array("ID" => 23, "period" => "2"),
		self::VERISIGN_SECURE_SITE			=> array("ID" => 180, "period" => "2"),
		self::VERISIGN_SECURE_SITE_PRO		=> array("ID" => 181, "period" => "2"),
		self::VERISIGN_SECURE_SITE_EV		=> array("ID" => 182, "period" => "2"),
		self::VERISIGN_SECURE_SITE_PRO_EV	=> array("ID" => 183, "period" => "2"),
		self::COMODO_ESSENTIAL				=> array("ID" => 211, "period" => "2"),
		self::COMODO_INSTANT				=> array("ID" => 212, "period" => "2"),
	);
}