<?php

namespace Module\Ssl\Opensrs;
use Billing\Module\Exception;

class Api
{
	public function __construct(Negotiator $negotiator)
	{
		$this->_negotiator = $negotiator;
	}

	public function queryApproverList($service, $domain)
	{
		$command = new Command($this->_negotiator, Command::COMMAND_QUERY_APPROVER_LIST);
		$command->service = $service;
		$command->domain = $domain;
		$response = $command->run();
		if (!$response->isSuccess) {
            $this->_processErrors($response, 'Request for getting approver email list was failed with error(s): "');
		}
		$nodes = $response->attributes->approver_list->xpath('//item/dt_assoc/item[@key="email"]');
		$emails = array();
		foreach ($nodes as $node) {
			$emails[] = (string)$node;
		}
		return $emails;
	}
	
	public function register($code, $attributes, $isRequiredContactOrganization, $isRequiredContactAdmin, $isRequiredContactBilling, $isRequiredContactTech, $isRequiredContactSigner)
	{
		$command = new Command($this->_negotiator, Command::COMMAND_REGISTER);
		$command->code = $code;
		foreach ($attributes as $name => $value) {
			if (strpos($name, \OpensrsSsl::PREFIX_ATTRIBUTES) === 0) {
				$parameter = substr($name, strlen(\OpensrsSsl::PREFIX_ATTRIBUTES));
				$command->$parameter = $value;
			}
		}
		$command->isRequiredContactOrganization = $isRequiredContactOrganization;
		$command->isRequiredContactAdmin = $isRequiredContactAdmin;
		$command->isRequiredContactBilling = $isRequiredContactBilling;
		$command->isRequiredContactTech = $isRequiredContactTech;
		$command->isRequiredContactSigner = $isRequiredContactSigner;
		$response = $command->run();
		if (!$response->isSuccess) {
            $this->_processErrors($response, 'Request for service registration was failed with error(s): "');
		}
		return (int)$response->attributes->order_id;
	}
	
	public function getOrderInfo($orderId)
	{
		$command = new Command($this->_negotiator, Command::COMMAND_GET_ORDER_INFO);
		$command->orderId = $orderId;
		$response = $command->run();
		if (!$response->isSuccess) {
            $this->_processErrors($response, 'Request for order info was failed with error(s): "');
		}
		try {
			$productId = (int)$response->attributes->product_id;
		} catch(Exception $ex) {
			$productId = NULL;
		}
		return $productId;
	}



    private function _processErrors(Response $response, $message)
    {
        $errors = array('[' . $response->code . '] ' . $response->message);
        foreach ($response->errors as $error) {
            $errors[] = '[' . $error->code . '] ' . $error->message;
        }
        $message .= implode('; ', $errors);
        throw new Exception($message);
    }
	
	private $_negotiator;
}