<?php

namespace Module\Ssl\Opensrs;
use Billing\Module\Exception;

class Negotiator
{
	const SOCKET_TIMEOUT = 5;
	const CLRF = "\r\n";
	
	public function __construct($username, $key, $host, $port)
	{
		$this->_username = $username;
		$this->_key = $key;
		if (!parse_url($host, PHP_URL_SCHEME)) {
			$host = 'ssl://' . $host;
		}
		$this->_socket = fsockopen($host, $port, $errorNumber, $errorMessage);
		socket_set_timeout($this->_socket, self::SOCKET_TIMEOUT);
	}
	
	public function __destruct()
	{
		fclose($this->_socket);
	}
	
	public function say($message)
	{
		$signature = md5(md5($message . $this->_key) . $this->_key);
		$messageLength = strlen($message);
		$header = "POST / HTTP/1.0" . self::CLRF;
		$header .= "Content-Type: text/xml" . self::CLRF;
		$header .= "X-Username: " . $this->_username . self::CLRF;
		$header .= "X-Signature: " . $signature . self::CLRF;
		$header .= "Content-Length: " . $messageLength . self::CLRF . self::CLRF;
		
		fputs($this->_socket, $header);
		fputs($this->_socket, $message, $messageLength);
		if (!$header = $this->_readHeader()) {
			throw new Exception('No Content-Length header provided. Please make sure that your server IP is whitelisted in OpenSRS Reseller Web Interface (RWI).');
		}
		$contentLength = isset($header['content-length']) ? (int)$header['content-length'] : 0;
		$response = '';
		while (strlen($response) < $contentLength) {
			$response .= fread($this->_socket, $contentLength);
		}
		return $response;
	}
	
	private function _readHeader()
	{
		$header = array();
		$httpLog = '';
		$line = fgets($this->_socket, 4000);
		$httpLog .= $line;
		if (!preg_match('/^HTTP\/1.1 ([0-9]{0,3}) (.*)\r\n$/', $line, $matches)) {
			throw new Exception('Unable to parse HTTP response code. Please make sure that your server IP is whitelisted in OpenSRS Reseller Web Interface (RWI).');
		}
		$header['http_response_code'] = $matches[1];
		$header['http_response_text'] = $matches[2];
		
		while ($line != self::CLRF) {
			$line = fgets($this->_socket, 4000);
			$httpLog .= $line;
			if (feof($this->_socket)) {
				throw new Exception('Error reading HTTP header. Please make sure that your server IP is whitelisted in OpenSRS Reseller Web Interface (RWI).');
			}
			$matches = explode(': ', $line, 2);
			if (sizeof($matches) == 2) {
				$header[trim(strtolower($matches[0]))] = $matches[1];
			}
		}
		
		$header['full_header'] = $httpLog;
		return $header;
	}
	
	private $_username;
	private $_key;
	private $_socket;
}