<?php

namespace Module\Ssl\Opensrs;

class Service
{
	public function __construct($service)
	{
		if (is_null($descriptor = Regisrty::getInstance()->servicesDescriptor->$service)) {
			throw new Exception('Unknown service "' . $service . '".');
		}
		$this->_service = $service;
		$this->_descriptor = $descriptor;
		$this->_serverTypes = Regisrty::getInstance()->servicesDescriptor->getServerTypes($this->_descriptor['server_types']);
	}
	
	public function __get($name)
	{
		switch ($name) {
			case 'name':
				return $this->_service;
			case 'code':
				return $this->_descriptor['code'];
			case 'period':
				return $this->_descriptor['period'];
			case 'serverTypes':
				return $this->_serverTypes;
			// validity settings
			case 'isRequiredDomain':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_DOMAIN]) ? $this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_DOMAIN] : false;
			case 'isAllowedDomain':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_DOMAIN]) ? true : false;
			case 'isRequiredApproverEmail':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_APPROVER_EMAIL]) ? $this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_DOMAIN] : false;
			case 'isAllowedApproverEmail':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_APPROVER_EMAIL]) ? true : false;
			case 'isRequiredServerType':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_SERVER_TYPE]) ? $this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_SERVER_TYPE] : false;
			case 'isAllowedServerType':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_SERVER_TYPE]) ? true : false;
			case 'isRequiredServerCount':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_SERVER_COUNT]) ? $this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_SERVER_COUNT] : false;
			case 'isAllowedServerCount':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_SERVER_COUNT]) ? true : false;
			case 'isAllowedValidationSettings':
				return $this->isAllowedDomain || $this->isAllowedApproverEmail || $this->isAllowedServerType || $this->isAllowedServerCount;
			// certificate settings
			case 'isRequiredCSR':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_CSR]) ? $this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_CSR] : false;
			case 'isAllowedCSR':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_CSR]) ? true : false;
			case 'isRequiredSpecialInstructions':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_SPECIAL_INSTRUCTIONS]) ? $this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_SPECIAL_INSTRUCTIONS] : false;
			case 'isAllowedSpecialInstructions':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_SPECIAL_INSTRUCTIONS]) ? true : false;
			case 'isAllowedCertificateSettings':
				return $this->isAllowedCSR || $this->isAllowedSpecialInstructions;
			// contacts
			case 'isRequiredContactOrganization':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_CONTACT_ORGANIZATION]) ? $this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_CONTACT_ORGANIZATION] : false;
			case 'isRequiredContactAdmin':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_CONTACT_ADMIN]) ? $this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_CONTACT_ADMIN] : false;
			case 'isRequiredContactBilling':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_CONTACT_BILLING]) ? $this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_CONTACT_BILLING] : false;
			case 'isRequiredContactTech':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_CONTACT_TECH]) ? $this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_CONTACT_TECH] : false;
			case 'isRequiredContactSigner':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_CONTACT_SIGNER]) ? $this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_CONTACT_SIGNER] : false;
			case 'isRequiredTitle':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_TITLE]) ? $this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_TITLE] : false;
			case 'isAllowedTitle':
				return isset($this->_descriptor['requirements'][ServicesDescriptor::REQUIREMENT_TITLE]) ? true : false;
			default:
				throw new Exception('Property "' . $name . '" is not supported.');
		}
	}
	
	private $_service;
	private $_descriptor;
	private $_serverTypes;
}