<?php

/**
 * This file maintains all information having to do with priorities, that will ultimately be
 * associated with various objects, scaling their interpreted significance. The order of the
 * $CONF["priorities"] array in this file is very important, do not change it, as the array keys
 * are considered the priority IDs.
 *
 * DO NOT ADD APPLICATION SPECIFC PRIORITIES TO THIS FILE UNLESS THEY ARE
 * SHARED BY ALL MBAPI APPLICATIONS!
 *
 * Application specifc priorities should be located here:
 *
 * /lib-APPLICATION/include/config/internal/priorities.php
 */

$CONF["priorities"] = array(
    blmsg("TRANS_LOWEST"),
    blmsg("TRANS_LOW"),
    blmsg("TRANS_NORMAL"),
    blmsg("TRANS_HIGH"),
    blmsg("TRANS_HIGHEST"),
);

/**
 * This function gets the ID of a priority by searching for the given name. If the given name is found
 * in the priority list, this function returns the ID, however if the given name is not found, this
 * function returns false. Note that a priority ID of 0 is possible.
 *
 * @param  name  The priority name to search for, case-insensitive.
 * @return  The priority ID of the given priority name, or false if the given name was not found.
 */
function getPriorityID($name) {
	global $CONF;
	$retval = false;
	$c = count($CONF["priorities"]);
	for($i=0; $i<$c; $i++) {
		if(strcasecmp($CONF["priorities"][$i], $name) === 0) {
			$retval = $i;
			break;
		}
	}
	return $retval;
}

/**
 * This function gets the name of a priority by searching for the given priority ID. If the given ID is
 * found in the priority list, this function returns the name associated with that ID, however if the given
 * ID is not found, this function returns false. Note that a priority ID of 0 is possible.
 *
 * @param  id  The priority ID to search for.
 * @return  The priority name of the given priority ID, or false if the given ID was not found.
 */
function getPriorityName($id) {
	global $CONF;
	$retval = false;
	if($id < count($CONF["priorities"])) {
		$retval = $CONF["priorities"][$id];
	}
	return $retval;
}
?>