<?php

/**
 * This file maintains the status constants that are used to describe various objects. This file also
 * contains functions to manage those constants. Note that the array keys of these constants must NEVER
 * change; as the system will break if those array keys change.
 *
 * DO NOT ADD APPLICATION SPECIFC STATUSES TO THIS FILE UNLESS THEY ARE
 * SHARED BY ALL MBAPI APPLICATIONS!
 *
 * Application specifc statuses should be located here:
 *
 * /lib-APPLICATION/include/config/internal/statuses.php
 */

$CONF["statuses"] = array();
$CONF["APIstatuses"] = array();

$CONF["statuses"]["active"] = array();
$CONF["statuses"]["active"][0] = blmsg('TRANS_INACTIVE');
$CONF["statuses"]["active"][1] = blmsg('TRANS_ACTIVE');

$CONF["APIstatuses"]["active"] = array();
$CONF["APIstatuses"]["active"][0] = "inactive";
$CONF["APIstatuses"]["active"][1] = "active";

/**
 * class for handling statuses
 */
class StatusOf {
	/**
	 * string  status
	 */
	public $status;
	
	/**
	 * string  interface
	 */
	public $interface;
	
	/**
	 * constructor
	 *
	 * @param  string  type
	 * @param  boolean  whether to use API names rather than display names
	 */
	function StatusOf($status, $api = false) {
		$this->status = $status;
		$this->interface = $api ? "APIstatuses" : "statuses";
	}
	
	/**
	 * id by name
	 *
	 * @param  string  name
	 * @return  int  id
	 */
	function id($name) {
		global $CONF;
		return isset($CONF[$this->interface][$this->status]) ? array_search($name, $CONF[$this->interface][$this->status]) : false;
	}
	
	/**
	 * name by id
	 *
	 * @param  int  id
	 * @return  string  name
	 */
	function name($id) {
		global $CONF;
		return isset($CONF[$this->interface][$this->status][$id]) ? $CONF[$this->interface][$this->status][$id] : false;
	}
	
	/**
	 * HTML options
	 *
	 * @return  array  of associated names by ID
	 */
	function options() {
		global $CONF;
		return  isset($CONF[$this->interface][$this->status]) ? $CONF[$this->interface][$this->status] : array();
	}

	/**
	 * multi-select HTML options 
	 *
	 * @ return  array  list($selected, $selectable)
	 */
	function select($IDs = array()) {
		global $CONF;
		$selected = $selectable = array();
		foreach($this->options() as $id => $name) {
			if(in_array($id, $IDs)) {
				$selected[$id] = $name;
			} else {
				$selectable[$id] = $name;
			}
		}
		return array($selected, $selectable);
	}
}

/**
 * get status ID from name
 *
 * @param  string  status
 * @param  string  name
 * @return  int  ID
 */
function getStatusID($status, $name) {
	$s = new StatusOf($status);
	return $s->id($name);
}

/**
 * get status name from ID
 *
 * @param  string  status
 * @param  int  ID
 * @return  string  name
 */
function getStatusName($status, $id) {
	$s = new StatusOf($status);
	return $s->name($id);
}

/**
 * get status options
 *
 * @param  string  status
 * @return  array  of associated names by ID
 */
function getStatuses($status) {
	$s = new StatusOf($status);
	return $s->options();
}

/**
 * get API status ID from name
 *
 * @param  string  status
 * @param  string  name
 * @return  int  ID
 */
function getAPIStatusID($status, $name) {
	$s = new StatusOf($status, true);
	return $s->id($name);
}

/**
 * get API status name from ID
 *
 * @param  string  status
 * @param  int  ID
 * @return  string  name
 */
function getAPIStatusName($status, $id) {
	$s = new StatusOf($status, true);
	return $s->name($id);
}

/**
 * get API status options
 *
 * @param  string  status
 * @return  array  of associated names by ID
 */
function getAPIStatuses($status) {
	$s = new StatusOf($status, true);
	return $s->options();
}

?>
