<?php

/**
 * This file maintains "type" constants and their helper functions.
 *
 * DO NOT ADD APPLICATION SPECIFC TYPES TO THIS FILE UNLESS THEY ARE
 * SHARED BY ALL MBAPI APPLICATIONS!
 *
 * Application specifc types should be located here:
 *
 * /lib-APPLICATION/include/config/internal/types.php
 */

$CONF["types"] = array();
$CONF["APItypes"] = array();

$CONF["types"]["DBDriver"] = array(
    blmsg('TRANS_MYSQL'),
    blmsg('TRANS_MYSQLI'),
    blmsg('TRANS_POSTGRES7'),
    blmsg('TRANS_POSTGRES8'),
    blmsg('TRANS_ORACLE'),
    blmsg('TRANS_MSSQL'),
);

$CONF["APItypes"]["DBDriver"] = array();
$CONF["APItypes"]["DBDriver"][0] = "mysql";
$CONF["APItypes"]["DBDriver"][1] = "mysqli";
$CONF["APItypes"]["DBDriver"][2] = "postgres7";
$CONF["APItypes"]["DBDriver"][3] = "postgres8";
$CONF["APItypes"]["DBDriver"][4] = "oci8po";
$CONF["APItypes"]["DBDriver"][5] = "mssql";

$CONF["types"]["paramInput"] = array(
    "h" => blmsg('TRANS_HIDDEN'),
    "t" => blmsg('TRANS_TEXT'),
    "p" => blmsg('TRANS_PASSWORD'),
    "a" => blmsg('TRANS_TEXTAREA'),
    "s" => blmsg('TRANS_SELECT'),
    "s" => blmsg('TRANS_MULTISELECT'),
    "c" => blmsg('TRANS_CHECKBOXES'),
    "r" => blmsg('TRANS_RADIO_BUTTONS'),
);

$CONF["APItypes"]["paramInput"] = array();
$CONF["APItypes"]["paramInput"]["h"] = "hidden";
$CONF["APItypes"]["paramInput"]["t"] = "text";
$CONF["APItypes"]["paramInput"]["p"] = "password";
$CONF["APItypes"]["paramInput"]["a"] = "textarea";
$CONF["APItypes"]["paramInput"]["s"] = "select";
$CONF["APItypes"]["paramInput"]["s"] = "multiselect";
$CONF["APItypes"]["paramInput"]["c"] = "checkbox";
$CONF["APItypes"]["paramInput"]["r"] = "radio";

$CONF["types"]["user"] = array();
$CONF["types"]["user"][0] = blmsg('TRANS_PUBLIC_USER');
$CONF["types"]["user"][1] = blmsg('TRANS_CLIENT_USER');
$CONF["types"]["user"][2] = blmsg('TRANS_ADMIN_USER');

$CONF["APItypes"]["user"] = array();
$CONF["APItypes"]["user"][0] = "public";
$CONF["APItypes"]["user"][1] = "client";
$CONF["APItypes"]["user"][2] = "admin";

$php_options = get_loaded_extensions();
$CONF["types"]["crypt"] = array();
$CONF["types"]["crypt"][0] = blmsg('TRANS_CRYPT_BLOWFISH');
if(in_array("mcrypt",$php_options)) {
	$CONF["types"]["crypt"][1] = blmsg('TRANS_CRYPT_3DES');
	$CONF["types"]["crypt"][2] = blmsg('TRANS_CRYPT_TWOFISH');
	$CONF["types"]["crypt"][3] = blmsg('TRANS_CRYPT_TWOFISH128');
	$CONF["types"]["crypt"][4] = blmsg('TRANS_CRYPT_TWOFISH192');
	$CONF["types"]["crypt"][5] = blmsg('TRANS_CRYPT_TWOFISH256');
	$CONF["types"]["crypt"][6] = blmsg('TRANS_CRYPT_RIJNDAEL_128');
	$CONF["types"]["crypt"][7] = blmsg('TRANS_CRYPT_RIJNDAEL_192');
	$CONF["types"]["crypt"][8] = blmsg('TRANS_CRYPT_RIJNDAEL_256');
}

$CONF["APItypes"]["crypt"] = array();
$CONF["APItypes"]["crypt"][0] = blmsg('TRANS_CRYPT_BLOWFISH');
if(in_array("mcrypt",$php_options)) {
	$CONF["APItypes"]["crypt"][1] = "3des";
	$CONF["APItypes"]["crypt"][2] = "twofish";
	$CONF["APItypes"]["crypt"][3] = "twofish128";
	$CONF["APItypes"]["crypt"][4] = "twofish192";
	$CONF["APItypes"]["crypt"][5] = "twofish256";
	$CONF["APItypes"]["crypt"][6] = "rijndael128";
	$CONF["APItypes"]["crypt"][7] = "rijndael192";
	$CONF["APItypes"]["crypt"][8] = "rijndael256";
}

/**
 * class for handling types
 */
class TypeOf {
	/**
	 * string  type
	 */
	public $type;

	/**
	 * string  interface
	 */
	public $interface;

	/**
	 * constructor
	 *
	 * @param  string  type
	 * @param  boolean  whether to use API names rather than display names
	 */
	function TypeOf($type, $api = false) {
		$this->type = $type;
		$this->interface = $api ? "APItypes" : "types";
	}

	/**
	 * id by name
	 *
	 * @param  string  name
	 * @return  int  id
	 */
	function id($name) {
		global $CONF;
		return isset($CONF[$this->interface][$this->type]) ? array_search($name, $CONF[$this->interface][$this->type]) : false;
	}

	/**
	 * name by id
	 *
	 * @param  int  id
	 * @return  string  name
	 */
	function name($id) {
		global $CONF;
		return isset($CONF[$this->interface][$this->type][$id]) ? $CONF[$this->interface][$this->type][$id] : false;
	}

	/**
	 * HTML options
	 *
	 * @return  array  of associated names by ID
	 */
	function options() {
		global $CONF;
		return  isset($CONF[$this->interface][$this->type]) ? $CONF[$this->interface][$this->type] : array();
	}

	/**
	 * multi-select HTML options 
	 *
	 * @ return  array  list($selected, $selectable)
	 */
	function select($IDs = array()) {
		global $CONF;
		$selected = $selectable = array();
		foreach($this->options() as $id => $name) {
			if(in_array($id, $IDs)) {
				$selected[$id] = $name;
			} else {
				$selectable[$id] = $name;
			}
		}
		return array($selected, $selectable);
	}
}

/**
 * get type ID from name
 *
 * @param  string  type
 * @param  string  name
 * @return  int  ID
 */
function getTypeID($type, $name) {
	$t = new TypeOf($type);
	return $t->id($name);
}

/**
 * get type name from ID
 *
 * @param  string  type
 * @param  int  ID
 * @return  string  name
 */
function getTypeName($type, $id) {
	$t = new TypeOf($type);
	return $t->name($id);
}

/**
 * get type options
 *
 * @param  string  type
 * @return  array  of associated names by ID
 */
function getTypes($type) {
	$t = new TypeOf($type);
	return $t->options();
}

/**
 * get API type ID from name
 *
 * @param  string  type
 * @param  string  name
 * @return  int  ID
 */
function getAPITypeID($type, $name) {
	$t = new TypeOf($type, true);
	return $t->id($name);
}

/**
 * get API type name from ID
 *
 * @param  string  type
 * @param  int  ID
 * @return  string  name
 */
function getAPITypeName($type, $id) {
	$t = new TypeOf($type, true);
	return $t->name($id);
}

/**
 * get API type options
 *
 * @param  string  type
 * @return  array  of associated names by ID
 */
function getAPITypes($type) {
	$t = new TypeOf($type, true);
	return $t->options();
}

?>
