<?php

class PurchaseManager {

	public static function preparePurchaseData($store, $purchase, &$amount, &$totalAmount, &$prorationAmount, &$minimalCycle, $withProration = false) {
		$subData = $ownDomains = array();
		$description = null;

		$plan = ResourceLocator::getInstance()->billingPlanService->getById(
			$purchase->getPlanId(),
			$store->getId(),
			$purchase->hasCoupon() ? $purchase->getCoupon()->getId() : false,
			false,
			$withProration
		);
		foreach($purchase->getOwnDomains() as $domainName => $domainData) {

			$fakeId = ResourceLocator::getInstance()->cart->generateFakeId();
			$ownDomains[$fakeId][$domainName] = array(
				'id' => $fakeId,
				'type' => __('TRANS_YOUR_OWN_DOMAIN'),
				'name' => $domainName,
				'description' => $description,
				'price' => null,
				'cycle' => null,
				'hasAddAddonAbility' => false,
				'hasAddDomainAbility' => false,
				'fake' => true,
				'purchases' => array()
			);
		}

		foreach ($purchase->getPurchases() as $subPurchase) {
			$subData[$subPurchase->getId()] = self::preparePurchaseData($store, $subPurchase, $amount, $totalAmount, $prorationAmount, $minimalCycle);
		}

		$useTransferFee = $plan instanceof BillingDomainPlan && $purchase->getParameter('type') == BillingDomainPlan::PURCHASE_TYPE_TRANSFER ? true : false;

        $price = 0;
        $cycle = 0;
        $hasAddAddonAbility = false;
        $addons = array();
        if (!$purchase->isAlreadyPurchased()) {
            $price = $plan->getPriceById($purchase->getPriceId());
            $cycle = $plan->getCycle($price->cycleID);
            if (!$minimalCycle || $minimalCycle->getLength() < $cycle->getLength()) {
                $minimalCycle = $cycle;
            }
            $amount += $price->getTotalFee(true, false, $useTransferFee);
            $totalAmount += $price->getTotalFee(true, true, $useTransferFee);
            $prorationAmount += $price->hasProrationFee() ? $price->getProrationFee() : $price->getTotalFee(true, true, $useTransferFee);
            $hasAddAddonAbility = $plan->mayHaveAddons() && $plan->hasAddAddonAvailability($cycle->cycleID);
            foreach ($purchase->getAddons() as $addon) {
                $addons[] = self::prepareAddonData($store, $addon, $amount, $totalAmount, $prorationAmount, $minimalCycle, $withProration);
            }
        }

		$hasAddDomainAbility = false;
		if ($plan->mayHaveDomains()) {
			if ($store->hasAbilityToUseOwnDomain()) {
				$hasAddDomainAbility = true;
			} else {
				$domainPlan = $plan instanceof BillingDomainPlan ? $plan : $store->getDefaultDomainPlan();
				if ($domainPlan && $store->hasAbilityToPurchaseDomainPlan($domainPlan)) {
					$hasAddDomainAbility = true;
				}
			}
		}
		if ($plan->productType == BillingPlan::TYPE_SSL_CERTIFICATE) {
			if ($purchase->hasParameter('data')) {
				$certificateData = $purchase->getParameter('data');
				$description = $certificateData["dn"]["commonName"];
			}
		}

        $hasAdditionalServices = false;
        $groupsWithServices = $plan->getSupplementary($store->getId(), $purchase->getCycleId(), $purchase->getSubscriptionId());
        foreach ($groupsWithServices as $group) {
            if ($group->hasPlans()) {
                $hasAdditionalServices = true;
                break;
            }
        }

        $data = array(
			'id' => $purchase->getId(),
			'type' => $useTransferFee ? __('TRANS_PRODUCT_TYPE_NAME_TRANSFER_DOMAIN') : __('TRANS_PRODUCT_TYPE_NAME_' . $plan->productType),
			'name' => $purchase->getName(),
			'description' => $purchase->isAlreadyPurchased() ? $purchase->getDescription() : $description,
			'price' => $price,
			'cycle' => $cycle,
			'hasAddAddonAbility' => $hasAddAddonAbility,
			'hasAddDomainAbility' => $hasAddDomainAbility,
            'hasAdditionalServices' => $hasAdditionalServices,
			'useTransferFee' => $useTransferFee,
			'fake' => false,
			'purchases' => $subData,
			'ownDomains' => $ownDomains,
			'hasAdditionalInformation' => $purchase->hasParameter('data'),
            'addons' => $addons,
            'isDomain' => $purchase->isDomain(),
            'isAlreadyPurchased' => $purchase->isAlreadyPurchased()
		);

		return $data;
	}

	public static function prepareAddonData($store, $addon, &$amount, &$totalAmount, &$prorationAmount, &$minimalCycle, $withProration = false) {

		$billingAddon = ResourceLocator::getInstance()->billingAddonService->getById(
			$addon->getAddonId(),
			$addon->getPlanId(),
			$store->getId(),
			$addon->getCycleId(),
			$addon->hasCoupon() ? $addon->getCoupon()->getId() : false,
			false,
			$withProration
		);

		$price = $billingAddon->getPriceById($addon->getPriceId());
		$cycle = $billingAddon->getCycle($price->cycleID);
		if (!$minimalCycle || $minimalCycle->getLength() < $cycle->getLength()) {
			$minimalCycle = $cycle;
		}
		$amount += $price->getTotalFee(true, false);
		$totalAmount += $price->getTotalFee();
		$prorationAmount += $price->hasProrationFee() ? $price->getProrationFee() : $price->getTotalFee(true, true, false);

		return array(
			'id' => $addon->getId(),
			'name' => $billingAddon->productName,
			'price' => $price,
			'cycle' => $cycle
		);
	}

	public static function purchaseDomains($request, $parentPurchase = null, $cart, $plan, &$duplicatedDomains = array()) {
		$SLD = $request->getParameter('SLD');

		$registerTLDs = $request->getParameter('registerTLDs', array());
		$transferTLDs = $request->getParameter('transferTLDs', array());

		$alreadyPurchasedDomains = array();
		$domainPurchases = $cart->getDomainPurchases();
		foreach ($domainPurchases as $purchase) {
			$data = $purchase->getParameter('domainData');
			$alreadyPurchasedDomains[] = $data['name'];
		}

		$currentlyPurchasingDomains = array();
		if ($currentCartItem = $cart->getCurrentItem()) {
			foreach ($currentCartItem->getPurchases(true) as $purchase) {
				if ($purchase->getType() == BillingPlan::TYPE_DOMAIN) {
					$data = $purchase->getParameter('domainData');
					$currentlyPurchasingDomains[] = $data['name'];
				} else {
					if ($data = $purchase->getParameter('domainData')) {
						$alreadyPurchasedDomains[] = $data['name'];
					}
				}
			}
		}

		$count = 0;

		$purchases = array();
		foreach ($registerTLDs as $TLD) {
			$domainName = $SLD . '.' . $TLD;
			if (array_search($domainName, $alreadyPurchasedDomains) !== false) {
				$duplicatedDomains[] = $domainName;
				continue;
			}
			if (array_search($domainName, $currentlyPurchasingDomains) !== false) {
				$count++;
				continue;
			}
			$priceId = $request->getParameter('price_' . str_replace(".", "_", $TLD));
			$cycleId = $plan->getPriceById($priceId)->cycleID;

			$purchase = self::prepareDomainPurchase($plan->getId(), $cycleId, $priceId, BillingDomainPlan::PURCHASE_TYPE_REGISTER, $SLD, $TLD);
			if ($coupon = $plan->getPreappliedCoupon()) {
				$purchase->setCoupon($coupon);
			}

			$purchases[] = $purchase;
			$count++;
		}

		foreach ($transferTLDs as $TLD) {
			$domainName = $SLD . '.' . $TLD;
			if (array_search($domainName, $alreadyPurchasedDomains) !== false) {
				$duplicatedDomains[] = $domainName;
				continue;
			}
			if (array_search($domainName, $currentlyPurchasingDomains) !== false) {
				$count++;
				continue;
			}
			$priceId = $request->getParameter('price_' . str_replace(".", "_", $TLD));
			$cycleId = $plan->getPriceById($priceId)->cycleID;

			$purchase = self::prepareDomainPurchase($plan->getId(), $cycleId, $priceId, BillingDomainPlan::PURCHASE_TYPE_TRANSFER, $SLD, $TLD);
			if ($coupon = $plan->getPreappliedCoupon()) {
				$purchase->setCoupon($coupon);
			}

			$purchases[] = $purchase;
			$count++;
		}

		if ($count && !$duplicatedDomains) {
			foreach ($purchases as $purchase) {
				// set id of related subscription
				if ($subscriptionId = sfContext::getInstance()->getUser()->getAttribute('subscriptionId')) {
					$purchase->setSubscriptionId($subscriptionId);
				}
                if (null != $parentPurchase) {
                    $parentPurchase->addPurchase($purchase);
                } else {
                    $cart->getCurrentItem()->addPurchase($purchase);
                }

			}
			sfContext::getInstance()->getUser()->setAttribute('subscriptionId', null);
			return true;
		} else {
			return false;
		}

	}

	private static function prepareDomainPurchase($planId, $cycleId, $priceId, $type, $SLD, $TLD) {
		$domainName = $SLD . '.' . $TLD;
		$purchase = new Purchase($planId, $cycleId, $priceId, $domainName, BillingPlan::TYPE_DOMAIN);
		$purchase->addParameter('type', $type);
		$domainData = array(
			'SLD' => $SLD,
			'TLD' => $TLD,
			'name' => $domainName
		);
		$purchase->addParameter('domainData', $domainData);
		return $purchase;
	}
}