<?php

abstract class CartAction extends sfAction {

	const ACTIVITY_BACK = '__back';

	protected $_router;

	protected $_store;

	protected $_container;

	protected $_cart;

	public function preExecute() {
		try {
			if (!$this->_router = ResourceLocator::getInstance()->routersStack->current()) {
				throw new StoreException('Router is not defined');
			}
			$this->_router->checkByAction($this->getActionName());

			$request = sfContext::getInstance()->getRequest();

			if (!$this->_container = sfContext::getInstance()->getUser()->getAttribute('container')) {
				throw new StoreException('Container is not defined');
			}

			if ($request->isMethod('post')) {
				switch ($request->getParameter('__activity')) {
					case self::ACTIVITY_BACK:
						if("catalog" == $this->_router->getPreviousPage()->getCode()) {
							sfContext::getInstance()->getConfiguration()->loadHelpers('BillingFormat');
							$this->redirect(formatContinueShoppingUrl($this->_container));
						} else {
							$this->_router->previous();
						}
						return;
						break;
				}
			}

			if (!$this->_store = sfContext::getInstance()->getUser()->getAttribute('store')) {
				throw new StoreException('Store is not defined');
			}

			$this->_cart = ResourceLocator::getInstance()->cart;

			if (isset($this->_store->locale)) {
				$this->getUser()->setCulture($this->_store->locale);
			}

			$previewTheme = $this->getUser()->getAttribute('theme');
			$theme = $previewTheme ? $previewTheme : $this->_store->theme;
			sfConfig::set('sf_web_css_dir_name', 'themes/' . $theme . '/css');
			sfConfig::set('sf_web_images_dir_name', 'themes/' . $theme . '/images');

			$this->router = $this->_router;
			$this->container = $this->_container;
			$this->currencyId = $this->_store->currencyID;

			$this->getResponse()->setTitle($this->_store->orderFormName);

		} catch (StoreException $exception) {
			throw new sfError404Exception();
		}

        sfContext::getInstance()->getConfiguration()->loadHelpers('I18N');
        sfContext::getInstance()->getConfiguration()->loadHelpers('BillingFormat');
	}

    protected function _initSelectedPurchase()
    {
        $purchaseId = $this->getRequest()->getParameter('purchaseId');
        $this->domainPlanId = empty($purchaseId) ? $this->getRequest()->getParameter('domainPlanId', 0) : 0;
        $this->parentSubscriptionId = empty($purchaseId) ? $this->getRequest()->getParameter('parentSubscriptionId', 0) : 0;

        if (!$this->cartItem = $this->_cart->getCurrentItem()) {
            // skip this step if there is no current cart item
            $this->_router->next(array(), false);
        }
        $this->selectedPurchase = null;
        $this->plan = null;
        try {
            if (($this->domainPlanId == 0) && ($this->parentSubscriptionId == 0)) {
                if (empty($purchaseId)) {
                    // skip this step if there is no current purchase
                    $this->_router->next(array(), false);
                }

                $this->selectedPurchase = $this->cartItem->getPurchase($purchaseId);
                if (!$this->selectedPurchase) {
                    $this->redirect(formatContinueShoppingUrl($this->_container));
                }
                $this->plan = $this->_store->getPlan($this->selectedPurchase->getPlanId(), true);
            } elseif ($this->domainPlanId > 0) {
                $this->plan = $this->_store->getPlan($this->domainPlanId, true);
            }
        } catch (Exception $e) {
            $this->redirect(formatContinueShoppingUrl($this->_container));
        }
    }
}