<?php

abstract class CatalogAction extends sfAction
{
	protected $_store;

	protected $_widget;

	protected $_container;

	public function preExecute()
    {
		try {
			if (!$this->_container = $this->getRequest()->getParameter('container', false)) {
				if ($store = ResourceLocator::getInstance()->billingStoreService->getDefault()) {
					sfContext::getInstance()->getConfiguration()->loadHelpers('BillingFormat');
					$this->redirect('@catalog?container=' . $store->orderFormCode);
				} else {
					throw new StoreException();
				}
			}
			if ($this->_widget = ResourceLocator::getInstance()->billingWidgetService->getByCode($this->_container)) {
				$this->_store = $this->_widget->getStore();
			} else {
				if (!$this->_store = ResourceLocator::getInstance()->billingStoreService->getByCode($this->_container)) {
					throw new StoreException();
				}
			}
            $purchaseRequestId = $this->getRequest()->getParameter('purchaseRequestId');
            if ($purchaseRequestId && $this->getUser()->getAttribute('purchaseRequestId') != $purchaseRequestId) {
                $purchaseRequestParams = ResourceLocator::getInstance()->billingPurchaseService->getRequestParams($purchaseRequestId);
                $this->getUser()->setAttribute('purchaseRequestId', $purchaseRequestId);
                $this->getUser()->setAttribute('purchaseRequestParams', $purchaseRequestParams);
                $this->_setPleskUrl($purchaseRequestParams);
            }
			if (isset($this->_store->locale)) {
				$this->getUser()->setCulture($this->_store->locale);
			}
			$previewTheme = $this->getUser()->getAttribute('theme');
			$theme = $previewTheme ? $previewTheme : $this->_store->theme;
			sfConfig::set('sf_web_css_dir_name', 'themes/' . $theme . '/css');
			sfConfig::set('sf_web_images_dir_name', 'themes/' . $theme . '/images');
			$this->getUser()->setAttribute('store', $this->_store);
			$this->getUser()->setAttribute('container', $this->_container);
			$this->getResponse()->setTitle($this->_store->orderFormName);
		} catch (StoreException $exception) {
			throw new sfError404Exception();
		}
	}

	protected function getPlans()
    {
		return $this->_widget ? $this->_widget->getPlans(true) : $this->_store->getPlans(true);
	}

	protected function getDefaultTaxZone()
    {
		$result = null;
		try {
			if ($defaultTaxZoneId = isset($this->_store->defaultTaxZoneID) ? $this->_store->defaultTaxZoneID : false) {
				$result = ResourceLocator::getInstance()->billingTaxZoneService->getById($defaultTaxZoneId);
			}
		} catch (BillingObjectException $exception) {
		}
		return $result;
	}

    private function _setPleskUrl($params)
    {
        $url = false;
        if (isset($params['backRedirectUrl'])) {
            $parsedUrl = parse_url($params['backRedirectUrl']);
            if ($parsedUrl) {
                $url = $parsedUrl['scheme'] . '://' . $parsedUrl['host'] . ':' . $parsedUrl['port'];
            }
        }

        $this->getUser()->setAttribute('pleskUrl', $url);
    }
}