<?php

class OrderDetailsAction extends CartAction {

	const OPERATION_SKIP = 'skip';

	public function execute($request)
    {
        $this->cartItem = $this->_cart->getCurrentItem();
        $this->plan = null;

		$domainPurchase = null;
		$domainPurchases = null;
		$certificatePurchases = null;
		$customQuestionPurchase = null;

		$totalPurchasesCount = 0;
		if($purchaseId = $request->getParameter('purchaseId', false)) {
			$purchases = ResourceLocator::getInstance()->cart->getPurchases();
			if(!isset($purchases[$purchaseId])) {
				$this->_router->next(array(), false);
			}
			$this->purchaseId = $purchaseId;
			switch($purchases[$purchaseId]->getType()) {
				case BillingPlan::TYPE_DOMAIN :
					$domainPurchase = $purchases[$purchaseId];
					$totalPurchasesCount = 1;
					break;
				case BillingPlan::TYPE_SSL_CERTIFICATE :
					$certificatePurchase = $purchases[$purchaseId];
					$totalPurchasesCount = 1;
					break;
				default :
					$this->_router->next(array(), false);
			}
		} else {
            $purchases = ResourceLocator::getInstance()->cart->getPurchases();
            $purchasesToCheck = array();
            foreach($purchases as $purchase) {
                if (!$purchase->getParameter('customQuestionData')) {
                    $parentPlanId = $purchase->hasParentPurchase() ? $purchase->getParentPurchase()->getPlanId() : null;
                    $purchasesToCheck[$purchase->getId()] = array('planId' => $purchase->getPlanId(), 'parentPlanId' => $parentPlanId);
                }
            }
            $customQuestionPurchasesIds = empty($purchasesToCheck) ? array() : ResourceLocator::getInstance()->billingContactService->getPurchasesWithCustomQuestions($purchasesToCheck);
            $customQuestionPurchases = array();
            foreach($purchases as $purchase) {
                if (in_array($purchase->getId(), $customQuestionPurchasesIds)) {
                    $customQuestionPurchases[] = $purchase;
                }
            }
            $totalPurchasesCount += count($customQuestionPurchases);
			$domainPurchases = ResourceLocator::getInstance()->cart->getDomainPurchases(true, $totalPurchasesCount);
			$certificatePurchases = ResourceLocator::getInstance()->cart->getCertificatePurchasesAndAddons(true, $totalPurchasesCount);
			$domainPurchase = $certificatePurchase = $customQuestionPurchase = NULL;
			if (!empty($domainPurchases)) {
				$domainPurchase = array_shift($domainPurchases);
			} elseif (!empty($certificatePurchases)) {
				$certificatePurchase = array_shift($certificatePurchases);
			} elseif (!empty($customQuestionPurchases)) {
				$customQuestionPurchase = array_shift($customQuestionPurchases);
			}
		}

		if (!$totalPurchasesCount) {
			$this->_router->next(array(), false);
		}

		if ($domainPurchase) {
			$plan = $this->_store->getPlan($domainPurchase->getPlanId());
			$price = $plan->getPriceById($domainPurchase->getPriceId());
			$variantId = $price->productVariantID;
			$domainData = $domainPurchase->getParameter('domainData');
			$SLD = $domainData['SLD'];
			$action = $domainPurchase->getParameter('type');
            $customerType[$SLD."_domain_ContractType"] = (isset($_REQUEST["domainDetails"][$SLD."_domain_ContractType"])) ? $_REQUEST["domainDetails"][$SLD."_domain_ContractType"] : 0;
			$this->form = ResourceLocator::getInstance()->billingDomainService->getDomainDetailsForm($this->_store->getId(), $variantId, $SLD, $action, $customerType);
			if($domainPurchase->hasParameter('data')) {
				$data = $this->form->getDefaults();
				foreach($domainPurchase->getParameter('data') as $key => $value) {
					$data["{$SLD}_{$key}"] = $value;
				}
				$this->form->bind($data);
			}
			$this->form->setAttribute('variantId', $variantId);
			$this->form->setAttribute('SLD', $SLD);
			$this->form->setAttribute('action', $action);
			$this->form->setAttribute('purchase', $domainPurchase);
			$domainData = $domainPurchase->getParameter('domainData');
			$this->title = __('TRANS_INFORMATION_FOR_DOMAIN', array('%NAME%' => formatProductName($domainData['name'])));
			if ( 1 < ( $domainCount = count(ResourceLocator::getInstance()->cart->getDomainPurchases(false))) ) {
				$this->subTitle = __('TRANS_ONLINE_STORE_DOMAIN_PURCHASES_COUNT', array("%ALL%" => $domainCount, '%CURRENT%' => ( $domainCount - count($domainPurchases))));
			}
			$this->description = 'TRANS_INFORMATION_FOR_DOMAIN_DESCRIPTION';
		} else if ($certificatePurchase) {
			if ($certificatePurchase instanceof Purchase && ! is_null($parentPurchase = $certificatePurchase->getParentPurchase())) {
				$domains = array();
				foreach (array_keys($parentPurchase->getOwnDomains()) as $domainName) {
					$domains[$domainName] = null;
				}
				$this->parentPlan = ResourceLocator::getInstance()->billingPlanService->getById($parentPurchase->getPlanId(), $this->_store->getId());
				if (! ($this->parentPlan instanceof BillingDomainPlan)) {
					$plans = $this->_store->getPlans(true);
					foreach ($parentPurchase->getPurchases() as $subPurchase) {
						$planId = $subPurchase->getPlanId();
						if ($plans[$planId]->productType == BillingPlan::TYPE_DOMAIN) {
							$data = $subPurchase->getParameter('domainData');
							$domains[$data['name']] = $subPurchase->getParameter('type');
						}
					}
					$this->parentDomains = $domains;
				}
			}
			$planId = $certificatePurchase instanceof Addon ? $certificatePurchase->getAddonId() : $certificatePurchase->getPlanId();
			$this->form = ResourceLocator::getInstance()->billingCertificateService->getCertificateDetailsForm($this->_store->getId(), $planId);
			if($certificatePurchase->hasParameter('data')) {
				$data = $this->form->getDefaults();
				$certificateData = $certificatePurchase->getParameter('data');
				foreach($certificateData["dn"] as $key => $value) {
					$data[$key] = $value;
				}
				$this->form->bind($data);
			} else if ($this->getUser()->hasAttribute('selectCertificateParams')) {
				$data = $this->form->getDefaults();
				foreach($this->getUser()->getAttribute('selectCertificateParams')->getAll() as $key => $value) {
					$data[$key] = $value;
				}
				$this->form->bind($data);
			} else if (($purchaseRequestParams = $this->getUser()->getAttribute('purchaseRequestParams')) && isset($purchaseRequestParams['certificate'])) {
				$data = $this->form->getDefaults();
				foreach ($purchaseRequestParams['certificate'] as $name => $value) {
                    if ('' != $value) {
                        $data[$name] = $value;
                    }
				}
                if (!empty($data['csr'])) {
                    $data['creationmethod'] = 'upload';
                }
				$this->form->setDefaults($data);
			}
			$this->form->setAttribute('planId', $planId);
			$this->form->setAttribute('purchase', $certificatePurchase);
			$this->title = __('TRANS_INFORMATION_FOR_SSL_CERTIFICATE');
			$this->subTitle = $certificatePurchase->getName();
			$this->description = 'TRANS_INFORMATION_FOR_SSL_CERTIFICATE_DESCRIPTION';
		} else if ($customQuestionPurchase) {
			$planId = $customQuestionPurchase->getPlanId();
            $parentPlanId = $customQuestionPurchase->hasParentPurchase() ? $customQuestionPurchase->getParentPurchase()->getPlanId() : null;
			$this->form = ResourceLocator::getInstance()->billingContactService->getPlanCustomQuestionsForm(
                    array($customQuestionPurchase->getId() => array('planId' => $planId, 'parentPlanId' => $parentPlanId)));
            $this->form->setAttribute('planId', $planId);
            $this->form->setAttribute('purchase', $customQuestionPurchase);
            $this->title = __($this->form->getSection('customQuestion')->getTitle());
            $this->description =  __('TRANS_ORDERFORM_ADDITIONAL_PRODUCT_INFORMATION_DESC', array('%PLAN_NAME%' => $customQuestionPurchase->getName()));
		}

		if ($request->isMethod('post')) {
			$operation = $request->getParameter('operation');
			switch ($operation) {
				case self::OPERATION_SKIP:
					$this->_router->next();
					break;
				default:
					$this->form->bind($request->getParameter($this->form->getName()));
					if ($this->form->isValid()) {
						if ($this->form->save()) {
							if (!is_null($domainPurchases) || !is_null($certificatePurchases) || !is_null($customQuestionPurchase)) {
								$this->_router->refresh();
							} else {
								$this->_router->next();
							}
						} else {
                            // form was not saved, probably because redirect to panel is required
                            if (!is_null($customQuestionPurchase) && $customQuestionPurchase->getParameter('redirectToPanelSubscription')) {
                                $cartItem = ResourceLocator::getInstance()->cart->getItemByPurchase($customQuestionPurchase->getId());
                                ResourceLocator::getInstance()->cart->setCurrentItem($cartItem);
                                $router = new Router();
                                $router->init(
                                    array(
                                        new RouterPage('login-to-panel', array('purchaseId' => $customQuestionPurchase->getId())),
                                        new RouterPage('summary'),
                                        new RouterPage('order-details'),
                                        new RouterPage('confirmation')
                                    ),
                                    new RouterPage('confirmation')
                                );
                                ResourceLocator::getInstance()->routersStack->push($router);
                                $router->next();
                            }
                        }

					} else {
					}
			}

		}
	}
}