<?php

class RegisterDomainAction extends CartAction
{
	const ACTIVITY_CHECK = 'check';

	const OPERATION_REGISTER = 'register';
	const OPERATION_TRANSFER = 'transfer';

	const ERROR_WRONG_DOMAIN = 1;
	const ERROR_CANNOT_PURCHASE_DOMAIN = 2;

	public function execute($request)
    {
        $this->_initSelectedPurchase();

		if ($request->isMethod('post')) {
			try {
				switch ($request->getParameter('activity')) {
					case self::ACTIVITY_CHECK:
						if (
							!($domainPlanId = $request->getParameter('domainPlanId')) ||
							!($operation = $request->getParameter('operation'))
						) {
							throw new InputException('TRANS_YOU_MUST_SPECIFY_DOMAIN_NAME');
						}
						$masterTLD = $request->getParameter('masterTLD');
						$SLD = trim($request->getParameter('SLD'));
						$selectDomainParameters = new sfParameterHolder();
						$selectDomainParameters->add(
							array(
								'domainPlanId' => $domainPlanId,
								'masterTLD' => $masterTLD,
								'SLD' => $SLD,
								'operation' => $operation
							)
						);
						sfContext::getInstance()->getUser()->setAttribute('selectDomainParameters', $selectDomainParameters);
						if (!$SLD) {
							throw new InputException('TRANS_DOMAIN_REQUIRED', self::ERROR_WRONG_DOMAIN);
						}
						if (!ResourceLocator::getInstance()->billingDomainService->isDomainValid("{$SLD}.{$masterTLD}", $this->_store->getId(), $domainPlanId)) {
							throw new InputException('TRANS_YOU_MUST_SPECIFY_DOMAIN_NAME', self::ERROR_WRONG_DOMAIN);
						}
						$this->_router->refresh();
						break;
					default:
						$plan = $this->_store->getPlan($request->getParameter('planId'));
						$duplicatedDomains = array();
						if (PurchaseManager::purchaseDomains($request, $this->selectedPurchase, $this->_cart, $plan, $duplicatedDomains)) {
							$this->_router->next();
						} else {
							if ($duplicatedDomains) {
								$errorMessage = __('TRANS_DUPLICATE_DOMAINS', array('%DOMAINS%' => implode(', ', $duplicatedDomains)));
							} else {
								$errorMessage = 'TRANS_YOU_MUST_SELECT_AT_LEAST_ONE_DOMAIN';
							}
							throw new InputException($errorMessage, self::ERROR_CANNOT_PURCHASE_DOMAIN);
						}
				}
			} catch (InputException $exception) {
				$this->errorMessage = $exception->getMessage();
				$this->errorCode = $exception->getCode();
			}

		}

		if (!$selectDomainParameters = sfContext::getInstance()->getUser()->getAttribute('selectDomainParameters')) {
			$this->_router->previous();
		}

		$domainPlanId = $selectDomainParameters->get('domainPlanId');
		$domainPlan = $this->_store->getPlan($domainPlanId, true);

		$masterTLD = $selectDomainParameters->get('masterTLD');
        $masterTldId = null;

		$domains = $domainPlan->getDomains($this->_store->getId());
		$TLDs = array();
		foreach ($domains as $domain) {
			if ($domain->tldAutoSearch) {
				$TLDs[$domain->tldExtension] = $domain->tldID;
			}
			if ($masterTLD == $domain->tldExtension) {
                $masterTldId = $domain->tldID;
			}
		}
        if (null !== $masterTldId) {
            $TLDs = array_merge(array($masterTLD => $masterTldId), $TLDs);
        }

		$SLD = $selectDomainParameters->get('SLD');

		$this->SLD = $SLD;
		$this->TLDs = $TLDs;
        $this->TLDsCount = count($TLDs);
        $this->TLDsString = implode("-", $TLDs);
		$this->masterTLD = $masterTLD;
		$this->domainName = $SLD . ($masterTLD ? '.' . $masterTLD : '');
		$this->domainPlan = $domainPlan;
		$this->domainPlanId = $domainPlanId;
		$this->operation = $selectDomainParameters->get('operation', self::OPERATION_REGISTER);
        if ($defaultTaxZoneId = isset($this->_store->defaultTaxZoneID) ? $this->_store->defaultTaxZoneID : false) {
            $this->taxZone = ResourceLocator::getInstance()->billingTaxZoneService->getById($defaultTaxZoneId);
        }

    }
}