<?php

class SelectParentServiceAction extends CartAction
{
    public function execute($request)
    {
        $this->_initSelectedPurchase();
        $customer = sfContext::getInstance()->getUser()->getAttribute('customer', null);
        $this->isParentServiceRequired = !is_null($customer) && BillingPlan::HOSTING_REQUIRED == $this->plan->productHostingRequired;

        if ($request->isMethod('post')) {
            $newPurchase = null;
            if ('' == ($selectedParentPlanVariantPriceId = $request->getParameter('parentPlanVariantPriceId'))) {
                if ($this->isParentServiceRequired) {
                    $this->errorMessage = __('TRANS_STORE_SUPPLEMENTARY_HOSTING_PLAN_REQUIRED_MSG');
                } else {
                    $this->getUser()->setAttribute('subscriptionId', '');
                }
            } else {
                if (empty($this->selectedPurchase)) {
                    if (false !== strstr($selectedParentPlanVariantPriceId, "subscription-")) {
                        $selectedExistingSubscriptionId = str_replace('subscription-', '', $selectedParentPlanVariantPriceId);
                        $selectedExistingSubscription = ResourceLocator::getInstance()->billingSubscriptionService->getSubscription($selectedExistingSubscriptionId);
                        $this->getUser()->setAttribute('subscriptionId', $selectedExistingSubscriptionId);
                        $selectedPlan = ResourceLocator::getInstance()->billingPlanService->getById($selectedExistingSubscription->getPlanId(), $this->_store->getId());
                        $newPurchase = new Purchase($selectedExistingSubscription->getPlanId(), $selectedExistingSubscription->getCycleId(), null, $selectedExistingSubscription->getName(), $selectedPlan->getType());
                        $newPurchase->setDescription($selectedExistingSubscription->additionalName);
                        $newPurchase->setAlreadyPurchasedId($selectedExistingSubscriptionId);
                        foreach ($this->_cart->getCurrentItem()->getPurchases() as $purchase) {
                            $purchase->setSubscriptionId($selectedExistingSubscriptionId);
                            $newPurchase->addPurchase($purchase);
                        }
                    } else {
                        $parentPlanId = $request->getParameter("plan_{$selectedParentPlanVariantPriceId}");
                        $parentPlan = $this->_store->getPlan($parentPlanId);
                        $price = $parentPlan->getPriceById($selectedParentPlanVariantPriceId);
                        $cycleId = $price->cycleID;
                        $newPurchase = new Purchase($parentPlanId, $cycleId, $selectedParentPlanVariantPriceId, $parentPlan->productName, $parentPlan->productType);
                        foreach ($this->_cart->getCurrentItem()->getPurchases() as $purchase) {
                            $newPurchase->addPurchase($purchase);
                        }
                        if ($coupon = $parentPlan->getPreappliedCoupon()) {
                            $newPurchase->setCoupon($coupon);
                        }
                    }
                } else {
                    if (false !== strstr($selectedParentPlanVariantPriceId, "subscription-")) {
                        $selectedExistingSubscriptionId = str_replace('subscription-', '', $selectedParentPlanVariantPriceId);
                        $selectedExistingSubscription = ResourceLocator::getInstance()->billingSubscriptionService->getSubscription($selectedExistingSubscriptionId);
                        $this->getUser()->setAttribute('subscriptionId', $selectedExistingSubscriptionId);
                        $selectedPlan = ResourceLocator::getInstance()->billingPlanService->getById($selectedExistingSubscription->getPlanId(), $this->_store->getId());
                        $newPurchase = new Purchase($selectedExistingSubscription->getPlanId(), $selectedExistingSubscription->getCycleId(), null, $selectedExistingSubscription->getName(), $selectedPlan->getType());
                        $newPurchase->setDescription($selectedExistingSubscription->additionalName);
                        $newPurchase->setAlreadyPurchasedId($selectedExistingSubscriptionId);
                        $this->selectedPurchase->setSubscriptionId($selectedExistingSubscriptionId);
                        $newPurchase->addPurchase($this->selectedPurchase);
                    } elseif (is_null($this->selectedPurchase->getParentPurchase()) || ($this->selectedPurchase->getParentPurchase() && $this->selectedPurchase->getParentPurchase()->getPriceId() != $selectedParentPlanVariantPriceId)) {
                        $parentPlanId = $request->getParameter("plan_{$selectedParentPlanVariantPriceId}");
                        $parentPlan = $this->_store->getPlan($parentPlanId);
                        $price = $parentPlan->getPriceById($selectedParentPlanVariantPriceId);
                        $cycleId = $price->cycleID;
                        $newPurchase = new Purchase($parentPlanId, $cycleId, $selectedParentPlanVariantPriceId, $parentPlan->productName, $parentPlan->productType);
                        $newPurchase->addPurchase($this->selectedPurchase);
                        if ($coupon = $parentPlan->getPreappliedCoupon()) {
                            $newPurchase->setCoupon($coupon);
                        }
                    }
                }
            }

            if ($newPurchase instanceof Purchase) {
                $newCartItem = new CartItem();
                $newCartItem->addPurchase($newPurchase);
                $this->_cart->setCurrentItem($newCartItem);
            } elseif (!is_null($this->selectedPurchase)) {
                // not for domain subscriptions
                $newCartItem = new CartItem();
                $newCartItem->addPurchase($this->selectedPurchase);
                $this->_cart->setCurrentItem($newCartItem);
                $this->_router->next();
            }

            if (!empty($this->selectedPurchase) && $this->selectedPurchase->getParentPurchase() && !$this->selectedPurchase->getParentPurchase()->isAlreadyPurchased()) {
                $parentPlan = empty($parentPlan) ? $this->_store->getPlan($this->selectedPurchase->getParentPurchase()->getPlanId()) : $parentPlan;
                if (BillingPlan::DOMAIN_NOT_NEEDED != $parentPlan->productDomainRequired) {
                    $router = new Router();
                    $router->init(
                        array(
                            new RouterPage('select-domain', array('purchaseId' => $this->selectedPurchase->getParentPurchase()->getId())),
                            new RouterPage('register-domain', array('purchaseId' => $this->selectedPurchase->getParentPurchase()->getId())),
                            new RouterPage('select-parent-service')
                        ),
                        new RouterPage('select-parent-service', array('purchaseId' => $this->selectedPurchase->getId()))
                    );
                    ResourceLocator::getInstance()->routersStack->push($router);
                    $router->next();
                }
            }

            if (is_null($this->errorMessage)) {
                $this->_router->next();
            }
        }

        $this->existingSubscriptions = array();
        $parentPlans = array();

        if (BillingPlan::TYPE_DOMAIN == $this->plan->productType) {
            if (BillingPlan::HOSTING_NOT_NEEDED != $this->plan->productHostingRequired) {
                foreach ($this->_store->getPlans(true) as $storePlan) {
                    if (BillingPlan::TYPE_HOSTING != $storePlan->productType || !$this->_store->hasAbilityToPurchasePlan($storePlan)) {
                        continue;
                    }
                    $parentPlans[] = $storePlan;
                }
                if (!is_null($customer)) {
                    $this->existingSubscriptions = ResourceLocator::getInstance()->billingSubscriptionService->getExistingHostingSubscriptions($this->_store->getId(), $customer->clientID);
                }
            }
        } else {
            $planInStore = $this->_store->getPlans();
            if (!is_null($customer)) {
                $this->existingSubscriptions = ResourceLocator::getInstance()->billingSubscriptionService->getSubscriptionsToAttachAsParent($customer->clientID, $this->selectedPurchase->getPlanId(), $this->selectedPurchase->getCycleId());
            }
            foreach ($this->plan->parentPlanIds as $planId) {
                if (!isset($planInStore[$planId]) || !$this->_store->hasAbilityToPurchasePlan($planInStore[$planId])) {
                    continue;
                }
                $parentPlans[$planId] = $planInStore[$planId];
            }
        }

        if (empty($parentPlans) && empty($this->existingSubscriptions)) {
            $this->_router->next(array(), false);
        }

        $this->_router->removeStep('select-services');
        $this->selectedPlan = ($this->selectedPurchase && $this->selectedPurchase->getParentPurchase()) ? $this->selectedPurchase->getParentPurchase()->getPriceId() : null;
        $this->selectedSubscription = ($this->selectedPurchase && $this->selectedPurchase->getParentPurchase() && $this->selectedPurchase->getParentPurchase()->isAlreadyPurchased()) ? $this->selectedPurchase->getParentPurchase()->getSubsciriptionId() : null;
        $this->parentPlans = $parentPlans;

        if ($defaultTaxZoneId = isset($this->_store->defaultTaxZoneID) ? $this->_store->defaultTaxZoneID : false) {
            $this->taxZone = ResourceLocator::getInstance()->billingTaxZoneService->getById($defaultTaxZoneId);
        }
    }
}