<?php

class SelectServiceAction extends CartAction
{
    public function execute($request)
    {
        $this->_initSelectedPurchase();

        $services = null;
        if ($request->isMethod('post')) {
            $purchase = $this->selectedPurchase;

            $planId = $purchase->getPlanId();
            $plan = ResourceLocator::getInstance()->billingPlanService->getById($planId, $this->_store->getId());
            $subscriptionId = $purchase->getSubscriptionId();
            $groupsWithServices = $plan->getSupplementary($this->_store->getId(), $purchase->getCycleId(), $subscriptionId);

            // retrive already added non-domain plans
            $alreadySelected = array();
            foreach ($purchase->getPurchases() as $childPurchase) {
                if (!$childPurchase->isDomain()) {
                    $alreadySelected[$childPurchase->getPlanId()]['purchaseId'] = $childPurchase->getId();
                    $alreadySelected[$childPurchase->getPlanId()]['cycleId'] = $childPurchase->getCycleId();
                    $alreadySelected[$childPurchase->getPlanId()]['priceId'] = $childPurchase->getPriceId();
                }
            }

            $noOneSupplementaryGroupNames = array();
            foreach ($groupsWithServices as $group) {
                $isPlanInGroupSelected = false;
                foreach ($group->getPlans() as $supplementaryPlan) {
                    $supplementary = $request->getParameter('supplementary_'.$purchase->getId().'_'.$group->getId());
                    if ($group->isMultipleChoice() && (!is_array($supplementary) || !in_array($supplementaryPlan->getId(), $supplementary))) {
                        continue;
                    }
                    if (!$group->isMultipleChoice() && $supplementaryPlan->getId() != $supplementary) {
                        continue;
                    }

                    $isPlanInGroupSelected = true;

                    list($cycleId, $priceId) = explode("_", $request->getParameter('supplementary_'.$purchase->getId().'_'.$supplementaryPlan->getId().'_cycle_price'));

                    if (isset($alreadySelected[$supplementaryPlan->getId()]) && ($alreadySelected[$supplementaryPlan->getId()]['cycleId'] == $cycleId) && ($alreadySelected[$supplementaryPlan->getId()]['priceId'] == $priceId)) {
                        unset($alreadySelected[$supplementaryPlan->getId()]);
                        continue;
                    }

                    $childPurchase = new Purchase($supplementaryPlan->getId(), $cycleId, $priceId, $supplementaryPlan->productName, $supplementaryPlan->productType);
                    if ($subscriptionId) {
                        $childPurchase->setSubscriptionId($subscriptionId);
                    }
                    if ($coupon = $supplementaryPlan->getPreappliedCoupon()) {
                        $childPurchase->setCoupon($coupon);
                    }
                    $purchase->addPurchase($childPurchase);
                }
                if (!$purchase->isAlreadyPurchased() && !$group->isPossiblyEmpty() && !$isPlanInGroupSelected) {
                    $noOneSupplementaryGroupNames[] = $group->name;
                }
            }

            if ($noOneSupplementaryGroupNames) {
                $this->errorMessage = __('TRANS_FOLLOWING_SUPPLEMENARY_GROUPS_MUST_HAVE_PLANS', array('%NAMES%' => implode(', ', array_unique($noOneSupplementaryGroupNames))));
            }

            // remove unselected plans
            foreach ($alreadySelected as $purchaseData) {
                $purchase->removePurchase($purchaseData['purchaseId']);
            }
            if (is_null($this->errorMessage)) {
                $this->getUser()->setAttribute('showOnlyCertificate', false);
                $this->_router->next();
            }
        }

        // master purchase was found in current cart item, so get supplementaries from corresponding plan
        $selectedServices = array();
        $purchase = $this->selectedPurchase;
        if ($groupsWithServices = $this->plan->getSupplementary($this->_store->getId(), $purchase->getCycleId(), $purchase->getSubscriptionId())) {
            $selectedServices[$purchase->getId()] = array();
            foreach ($groupsWithServices as $groupId => $group) {
                foreach ($group->getPlans() as $supplementaryPlan) {
                    if ($subPurchase = $purchase->getPurchaseByPlanId($supplementaryPlan->getId())) {
                        $selectedServices[$purchase->getId()][$supplementaryPlan->getId()] = array(
                            'cycleId' => $subPurchase->getCycleId(),
                            'priceId' => $subPurchase->getPriceId()
                        );

                    }
                    if ($this->getUser()->getAttribute('showOnlyCertificate')) {
                        if (!$supplementaryPlan->isCertPlan()) {
                            $group->removePlanById($supplementaryPlan->getId());
                        }
                    }
                }
                if (!$group->hasPlans()) {
                    unset($groupsWithServices[$groupId]);
                }
            }
        }

        if (0 == count($groupsWithServices)) {
            $this->_router->next(array(), false);
        }

        $this->purchaseId = $purchase->getId();
        $this->groupsWithServices = $groupsWithServices;
        $this->selectedServices = $selectedServices;
        $this->parentCycleId = $purchase->getCycleId();

       if ($defaultTaxZoneId = isset($this->_store->defaultTaxZoneID) ? $this->_store->defaultTaxZoneID : false) {
            $this->taxZone = ResourceLocator::getInstance()->billingTaxZoneService->getById($defaultTaxZoneId);
        }
    }
}