<?php
class BuyAction extends CatalogAction
{
    public function execute($request)
    {
        $plan = $this->_store->getPlan($request->getParameter('plan'));
        $this->getUser()->setAttribute('showOnlyCertificate', false);

        $cart = ResourceLocator::getInstance()->cart;
        $cartItem = new CartItem();

        if (BillingPlan::TYPE_DOMAIN != $plan->productType) {
            $price = $plan->getPriceById($request->getParameter('price'));
            $purchase = new Purchase($plan->getId(), $price->cycleID, $price->getId(), $plan->productName, $plan->productType);
            if ($coupon = $plan->getPreappliedCoupon()) {
                $purchase->setCoupon($coupon);
            }
            $cartItem->addPurchase($purchase);
        }
        $cart->setCurrentItem($cartItem);

        $this->_attachSiteBuilderParams($cart);
        $this->_attachAffiliateProgramParams($cart);

        if (BillingPlan::TYPE_DOMAIN != $plan->productType) {
            $this->_addAddonsToPurchaseFromRequest($request, $plan, $purchase);
            $steps = $this->_getSteps($plan, $purchase);
        } else {
            $this->_addDomainsToCartFromRequest($request, $plan, $cart);
            $steps = $this->_getDomainSteps($request, $plan);
        }

        $router = new Router();
        $router->init($steps, new RouterPage('catalog', array('container' => $this->_container)));

        ResourceLocator::getInstance()->routersStack->push($router);

        // go to next page, but supress redirection via router method,
        // instead this below we apply workaround to avoid caching of this action
        $router->next(array(), true, true);

        // set no cache and localtion headers manually, because simple $this->redirect call entail clear
        // of http headers, so some browsers may cache result of this action and prevent it execution
        sfContext::getInstance()->getConfiguration()->loadHelpers('Url');
        $response = $this->getResponse();
        $response->addCacheControlHttpHeader('no-cache');
        $response->setHttpHeader('Location', url_for(reset($steps)->getUrl()));

        // send only headers to perform redirect
        return sfView::HEADER_ONLY;
    }

    private function _attachSiteBuilderParams(Cart $cart)
    {
        if ($tryBeforeBuySiteBuilderParameters = $this->getUser()->getAttribute('tryBeforeBuySiteBuilderParameters')) {
            $cart->addParameter('tryBeforeBuySiteBuilderParameters', $tryBeforeBuySiteBuilderParameters);
            $this->getUser()->getAttributeHolder()->remove('tryBeforeBuySiteBuilderParameters');
        }
    }

    private function _attachAffiliateProgramParams(Cart $cart)
    {
        if (($partner = $this->getUser()->getAttribute('partner')) && ($program = $this->getUser()->getAttribute('program'))) {
            $cart->addParameter('partner', $partner);
            $this->getUser()->getAttributeHolder()->remove('partner');
            $cart->addParameter('program', $program);
            $this->getUser()->getAttributeHolder()->remove('program');
        }
    }

    private  function _addAddonsToPurchaseFromRequest(sfWebRequest $request, BillingPlan $plan, Purchase $purchase)
    {
        if ($plan->productAddonRequired != BillingPlan::ADDON_NOT_NEEDED) {
            if (($addons = $request->getParameter('addons')) && $purchase) {
                $addonIds = array_unique(explode(',', $addons));
                $groupAddons = array();
                foreach ($addonIds as $addonId) {
                    $billingAddon = ResourceLocator::getInstance()->BillingAddonService->getById(
                        $addonId,
                        $purchase->getPlanId(),
                        $this->_store->getId(),
                        $purchase->getCycleId()
                    );
                    $addon = new Addon(
                        $addonId,
                        $purchase->getPlanId(),
                        $purchase->getCycleId(),
                        $billingAddon->getPrice($purchase->getCycleId(), $this->_store->currencyID)->getId(),
                        $billingAddon->productName,
                        $billingAddon->productType
                    );
                    $groupAddons[$billingAddon->productAddonGroupID][] = $addon;
                }
                foreach ($groupAddons as $groupId => $addons) {
                    $addonGroup = $plan->getAddonGroup($groupId, $this->_store->getId(), $purchase->getCycleId());
                    if ($addonGroup->multichoice()) {
                        foreach ($addons as $addon) {
                            $purchase->addAddon($groupId, $addon);
                        }
                    } else {
                        $purchase->addAddon($groupId, reset($addons));
                    }
                }
            }
        }
    }

    private function _getSteps(BillingPlan $plan, Purchase $purchase)
    {
        $steps = array();

        $params = array('purchaseId' => $purchase->getId());
        if ($plan->productDomainRequired == BillingPlan::DOMAIN_NOT_NEEDED) {
            array_unshift($steps, new RouterPage('select-services', $params));
            array_unshift($steps, new RouterPage('select-parent-service', $params));
        } else {
            array_unshift($steps, new RouterPage('select-services', $params));
            array_unshift($steps, new RouterPage('select-parent-service', $params));
            array_unshift($steps, new RouterPage('register-domain', $params));
            array_unshift($steps, new RouterPage('select-domain', $params));
        }

        foreach(array('prepare-select-addons', 'add-purchase', 'summary', 'contact-information') as $item) {
            $steps[] = new RouterPage($item);
        }
        $steps[] = new RouterPage('select-subscription', $params);
        $steps[] = new RouterPage('order-details');
        $steps[] = new RouterPage('confirmation');
        $steps[] = new RouterPage('thank-you');

        return $steps;
    }

    private function _getDomainSteps(sfWebRequest $request, BillingPlan $plan)
    {
        $steps = array();
        $params = array('domainPlanId' => $plan->getId());

        if (!($domainName = $request->getParameter('domain_name')) || !($domainPrice = $request->getParameter('domain_price'))) {
            array_unshift($steps, new RouterPage('select-parent-service', $params));
            array_unshift($steps, new RouterPage('register-domain', $params));
            array_unshift($steps, new RouterPage('select-domain', $params));
        }

        foreach(array('prepare-select-addons', 'add-purchase', 'summary', 'contact-information') as $item) {
            $steps[] = new RouterPage($item);
        }
        $steps[] = new RouterPage('select-subscription', $params);
        $steps[] = new RouterPage('order-details');
        $steps[] = new RouterPage('confirmation');
        $steps[] = new RouterPage('thank-you');

        return $steps;
    }

    private function _addDomainsToCartFromRequest(sfWebRequest $request, BillingPlan $plan, Cart $cart)
    {
        if (($domainName = $request->getParameter('domain_name')) && ($domainPrice = $request->getParameter('domain_price'))) {
            $domainPlan = $plan instanceof BillingDomainPlan ? $plan : $this->_store->getDefaultDomainPlan(true);
            $dotPosition = strrpos($domainName, '.');
            $sld = substr($domainName, 0, $dotPosition);
            $tld = substr($domainName, $dotPosition + 1);
            $request->setParameter('SLD', $sld);
            $request->setParameter('registerTLDs', array($tld));
            $request->setParameter('price_' . $tld, $domainPrice);
            $duplicatedDomains = array();
            PurchaseManager::purchaseDomains($request, null, $cart, $domainPlan, $duplicatedDomains);
        }
    }
}