<?php

class LoginAction extends sfAction
{
	const TARGET_PURCHASE_DOMAIN = 'purchase-domain';
	const TARGET_PURCHASE_CERTIFICATE = 'purchase-certificate';
    const TARGET_PURCHASE_UPSELL = 'purchase-upsell';

	public function execute($request)
    {
		$shoppingSessionId = $request->getParameter('shoppingSessionId');
		try {
			ResourceLocator::getInstance()->billingContactService->loginByShoppingSessionId($shoppingSessionId);
		} catch (StoreException $exception) {
			$this->logMessage('Unable to login [shoppingSessionId=' . $shoppingSessionId . ']; ' . get_class($exception) . ' message: ' . $exception->getMessage(), 'err');
		}

		$container = $request->getParameter('container');

		// check specified target and forward to appropriate action
		if ($container && $target = $request->getParameter('target')) {
			switch ($target) {
				case self::TARGET_PURCHASE_DOMAIN:
					$this->redirect('@purchase-domain?container=' . $container . '&subscription=' . $request->getParameter('subscription') . '&domain=' . $request->getParameter('domain'));
				case self::TARGET_PURCHASE_CERTIFICATE:
                    ResourceLocator::getInstance()->cart->clean();
                    $params = array();
                    foreach($request->getParameterHolder()->getAll() as $key => $value) {
                        if (in_array($key, $this->_certificateParams)) {
                            $params[$key] = $value;
                        }
                    }
                    $selectCertificateParams = new sfParameterHolder();
                    $selectCertificateParams->add($params);
                    sfContext::getInstance()->getUser()->setAttribute('selectCertificateParams', $selectCertificateParams);
                    $this->redirect('@purchase-certificate?container=' . $container . '&planId=' . $request->getParameter('planId'). '&subscription=' . $request->getParameter('subscription'));
                case self::TARGET_PURCHASE_UPSELL:
                    ResourceLocator::getInstance()->cart->clean();
                    $this->redirect('@purchase-upsell?container=' . $container . '&subscription=' . $request->getParameter('subscription'));
			}          
		}

		// if no valid target specified, redirect to catalog
        if ($catalogUrl = $request->getParameter('catalogUrl')) {
            $this->redirect($catalogUrl);
        } else if ($container) {
			$this->redirect($this->generateUrl('catalog', array('container' => $container)));
		} else {
			$this->redirect('@default');
		}
	}

    private $_certificateParams = array(
        'commonName',
        'countryName',
        'stateOrProvinceName',
        'localityName',
        'organizationName',
        'organizationalUnitName',
        'emailAddress',
    	'backUrl',
    );
}