<?php

abstract class BillingService implements Resource  {
	
	protected $_client;
    protected $_isCacheEnabled = false;
	
	private static $_totalExecutionTime = 0;
	
	protected function __construct(SoapClient $client) {
		$this->_client = $client;
	}
	
	protected static function prepareSoapClient($code) {
		$url = sfConfig::get('app_billing_service_url') . '?service=' . $code;
		return new SoapClient(
			null,
			array(
				'location' => $url,
				'uri' => $url,
				'trace' => 1
			)
		); 
	}

	public function __call($name, $arguments) {
		if (strpos($name, 'execute') === 0) {
			// parse name of method of SOAP service
			$command = substr($name, 7);
			$command{0} = strtolower($command{0});
			// make call of method to SOAP service
			try {
				$time = microtime(true);
                $response = new BillingServiceResponse($this->_makeSoapCall($command, $arguments));
                $executionTime = microtime(true) - $time;
				self::$_totalExecutionTime += $executionTime;
				sfContext::getInstance()->getLogger()->debug('[profile] Duration of command "' . $command . '" is ' . $executionTime . ' ms. (total: ' . self::$_totalExecutionTime . ' ms.)');
                if (!is_null($redirectUrl = $response->getRedirectUrl())) {
                    sfContext::getInstance()->getController()->redirect($redirectUrl);
                    exit;
                }
                return $response;
            }  catch (SoapFault $exception) {
				throw new BillingServiceException($exception->getMessage());
			}
		}
	}

    private function _makeSoapCall($command, $arguments) {
        if (sfConfig::get('app_enable_api_call_cache') && $this->_isCacheEnabled && strpos($command, 'get') === 0) {
            $cache = new sfFileCache(array('cache_dir' => sfConfig::get('sf_cache_dir').'/function'));
            $fc = new sfFunctionCache($cache);
            return $fc->call(array($this->_client, $command), $arguments);
        }
        return call_user_func_array(array($this->_client, $command), $arguments);
    }
}