<?php

class BillingAddon extends BillingProduct {
	
	private $_variants = null;
	
	private $_cycles = null;
	
	public function getVariants() {
		if (is_null($this->_variants)) {
			if (!isset($this->productVariants)) {
				throw new BillingObjectException('Variants section does not exist in plan data');
			}
			$this->_variants = array();
			foreach ($this->productVariants as $data) {
				$variant = new BillingVariant($data);
				$this->_variants[$variant->getId()] = $variant;
			}
		}
		return $this->_variants;
	}
	
	public function getCycles() {
		if (is_null($this->_cycles)) {
			if (!isset($this->cycles)) {
				throw new BillingObjectException('Cycles section does not exist in plan data');
			}
			$this->_cycles = array();
			foreach ($this->cycles as $data) {
				$cycle = new BillingCycle($data);
				$this->_cycles[$cycle->getId()] = $cycle;
			}
		}
		return $this->_cycles;
	}
	
	public function getCycle($id) {
		$cycles = $this->getCycles();
		return isset($cycles[$id]) ? $cycles[$id] : null;
	}
	
	public function getPrice($cycleId, $currencyId) {
        foreach ($this->getVariants() as $variant) {
            if ($result = $variant->getPrice($cycleId, $currencyId)) {
                return $result;
            }
        }

        throw new BillingObjectException('Price for cycle [' . $cycleId . '] not found (or zero) for addon [' . $this->getId() . ']');
	}

    public function getPriceById($id) {
        foreach ($this->getVariants() as $variant) {
            if ($result = $variant->getPriceById($id)) {
                return $result;
            }
        }

        throw new BillingObjectException('Price [' . $id . '] not found (or zero) for addon [' . $this->getId() . ']');
    }

    public function getPricesCount($currencyId) {
        $count = 0;
        foreach ($this->getVariants() as $variant) {
            $count += $variant->getPricesCount($currencyId);
        }
        return $count;
    }
	
	protected function _extractId() {
		return $this->productID;
	}
	
}