<?php

class CartItem {

    private $_id;
    private $_purchases;

    public function __construct()
    {
        $this->_id = ResourceLocator::getInstance()->cart->generateItemId();
        $this->_purchases = array();
    }

    public function getId() {
        return $this->_id;
    }

    public function getPurchases($recursively = false) {
        if ($recursively) {
            $result = array();
            foreach ($this->_purchases as $purchase) {
                $result[$purchase->getId()] = $purchase;
                foreach ($purchase->getPurchases(true) as $subPurchase) {
                    $result[$subPurchase->getId()] = $subPurchase;
                }
            }
            return $result;
        } else {
            return $this->_purchases;
        }
    }

    public function getAddons() {
        $result = array();
        foreach ($this->_purchases as $purchase) {
            $result = array_merge($result, $purchase->getAddons(true));
        }
        return $result;
    }

    public function hasPurchase($id) {
        return isset($this->_purchases[$id]) ? true : false;
    }

    public function getPurchase($id) {
        if (isset($this->_purchases[$id])) {
            return $this->_purchases[$id];
        }
        foreach ($this->_purchases as $purchase) {
            if ($target = $purchase->getPurchase($id)) {
                return $target;
            }
        }
        return null;
    }

    public function addPurchase(Purchase $purchase) {
        $this->_purchases[$purchase->getId()] = $purchase;
    }

    public function removePurchase($purchaseId) {
        if (isset($this->_purchases[$purchaseId])) {
            unset($this->_purchases[$purchaseId]);
        } else {
            foreach ($this->_purchases as $purchase) {
                $purchase->removePurchase($purchaseId);
            }
        }
    }
}