<?php

function formatPrice($amount, $currencyId) {
	$formatter = new NumberFormatter(sfContext::getInstance()->getUser()->getCulture(), NumberFormatter::CURRENCY);
	return $formatter->formatCurrency($amount, $currencyId);
}

function formatRate($rate) {
	$formatter = new NumberFormatter(sfContext::getInstance()->getUser()->getCulture(), NumberFormatter::PERCENT);
	$formatter->setAttribute(NumberFormatter::MAX_FRACTION_DIGITS, 2);
	return $formatter->format($rate, NumberFormatter::TYPE_DOUBLE);
}

function formatTax(BillingTaxZone $taxZone, $currencyId = false) {
	if (!(int)$taxZone->taxZoneAmountType) {
		$formatter = new NumberFormatter(sfContext::getInstance()->getUser()->getCulture(), NumberFormatter::PERCENT);
		$formatter->setAttribute(NumberFormatter::MAX_FRACTION_DIGITS, 2);
		return $formatter->format((float)$taxZone->taxZoneAmount / 100, NumberFormatter::TYPE_DOUBLE);
	} else {
		$formatter = new NumberFormatter(sfContext::getInstance()->getUser()->getCulture(), NumberFormatter::CURRENCY);
		$formatter->setAttribute(NumberFormatter::MAX_FRACTION_DIGITS, 2);
		return $formatter->formatCurrency((float)$taxZone->taxZoneAmount, $currencyId);
	}
}

function formatProductName($productName, $maxLength = 20) {
	$words = explode(' ', $productName);
	$resultWords = array();
	foreach ($words as $word) {
		$leftLength = (int)$maxLength / 2;
		$rightLength = $maxLength - $leftLength;
		$resultWords[] = (
			mb_strlen($word, "UTF-8") > $maxLength ?
			mb_substr($word, 0, $leftLength, "UTF-8") . "..." . mb_substr($word, mb_strlen($word, "UTF-8") - $rightLength, mb_strlen($word, "UTF-8"), "UTF-8") :
			$word
		);
	}
	return implode(' ', $resultWords);
}

function formatContinueShoppingUrl($container) {
	if ($widget = ResourceLocator::getInstance()->billingWidgetService->getByCode($container)) {
		return $widget->continueShoppingUrl;
	}

	if ($store = ResourceLocator::getInstance()->billingStoreService->getByCode($container)) {
		return $store->continueShoppingUrl;
	}

	return url_for('@catalog?container='.$container);
}

function formatCycle($name, $supressOutput = false) {
	if ($supressOutput) {
		return '';
	}
	sfContext::getInstance()->getConfiguration()->loadHelpers('I18N');
	return ' ' . __('TRANS_STORE_FOR_CYCLE') . ' ' . __($name);
}