<?php

function preparePlanAttributes(BillingPlan $plan, $formatType = false) {
	$moduleName = sfContext::getInstance()->getModuleName();
	$attributesDescriptor = sfConfig::get('mod_' . $moduleName . '_plan_attributes', array());
	$attributes = array();
	foreach ($attributesDescriptor as $name => $descriptor) {
        $attrName = getRealPlanAttrName($name);
		if (isset($plan->attributes[$attrName])) {
			$value = $plan->attributes[$attrName];
			
			$display = isset($descriptor['display']) ? $descriptor['display'] : true;

			if (!$display) {
				continue;
			}
			
			$constraints = isset($descriptor['constraints']) ? $descriptor['constraints'] : array();

            $skip = false;
			if ($constraints) {
				foreach ($constraints as $constraint) {
					if (isset($constraint['type'])) {
						switch ($constraint['type']) {
							case 'value':
								$comparison = isset($constraint['comparison']) ? $constraint['comparison'] : '';
								$condition = isset($constraint['condition']) ? $constraint['condition'] : '';
								switch ($comparison) {
									case 'above':
										if ($value <= $condition) {
											$skip = true;
										}
										break;
									case 'above-equal':
										if ($value < $condition) {
											$skip = true;
										}
										break;
									case 'below':
										if ($value >= $condition) {
											$skip = true;
										}
										break;
									case 'below-equal':
										if ($value > $condition) {
											$skip = true;
										}
										break;
									case 'not-equal':
										if ($value == $condition) {
											$skip = true;
										}
										break;
									default:
										if ($value != $condition) {
											$skip = true;
										}
								}
								break;
						}
					}
				}
			}
			if ($skip) {
				continue;
			}
			
			if ($formatType) {
				$format = isset($descriptor[$formatType . 'Format']) ? $descriptor[$formatType . 'Format'] : sfConfig::get('mod_' . $moduleName . '_plan_defaultAttributes' . ucfirst($formatType) . 'Format');
			} else {
				$format = isset($descriptor['format']) ? $descriptor['format'] : sfConfig::get('mod_' . $moduleName . '_plan_defaultAttributesFormat');
			}


			$attributes[$name] = formatPlanAttributeValue($name, $format, $formatType, $plan);
		}
	}
	return $attributes;
}

function formatPlanAttributeValue($name, $format, $formatType, $plan) {
	sfContext::getInstance()->getConfiguration()->loadHelpers('I18N');
	
	$moduleName = sfContext::getInstance()->getModuleName();
	$attributesDescriptor = sfConfig::get('mod_' . $moduleName . '_plan_attributes', array());
	// set special format in case if there are suitable replacement of target attribute value

	$targetAttributeDescriptor = $attributesDescriptor[$name];
	$targetAttributeReplacements = isset($targetAttributeDescriptor['replacements']) ? $targetAttributeDescriptor['replacements'] : array();
    foreach ($targetAttributeReplacements as $replacement) {
		if ($replacement['from'] == $plan->attributes[getRealPlanAttrName($name)]) {
			if ($formatType) {
				$format = sfConfig::get('mod_' . $moduleName . '_plan_defaultAttributes' . ucfirst($formatType) . 'Format');
			} else {
				$format = sfConfig::get('mod_' . $moduleName . '_plan_defaultAttributesFormat');
			}

			break;
		}
	}
	$output = '';
	foreach ($format as $item) {
		$tmp = explode('#', $item);
		$transformFunction = isset($tmp[1]) ? $tmp[1] : false;
		$tmp = explode('@', $tmp[0]);
		if (sizeof($tmp) == 1) {
			$output .= trim($tmp[0]) ? __($tmp[0]) : $tmp[0];
			continue;
		} else {
			$attributeName = $tmp[0] ? $tmp[0] : $name;
            $realAttrName = getRealPlanAttrName($attributeName);
			$attributeItem = $tmp[1];
			$field = '';
			switch ($attributeItem) {
				case 'name':
					$field = getLocalePlanAttrName($realAttrName);
					break;
				case 'value':

					$descriptor = $attributesDescriptor[$attributeName];
					$field = $plan->attributes[$realAttrName];
					$replacements = isset($descriptor['replacements']) ? $descriptor['replacements'] : array();
					if ($replacements) {
						foreach ($replacements as $replacement) {
							$from = isset($replacement['from']) ? $replacement['from'] : '';
							$to = isset($replacement['to']) ? $replacement['to'] : '';
							if ($field == $from) {
								$field = __($to);
								break;
							}
						}
					}

					$type = isset($descriptor['type']) ? $descriptor['type'] : false;
					switch ($type) {
						case 'double':
							$field = (double)$field;
							break;
						case 'integer':
							$field = (int)$field;
							break;
						case 'string':
							$field = (string)$field;
							break;
						case 'boolean':
							$field = $field ? true : false;
							break;
						case 'empty':
							$field = null;
							break;
					}
					
					break;
			}
			$output .= function_exists($transformFunction) ? $transformFunction($field) : $field;
		}
	}
	return $output;
}

function getRealPlanAttrName($name)
{
    return str_replace('__dot', '.', $name);
}

function getLocalePlanAttrName($name)
{
    return __('TRANS_PACKAGE_ATTRIBUTES__' . strtoupper(str_replace(array('.','-'), '_', $name)));
}