#!/bin/bash

CONFIG=
VERBOSE=0

usage()
{
	echo "Usage: run-all.sh [--verbose] [--help] --config path\to\config\file";
}

log()
{
	if [ 1 -eq $VERBOSE ]; then
		echo $1
	fi
}

while true; do
	case "$1" in
		--verbose) VERBOSE=1; shift;;
		--help) usage; exit 0;;
		--config) CONFIG="$2"; shift 2;;
		"") break ;;
	esac
done

if [ -z "$CONFIG" ]; then
	usage
	exit 1
fi
if [ ! -f "$CONFIG" ]; then
	echo "Config file does not exist: $CONFIG"
	exit 1
fi

while IFS="=" read -r key value; do
	if [[ "$key" =~ 'authToken' ]]; then
		TOKEN=${value//[\" ]/}
	elif [[ "$key" =~ 'url' ]]; then
		URL=${value//[\" ]/}
	fi
done < $CONFIG

while true; do
	REQUEST='{"jsonrpc":"2.0","method":"runAll","params":[],"id":null}'
	RESPONSE=`curl -k -s -S -w ' {"http_code": %{http_code}}' \
		-X POST \
		-H "Authorization: ApplicationToken $TOKEN" \
		-H 'Expect: ' \
		-d $REQUEST \
		"$URL"`
	log "Request: $REQUEST"

	if [ "0" != "$?" ]; then
		exit 1
	fi

	log "Response: $RESPONSE"
	if [[ ! "$RESPONSE" =~ '"http_code": 200' ]]; then
		echo ${RESPONSE/% \{\"http_code\": [0-9]*\}/}
		exit 1
	fi
	if [[ "$RESPONSE" =~ '"error":\{' ]] || [[ "$RESPONSE" =~ '"error":{' ]]; then
		echo ${RESPONSE/% \{\"http_code\": [0-9]*\}/}
		exit 1
	fi

	if [[ ! "$RESPONSE" =~ '"result":1' ]]; then
		exit 0
	fi
done
